@@ -98,7 +98,7 @@ ident ( ) {
     fi
 
     # extract version from CVS Id string
-    __id="$Id: autogen.sh 15874 2007-09-13 01:45:41Z brlcad $"
+    __id="$Id: autogen.sh 21940 2011-06-18 02:51:16Z bullet_catcher $"
     __version="`echo $__id | sed 's/.*\([0-9][0-9][0-9][0-9]\)[-\/]\([0-9][0-9]\)[-\/]\([0-9][0-9]\).*/\1\2\3/'`"
     if [ "x$__version" = "x" ] ; then
 	__version=""
@@ -1449,6 +1449,10 @@ if [ "x$reconfigure_manually" = "xyes" ] ; then
     recursive_manual_autogen
 fi
 
+# ugly hack to provide curl with the "missing" script
+if [ -x "${_aux_dir}/missing" -a ! -f src/other/curl/missing ] ; then
+    cp -p "${_aux_dir}/missing" src/other/curl
+fi
 
 #########################
 # restore and summarize #
