@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -28,6 +28,7 @@
 
 // local headers
 #include "common.h"
+#include "global.h"
 #include "network.h"
 #include "Pack.h"
 
@@ -70,9 +71,9 @@ const int		ServerIdPLen = 8;
 const PlayerId		NoPlayer = 255;
 const PlayerId		AllPlayers = 254;
 const PlayerId		ServerPlayer = 253;
-const PlayerId		AdminPlayers = 252; // These two ids are unused at present
-const PlayerId		UnusedSpecialPlayer2 = 251; // Available for special needs in the future
-const PlayerId		LastRealPlayer = 243;
+const PlayerId		AdminPlayers = 252;
+const PlayerId		FirstTeam = 251;
+const PlayerId		LastRealPlayer = FirstTeam - NumTeams;
 
 class ServerId {
   public:
