@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -21,27 +21,27 @@ class BZDBCache
   public:
     static void init();
 
-    // prohibit external write access    
+    // prohibit external write access
     template <class T>
     class ReadOnly {
       friend class BZDBCache;
       public:
-        inline operator T() const { return data; }
+	inline operator T() const { return data; }
       private:
-        ReadOnly() {}
-        ReadOnly& operator=(const T& value) { data = value; return *this; }
+	ReadOnly() {}
+	ReadOnly& operator=(const T& value) { data = value; return *this; }
       private:
-        ReadOnly(const ReadOnly&);
-        ReadOnly& operator=(const ReadOnly&);
+	ReadOnly(const ReadOnly&);
+	ReadOnly& operator=(const ReadOnly&);
       private:
-        T data;
+	T data;
     };
 
     // our basics types
     typedef ReadOnly<int>   Int;
     typedef ReadOnly<bool>  Bool;
     typedef ReadOnly<float> Float;
-      
+
     static Bool  displayMainFlags;
     static Bool  blend;
     static Bool  texture;
@@ -59,6 +59,7 @@ class BZDBCache
     static Bool  leadingShotLine;
     static Float linedRadarShots;
     static Float sizedRadarShots;
+    static Float shotLength;
     static Float pulseRate;
     static Float pulseDepth;
     static Bool  showCollisionGrid;
@@ -85,12 +86,14 @@ class BZDBCache
     static Float flagPoleSize;
     static Float flagPoleWidth;
 
+    static Float gmSize;
+
     static Float hudGUIBorderOpacityFactor;
 
   public:
     /** public method to update cached variable
-        has to be called at best opportunity
-        (e.g. at beginnig of main loop)
+	has to be called at best opportunity
+	(e.g. at beginnig of main loop)
     */
     static void update();
 
