@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* BzfDisplay:
@@ -29,6 +29,7 @@ class BzfDisplay {
     virtual bool	isValid() const = 0;
     virtual bool	isEventPending() const = 0;
     virtual bool	getEvent(BzfEvent&) const = 0;
+    virtual bool	peekEvent(BzfEvent&) const = 0;
 
     virtual bool	hasGetKeyMode() {return false;};
     virtual void	getModState(bool &shift, bool &control, bool &alt) {
