@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /** @file
@@ -109,26 +109,31 @@ typedef std::set<FlagType*> FlagSet;
 /** This class represents a flagtype, like "GM" or "CL". */
 class FlagType {
 public:
-  FlagType( const char* name, const char* abbv, FlagEndurance _endurance,
-	    ShotType sType, FlagQuality quality, TeamColor team, const char* help ) {
-    flagName = name;
-    flagAbbv = abbv;
+  FlagType( const std::string& name, const std::string& abbv, FlagEndurance _endurance,
+	    ShotType sType, FlagQuality quality, TeamColor team, const std::string& help,
+	    bool _custom = false ) :
+    flagName(name),
+    flagAbbv(abbv),
+    flagHelp(help)
+  {
     endurance = _endurance;
     flagShot = sType;
     flagQuality = quality;
-    flagHelp = help;
     flagTeam = team;
+    custom = _custom;
 
     /* allocate flagset array on first use to work around mipspro
      * std::set compiler bug of making flagSets a fixed array.
      */
     if (flagSets == NULL) {
-      flagSets = new FlagSet[2];
+      flagSets = new FlagSet[NumQualities];
     }
 
+    if (custom)
+      customFlags.insert(this);
+
     flagSets[flagQuality].insert(this);
     getFlagMap()[flagAbbv] = this;
-    flagCount++;
   }
 
   /** returns a label of flag name and abbreviation with the flag name
@@ -138,7 +143,7 @@ public:
 
   /** returns information about a flag including the name, abbreviation, and
    * description.  format is "name ([+|-]abbrev): description" where +|-
-   * indicates whether the flag is inherintly good or bad by default.
+   * indicates whether the flag is inherently good or bad by default.
    */
   const std::string information() const;
 
@@ -148,25 +153,28 @@ public:
   /** network serialization */
   void* pack(void* buf) const;
   void* fakePack(void* buf) const;
+  void* packCustom(void* buf) const;
 
   /** network deserialization */
   static void* unpack(void* buf, FlagType* &desc);
+  static void* unpackCustom(void* buf, FlagType* &desc);
 
   /** Static wrapper function that makes sure that the flag map is
    * initialized before it's used.
    */
   static FlagTypeMap& getFlagMap();
 
-  const char* flagName;
-  const char* flagAbbv;
+  const std::string flagName;
+  const std::string flagAbbv;
+  const std::string flagHelp;
   FlagEndurance	endurance;
-  const char* flagHelp;
   FlagQuality flagQuality;
   ShotType flagShot;
   TeamColor flagTeam;
+  bool custom;
 
-  static int flagCount;
   static FlagSet *flagSets;
+  static FlagSet customFlags;
   static const int packSize;
 };
 
@@ -235,12 +243,15 @@ namespace Flags {
     *Wings, *ReverseControls, *Agility,
     *Colorblindness, *Obesity, *LeftTurnOnly, *RightTurnOnly, *Momentum,
     *Blindness, *Jamming, *WideAngle, *NoJumping, *TriggerHappy,
-    *ReverseOnly, *ForwardOnly, *Bouncy, *Lag;
+    *ReverseOnly, *ForwardOnly, *Bouncy;
 
   /** This function initializes all the FlagType objects in the Flags
       namespace. */
   void init();
   void kill();
+
+  /** Clear all the custom flags (i.e. when switching servers) */
+  void clearCustomFlags();
 }
 
 #endif // BZF_FLAG_H
