@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -38,7 +38,7 @@ class MeshFace : public Obstacle {
     MeshFace(MeshObstacle* mesh, int vertexCount,
 	     float** vertices, float** normals, float** texcoords,
 	     const BzMaterial* material, int physics,
-	     bool noclusters, bool smoothBounce, bool drive, bool shoot);
+	     bool noclusters, bool smoothBounce, bool drive, bool shoot, bool ricochet);
     ~MeshFace();
 
     const char* getType() const;
@@ -114,8 +114,8 @@ class MeshFace : public Obstacle {
     bool noclusters;
     int phydrv;
 
-    fvec4 plane;
-    fvec4* edgePlanes;
+    afvec4 plane;
+    afvec4* edgePlanes;
 
     MeshFace* edges; // edge 0 is between vertex 0 and 1, etc...
 		     // not currently used for anything
