@@ -1,9 +1,9 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
@@ -38,7 +38,7 @@ namespace MsgStrings {
   void init();
   void reset();			// clean up all tracked state
   void useDNS(bool);		// look up hostnames based on IPs?
-  void showEmail(bool);		// show player emails next to names?
+  void showMotto(bool);		// show player mottos next to names?
   void colorize(bool);		// use ANSI color codes?
   void trackState(bool);	// track game state?
 
