@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -62,6 +62,7 @@ class GroupInstance {
     void setMaterial(const BzMaterial*);
     void setDriveThrough();
     void setShootThrough();
+    void setCanRicochet();
     void addMaterialSwap(const BzMaterial* src,
 			 const BzMaterial* dst);
 
@@ -91,6 +92,7 @@ class GroupInstance {
     const BzMaterial* material;
     bool driveThrough;
     bool shootThrough;
+    bool ricochet;
     MaterialMap matMap;
 };
 
