@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -24,8 +24,8 @@
 #include "multicast.h"
 
 
-// 4 uint16's and 13 uint8's hex encoded
-static const int	PingPacketHexPackedSize = 4 * 7 + 2 * 13;
+// 8 uint16's and 13 uint8's hex encoded
+static const int	PingPacketHexPackedSize = 4 * 8 + 2 * 13;
 
 
 /** PingPacket:
@@ -57,7 +57,8 @@ class PingPacket {
   public:
     ServerId		serverId;
     Address		sourceAddr;
-    uint16_t		gameStyle;
+    uint16_t	    gameOptions;
+    uint16_t		gameType;
     uint16_t		maxShots;
     uint16_t		shakeWins;
     uint16_t		shakeTimeout;		// 1/10ths of second
