@@ -1,9 +1,9 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
@@ -29,6 +29,7 @@
 #include "Protocol.h"
 #include "Flag.h"
 #include "WordFilter.h"
+#include "StateDatabase.h"
 
 
 enum ClientState {
@@ -80,9 +81,13 @@ public:
   void  *packUpdate(void *buf);
   void  *packId(void *buf);
   bool	unpackEnter(void *buf, uint16_t &rejectCode, char *rejectMsg);
+  bool  processEnter ( uint16_t &rejectCode, char *rejectMsg );
   const char *getCallSign() const;
-  const char *getEMail() const;
+  void setCallSign(const char * c);
+  const char *getMotto() const;
+  void  setMotto(const char* _motto);
   const char *getToken() const;
+  void setToken(const char * c);
   void	clearToken();
   void       *packVirtualFlagCapture(void *buf);
   bool	isTeam(TeamColor team) const;
@@ -98,6 +103,7 @@ public:
   void	setFlag(int flag);
   bool	isFlagTransitSafe();
   const char *getClientVersion();
+  void setClientVersion(const char * c);
   void getClientVersionNumbers(int& major, int& minor, int& revision);
   std::string getIdleStat();
   bool	canBeRabbit(bool relaxing = false);
@@ -107,7 +113,12 @@ public:
   bool	hasStartedToNotRespond();
   void	hasSent();
   bool	hasPlayedEarly();
-  bool	hasSentEnter() {return validEnter;}
+  void  setCompletelyAdded() {completelyAdded = true;}
+  bool	isCompletelyAdded() {return completelyAdded;}
+  TimeKeeper  getNextSpawnTime() const;
+  void	setSpawnDelay(double delay);
+  bool	waitingToSpawn() const;
+  void	queueSpawn();
   void	setPlayedEarly(bool early = true);
   void	setReplayState(PlayerReplayState state);
   void	updateIdleTime();
@@ -121,14 +132,16 @@ public:
   unsigned short int trackerID();
   static TimeKeeper now;
   int endShotCredit;
+  int endShotShieldCredit;
 
   PlayerType getType( void ) {return type;}
+  void setType( PlayerType t ) {type = t;}
 
 private:
 
-  void	cleanEMail();
+  void	cleanMotto();
   bool	isCallSignReadable();
-  bool	isEMailReadable();
+  bool	isMottoReadable();
 
   int	 playerIndex;
 
@@ -136,14 +149,16 @@ private:
 
   // current state of player
   ClientState state;
+  // Need to know if entered is already done
+  bool       hasDoneEntering;
   // type of player
   PlayerType type;
   // player's pseudonym
   char callSign[CallSignLen];
   // token from db server
   char token[TokenLen];
-  // player's email address
-  char email[EmailLen];
+  // player's motto
+  char motto[MottoLen];
   // version information from client
   char clientVersion[VersionLen];
   int clientVersionMajor;
@@ -157,11 +172,17 @@ private:
   // flag index player has
   int flag;
 
-  // has the player sent an enter
-  bool validEnter;
+  // addPlayer has completed for this player
+  bool completelyAdded;
 
   TimeKeeper lastFlagDropTime;
 
+  // time of player's next allowed spawn
+  TimeKeeper nextSpawnTime;
+
+  // Requested a spawn?
+  bool wantsToSpawn;
+
 
   // spam prevention
   std::string lastMsgSent;
@@ -204,7 +225,7 @@ inline bool PlayerInfo::isPlaying() const {
 }
 
 inline bool PlayerInfo::isHuman() const {
-  return type == TankPlayer;
+  return (type == TankPlayer) && hasDoneEntering;
 }
 
 inline bool PlayerInfo::haveFlag() const {
@@ -256,6 +277,24 @@ inline bool PlayerInfo::isARabbitKill(PlayerInfo &victim) const {
   return wasRabbit || victim.team == RabbitTeam;
 }
 
+inline TimeKeeper PlayerInfo::getNextSpawnTime() const {
+  return nextSpawnTime;
+}
+
+inline void PlayerInfo::setSpawnDelay(double delay) {
+  nextSpawnTime = TimeKeeper::getCurrent();
+  nextSpawnTime += delay;
+}
+
+inline bool PlayerInfo::waitingToSpawn() const {
+  return wantsToSpawn;
+}
+
+inline void PlayerInfo::queueSpawn() {
+  wantsToSpawn = true;
+}
+
+
 #endif
 
 // Local Variables: ***
