@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -25,7 +25,7 @@ class PyramidBuilding : public Obstacle {
   public:
 			PyramidBuilding();
 			PyramidBuilding(const float* pos, float rotation,
-				float width, float breadth, float height, bool drive = false, bool shoot = false);
+				float width, float breadth, float height, bool drive = false, bool shoot = false, bool ricochet = false);
 			~PyramidBuilding();
 
     virtual Obstacle*	copyWithTransform(const MeshTransform&) const;
