@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* QuadWallSceneNode:
@@ -27,7 +27,8 @@ class QuadWallSceneNode : public WallSceneNode {
 				const GLfloat tEdge[3],
 				float uRepeats = 1.0,
 				float vRepeats = 1.0,
-				bool makeLODs = true);
+				bool makeLODs = true,
+				bool fixedUVs = false);
 			QuadWallSceneNode(const GLfloat base[3],
 				const GLfloat sEdge[3],
 				const GLfloat tEdge[3],
@@ -42,14 +43,15 @@ class QuadWallSceneNode : public WallSceneNode {
 
     void		addRenderNodes(SceneRenderer&);
     void		addShadowNodes(SceneRenderer&);
+    void		renderRadar();
+
 
     bool		inAxisBox (const Extents& exts) const;
 
     int			getVertexCount () const;
     const		GLfloat* getVertex (int vertex) const;
 
-    RenderNode*	getRenderNode(int) { return nodes[0]; }
-    int		getRenderNodeCount() { return 1; }
+    virtual void	getRenderNodes(std::vector<RenderSet>& rnodes);
 
   private:
     void		init(const GLfloat base[3],
@@ -59,7 +61,7 @@ class QuadWallSceneNode : public WallSceneNode {
 				float vOffset,
 				float uRepeats,
 				float vRepeats,
-				bool makeLODs);
+				bool makeLODs, bool fixedUVs);
 
   protected:
     class Geometry : public RenderNode {
@@ -71,7 +73,8 @@ class QuadWallSceneNode : public WallSceneNode {
 				const GLfloat vEdge[3],
 				const GLfloat* normal,
 				float uOffset, float vOffset,
-				float uRepeats, float vRepeats);
+				float uRepeats, float vRepeats,
+				bool fixedUVs);
 			~Geometry();
 	void		setStyle(int _style) { style = _style; }
 	void		render();
@@ -90,6 +93,7 @@ class QuadWallSceneNode : public WallSceneNode {
       public:
 	GLfloat3Array	vertex;
 	GLfloat2Array	uv;
+	int	     triangles;
     };
 
   private:
