@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2004 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /*
@@ -17,30 +17,41 @@
 #ifndef BZF_MYIO
 #define BZF_MYIO
 
-void formatDebug(const char* fmt, ...);
+#include "common.h"
+
+void setDebugTimestamp (bool enable, bool doMicros, bool utc);
+void logDebugMessage(int level, const char* fmt, ...);
+
+class LoggingCallback
+{
+public:
+	virtual ~LoggingCallback(){};
+
+	virtual void log ( int level, const char* message ) = 0;
+};
+
+extern LoggingCallback	*loggingCallback;
 
 /* egcs headers on linux define NULL as (void*)0.  that's a no no in C++. */
 #if defined(NULL)
-#undef NULL
+#  undef NULL
 #endif
 #define NULL 0
 
 /* insert any other broken OS conditionals here */
-#if !defined(_WIN32)
-#endif
 #include <iostream>
 
-extern int debugLevel;
-
 #endif
 
 
+
 /*
  * Local Variables: ***
- * mode: C++ ***
+ * mode:C++ ***
  * tab-width: 8 ***
  * c-basic-offset: 2 ***
  * indent-tabs-mode: t ***
  * End: ***
  * ex: shiftwidth=2 tabstop=8
  */
+
