@@ -1,14 +1,14 @@
 /* bzflag
-* Copyright (c) 1993 - 2006 Tim Riker
-*
-* This package is free software;  you can redistribute it and/or
-* modify it under the terms of the license found in the file
-* named COPYING that should have accompanied this file.
-*
-* THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
-* IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
-* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-*/
+ * Copyright (c) 1993-2011 Tim Riker
+ *
+ * This package is free software;  you can redistribute it and/or
+ * modify it under the terms of the license found in the file
+ * named COPYING that should have accompanied this file.
+ *
+ * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ */
 
 /*
 * Communication protocol constants
@@ -25,7 +25,7 @@
 
 #define _CallSignLen		32		// including terminating NUL
 #define _PasswordLen		32		// including terminating NUL
-#define _EmailLen				128		// including terminating NUL
+#define _MottoLen				128		// including terminating NUL
 #define _TokenLen				22		// opaque string (now int(10)) and terminating NUL
 #define _VersionLen			60		// including terminating NUL
 #define _MessageLen			128		// including terminating NUL
@@ -45,7 +45,7 @@ public:
 	int losses;
 	int	tks;
 	std::string callsign;
-	std::string email;
+	std::string motto;
 };
 
 #endif // BZF_MESSAGES_H
