@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 #
 # bzflag
-# Copyright (c) 1993 - 2005 Tim Riker
+# Copyright (c) 1993-2011 Tim Riker
 #
 # This package is free software;  you can redistribute it and/or
 # modify it under the terms of the license found in the file
@@ -9,7 +9,7 @@
 #
 # THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 # IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
-# WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
 =pod
 
@@ -56,6 +56,9 @@ $server = pack($sockaddr, AF_INET, $port, $serveraddr);
 die $! unless socket(S, AF_INET, SOCK_STREAM, $proto);
 die $! unless connect(S, $server);
 
+# Send header
+print S "BZFLAG\r\n\r\n";
+
 # don't buffer
 select(S); $| = 1; select(STDOUT);
 
@@ -67,7 +70,7 @@ die $! unless sysread(S, $buffer, 9) == 9;
 
 # quit if version isn't valid
 die "not a bzflag server" if ($magic ne "BZFS");
-die "incompatible version" if ($protocol ne "0025");
+die "incompatible version" if ($protocol ne "0221");
 
 # quit if rejected
 die "rejected by server" if ($id == 255);
@@ -75,32 +78,42 @@ die "rejected by server" if ($id == 255);
 # send game request
 print S pack("n2", 0, 0x7167);
 
-# get reply
-die $! unless sysread(S, $buffer, 46) == 46;
-($len,$code,$style,$maxPlayers,$maxShots,
+my $nbytes = sysread(S, $buffer, 46);
+if ($nbytes == 12) {  # if MsgGameTime rxed ... ignore it
+  my ($len, $code) = unpack("n2", $buffer);
+  $nbytes = sysread(S, $buffer, 46);
+}
+if ($nbytes != 46) {
+  die "Error: $nbytes bytes, expecting 46: $^E\n";
+}
+
+($len,$code,$style,$options,$maxPlayers,$maxShots,
 	$rogueSize,$redSize,$greenSize,$blueSize,$purpleSize,$obsSize,
 	$rogueMax,$redMax,$greenMax,$blueMax,$purpleMax,$obsMax,
 	$shakeWins,$shakeTimeout,
-	$maxPlayerScore,$maxTeamScore,$maxTime,$timeElapsed) = unpack("n23", $buffer);
+	$maxPlayerScore,$maxTeamScore,$maxTime,$timeElapsed) = unpack("n24", $buffer);
 die $! unless $code == 0x7167;
 
 # print info
-print "style:";
-print " CTF" if $style & 0x0001;
-print " flags" if $style & 0x0002;
-print " jumping" if $style & 0x0008;
-print " inertia" if $style & 0x0010;
-print " ricochet" if $style & 0x0020;
-print " shaking" if $style & 0x0040;
-print " antidote" if $style & 0x0080;
-print " handicap" if $style & 0x0100;
-print " rabbit-hunt" if $style & 0x0200;
+print "style: ";
+@styles=("TeamFFA", "ClassicCTF", "OpenFFA", "RabbitChase");
+print $styles[$style];
+print "\n";
+print "options:";
+print " flags" if $options & 0x0002;
+print " jumping" if $options & 0x0008;
+print " inertia" if $options & 0x0010;
+print " ricochet" if $options & 0x0020;
+print " shaking" if $options & 0x0040;
+print " antidote" if $options & 0x0080;
+print " handicap" if $options & 0x0100;
+print " no-team-kills" if $options & 0x0200;
 print "\n";
 print "maxPlayers: $maxPlayers\nmaxShots: $maxShots\n";
 print "team sizes: $rogueSize $redSize $greenSize $blueSize $purpleSize $obsSize" .
 	" (rogue red green blue purple observer)\n";
 print "max sizes:  $rogueMax $redMax $greenMax $blueMax $purpleMax $obsMax\n";
-if ($style & 0x0040) {
+if ($options & 0x0040) {
   print "wins to shake bad flag: $shakeWins\n";
   print "time to shake bad flag: " . $shakeTimeout / 10 . "\n";
 }
@@ -114,13 +127,15 @@ print "time elapsed: " . $timeElapsed / 10 . "\n\n";
 print S pack("n2", 0, 0x7170);
 
 # get number of teams and players we'll be receiving
+sysread(S, $buffer, 2);
 die $! unless sysread(S, $buffer, 8) == 8;
 ($len,$code,$numTotalTeams,$numPlayers) = unpack("n4", $buffer);
 die $! unless $code == 0x7170;
 
-# get the teams
+# get the teams (for all except OpenFFA)
 # TimRiker: MsgTeamUpdate has numTotalTeams but this is how many we will get
 die $! unless sysread(S, $buffer, 5) == 5;
+
 ($len,$code,$numTeams) = unpack("n n C", $buffer);
 die $! unless $code == 0x7475;
 @teamName = ("Rogue", "Red", "Green", "Blue", "Purple", "Observer", "Rabbit");
@@ -142,13 +157,13 @@ for (1..$numPlayers) {
  }
  if ($bytesRead == undef || $bytesRead < 175){ die $!; }
 
- ($len,$code,$pID,$type,$team,$won,$lost,$tks,$sign,$email) =
+ ($len,$code,$pID,$type,$team,$won,$lost,$tks,$sign,$motto) =
 					unpack("n2Cn5A32A128", $buffer);
  die $! unless $code == 0x6170;
  $score = $won - $lost;
  print "player $sign ($teamName[$team] team) is a $playerType[$type]:\n";
  print "  score: $score ($won wins, $lost losses)\n";
- print "  $email\n";
+ print "  $motto\n";
 }
 
 # close socket
