@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -65,7 +65,7 @@ CursesUI::CursesUI(BZAdminClient& c) :
   menu.setUpdateCallback(initMainMenu);
 
   // add additional chat targets
-  additionalTargets[PlayerId(250 - ObserverTeam)] = PlayerInfo("teammates");
+  additionalTargets[PlayerId(FirstTeam - ObserverTeam)] = PlayerInfo("teammates");
   additionalTargets[AdminPlayers] = PlayerInfo("admins");
 }
 
