@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -17,6 +17,8 @@
 /* interface header */
 #include "OptionParser.h"
 
+// system header
+#include <string.h>
 
 OptionParser::OptionParser(const std::string& helpPrefix,
 			   const std::string& usageSuffix)
