@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -659,7 +659,7 @@ void BackgroundRenderer::renderGroundEffects(SceneRenderer& renderer,
   }
 
   if (!blank) {
-    if (doShadows && shadowsVisible && !drawingMirror) {
+	  if (doShadows && shadowsVisible && !drawingMirror) {
       drawGroundShadows(renderer);
     }
 
@@ -668,7 +668,7 @@ void BackgroundRenderer::renderGroundEffects(SceneRenderer& renderer,
     // performed only at a vertex, and the ground's vertices are a few
     // kilometers away.
     if (BZDBCache::blend && BZDBCache::lighting &&
-	!drawingMirror && BZDBCache::drawGroundLights) {
+		!drawingMirror && BZDBCache::drawGroundLights) {
       if (BZDBCache::tesselation && (renderer.useQuality() >= 3)) {
 //	  (BZDB.get(StateDatabase::BZDB_FOGMODE) == "none")) {
 	// not really tesselation, but it is tied to the "Best" lighting,
@@ -684,12 +684,12 @@ void BackgroundRenderer::renderGroundEffects(SceneRenderer& renderer,
       // light the mountains (so that they get dark when the sun goes down).
       // don't do zbuffer test since they occlude all drawn before them and
       // are occluded by all drawn after.
-      if (mountainsVisible && BZDBCache::drawMountains) {
+		if (mountainsVisible && BZDBCache::drawMountains) {
 	drawMountains();
       }
 
       // draw clouds
-      if (cloudsVisible && BZDBCache::drawClouds) {
+		if (cloudsVisible && BZDBCache::drawClouds) {
 	cloudsGState.setState();
 	glMatrixMode(GL_TEXTURE);
 	glPushMatrix();
@@ -706,14 +706,19 @@ void BackgroundRenderer::renderGroundEffects(SceneRenderer& renderer,
 
 void BackgroundRenderer::renderEnvironment(SceneRenderer& renderer, bool update)
 {
-  if (!blank) {
-    if (update) {
-      weather.update();
-      EFFECTS.update();
-    }
-    EFFECTS.draw(renderer);
-    weather.draw(renderer);
-  }
+	if (blank) {
+		return;
+	}
+
+	if (update) {
+		weather.update();
+	}
+	weather.draw(renderer);
+
+	if (update) {
+		EFFECTS.update();
+	}
+	EFFECTS.draw(renderer);
 }
 
 
@@ -1035,7 +1040,10 @@ void BackgroundRenderer::drawSky(SceneRenderer& renderer, bool mirror)
 
 void BackgroundRenderer::drawGround()
 {
-  if (BZDBCache::drawGround) {
+  if (!BZDBCache::drawGround)
+  return;
+
+  {
     // draw ground
     glNormal3f(0.0f, 0.0f, 1.0f);
     if (invert) {
@@ -1052,7 +1060,7 @@ void BackgroundRenderer::drawGround()
       groundGState[styleIndex].setState();
     }
 
-    if (RENDERER.useQuality() >= 3) {
+    if (RENDERER.useQuality() >= 2) {
       drawGroundCentered();
     } else {
       glCallList(simpleGroundList[styleIndex]);
@@ -1062,45 +1070,49 @@ void BackgroundRenderer::drawGround()
 
 void BackgroundRenderer::drawGroundCentered()
 {
-  const ViewFrustum& frustum = RENDERER.getViewFrustum();
-  const float* center = frustum.getEye();
-
   const float groundSize = 10.0f * BZDBCache::worldSize;
+  const float centerSize = 128.0f;
+
+  const ViewFrustum& frustum = RENDERER.getViewFrustum();
+  float center[2] = { frustum.getEye()[0], frustum.getEye()[1] };
+  const float minDist = -groundSize + centerSize;
+  const float maxDist = +groundSize - centerSize;
+  if (center[0] < minDist) { center[0] = minDist; }
+  if (center[0] > maxDist) { center[0] = maxDist; }
+  if (center[1] < minDist) { center[1] = minDist; }
+  if (center[1] > maxDist) { center[1] = maxDist; }
+
+  const float vertices[8][2] = {
+    { -groundSize, -groundSize },
+    { +groundSize, -groundSize },
+    { +groundSize, +groundSize },
+    { -groundSize, +groundSize },
+    { center[0] - centerSize, center[1] - centerSize },
+    { center[0] + centerSize, center[1] - centerSize },
+    { center[0] + centerSize, center[1] + centerSize },
+    { center[0] - centerSize, center[1] + centerSize }
+  };
+
   const float repeat = BZDB.eval("groundHighResTexRepeat");
-
-  // vertices
-  const float vXmin = -groundSize;
-  const float vXmax = +groundSize;
-  const float vYmin = -groundSize;
-  const float vYmax = +groundSize;
-  const GLfloat vertices[5][2] = {
-    {center[0], center[1]},
-    {vXmin, vYmin}, {vXmax, vYmin}, {vXmax, vYmax}, {vXmin, vYmax}
+  const int indices[5][4] = {
+    { 4, 5, 6, 7 },
+    { 0, 1, 5, 4 },
+    { 1, 2, 6, 5 },
+    { 2, 3, 7, 6 },
+    { 3, 0, 4, 7 },
   };
 
-  // texcoords
-  const float tcenterX = center[0] * repeat;
-  const float tcenterY = center[1] * repeat;
-  const float tXmin = -groundSize * repeat;
-  const float tXmax = +groundSize * repeat;
-  const float tYmin = -groundSize * repeat;
-  const float tYmax = +groundSize * repeat;
-  const GLfloat texcoords[5][2] = {
-    {tcenterX, tcenterY},
-    {tXmin, tYmin}, {tXmax, tYmin}, {tXmax, tYmax}, {tXmin, tYmax}
-  };
-
-//  const GLubyte fan[6] = { 0, 1, 2, 3, 4, 1};
-
   glNormal3f(0.0f, 0.0f, 1.0f);
-  glBegin(GL_TRIANGLE_FAN);
+  glBegin(GL_QUADS);
   {
-    for (int i = 0; i < 5; i++) {
-      glTexCoord2fv(texcoords[i]);
-      glVertex2fv(vertices[i]);
+    for (int q = 0; q < 5; q++) {
+      for (int c = 0; c < 4; c++) {
+	const int index = indices[q][c];
+	glTexCoord2f(vertices[index][0] * repeat,
+		     vertices[index][1] * repeat);
+	glVertex2fv(vertices[index]);
+      }
     }
-    glTexCoord2fv(texcoords[1]);
-    glVertex2fv(vertices[1]);
   }
   glEnd();
 
@@ -1213,6 +1225,14 @@ void			BackgroundRenderer::drawGroundShadows(
 }
 
 
+static void setupBlackFog(float fogColor[4])
+{
+  static const float black[4] = { 0.0f, 0.0f, 0.0f, 0.0f };
+  glGetFloatv(GL_FOG_COLOR, fogColor);
+  glFogfv(GL_FOG_COLOR, black);
+}
+
+
 void BackgroundRenderer::drawGroundReceivers(SceneRenderer& renderer)
 {
   static const int receiverRings = 4;
@@ -1240,6 +1260,10 @@ void BackgroundRenderer::drawGroundReceivers(SceneRenderer& renderer)
 
   receiverGState.setState();
 
+  // setup black fog
+  float fogColor[4];
+  setupBlackFog(fogColor);
+
   glPushMatrix();
   int i, j;
   for (int k = 0; k < count; k++) {
@@ -1335,6 +1359,8 @@ void BackgroundRenderer::drawGroundReceivers(SceneRenderer& renderer)
     glTranslatef(-pos[0], -pos[1], 0.0f);
   }
   glPopMatrix();
+
+  glFogfv(GL_FOG_COLOR, fogColor);
 }
 
 
@@ -1380,6 +1406,10 @@ void BackgroundRenderer::drawAdvancedGroundReceivers(SceneRenderer& renderer)
   advGState = builder.getState();
   advGState.setState();
 
+  // setup black fog
+  float fogColor[4];
+  setupBlackFog(fogColor);
+
   // lazy way to get texcoords
   if (useTexture) {
     const float repeat = BZDB.eval("groundHighResTexRepeat");
@@ -1508,6 +1538,8 @@ void BackgroundRenderer::drawAdvancedGroundReceivers(SceneRenderer& renderer)
     glDisable(GL_TEXTURE_GEN_S);
     glDisable(GL_TEXTURE_GEN_T);
   }
+
+  glFogfv(GL_FOG_COLOR, fogColor);
 }
 
 
@@ -1570,10 +1602,6 @@ void BackgroundRenderer::doInitDisplayLists()
   weather.rebuildContext();
   EFFECTS.rebuildContext();
 
-  // need some workarounds on RIVA 128
-  bool isRiva128 = (strncmp((const char*)glGetString(GL_RENDERER),
-						"RIVA 128", 8) == 0);
-
   //
   // sky stuff
   //
@@ -1615,9 +1643,6 @@ void BackgroundRenderer::doInitDisplayLists()
   // ground
   //
 
-  // RIVA 128 can't repeat texture uv's too much.  if we're using one
-  // of those, only texture the ground inside the game area.
-  float uv[2];
   const GLfloat groundSize = 10.0f * worldSize;
   const GLfloat gameSize = 0.5f * worldSize;
   GLfloat groundPlane[4][3];
@@ -1631,45 +1656,7 @@ void BackgroundRenderer::doInitDisplayLists()
     gameArea[i][2] = 0.0f;
   }
 
-  if (isRiva128) {
-    simpleGroundList[2] = glGenLists(1);
-    glNewList(simpleGroundList[2], GL_COMPILE);
-    {
-      glBegin(GL_TRIANGLE_STRIP);
-	renderer.getGroundUV(gameArea[0], uv);
-	glTexCoord2f(uv[0], uv[1]);
-	glVertex2fv(gameArea[0]);
-	renderer.getGroundUV(gameArea[1], uv);
-	glTexCoord2f(uv[0], uv[1]);
-	glVertex2fv(gameArea[1]);
-	renderer.getGroundUV(gameArea[3], uv);
-	glTexCoord2f(uv[0], uv[1]);
-	glVertex2fv(gameArea[3]);
-	renderer.getGroundUV(gameArea[2], uv);
-	glTexCoord2f(uv[0], uv[1]);
-	glVertex2fv(gameArea[2]);
-      glEnd();
-
-      glTexCoord2f(0.0f, 0.0f);
-      glBegin(GL_TRIANGLE_STRIP);
-	glVertex2fv(gameArea[0]);
-	glVertex2fv(groundPlane[0]);
-
-	glVertex2fv(gameArea[1]);
-	glVertex2fv(groundPlane[1]);
-
-	glVertex2fv(gameArea[2]);
-	glVertex2fv(groundPlane[2]);
-
-	glVertex2fv(gameArea[3]);
-	glVertex2fv(groundPlane[3]);
-
-	glVertex2fv(gameArea[0]);
-	glVertex2fv(groundPlane[0]);
-      glEnd();
-    }
-    glEndList();
-  } else {
+  {
     GLfloat xmin, xmax;
     GLfloat ymin, ymax;
     GLfloat xdist, ydist;
@@ -1761,12 +1748,6 @@ void BackgroundRenderer::doInitDisplayLists()
       cloudsInner[i][2] = cloudsOuter[i][2];
     }
 
-    // make cloud display list.  RIVA 128 doesn't interpolate alpha,
-    // so on that system use full alpha everywhere.
-    GLfloat minAlpha = 0.0f;
-    if (isRiva128)
-      minAlpha = 1.0f;
-
     cloudsList = glGenLists(1);
     glNewList(cloudsList, GL_COMPILE);
     {
@@ -1790,7 +1771,7 @@ void BackgroundRenderer::doInitDisplayLists()
 
       // outer clouds -- fade to zero opacity at outer edge
       glBegin(GL_TRIANGLE_STRIP);
-	glColor4f(1.0f, 1.0f, 1.0f, minAlpha);
+	glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
 	glTexCoord2f(cloudRepeats * squareShape[1][0],
 		     cloudRepeats * squareShape[1][1]);
 	glVertex3fv(cloudsOuter[1]);
@@ -1799,7 +1780,7 @@ void BackgroundRenderer::doInitDisplayLists()
 		     uvScale * cloudRepeats * squareShape[1][1]);
 	glVertex3fv(cloudsInner[1]);
 
-	glColor4f(1.0f, 1.0f, 1.0f, minAlpha);
+	glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
 	glTexCoord2f(cloudRepeats * squareShape[2][0],
 		     cloudRepeats * squareShape[2][1]);
 	glVertex3fv(cloudsOuter[2]);
@@ -1808,7 +1789,7 @@ void BackgroundRenderer::doInitDisplayLists()
 		     uvScale * cloudRepeats * squareShape[2][1]);
 	glVertex3fv(cloudsInner[2]);
 
-	glColor4f(1.0f, 1.0f, 1.0f, minAlpha);
+	glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
 	glTexCoord2f(cloudRepeats * squareShape[3][0],
 		     cloudRepeats * squareShape[3][1]);
 	glVertex3fv(cloudsOuter[3]);
@@ -1817,7 +1798,7 @@ void BackgroundRenderer::doInitDisplayLists()
 		     uvScale * cloudRepeats * squareShape[3][1]);
 	glVertex3fv(cloudsInner[3]);
 
-	glColor4f(1.0f, 1.0f, 1.0f, minAlpha);
+	glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
 	glTexCoord2f(cloudRepeats * squareShape[0][0],
 		     cloudRepeats * squareShape[0][1]);
 	glVertex3fv(cloudsOuter[0]);
@@ -1826,7 +1807,7 @@ void BackgroundRenderer::doInitDisplayLists()
 		     uvScale * cloudRepeats * squareShape[0][1]);
 	glVertex3fv(cloudsInner[0]);
 
-	glColor4f(1.0f, 1.0f, 1.0f, minAlpha);
+	glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
 	glTexCoord2f(cloudRepeats * squareShape[1][0],
 		     cloudRepeats * squareShape[1][1]);
 	glVertex3fv(cloudsOuter[1]);
