@@ -1,9 +1,9 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
@@ -127,6 +127,18 @@ class SaveWorldCommand : LocalCommand {
     bool operator() (const char *commandLine);
 };
 
+class ForceRadarCommand : LocalCommand {
+  public:
+    ForceRadarCommand();
+    bool operator() (const char *commandLine);
+};
+
+class DebugLevelCommand : LocalCommand {
+  public:
+    DebugLevelCommand();
+    bool operator() (const char *commandLine);
+};
+
 
 // class instantiations
 static CommandList	  commandList;
@@ -144,29 +156,33 @@ static RoamPosCommand     RoamPosCommand;
 static ReTextureCommand   reTextureCommand;
 static SaveMsgsCommand	  saveMsgsCommand;
 static SaveWorldCommand   saveWorldCommand;
+static ForceRadarCommand  forceRadarCommand;
+static DebugLevelCommand  DebugLevelCommand;
 
 
 // class constructors
-BindCommand::BindCommand() :		LocalCommand("/bind") {}
-CommandList::CommandList() :		LocalCommand("/cmds") {}
-DiffCommand::DiffCommand() :		LocalCommand("/diff") {}
-DumpCommand::DumpCommand() :		LocalCommand("/dumpvars") {}
-HighlightCommand::HighlightCommand() :	LocalCommand("/highlight") {}
-LocalSetCommand::LocalSetCommand() :	LocalCommand("/localset") {}
-QuitCommand::QuitCommand() :		LocalCommand("/quit") {}
-ReTextureCommand::ReTextureCommand() :	LocalCommand("/retexture") {}
-RoamPosCommand::RoamPosCommand() :	LocalCommand("/roampos") {}
-SaveMsgsCommand::SaveMsgsCommand() :	LocalCommand("/savemsgs") {}
-SaveWorldCommand::SaveWorldCommand() :	LocalCommand("/saveworld") {}
-SetCommand::SetCommand() :		LocalCommand("/set") {}
-SilenceCommand::SilenceCommand() :	LocalCommand("/silence") {}
-UnsilenceCommand::UnsilenceCommand() :	LocalCommand("/unsilence") {}
-ClientQueryCommand::ClientQueryCommand() : LocalCommand("CLIENTQUERY") {}
+BindCommand::BindCommand()			: LocalCommand("/bind") {}
+CommandList::CommandList()			: LocalCommand("/cmds") {}
+DiffCommand::DiffCommand()			: LocalCommand("/diff") {}
+DumpCommand::DumpCommand()			: LocalCommand("/dumpvars") {}
+HighlightCommand::HighlightCommand()		: LocalCommand("/highlight") {}
+LocalSetCommand::LocalSetCommand()		: LocalCommand("/localset") {}
+QuitCommand::QuitCommand()			: LocalCommand("/quit") {}
+ReTextureCommand::ReTextureCommand()		: LocalCommand("/retexture") {}
+RoamPosCommand::RoamPosCommand()		: LocalCommand("/roampos") {}
+SaveMsgsCommand::SaveMsgsCommand()		: LocalCommand("/savemsgs") {}
+SaveWorldCommand::SaveWorldCommand()		: LocalCommand("/saveworld") {}
+ForceRadarCommand::ForceRadarCommand()		: LocalCommand("/forceradar") {}
+DebugLevelCommand::DebugLevelCommand()		: LocalCommand("/debug") {}
+SetCommand::SetCommand()			: LocalCommand("/set") {}
+SilenceCommand::SilenceCommand()		: LocalCommand("/silence") {}
+UnsilenceCommand::UnsilenceCommand()		: LocalCommand("/unsilence") {}
+ClientQueryCommand::ClientQueryCommand()	: LocalCommand("CLIENTQUERY") {}
 
 
 // the meat of the matter
 
-bool CommandList::operator() (const char * /*cmdLine*/)
+bool CommandList::operator() (const char * UNUSED(cmdLine))
 {
   int i;
   const int maxLineLen = 64;
@@ -269,7 +285,7 @@ bool BindCommand::operator() (const char *commandLine)
     addMessage(NULL, "bad <press> type");
     return true;
   }
-  
+
   if (both) {
     KEYMGR.unbind(ev, true);
     KEYMGR.bind(ev, true, tokens[2]);
@@ -279,7 +295,7 @@ bool BindCommand::operator() (const char *commandLine)
     KEYMGR.unbind(ev, press);
     KEYMGR.bind(ev, press, tokens[2]);
   }
-  
+
   return true;
 }
 
@@ -437,13 +453,14 @@ static void listSetVars(const std::string& name, void* varDispPtr)
   char message[MessageLen];
   if ((serverVar && varDisp->server) || (!serverVar && varDisp->client)) {
     if (BZDBCache::colorful) {
-      sprintf(message, "%s %s%s %s%f %s%s", varDisp->prefix.c_str(),
+      sprintf(message, "%s %s%s %s%g %s%s %s%s", varDisp->prefix.c_str(),
 	      ColorStrings[RedColor].c_str(), name.c_str(),
 	      ColorStrings[GreenColor].c_str(), BZDB.eval(name),
-	      ColorStrings[BlueColor].c_str(), BZDB.get(name).c_str());
+	      ColorStrings[BlueColor].c_str(), BZDB.get(name).c_str(),
+	      ColorStrings[CyanColor].c_str(), BZDB.getDefault(name).c_str());
     } else {
-      sprintf(message, "%s %s <%f> %s", name.c_str(), varDisp->prefix.c_str(),
-	      BZDB.eval(name), BZDB.get(name).c_str());
+      sprintf(message, "%s %s <%g> %s %s", name.c_str(), varDisp->prefix.c_str(),
+	      BZDB.eval(name), BZDB.get(name).c_str(), BZDB.getDefault(name).c_str());
     }
     addMessage(LocalPlayer::getMyTank(), message, 2);
   }
@@ -761,6 +778,47 @@ bool SaveWorldCommand::operator() (const char *commandLine)
 }
 
 
+bool ForceRadarCommand::operator() (const char*)
+{
+  LocalPlayer* myTank = LocalPlayer::getMyTank();
+  if (myTank == NULL) {
+    return true;
+  }
+  if (myTank->getTeam() != ObserverTeam) {
+    addMessage(NULL, "the /forceradar command only works for observers");
+    return true;
+  }
+  float value;
+  if (BZDBCache::radarLimit == 0.0f) {
+    value = +BZDBCache::worldSize;
+  } else {
+    value = -BZDBCache::radarLimit;
+  }
+  BZDB.setFloat(StateDatabase::BZDB_RADARLIMIT, value, StateDatabase::Server);
+  return true;
+}
+
+
+bool DebugLevelCommand::operator() (const char* cmdLine)
+{
+  const std::vector<std::string> args = TextUtils::tokenize(cmdLine, " ");
+  if (args.size() < 2) {
+    std::string msg = "debug level is ";
+    msg += TextUtils::itoa(debugLevel);
+    addMessage(NULL, msg);
+    return true;
+  }
+
+  debugLevel = atoi(args[1].c_str());
+
+  char buf[128];
+  snprintf(buf, sizeof(buf), "debug level set to %i", debugLevel);
+  addMessage(NULL, buf);
+
+  return true;
+}
+
+
 // Local Variables: ***
 // mode:C++ ***
 // tab-width: 8 ***
