@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -20,6 +20,9 @@
 #include <assert.h>
 #include <time.h>
 #include <string.h>
+#ifdef HAVE_PTHREADS
+#  include <pthread.h>
+#endif
 
 /* common implementation headers */
 #include "BZDBCache.h"
@@ -127,6 +130,27 @@ void ControlPanelMessage::breakLines(float maxLength, int fontFace, float fontSi
   }
 }
 
+
+static std::deque<std::string> mutexMsgs;
+
+#if defined(HAVE_PTHREADS)
+static pthread_mutex_t controlpanel_mutex = PTHREAD_MUTEX_INITIALIZER;
+#elif defined(_WIN32)
+static CRITICAL_SECTION controlpanel_critical;
+#endif
+
+#if defined(HAVE_PTHREADS)
+# define LOCK_CONTROLPANEL_MUTEX   pthread_mutex_lock(&controlpanel_mutex);
+# define UNLOCK_CONTROLPANEL_MUTEX pthread_mutex_unlock(&controlpanel_mutex);
+#elif defined(_WIN32)
+# define LOCK_CONTROLPANEL_MUTEX   EnterCriticalSection(&controlpanel_critical);
+# define UNLOCK_CONTROLPANEL_MUTEX LeaveCriticalSection(&controlpanel_critical);
+#else
+# define LOCK_CONTROLPANEL_MUTEX
+# define UNLOCK_CONTROLPANEL_MUTEX
+#endif
+
+
 //
 // ControlPanel
 //
@@ -184,6 +208,13 @@ ControlPanel::ControlPanel(MainWindow& _mainWindow, SceneRenderer& _renderer) :
     tabs->push_back("Misc");
   }
 
+  // construct critical section for threadsafe messages
+#if defined(HAVE_PTHREADS)
+  pthread_mutex_init(&controlpanel_mutex, NULL);
+#elif defined(_WIN32)
+  InitializeCriticalSection(&controlpanel_critical);
+#endif
+
   resize(); // need resize to set up font and window dimensions
 }
 
@@ -195,6 +226,13 @@ ControlPanel::~ControlPanel()
   BZDB.removeCallback("displayRadar", bzdbCallback, this);
   BZDB.removeCallback(StateDatabase::BZDB_RADARLIMIT, bzdbCallback, this);
 
+  // destroy critical section for threadsafe messages
+#if defined(HAVE_PTHREADS)
+  pthread_mutex_destroy(&controlpanel_mutex);
+#elif defined(_WIN32)
+  DeleteCriticalSection(&controlpanel_critical);
+#endif
+
   if (echoToConsole && echoAnsi) {
     std::cout << ColorStrings[FinalResetColor] << std::flush;
   }
@@ -206,7 +244,7 @@ ControlPanel::~ControlPanel()
   }
 }
 
-void ControlPanel::bzdbCallback(const std::string& /*name*/, void* data)
+void ControlPanel::bzdbCallback(const std::string& UNUSED(name), void* data)
 {
   ((ControlPanel*)data)->resize();
   return;
@@ -222,6 +260,11 @@ void			ControlPanel::setControlColor(const GLfloat *color)
 
 void			ControlPanel::render(SceneRenderer& _renderer)
 {
+  while (!mutexMsgs.empty()) {
+    addMessage(mutexMsgs.front());
+    mutexMsgs.pop_front();
+  }
+
   if (!BZDB.isTrue("displayConsole")) {
     // always draw the console if its fully opaque
     const float opacity = RENDERER.getPanelOpacity();
@@ -544,39 +587,41 @@ void			ControlPanel::render(SceneRenderer& _renderer)
 
   } glEnd();
 
-  // some engines miss the corners
-  glBegin(GL_POINTS); {
-    glVertex2f((float) (x + messageAreaPixels[0] - 1),
-	       (float) (y + messageAreaPixels[1] - 1));
-    glVertex2f((float) (x + messageAreaPixels[0] + messageAreaPixels[2]),
-	       (float) (y + messageAreaPixels[1] - 1));
-    glVertex2f((float) (x + messageAreaPixels[0] + messageAreaPixels[2]),
-	       (float) (y + messageAreaPixels[1] + messageAreaPixels[3]));
-    glVertex2f((float) (x + messageAreaPixels[0] - 1),
-	       (float) (y + messageAreaPixels[1] + messageAreaPixels[3]));
-    long int tabPosition = 0;
-    for (int tab = 0; tab < (int)tabs->size(); tab++) {
-      if (messageMode == MessageModes(tab)) {
-	if (tabsOnRight) {
-	  glVertex2f((float) (x + messageAreaPixels[0] + messageAreaPixels[2] - totalTabWidth + tabPosition),
-		     (float) (y + messageAreaPixels[1] + messageAreaPixels[3] + ay));
-	  glVertex2f((float) (x + messageAreaPixels[0] + messageAreaPixels[2] - totalTabWidth + tabPosition + long(tabTextWidth[tab])),
-		     (float) (y + messageAreaPixels[1] + messageAreaPixels[3] + ay));
-	} else {
-	  glVertex2f((float) (x + messageAreaPixels[0] + tabPosition),
-		     (float) (y + messageAreaPixels[1] + messageAreaPixels[3] + ay));
-	  glVertex2f((float) (x + messageAreaPixels[0] + tabPosition + long(tabTextWidth[tab])),
-		     (float) (y + messageAreaPixels[1] + messageAreaPixels[3] + ay));
+  if (0){
+    // some engines miss the corners
+    glBegin(GL_POINTS); {
+      glVertex2f((float) (x + messageAreaPixels[0] - 1),
+		 (float) (y + messageAreaPixels[1] - 1));
+      glVertex2f((float) (x + messageAreaPixels[0] + messageAreaPixels[2]),
+		 (float) (y + messageAreaPixels[1] - 1));
+      glVertex2f((float) (x + messageAreaPixels[0] + messageAreaPixels[2]),
+		 (float) (y + messageAreaPixels[1] + messageAreaPixels[3]));
+      glVertex2f((float) (x + messageAreaPixels[0] - 1),
+		 (float) (y + messageAreaPixels[1] + messageAreaPixels[3]));
+      long int tabPosition = 0;
+      for (int tab = 0; tab < (int)tabs->size(); tab++) {
+	if (messageMode == MessageModes(tab)) {
+	  if (tabsOnRight) {
+	    glVertex2f((float) (x + messageAreaPixels[0] + messageAreaPixels[2] - totalTabWidth + tabPosition),
+		       (float) (y + messageAreaPixels[1] + messageAreaPixels[3] + ay));
+	    glVertex2f((float) (x + messageAreaPixels[0] + messageAreaPixels[2] - totalTabWidth + tabPosition + long(tabTextWidth[tab])),
+		       (float) (y + messageAreaPixels[1] + messageAreaPixels[3] + ay));
+	  } else {
+	    glVertex2f((float) (x + messageAreaPixels[0] + tabPosition),
+		       (float) (y + messageAreaPixels[1] + messageAreaPixels[3] + ay));
+	    glVertex2f((float) (x + messageAreaPixels[0] + tabPosition + long(tabTextWidth[tab])),
+		       (float) (y + messageAreaPixels[1] + messageAreaPixels[3] + ay));
+	  }
 	}
+	tabPosition += long(tabTextWidth[tab]);
       }
-      tabPosition += long(tabTextWidth[tab]);
-    }
-  } glEnd();
+    } glEnd();
+  }
 
   if (BZDBCache::blend)
 	  glDisable(GL_BLEND);
 
-  glColor4f(teamColor[0], teamColor[1], teamColor[2],1.0f );
+glColor4f(teamColor[0], teamColor[1], teamColor[2],1.0f );
 
   glPopMatrix();
 
@@ -704,10 +749,10 @@ void			ControlPanel::setMessagesOffset(int offset, int whence, bool paged)
       offset = offset * maxLines;
     }
   }
-  
+
   // offset = offset from whence (offset of 0 is the bottom/most recent)
   // whence = 0, 1, or 2 (akin to SEEK_SET, SEEK_CUR, SEEK_END)
-  
+
   switch (whence) {
     case 0:
       if (offset < (int)messages[messageMode].size())
@@ -751,6 +796,13 @@ void			ControlPanel::setMessagesMode(int _messageMode)
   invalidate();
 }
 
+void ControlPanel::addMutexMessage(const std::string& msg)
+{
+  LOCK_CONTROLPANEL_MUTEX
+  mutexMsgs.push_back(msg);
+  UNLOCK_CONTROLPANEL_MUTEX
+}
+
 void			ControlPanel::addMessage(const std::string& line,
 						 const int realmode)
 {
