@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993-2010 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -35,7 +35,7 @@ GuidedMissileStrategy::GuidedMissileStrategy(ShotPath* _path) :
   renderTimes(0),
   needUpdate(true)
 {
-  ptSceneNode = new BoltSceneNode(_path->getPosition());
+  ptSceneNode = new BoltSceneNode(_path->getPosition(),_path->getVelocity(),false);
   TextureManager &tm = TextureManager::instance();
   int texture = tm.getTextureID("missile");
 
@@ -43,6 +43,7 @@ GuidedMissileStrategy::GuidedMissileStrategy(ShotPath* _path) :
     ptSceneNode->setTexture(texture);
     ptSceneNode->setTextureAnimation(4, 4);
     ptSceneNode->setColor(1.0f, 0.2f, 0.0f);
+    ptSceneNode->setTeamColor(Team::getRadarColor(_path->getTeam()));
     ptSceneNode->setFlares(true);
   }
 
@@ -94,7 +95,10 @@ GuidedMissileStrategy::GuidedMissileStrategy(ShotPath* _path) :
   lastTarget = NoPlayer;
 
   lastPuff = currentTime;
-  puffTime = BZDB.eval("gmPuffTime");
+  rootPuff = BZDB.eval("gmPuffTime");
+  puffTime = -1;
+  if (RENDERER.useQuality() >= 3)
+    rootPuff /= (10.0f + ((float)bzfrand()* 5.0f));
 }
 
 GuidedMissileStrategy::~GuidedMissileStrategy()
@@ -200,10 +204,16 @@ void GuidedMissileStrategy::update(float dt)
   Ray ray = Ray(nextPos, newDirection);
 
   renderTimes++;
+  if (puffTime < 0 )
+    puffTime =  (float)bzfrand()*rootPuff;
+
   // Changed: GM smoke trail, leave it every seconds, none of this per frame crap
   if (currentTime.getSeconds() - lastPuff.getSeconds() > puffTime ) {
     lastPuff = currentTime;
     addShotPuff(nextPos,azimuth,elevation);
+
+    // pick a new time for the next puff so it's not so orderd.
+    puffTime = (float)bzfrand()*rootPuff;
   }
 
   // get next position
