@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -67,7 +67,7 @@ HUDRenderer::HUDRenderer(const BzfDisplay* _display,
 				showCompose(false),
 				showCracks(true),
 				dater(false),
-				lastTimeChange(time(NULL)),
+				lastTimeChange((unsigned long)time(NULL)),
 				triangleCount(0),
 				radarTriangleCount(0)
 {
@@ -113,6 +113,8 @@ HUDRenderer::HUDRenderer(const BzfDisplay* _display,
   // create scoreboard renderer
   scoreboard = new ScoreboardRenderer();
 
+  friendlyMarkerList = DisplayListSystem::Instance().newList(this);
+
   // initialize fonts
   resize(true);
 }
@@ -157,7 +159,7 @@ void			HUDRenderer::resize(bool firstTime)
   }
 
   // initialize readout spacings
-  headingMarkSpacing = (int)(5.0f * float(maxMotionSize) / headingOffset);
+  headingMarkSpacing = 5.0f * float(maxMotionSize) / headingOffset;
   altitudeMarkSpacing = floorf(5.0f * float(maxMotionSize) / altitudeOffset);
 
   setBigFontSize(w, vh);
@@ -430,7 +432,7 @@ void			HUDRenderer::setFlagHelp(FlagType* desc, float duration)
   flagHelpClock.setClock(duration);
 
   // Generate the formatted help for this flag
-  flagHelpText = makeHelpString(desc->flagHelp);
+  flagHelpText = makeHelpString(desc->flagHelp.c_str());
 
   // count the number of lines in the help message
   flagHelpLines = 0;
@@ -475,6 +477,40 @@ void			HUDRenderer::addMarker(float _heading, const float *_color )
   memcpy(m.color, _color, sizeof(m.color));
 }
 
+
+void HUDRenderer::AddEnhancedNamedMarker(const fvec3& pos, const fvec4& color,
+					 std::string name, bool friendly,
+					 float zShift)
+{
+  return;
+
+  EnhancedHUDMarker newMarker(pos, color);
+  newMarker.pos.z += zShift;
+  newMarker.name = name;
+  newMarker.friendly = friendly;
+  enhancedMarkers.push_back(newMarker);
+}
+
+void HUDRenderer::AddEnhancedMarker(const fvec3& pos, const fvec4& color,
+				    bool friendly, float zShift )
+{
+  EnhancedHUDMarker newMarker(pos, color);
+  newMarker.pos.z += zShift;
+  newMarker.friendly = friendly;
+  enhancedMarkers.push_back(newMarker);
+}
+
+void HUDRenderer::AddLockOnMarker(const fvec3& pos, std::string name,
+				  bool friendly, float zShift )
+{
+  const fvec4 color(0.75f, 0.125f, 0.125f, 1.0f);
+  EnhancedHUDMarker newMarker(pos, color);
+  newMarker.pos.z += zShift;
+  newMarker.name = name;
+  newMarker.friendly = friendly;
+  lockOnMarkers.push_back(newMarker);
+}
+
 void			HUDRenderer::setRestartKeyLabel(const std::string& label)
 {
   char buffer[250];
@@ -596,6 +632,14 @@ void			HUDRenderer::hudColor3fv(const GLfloat* c)
     glColor3fv(c);
 }
 
+void HUDRenderer::hudColor3Afv(const float * c, const float a)
+{
+  if( dim )
+    glColor4f( dimFactor *c[0], dimFactor *c[1], dimFactor *c[2], a );
+  else
+    glColor4f( c[0],c[1],c[2],a );
+}
+
 void			HUDRenderer::hudSColor3fv(const GLfloat* c)
 {
   if (dim)
@@ -612,6 +656,64 @@ void			HUDRenderer::hudColor4fv(const GLfloat* c)
     glColor4fv(c);
 }
 
+
+void HUDRenderer::buildGeometry ( GLDisplayList displayList )
+{
+  if (displayList == friendlyMarkerList) {
+    float lockonSize = 40;
+
+    float segmentation = 32.0f/360.0f;
+    float rad = lockonSize * 0.125f;
+
+    // white outline
+    hudColor4f( 1,1,1, 0.85f );
+    glLineWidth(4.0f);
+    glBegin(GL_LINES);
+    glVertex3f(-rad,rad+rad,0.03f);
+    glVertex3f(rad,-rad+rad,0.02f);
+    // glVertex3f(-lockonSize*xFactor,lockonSize,0.02f);
+    // glVertex3f(lockonSize*xFactor,0,0.02f);
+    glEnd();
+
+    glBegin(GL_LINE_LOOP);
+    for (float t = 0; t < 360; t += segmentation) {
+      if (t != 0) {
+	const float s = (t - segmentation);
+	const float tRads = t * DEG2RADf;
+	const float sRads = s * DEG2RADf;
+	glVertex3f(sinf(sRads) * rad, (cosf(sRads) * rad) + rad, 0.02f);
+	glVertex3f(sinf(tRads) * rad, (cosf(tRads) * rad) + rad, 0.02f);
+      }
+    }
+    glEnd();
+
+    // red X
+    hudColor4f( 1,0,0, 0.85f );
+    glLineWidth(2.0f);
+    glBegin(GL_LINES);
+    glVertex3f(-rad,rad+rad,0.03f);
+    glVertex3f(rad,-rad+rad,0.02f);
+    // glVertex3f(-lockonSize*xFactor,lockonSize,0.03f);
+    //  glVertex3f(lockonSize*xFactor,0,0.02f);
+    glEnd();
+
+    glBegin(GL_LINE_LOOP);
+    for (float t = 0; t < 360; t += segmentation) {
+      if (t != 0) {
+	const float s = (t - segmentation);
+	const float tRads = t * DEG2RADf;
+	const float sRads = s * DEG2RADf;
+	glVertex3f(sinf(sRads) * rad, (cosf(sRads) * rad) + rad, 0.02f);
+	glVertex3f(sinf(tRads) * rad, (cosf(tRads) * rad) + rad, 0.02f);
+      }
+    }
+    glEnd();
+
+    glLineWidth(2.0f);
+  }
+
+}
+
 void			HUDRenderer::render(SceneRenderer& renderer)
 {
   if (firstRender) {
@@ -634,7 +736,8 @@ void			HUDRenderer::render(SceneRenderer& renderer)
   else {
     const bool showTimes = (fps > 0.0f) || (drawTime > 0.0f) ||
 			   (triangleCount > 0) || (radarTriangleCount > 0);
-    const bool showTankLabels = BZDB.isTrue("displayLabels");
+    const bool showTankLabels = roaming &&
+				BZDB.isTrue("displayLabels");
 
     if (showCompose || showTimes || showTankLabels) {
       // get view metrics
@@ -712,7 +815,7 @@ void			HUDRenderer::renderStatus(void)
   // print player name and score in upper left corner in team (radar) color
   if (!roaming && (!playerHasHighScore || scoreClock.isOn())) {
     sprintf(buffer, "%s: %d", myTank->getCallSign(), myTank->getScore());
-    hudColor3fv(Team::getRadarColor(teamIndex, World::getWorld()->allowRabbit()));
+    hudColor3fv(Team::getRadarColor(teamIndex));
     fm.drawString(x, y, 0, majorFontFace, majorFontSize, buffer);
   }
 
@@ -735,7 +838,7 @@ void			HUDRenderer::renderStatus(void)
     if (BZDB.eval("timedate") == 2) {
       if (time(NULL) - lastTimeChange >= 2) {
 	dater = !dater;
-	lastTimeChange = time(NULL);
+	lastTimeChange = (unsigned int)time(NULL);
       }
     } else {
       dater = (BZDB.eval("timedate") == 1);
@@ -752,14 +855,15 @@ void			HUDRenderer::renderStatus(void)
   }
 
   // print current position of tank
-  if (BZDB.isTrue("showCoordinates") && !(roaming && BZDB.isTrue("showVelocities")) ) {
-    y -= float(1.0*h);
-    sprintf(buffer, "[%d %d %d]", (int)myTank->getPosition()[0],
-	    (int)myTank->getPosition()[1], (int)myTank->getPosition()[2]);
-    x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(majorFontFace, majorFontSize, buffer);
-    fm.drawString(x, y, 0, majorFontFace, majorFontSize, buffer);
-    y += float(1.5*h);
-  }
+  if (BZDB.isTrue("showCoordinates")) {
+	Player *target = ROAM.getTargetTank();
+	if(!target){
+      sprintf(buffer, "[%d %d %d]", (int)myTank->getPosition()[0],
+	      (int)myTank->getPosition()[1], (int)myTank->getPosition()[2]);
+      x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(majorFontFace, majorFontSize, buffer);
+      fm.drawString(x, (y - h), 0, majorFontFace, majorFontSize, buffer);
+	}
+   }
 
   if (roaming && BZDB.isTrue("showVelocities"))
   {
@@ -767,10 +871,10 @@ void			HUDRenderer::renderStatus(void)
     if (target) {
       float vel[3] = {0};
       memcpy(vel,target->getVelocity(),sizeof(float)*3);
-  
+
       float apparentVel[3] = {0};
       memcpy(apparentVel,target->getApparentVelocity(),sizeof(float)*3);
-  
+
       float linSpeed = sqrt(vel[0]*vel[0]+vel[1]*vel[1]);
       float vertSpeed = vel[2];
       float rotSpeed = fabs(target->getAngularVelocity());
@@ -781,14 +885,14 @@ void			HUDRenderer::renderStatus(void)
       static TimeKeeper maxApparentLinTime = TimeKeeper::getStartTime();
 
       if (maxApparentLinSpeed > apparentLinSpeed &&
-          TimeKeeper::getTick() - maxApparentLinTime < 0.5f) {
-        apparentLinSpeed = maxApparentLinSpeed;
+	  TimeKeeper::getTick() - maxApparentLinTime < 0.5f) {
+	apparentLinSpeed = maxApparentLinSpeed;
       }
       else {
-        maxApparentLinSpeed = apparentLinSpeed;
-        maxApparentLinTime = TimeKeeper::getTick();
+	maxApparentLinSpeed = apparentLinSpeed;
+	maxApparentLinTime = TimeKeeper::getTick();
       }
-  
+
       float smallZHeight = fm.getStrHeight(minorFontFace,minorFontSize,std::string("X"))*1.125f;
       float drawY = y - smallZHeight;
       // draw header
@@ -802,44 +906,44 @@ void			HUDRenderer::renderStatus(void)
 
       std::string label = "Linear Speed:";
       if ( linSpeed > target->getMaxSpeed() )
-        label += "!";
-  
+	label += "!";
+
       sprintf(buffer,"%s%5.2f",label.c_str(),linSpeed);
       if (BZDB.evalInt("showVelocities") > 1)
-        sprintf(buffer,"%s%5.2f(%5.2f)",label.c_str(),linSpeed,apparentLinSpeed);
-  
+	sprintf(buffer,"%s%5.2f(%5.2f)",label.c_str(),linSpeed,apparentLinSpeed);
+
       x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
       fm.drawString(x,drawY-smallZHeight*2.0f, 0, minorFontFace, minorFontSize, buffer);
-  
+
       sprintf(buffer,"Vertical Speed:%5.2f",vertSpeed);
       if (BZDB.evalInt("showVelocities") > 1)
-        sprintf(buffer,"Vertical Speed:%5.2f(%5.2f)",vertSpeed,apparentVel[2]);
-  
+	sprintf(buffer,"Vertical Speed:%5.2f(%5.2f)",vertSpeed,apparentVel[2]);
+
       x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
       fm.drawString(x, drawY-smallZHeight*3.0f, 0, minorFontFace, minorFontSize, buffer);
-  
+
       label = "Angular Speed:";
       if (rotSpeed > BZDB.eval(StateDatabase::BZDB_TANKANGVEL))
-        label += "!";
+	label += "!";
       sprintf(buffer,"%s%5.2f",label.c_str(),rotSpeed);
       x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
       fm.drawString(x,drawY-smallZHeight*4.0f, 0, minorFontFace, minorFontSize, buffer);
-  
+
       float shotTime = (float)target->getShotStatistics()->getLastShotTimeDelta();
       float shotDeviation = (float)target->getShotStatistics()->getLastShotDeviation();
-  
+
       sprintf(buffer,"Last Shot Info  Time:%6.4f  Deviation:%6.3f", shotTime, shotDeviation);
       x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
       fm.drawString(x,drawY-smallZHeight*5.0f, 0, minorFontFace, minorFontSize, buffer);
-  
+
       float offset = 6.0f;
       if (BZDB.evalInt("showVelocities") > 2) {
-        offset += 1.0f;
-        sprintf(buffer,"ReportedHits %d ComputedHits %d ratio %f", target->reportedHits, target->computedHits,(float)target->reportedHits/(float)target->computedHits);
-        x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
-        fm.drawString(x,drawY-smallZHeight*offset, 0, minorFontFace, minorFontSize, buffer);
+	offset += 1.0f;
+	sprintf(buffer,"ReportedHits %d ComputedHits %d ratio %f", target->reportedHits, target->computedHits,(float)target->reportedHits/(float)target->computedHits);
+	x = (float)window.getWidth() - 0.25f * h - fm.getStrLength(minorFontFace, minorFontSize,buffer);
+	fm.drawString(x,drawY-smallZHeight*offset, 0, minorFontFace, minorFontSize, buffer);
       }
-  
+
       offset += 1.5f;
       scoreboard->setTeamScoreY(drawY - smallZHeight*offset);
     }
@@ -916,25 +1020,6 @@ void			HUDRenderer::renderStatus(void)
   fm.drawString(x, y, 0, majorFontFace, majorFontSize, buffer);
 }
 
-int HUDRenderer::tankScoreCompare(const void* _a, const void* _b)
-{
-  RemotePlayer* a = World::getWorld()->getPlayer(*(int*)_a);
-  RemotePlayer* b = World::getWorld()->getPlayer(*(int*)_b);
-  if (World::getWorld()->allowRabbit())
-    return b->getRabbitScore() - a->getRabbitScore();
-  else
-    return b->getScore() - a->getScore();
-}
-
-int HUDRenderer::teamScoreCompare(const void* _c, const void* _d)
-{
-  Team* c = World::getWorld()->getTeams()+*(int*)_c;
-  Team* d = World::getWorld()->getTeams()+*(int*)_d;
-
-  return (d->won-d->lost) - (c->won-c->lost);
-}
-
-
 void			HUDRenderer::renderTankLabels(SceneRenderer& renderer)
 {
   if (!World::getWorld()) return;
@@ -958,7 +1043,7 @@ void			HUDRenderer::renderTankLabels(SceneRenderer& renderer)
     if (pl && pl->isAlive()) {
       const char *name = pl->getCallSign();
       double x, y, z;
-      hudSColor3fv(Team::getRadarColor(pl->getTeam(), World::getWorld()->allowRabbit()));
+      hudSColor3fv(Team::getRadarColor(pl->getTeam()));
       gluProject(pl->getPosition()[0], pl->getPosition()[1],
 		 pl->getPosition()[2]/*+BZDB.eval(StateDatabase::BZDB_MUZZLEHEIGHT)*3.0f*/, model, proj, view, &x, &y, &z);
       if (z >= 0.0 && z <= 1.0) {
@@ -978,7 +1063,7 @@ void			HUDRenderer::renderTankLabels(SceneRenderer& renderer)
 			0, labelsFontFace, labelsFontSize, fname);
 	}
 
-		if (roaming && BZDB.isTrue("showVelocities")) 
+		if (roaming && BZDB.isTrue("showVelocities"))
 		{
 			float vel[3] = {0};
 			memcpy(vel,pl->getVelocity(),sizeof(float)*3);
@@ -1073,6 +1158,224 @@ void			HUDRenderer::renderTimes(void)
   }
 }
 
+void HUDRenderer::saveMatrixes(const float *mm, const float *pm )
+{
+  // ssave off the stuff before we reset it
+  for(int i = 0; i < 16; i++) {
+    modelMatrix[i] = mm[i];
+    projMatrix[i] = pm[i];
+  }
+  glGetIntegerv(GL_VIEWPORT,(GLint*)viewport);
+}
+
+
+void HUDRenderer::drawWaypointMarker(float* color, float alpha, float* object,
+				     const float* viewPos, std::string name,
+				     bool friendly)
+{
+  double map[3] = {0,0,0};
+  double o[3];
+  o[0] = object[0];
+  o[1] = object[1];
+  o[2] = object[2];
+
+  hudColor3Afv( color, alpha );
+
+  glPushMatrix();
+  gluProject(o[0], o[1], o[2], modelMatrix, projMatrix,
+    (GLint*)viewport, &map[0], &map[1], &map[2]);
+  glPopMatrix();
+
+  float halfWidth = window.getWidth( )* 0.5f;
+  float halfHeight = window.getHeight() * 0.5f;
+
+  // comp us back to the view
+  map[0] -= halfWidth;
+  map[1] -= halfHeight;
+
+  const fvec2 headingVec(sinf(heading * DEG2RADf),
+    cosf(heading * DEG2RADf));
+
+  const fvec2 toPosVec((float)object[0] - viewPos[0],
+    (float)object[1] - viewPos[1]);
+
+  if (fvec2::dot(toPosVec, headingVec) <= 1.0f /*0.866f*/) {
+    if (NEAR_ZERO(map[0], ZERO_TOLERANCE)) {
+      map[0] = -halfWidth;
+      map[1] = 0;
+    } else {
+      map[0] = -halfWidth * (fabs(map[0])/map[0]);
+      map[1] = 0;
+    }
+  } else {
+    if (map[0] < -halfWidth)
+      map[0] = -halfWidth;
+    if (map[0] > halfWidth)
+      map[0] = halfWidth;
+
+    if (map[1] < -halfHeight)
+      map[1] = -halfHeight;
+    if (map[1] > halfHeight)
+      map[1] = halfHeight;
+  }
+
+  glPushMatrix();
+  glTranslatef((float)map[0],(float)map[1],0);
+  glPushMatrix();
+  float triangleSize = BZDB.eval("hudWayPMarkerSize");
+  if (name.size())
+    triangleSize *= 0.75f;
+
+  if (map[0] == halfWidth && map[1] != -halfHeight && map[1] != halfHeight)	// right side
+    glRotatef(90,0,0,1);
+
+  if (map[0] == -halfWidth && map[1] != -halfHeight && map[1] != halfHeight)	// Left side
+    glRotatef(-90,0,0,1);
+
+  if (map[1] == halfHeight && map[0] != -halfWidth && map[0] != halfWidth)	// Top side
+    glRotatef(180,0,0,1);
+
+  if (map[0] == halfWidth && map[1] == -halfHeight)	// Lower right
+    glRotatef(45,0,0,1);
+
+  if (map[0] == -halfWidth && map[1] == -halfHeight)	// Lower left
+    glRotatef(-45,0,0,1);
+
+  if (map[0] == halfWidth && map[1] == halfHeight)	// upper right
+    glRotatef(180-45,0,0,1);
+
+  if (map[0] == -halfWidth && map[1] == halfHeight)	// upper left
+    glRotatef(180+45,0,0,1);
+
+  glBegin(GL_TRIANGLES);
+  glVertex2f(0,0);
+  glVertex2f(triangleSize,triangleSize);
+  glVertex2f(-triangleSize,triangleSize);
+  glEnd();
+
+  glBegin(GL_LINE_STRIP);
+  glVertex3f(0,0,0.01f);
+  glVertex3f(triangleSize,triangleSize,0.01f);
+  glVertex3f(-triangleSize,triangleSize,0.01f);
+  glEnd();
+
+  if (friendly)
+    DisplayListSystem::Instance().callList(friendlyMarkerList);
+
+  glPopMatrix();
+
+  if (name.size()) {
+    hudColor3Afv( color, alpha );
+    float textOffset = 5.0f;
+    float width = FontManager::instance().getStrLength(headingFontFace, headingFontSize, name);
+    glEnable(GL_TEXTURE_2D);
+    FontManager::instance().drawString(-width * 0.5f , textOffset + triangleSize,
+      0, headingFontFace, headingFontSize, name);
+    glDisable(GL_TEXTURE_2D);
+  }
+
+  glPopMatrix();
+}
+
+
+//-------------------------------------------------------------------------
+// HUDRenderer::drawLockonMarker
+//-------------------------------------------------------------------------
+
+void HUDRenderer::drawLockonMarker(float* color ,float alpha, float* object,
+				   const float *viewPos, std::string name,
+				   bool friendly )
+{
+  double map[3] = {0,0,0};
+  double o[3];
+  o[0] = object[0];
+  o[1] = object[1];
+  o[2] = object[2];
+
+  hudColor3Afv( color, alpha );
+
+  glPushMatrix();
+  gluProject(o[0], o[1], o[2], modelMatrix,projMatrix,
+    (GLint*)viewport, &map[0], &map[1], &map[2]);
+  glPopMatrix();
+
+  float halfWidth = window.getWidth( )* 0.5f;
+  float halfHeight = window.getHeight() * 0.5f;
+
+  // comp us back to the view
+  map[0] -= halfWidth;
+  map[1] -= halfHeight;
+
+  const fvec2 headingVec(sinf(heading * DEG2RADf),
+    cosf(heading * DEG2RADf));
+
+  const fvec2 toPosVec((float)object[0] - viewPos[0],
+    (float)object[1] - viewPos[1]);
+
+  if (fvec2::dot(toPosVec, headingVec) <= 1.0f) {
+    if (NEAR_ZERO(map[0], ZERO_TOLERANCE)) {
+      map[0] = -halfWidth;
+      map[1] = 0;
+    } else {
+      map[0] = -halfWidth * (fabs(map[0])/map[0]);
+      map[1] = 0;
+    }
+  } else {
+    if ( map[0] < -halfWidth )
+      map[0] = -halfWidth;
+    if ( map[0] > halfWidth )
+      map[0] = halfWidth;
+
+    if ( map[1] < -halfHeight )
+      map[1] = -halfHeight;
+    if ( map[1] > halfHeight )
+      map[1] = halfHeight;
+  }
+
+  glPushMatrix();
+  glTranslatef((float)map[0],(float)map[1],0);
+  glPushMatrix();
+
+  float lockonSize = 40;
+  float lockonInset = 15;
+  float lockonDeclination = 15;
+
+  glLineWidth(3.0f);
+
+  glBegin(GL_LINE_STRIP);
+  glVertex2f(-lockonInset,lockonSize-lockonDeclination);
+  glVertex2f(-lockonSize,lockonSize);
+  glVertex2f(-lockonSize,-lockonSize);
+  glVertex2f(-lockonInset,-lockonSize+lockonDeclination);
+  glEnd();
+
+  glBegin(GL_LINE_STRIP);
+  glVertex2f(lockonInset,lockonSize-lockonDeclination);
+  glVertex2f(lockonSize,lockonSize);
+  glVertex2f(lockonSize,-lockonSize);
+  glVertex2f(lockonInset,-lockonSize+lockonDeclination);
+  glEnd();
+
+  if (friendly)
+    DisplayListSystem::Instance().callList(friendlyMarkerList);
+
+  glLineWidth(1.0f);
+
+  glPopMatrix();
+
+  if (name.size()) {
+    hudColor3Afv( color, alpha );
+    float textOffset = 5.0f;
+    float width = FontManager::instance().getStrLength(headingFontFace, headingFontSize, name);
+    glEnable(GL_TEXTURE_2D);
+    FontManager::instance().drawString(-width * 0.5f, textOffset + lockonSize,
+      0, headingFontFace, headingFontSize, name);
+    glDisable(GL_TEXTURE_2D);
+  }
+
+  glPopMatrix();
+}
+
 void			HUDRenderer::renderBox(SceneRenderer&)
 {
   // get view metrics
@@ -1099,24 +1402,28 @@ void			HUDRenderer::renderBox(SceneRenderer&)
     glVertex2i(centerx + noMotionSize, centery + noMotionSize);
     glVertex2i(centerx - noMotionSize, centery + noMotionSize);
   } glEnd();
-  glBegin(GL_POINTS); {
-    glVertex2i(centerx - noMotionSize, centery - noMotionSize);
-    glVertex2i(centerx + noMotionSize, centery - noMotionSize);
-    glVertex2i(centerx + noMotionSize, centery + noMotionSize);
-    glVertex2i(centerx - noMotionSize, centery + noMotionSize);
-  } glEnd();
+  if (0){
+      glBegin(GL_POINTS); {
+      glVertex2i(centerx - noMotionSize, centery - noMotionSize);
+      glVertex2i(centerx + noMotionSize, centery - noMotionSize);
+      glVertex2i(centerx + noMotionSize, centery + noMotionSize);
+      glVertex2i(centerx - noMotionSize, centery + noMotionSize);
+    } glEnd();
+  }
   glBegin(GL_LINE_LOOP); {
     glVertex2i(centerx - maxMotionSize, centery - maxMotionSize);
     glVertex2i(centerx + maxMotionSize, centery - maxMotionSize);
     glVertex2i(centerx + maxMotionSize, centery + maxMotionSize);
     glVertex2i(centerx - maxMotionSize, centery + maxMotionSize);
   } glEnd();
-  glBegin(GL_POINTS); {
-    glVertex2i(centerx - maxMotionSize, centery - maxMotionSize);
-    glVertex2i(centerx + maxMotionSize, centery - maxMotionSize);
-    glVertex2i(centerx + maxMotionSize, centery + maxMotionSize);
-    glVertex2i(centerx - maxMotionSize, centery + maxMotionSize);
-  } glEnd();
+  if (0){
+      glBegin(GL_POINTS); {
+      glVertex2i(centerx - maxMotionSize, centery - maxMotionSize);
+      glVertex2i(centerx + maxMotionSize, centery - maxMotionSize);
+      glVertex2i(centerx + maxMotionSize, centery + maxMotionSize);
+      glVertex2i(centerx - maxMotionSize, centery + maxMotionSize);
+    } glEnd();
+  }
 
   // draw heading strip
   if (true /* always draw heading strip */) {
@@ -1307,6 +1614,43 @@ void			HUDRenderer::renderBox(SceneRenderer&)
   }
 }
 
+void HUDRenderer::drawMarkersInView( int centerx, int centery, const LocalPlayer* myTank )
+{
+  if (myTank) {
+    glPushMatrix();
+
+    hudColor3Afv( hudColor, 0.5f );
+
+    glTranslatef((float)centerx,(float)centery,0);
+    glLineWidth(2.0f);
+
+    // draw any waypoint markers
+    for (int i = 0; i < (int)enhancedMarkers.size(); i++) {
+      drawWaypointMarker(enhancedMarkers[i].color, 0.45f,
+	enhancedMarkers[i].pos, myTank->getPosition(),
+	enhancedMarkers[i].name, enhancedMarkers[i].friendly);
+    }
+
+    enhancedMarkers.clear();
+
+    // draw any lockon markers
+    for (int i = 0; i < (int)lockOnMarkers.size(); i++) {
+      drawLockonMarker(lockOnMarkers[i].color, 0.45f,
+	lockOnMarkers[i]. pos,myTank->getPosition(),
+	lockOnMarkers[i].name, lockOnMarkers[i].friendly);
+    }
+
+    lockOnMarkers.clear();
+
+    glLineWidth(1.0f);
+
+    glPopMatrix();
+
+    hudColor3Afv( hudColor, 0.5f );
+  }
+}
+
+
 void			HUDRenderer::renderPlaying(SceneRenderer& renderer)
 {
   // get view metrics
@@ -1316,6 +1660,7 @@ void			HUDRenderer::renderPlaying(SceneRenderer& renderer)
   const int ox = window.getOriginX();
   const int oy = window.getOriginY();
   const int centerx = width >> 1;
+  const int centery = viewHeight >> 1;
   int i;
   float y;
 
@@ -1346,6 +1691,19 @@ void			HUDRenderer::renderPlaying(SceneRenderer& renderer)
   if (showCracks)
     renderCracks();
 
+  // draw the markers, if we should
+  const bool  experimental = (RENDERER.useQuality() >= 3);
+
+  if (!BZDB.isTrue("_forbidMarkers") && experimental) {
+      bool enableTex = glIsEnabled(GL_TEXTURE_2D) != 0;
+      glDisable(GL_TEXTURE_2D);
+
+      drawMarkersInView(centerx,centery,myTank);
+
+      glEnable(GL_TEXTURE_2D);
+  }
+
+
   // draw status line
   renderStatus();
 
