@@ -1,9 +1,9 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
@@ -40,7 +40,8 @@ bool LocalCommand::execute(const char *commandLine)
   if (!mapOfCommands)
     return false;
   int i;
-  for (i = 0; commandLine[i] && !isspace(commandLine[i]); i++);
+  for (i = 0; commandLine[i] && !isspace(commandLine[i]); i++)
+    ;
   std::string commandToken(commandLine, i);
 
   std::map<std::string, LocalCommand *>::iterator it
