@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -32,8 +32,8 @@
 LocalPlayer*		LocalPlayer::mainPlayer = NULL;
 
 LocalPlayer::LocalPlayer(const PlayerId& _id,
-			 const char* name, const char* _email) :
-  BaseLocalPlayer(_id, name, _email),
+			 const char* name, const char* _motto) :
+  BaseLocalPlayer(_id, name, _motto),
   gettingSound(true),
   server(ServerLink::getServer()),
   location(Dead),
@@ -52,7 +52,6 @@ LocalPlayer::LocalPlayer(const PlayerId& _id,
   stuckFrameCount(0),
   spawning(false),
   wingsFlapCount(0),
-  handicap(0.0f),
   left(false),
   right(false),
   up(false),
@@ -1037,7 +1036,6 @@ void			LocalPlayer::restart(const float* pos, float _azimuth)
   down  = false;
   wantJump = false;
   doUpdateMotion(0.0f);
-  updateHandicap();
   entryDrop = true;
 
   // make me alive now
@@ -1158,6 +1156,7 @@ void			LocalPlayer::activateAutoPilot(bool autopilot)
   else if (!autopilot && isAutoPilot()) {
     setAutoPilot(false);
     server->sendAutoPilot(false);
+    setTarget(NULL);
   }
 }
 
@@ -1220,6 +1219,10 @@ bool			LocalPlayer::fireShot()
   const float timeStamp = float(TimeKeeper::getCurrent() - TimeKeeper::getNullTime());
   firingInfo.timeSent = timeStamp;
 
+  // always send a player-update message. To synchronize movement and
+  // shot start. They should generally travel on the same frame, when
+  // flushing the output queues.
+  server->sendPlayerUpdate(this);
   server->sendBeginShot(firingInfo);
 
   if (BZDB.isTrue("enableLocalShotEffect") && SceneRenderer::instance().useQuality() >= 2)
@@ -1373,14 +1376,20 @@ void			LocalPlayer::doJump()
     return;
   }
 
-  // add jump velocity (actually, set the vertical component since you
-  // can only jump if resting on something)
+  // jump velocity
   const float* oldVelocity = getVelocity();
   float newVelocity[3];
   newVelocity[0] = oldVelocity[0];
   newVelocity[1] = oldVelocity[1];
   if (flag == Flags::Wings) {
     newVelocity[2] = BZDB.eval(StateDatabase::BZDB_WINGSJUMPVELOCITY);
+    // if you're falling, wings will just slow you down
+    if (oldVelocity[2] < 0) {
+      newVelocity[2] += oldVelocity[2];
+    // if you're already going up faster, just keep doing that
+    } else if (oldVelocity[2] > newVelocity[2]) {
+      newVelocity[2] = oldVelocity[2];
+    }
   } else if (flag == Flags::Bouncy) {
     const float factor = 0.25f + ((float)bzfrand() * 0.75f);
     newVelocity[2] = factor * BZDB.eval(StateDatabase::BZDB_JUMPVELOCITY);
@@ -1429,6 +1438,7 @@ void			LocalPlayer::explodeTank()
   if (location == Dead || location == Exploding) return;
   float gravity      = BZDBCache::gravity;
   float explodeTim   = BZDB.eval(StateDatabase::BZDB_EXPLODETIME);
+  setExplodePos(getPosition());
   // Limiting max height increment to this value (the old default value)
   const float zMax  = 49.0f;
   setExplode(TimeKeeper::getTick());
@@ -1449,6 +1459,17 @@ void			LocalPlayer::explodeTank()
   } else {
     newVelocity[2] = oldVelocity[2];
   }
+  TankDeathOverride* death = getDeathEffect();
+  if (death)
+  {
+	  fvec3 v(newVelocity[0],newVelocity[1],newVelocity[2]);
+	  if(death->GetDeathVector(v))
+	  {
+		  newVelocity[0] = v.x;
+		  newVelocity[1] = v.y;
+		  newVelocity[2] = v.z;
+	  }
+  }
   setVelocity(newVelocity);
   location = Exploding;
   target = NULL;		// lose lock when dead
@@ -1500,9 +1521,9 @@ void			LocalPlayer::doFriction(float dt,
   }
 }
 
-void			LocalPlayer::doForces(float /*dt*/,
-					      float* /*velocity*/,
-					      float& /*angVel*/)
+void			LocalPlayer::doForces(float UNUSED(dt),
+					      float* UNUSED(velocity),
+					      float& UNUSED(angVel))
 {
   // apply external forces
   // do nothing -- no external forces right now
@@ -1526,6 +1547,12 @@ bool			LocalPlayer::checkHit(const Player* source,
     // my own shock wave cannot kill me
     if (source == this && ((shot->getFlag() == Flags::ShockWave) || (shot->getFlag() == Flags::Thief))) continue;
 
+    // if no team kills, shots of my team cannot kill me. Thief can still take
+	// a teammate's flag.
+    if (getTeam() != RogueTeam && !World::getWorld()->allowTeamKills() &&
+		shot->getFlag() != Flags::Thief && shot->getTeam() == getTeam() &&
+		source != this) continue;
+
     // short circuit test if shot can't possibly hit.
     // only superbullet or shockwave can kill zoned dude
     const FlagType* shotFlag = shot->getFlag();
@@ -1617,51 +1644,6 @@ void			LocalPlayer::changeScore(short deltaWins,
   }
 }
 
-// calculate overall relative score to other players
-int			 LocalPlayer::getHandicapScoreBase() const
-{
-  int winners = 0;
-  int loosers = 0;
-  const int maxplayers = World::getWorld()->getMaxPlayers();
-
-  // compute overall wins and losses in relation to other players
-  for (int i = 0; i < maxplayers; i++) {
-    RemotePlayer *player = World::getWorld()->getPlayer(i);
-    if ((player == NULL) || (player->getId() == LocalPlayer::getMyTank()->getId())) {
-      continue;
-    }
-    winners += player->getLocalWins();
-    loosers += player->getLocalLosses();
-  }
-  return loosers - winners;
-}
-
-float			LocalPlayer::updateHandicap()
-{
-  /* compute and return the handicap */
-
-  if (World::getWorld()->allowHandicap()) {
-    // a relative score of -50 points will provide maximum handicap
-    handicap = float(getHandicapScoreBase()) / BZDB.eval(StateDatabase::BZDB_HANDICAPSCOREDIFF);
-
-    /* limit how much of a handicap is afforded, and only provide
-     * handicap advantages instead of disadvantages.
-     */
-    if (handicap > 1.0f) {
-      // advantage
-      handicap = 1.0f;
-    } else if (handicap < 0.0f) {
-      // disadvantage
-      handicap = 0.0f;
-    }
-
-    return handicap;
-  }
-
-  // no handicap gameplay
-  return 0.0f;
-}
-
 void			LocalPlayer::addAntidote(SceneDatabase* scene)
 {
   if (antidoteFlag)
