@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __LOCALPLAYER_H__
@@ -23,7 +23,6 @@
 /* common interface headers */
 #include "Obstacle.h"
 #include "TimeKeeper.h"
-#include "BzfEvent.h"
 
 /* local interface headers */
 #include "Player.h"
@@ -55,12 +54,11 @@ public:
   };
 
   LocalPlayer(const PlayerId&,
-	      const char* name, const char* email);
+	      const char* name, const char* motto);
   ~LocalPlayer();
 
   Location	getLocation() const;
   FiringStatus	getFiringStatus() const;
-  float		getReloadTime() const;
   float		getFlagShakingTime() const;
   int		getFlagShakingWins() const;
   const float*	getAntidoteLocation() const;
@@ -75,13 +73,15 @@ public:
   void		setPause(bool = true);
   void		activateAutoPilot(bool = true);
   bool		fireShot();
-  void		forceReload(float time = 0.0f);
   void		explodeTank();
   void		doJump();
   void		setJump();
   void		setJumpPressed(bool value);
   void		setTarget(const Player*);
 
+  float		getReloadTime() const;
+
+
   void		setNemesis(const Player*);
   const Player*	getNemesis() const;
 
@@ -129,7 +129,6 @@ protected:
   void		doFriction(float dt, const float *oldVelocity, float *newVelocity);
   void		doForces(float dt, float* velocity, float& angVel);
   float		updateHandicap();
-  virtual int   getHandicapScoreBase() const;
   LocalShotPath**	shots;
   bool	  gettingSound;
   ServerLink*	server;
@@ -143,7 +142,6 @@ private:
 private:
   Location	location;
   FiringStatus	firingStatus;
-  TimeKeeper	jamTime;
   TimeKeeper	bounceTime;
   TimeKeeper	agilityTime;
   float		flagShakingTime;
@@ -164,7 +162,6 @@ private:
   int		stuckFrameCount;
   bool		spawning;
   int		wingsFlapCount;
-  float		handicap;
   bool		left;
   bool		right;
   bool		up;
@@ -267,6 +264,11 @@ inline int LocalPlayer::getSpeed() {
     return 0;
 }
 
+inline LocalPlayer* LocalPlayer::getMyTank()
+{
+  return mainPlayer;
+}
+
 #endif /* __LOCALPLAYER_H__ */
 
 // Local Variables: ***
