@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,26 +7,16 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 
-/* BZFlag common header */
-#include "common.h"
-
 /* interface headers */
 #include "MainWindow.h"
-#include "BzfWindow.h"
-
-/* system headers */
-#include <string>
 
 /* common implementation headers */
 #include "global.h"
 #include "SceneRenderer.h"
-#include "bzfgl.h"
-#include "BzfJoystick.h"
-#include "OpenGLGState.h"
 
 //
 // MainWindow
@@ -74,9 +64,9 @@ void			MainWindow::setPosition(int x, int y)
   window->setPosition(x, y);
 }
 
-void			MainWindow::setSize(int width, int height)
+void			MainWindow::setSize(int _width, int _height)
 {
-  window->setSize(width, height);
+  window->setSize(_width, _height);
   resize();
 }
 
@@ -86,20 +76,46 @@ void			MainWindow::showWindow(bool on)
   if (on) resize();
 }
 
+
+inline int MainWindow::getYOffset() const
+{
+  if (quadrant == FullWindow) {
+    return 0;
+  }
+  return ((trueHeight + 1) >> 1) - yOrigin;
+}
+
 void			MainWindow::warpMouse()
 {
   // move mouse to center of view window (zero motion box)
-  int y = viewHeight >> 1;
-  if (quadrant != FullWindow) y += ((trueHeight+1) >> 1) - yOrigin;
-  window->warpMouse((width >> 1) + xOrigin, y);
+  window->warpMouse((width >> 1) + xOrigin, (viewHeight >> 1) + getYOffset());
+}
+
+void			MainWindow::warpMouse(int mx, int my)
+{
+  window->warpMouse(mx, my);
+}
+
+void			MainWindow::warpMouseCenterX()
+{
+  int mx, my;
+  getMousePosition(mx, my);
+  window->warpMouse((width >> 1) + xOrigin, my + (viewHeight >> 1) + getYOffset());
+}
+
+void			MainWindow::warpMouseCenterY()
+{
+  int mx, my;
+  getMousePosition(mx, my);
+  const int y = (viewHeight >> 1) + getYOffset();
+  window->warpMouse(mx + (width >> 1) + xOrigin, y);
 }
 
 void			MainWindow::getMousePosition(int& mx, int& my) const
 {
   window->getMouse(mx, my);
   mx -= (width >> 1) + xOrigin;
-  my -= (viewHeight >> 1);
-  if (quadrant != FullWindow) my -= ((trueHeight+1) >> 1) - yOrigin;
+  my -= (viewHeight >> 1) + getYOffset();
 }
 
 void			MainWindow::grabMouse()
@@ -275,9 +291,25 @@ void		    MainWindow::getJoyDevices(std::vector<std::string>
   joystick->getJoyDevices(list);
 }
 
+void		    MainWindow::getJoyDeviceAxes(std::vector<std::string>
+						 &list) const
+{
+  joystick->getJoyDeviceAxes(list);
+}
+
+void		    MainWindow::setJoyXAxis(const std::string axis)
+{
+  joystick->setXAxis(axis);
+}
+
+void		    MainWindow::setJoyYAxis(const std::string axis)
+{
+  joystick->setYAxis(axis);
+}
+
 void			MainWindow::initJoystick(std::string &joystickName) {
   joystick->initJoystick(joystickName.c_str());
-};
+}
 
 // Local Variables: ***
 // mode:C++ ***
