@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,37 +7,32 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-// bzflag global header
-#include "common.h"
-
-// system headers
-#include <string.h>
-#include <string>
+/* interface header */
+#include "Player.h"
 
 // common interface headers
 #include "TankSceneNode.h"
-#include "SceneDatabase.h"
+#include "SceneRenderer.h"
 #include "SphereSceneNode.h"
 #include "OpenGLMaterial.h"
 #include "BZDBCache.h"
 #include "TextureManager.h"
 #include "CollisionManager.h"
-#include "Obstacle.h"
 #include "ObstacleMgr.h"
 #include "PhysicsDriver.h"
+#include "ObstacleList.h"
+#include "WallObstacle.h"
 
-// implementation-specific bzfs-specific headers
+// local implementation headers
 #include "playing.h"
-#include "Player.h"
 #include "World.h"
-#include "ShotPath.h"
-#include "ShotStatistics.h"
 #include "TrackMarks.h"
 #include "sound.h"
-
+#include "effectsRenderer.h"
+#include "Roaming.h"
 
 // for dead reckoning
 static const float	MaxUpdateTime = 1.0f;		// seconds
@@ -49,8 +44,10 @@ static const float	MaxUpdateTime = 1.0f;		// seconds
 int		Player::tankTexture = -1;
 
 Player::Player(const PlayerId& _id, TeamColor _team,
-	       const char* name, const char* _email, const PlayerType _type) :
+	       const char* name, const char* _motto, const PlayerType _type) :
   lastObstacle(NULL),
+  pauseMessageState(false),
+  handicap(0.0f),
   notResponding(false),
   hunted(false),
   id(_id),
@@ -88,20 +85,29 @@ Player::Player(const PlayerId& _id, TeamColor _team,
   setUserSpeed(0.0f);
   setUserAngVel(0.0f);
 
+  apparentVelocity[0] = apparentVelocity[1] = apparentVelocity[2] = 0.0f;
+  inputTimestamp = 0.0f;
+
+  reportedHits = 0;
+  computedHits = 0;
+
   // set call sign
   ::strncpy(callSign, name, CallSignLen);
   callSign[CallSignLen-1] = '\0';
 
-  // set email address
-  ::strncpy(email, _email, EmailLen);
-  email[EmailLen-1] = '\0';
+  setMotto(_motto);	// will be superseded by the server
 
   if (id != ServerPlayer) {
     // make scene nodes
     tankNode = new TankSceneNode(state.pos, forward);
     tankIDLNode = new TankIDLSceneNode(tankNode);
     changeTeam(team);
-    pausedSphere = new SphereSceneNode(state.pos, 1.5f * BZDBCache::tankRadius);
+    const float sphereRad = (1.5f * BZDBCache::tankRadius);
+    if (RENDERER.useQuality() >= 2) {
+      pausedSphere = new SphereLodSceneNode(state.pos, sphereRad);
+    } else {
+      pausedSphere = new SphereBspSceneNode(state.pos, sphereRad);
+    }
     pausedSphere->setColor(0.0f, 0.0f, 0.0f, 0.5f);
   }
 
@@ -122,7 +128,10 @@ Player::Player(const PlayerId& _id, TeamColor _team,
   alphaTarget = 1.0f;
   teleAlpha = 1.0f;
 
-  lastTrackDraw = TimeKeeper::getCurrent();
+  haveIpAddr = false; // no IP address yet
+  lastTrackDraw = TimeKeeper::getTick();
+
+  spawnTime = TimeKeeper::getTick();
 
   return;
 }
@@ -149,6 +158,33 @@ static float rabbitRank (int wins, int losses) {
   return average * penalty;
 }
 
+void Player::setMotto(const char* _motto)
+{
+  strncpy(motto, _motto, MottoLen);
+  motto[MottoLen - 1] = '\0';	// ensure null termination
+}
+
+float Player::getTKRatio() const
+{
+  if (wins == 0) {
+    // well, we have to return *something* if they have no wins
+    return (float)tks/1.0f;
+  } else {
+    return (float)tks/(float)wins;
+  }
+}
+
+// returns a value between 1.0 and -1.0
+float Player::getNormalizedScore() const
+{
+  return ((float)wins - losses) / ((wins+losses>20) ? wins+losses : 20);
+}
+
+float Player::getLocalNormalizedScore() const
+{
+  return ((float)localWins - localLosses)
+	/ ((localWins+localLosses>5) ? localWins+localLosses : 5);
+}
 
 short Player::getRabbitScore() const
 {
@@ -164,6 +200,18 @@ float Player::getRadius() const
   return (dimensionsScale[0] * BZDBCache::tankRadius);
 }
 
+float Player::getMaxSpeed ( void ) const
+{
+	// BURROW and AGILITY will not be taken into account
+	const FlagType* flag = getFlag();
+	float maxSpeed = BZDBCache::tankSpeed;
+	if (flag == Flags::Velocity) {
+		maxSpeed *= BZDB.eval(StateDatabase::BZDB_VELOCITYAD);
+	} else if (flag == Flags::Thief) {
+		maxSpeed *= BZDB.eval(StateDatabase::BZDB_THIEFVELAD);
+	}
+	return maxSpeed;
+}
 
 void Player::getMuzzle(float* m) const
 {
@@ -190,6 +238,11 @@ float Player::getMuzzleHeight() const
   return (dimensionsScale[2] * BZDB.eval(StateDatabase::BZDB_MUZZLEHEIGHT));
 }
 
+void Player::forceReload(float time)
+{
+  jamTime  = TimeKeeper::getTick();
+  jamTime += time;
+}
 
 void Player::move(const float* _pos, float _azimuth)
 {
@@ -341,6 +394,19 @@ void Player::setExplode(const TimeKeeper& t)
   updateFlagEffect(Flags::Null);
 }
 
+void Player::setDeathEffect ( TankDeathOverride *e )
+{
+  if (tankNode)
+    tankNode->setDeathOverride(e);
+}
+
+TankDeathOverride* Player::getDeathEffect ( void )
+{
+  if (tankNode)
+    return tankNode->getDeathOverride();
+
+  return NULL;
+}
 
 void Player::setTeleport(const TimeKeeper& t, short from, short to)
 {
@@ -386,7 +452,7 @@ void Player::updateTrackMarks()
   const float minSpeed = 0.1f; // relative speed slop
 
   if (isAlive() && !isFalling() && !isPhantomZoned()) {
-    const float lifeTime = TimeKeeper::getCurrent() - lastTrackDraw;
+    const float lifeTime = float(TimeKeeper::getTick() - lastTrackDraw);
     if (lifeTime > TrackMarks::updateTime) {
       bool drawMark = true;
       float markPos[3];
@@ -412,7 +478,7 @@ void Player::updateTrackMarks()
       if (drawMark) {
 	TrackMarks::addMark(markPos, dimensionsScale[1],
 			    state.azimuth, state.phydrv);
-	lastTrackDraw = TimeKeeper::getCurrent();
+	lastTrackDraw = TimeKeeper::getTick();
       }
     }
   }
@@ -516,7 +582,6 @@ bool Player::hitObstacleResizing()
   return false;
 }
 
-
 void Player::updateTranslucency(float dt)
 {
   // update the alpha value
@@ -577,7 +642,7 @@ void Player::updateTreads(float dt)
   }
 
   // setup the angular component
-  const float angularScale = 2.0f; // spin factor (at 1.0, the edges line up)
+  const float angularScale = 4.0f; // spin factor (at 1.0, the edges line up)
   angularFactor *= angularScale;
   const float halfWidth = 0.5f * BZDBCache::tankWidth;
   // not using dimensions[1], because it may be set to 0.001 by a Narrow flag
@@ -705,15 +770,9 @@ void Player::setVisualTeam (TeamColor visualTeam)
     shininess = tankShininess;
   }
 
-  // get the texture each time, since it's just a refrence
-  const bool hunter = World::getWorld()->allowRabbit() && visualTeam != RabbitTeam;
-
   TextureManager &tm = TextureManager::instance();
   std::string texName;
-  if (hunter)
-    texName = BZDB.get("hunterTeamPrefix");
-  else
-    texName = Team::getImagePrefix(visualTeam);
+  texName = Team::getImagePrefix(visualTeam);
 
   texName += BZDB.get("tankTexture");
 
@@ -726,18 +785,10 @@ void Player::setVisualTeam (TeamColor visualTeam)
   if (tankTexture < 0)
     tankTexture = tm.getTextureID(texName.c_str(),false);
 
-  // we are the hunter, we are orange..
-  // TODO this is cheap, just untill a "hunter" team is made
-  if (hunter) {
-    color[0] = 1.0f;
-    color[1] = 0.5f;
-    color[2] = 0.0f;
-  } else {
-    const float* _color = Team::getTankColor(visualTeam);
-    color[0] = _color[0];
-    color[1] = _color[1];
-    color[2] = _color[2];
-  }
+  const float* _color = Team::getTankColor(visualTeam);
+  color[0] = _color[0];
+  color[1] = _color[1];
+  color[2] = _color[2];
   tankNode->setMaterial(OpenGLMaterial(tankSpecular, emissive, shininess));
   tankNode->setTexture(tankTexture);
 
@@ -776,6 +827,8 @@ void Player::addToScene(SceneDatabase* scene, TeamColor effectiveTeam,
 			bool inCockpit, bool seerView,
 			bool showTreads, bool showIDL)
 {
+  const GLfloat groundPlane[4] = {0.0f, 0.0f, 1.0f, 0.0f};
+
   if (!isAlive() && !isExploding()) {
     return; // don't draw anything
   }
@@ -843,9 +896,9 @@ void Player::addToScene(SceneDatabase* scene, TeamColor effectiveTeam,
 	World::getWorld()->hitBuilding(state.pos, a,
 				       dimensions[0], dimensions[1],
 				       dimensions[2]);
-      if (obstacle && obstacle->isCrossing(state.pos, a,
+      if ((obstacle && obstacle->isCrossing(state.pos, a,
 					   dimensions[0], dimensions[1],
-					   dimensions[2], plane) ||
+					   dimensions[2], plane)) ||
 	  World::getWorld()->crossingTeleporter(state.pos, a,
 						dimensions[0], dimensions[1],
 						dimensions[2], plane)) {
@@ -863,16 +916,12 @@ void Player::addToScene(SceneDatabase* scene, TeamColor effectiveTeam,
     }
     else if (getPosition()[2] < 0.0f) {
       // this should only happen with Burrow flags
-      GLfloat plane[4];
-      plane[0] = plane[1] = 0.0f;
-      plane[2] = 1.0f;
-      plane[3] = 0.0f;
-      tankNode->setClipPlane(plane);
+      tankNode->setClipPlane(groundPlane);
     } // isCrossingWall()
   }   // isAlive()
   else if (isExploding() && (state.pos[2] > ZERO_TOLERANCE)) {
-    float t = (TimeKeeper::getTick() - explodeTime) /
-	      BZDB.eval(StateDatabase::BZDB_EXPLODETIME);
+    float t = float((TimeKeeper::getTick() - explodeTime) /
+	      BZDB.eval(StateDatabase::BZDB_EXPLODETIME));
     if (t > 1.0f) {
       // FIXME - setStatus(DeadStatus);
       t = 1.0f;
@@ -880,7 +929,17 @@ void Player::addToScene(SceneDatabase* scene, TeamColor effectiveTeam,
       // shouldn't happen but why take chances
       t = 0.0f;
     }
+    // fade at the end of the explosion
+    const float fadeRatio = 0.8f;
+    if (t > fadeRatio) {
+      GLfloat newColor[4];
+      memcpy(newColor, color, sizeof(float[3]));
+      const float fadeFactor = (1.0f - t) / (1.0f - fadeRatio);
+      newColor[3] = color[3] * fadeFactor;
+      tankNode->setColor(newColor);
+    }
     tankNode->setExplodeFraction(t);
+    tankNode->setClipPlane(groundPlane); // shadows are not clipped
     scene->addDynamicNode(tankNode);
   }
 
@@ -949,6 +1008,7 @@ void Player::spawnEffect()
       dimensionsScale[i] = 0.01f;
     }
   }
+  spawnTime = TimeKeeper::getTick();
   return;
 }
 
@@ -973,10 +1033,10 @@ void Player::addShots(SceneDatabase* scene, bool colorblind) const
 void* Player::unpack(void* buf, uint16_t code)
 {
   float timestamp;
-  PlayerId id;
+  PlayerId ident;
 
   buf = nboUnpackFloat(buf, timestamp);
-  buf = nboUnpackUByte(buf, id);
+  buf = nboUnpackUByte(buf, ident);
   buf = state.unpack(buf, code);
 
   setDeadReckoning(timestamp);
@@ -990,7 +1050,7 @@ bool Player::validTeamTarget(const Player *possibleTarget) const
 {
   TeamColor myTeam = getTeam();
   TeamColor targetTeam = possibleTarget->getTeam();
-  if (myTeam != targetTeam) {
+  if (myTeam != targetTeam || !World::getWorld()->allowTeams()) {
     return true;
   }
 
@@ -1122,8 +1182,11 @@ bool Player::isDeadReckoningWrong() const
     return true;
   }
 
+  // time since setdeadreckoning
+  const float dt = float(TimeKeeper::getTick() - inputTime);
+
   // otherwise always send at least one packet per second
-  if ((TimeKeeper::getTick() - inputTime) >= MaxUpdateTime) {
+  if (dt >= MaxUpdateTime) {
     return true;
   }
 
@@ -1131,7 +1194,6 @@ bool Player::isDeadReckoningWrong() const
   float predictedPos[3];
   float predictedVel[3];
   float predictedAzimuth;
-  float dt = TimeKeeper::getTick() - inputTime;
   getDeadReckoning(predictedPos, &predictedAzimuth, predictedVel, dt);
 
   // always send a new packet on reckoned touchdown
@@ -1146,7 +1208,7 @@ bool Player::isDeadReckoningWrong() const
   // client side throttling
   const int throttleRate = int(BZDB.eval(StateDatabase::BZDB_UPDATETHROTTLERATE));
   const float minUpdateTime = (throttleRate > 0) ? (1.0f / throttleRate) : 0.0f;
-  if (TimeKeeper::getTick() - inputTime < minUpdateTime) {
+  if (dt < minUpdateTime) {
     return false;
   }
 
@@ -1174,7 +1236,7 @@ bool Player::isDeadReckoningWrong() const
 
   float angleTolerance = BZDB.eval(StateDatabase::BZDB_ANGLETOLERANCE);
   if (fabsf(state.azimuth - predictedAzimuth) > angleTolerance) {
-    DEBUG4 ("state.azimuth = %f, predictedAzimuth = %f\n",
+    logDebugMessage(4,"state.azimuth = %f, predictedAzimuth = %f\n",
 	    state.azimuth, predictedAzimuth);
     return true;
   }
@@ -1194,7 +1256,7 @@ void Player::doDeadReckoning()
   float predictedPos[3];
   float predictedVel[3];
   float predictedAzimuth;
-  float dt = TimeKeeper::getTick() - inputTime;
+  float dt = float(TimeKeeper::getTick() - inputTime);
   getDeadReckoning(predictedPos, &predictedAzimuth, predictedVel, dt);
 
   // setup notResponding
@@ -1220,34 +1282,43 @@ void Player::doDeadReckoning()
   // setup remote players' landing sounds and graphics, and jumping sounds
   if (isAlive()) {
     // the importance level of the remote sounds
-    const bool remoteImportant = false;
+    const bool soundImportance = false;
+    const bool localSound = (ROAM.isRoaming()
+			     && (ROAM.getMode() == Roaming::roamViewFP)
+			     && (ROAM.getTargetTank() == this));
 
     // check for a landing
     if (((oldStatus & PlayerState::Falling) != 0) &&
 	((inputStatus & PlayerState::Falling) == 0)) {
       // setup the squish effect
       setLandingSpeed(oldZSpeed);
+
+      // make it "land"
+      EFFECTS.addLandEffect(getColor(),state.pos,state.azimuth);
+
       // setup the sound
       if (BZDB.isTrue("remoteSounds")) {
 	if ((getFlag() != Flags::Burrow) || (predictedPos[2] > 0.0f)) {
-	  playWorldSound(SFX_LAND, state.pos, remoteImportant);
+	  playSound(SFX_LAND, state.pos, soundImportance, localSound);
 	} else  {
 	  // probably never gets played
-	  playWorldSound(SFX_BURROW,  state.pos, remoteImportant);
+	  playSound(SFX_BURROW, state.pos, soundImportance, localSound);
 	}
       }
     }
 
     // play jumping type sounds, and then clear them
     if (state.sounds != PlayerState::NoSounds) {
-      if ((state.sounds & PlayerState::JumpSound) != 0) {
-	playWorldSound(SFX_JUMP, state.pos, remoteImportant);
-      }
-      if ((state.sounds & PlayerState::WingsSound) != 0) {
-	playWorldSound(SFX_FLAP, state.pos, remoteImportant);
-      }
-      if ((state.sounds & PlayerState::BounceSound) != 0) {
-	playWorldSound(SFX_BOUNCE, state.pos, remoteImportant);
+      if (BZDB.isTrue("remoteSounds")) {
+	if ((state.sounds & PlayerState::JumpSound) != 0) {
+	  playSound(SFX_JUMP, state.pos, soundImportance, localSound);
+	}
+	if ((state.sounds & PlayerState::WingsSound) != 0) {
+	  playSound(SFX_FLAP, state.pos, soundImportance, localSound);
+	}
+	if ((state.sounds & PlayerState::BounceSound) != 0) {
+	  playSound(SFX_BOUNCE, state.pos, soundImportance, localSound);
+	}
       }
       state.sounds = PlayerState::NoSounds;
     }
@@ -1266,66 +1337,25 @@ void Player::doDeadReckoning()
 
 
 // How long does the filter takes to be considered "initialized"
-const int   DRStateStable      = 10;
+const int   DRStateStable      = 100;
 const float maxToleratedJitter = 1.0f;
 
 void Player::setDeadReckoning(float timestamp)
 {
-  // offset should be the time packet has been delayed above average
-  offset = timestamp - (TimeKeeper::getTick() - TimeKeeper::getNullTime())
-    - deltaTime;
-
-  bool discardUpdate = false;
-  // at first stage, Delta time is computed as the average of the last
-  // differences in time (local & remote) the values is then updated
-  // with the new samples, smoothed with the old values
-  float alphaFactor = 1.0f / float(deadReckoningState + 1);
-  if (deadReckoningState >= DRStateStable) {
-    if (fabs(offset) > maxToleratedJitter) {
-      // Put a threshold on untimed measurement
-      offset = (offset > 0) ? maxToleratedJitter : -maxToleratedJitter;
-      // and discard, but before adjust delta a little
-      discardUpdate = true;
-    }
-    else if (offset > 0) {
-      // fast alignment to the packet that take less travel time
-      // that's for trying to have less lag
-      alphaFactor = 1.0f;
-    }
-  }
-  // alpha filtering
-  deltaTime = deltaTime + (offset * alphaFactor);
-  if (discardUpdate) {
-    return;
-  }
-  // when alphaFactor is 1, that really means we are
-  // re-initializing deltaTime so offset should be zero
-  if (alphaFactor == 1.0f) {
-    offset = 0.0f;
-  }
-  if (deadReckoningState < DRStateStable) {
-    deadReckoningState++;
+  // calculate apparent speed
+  //  const float timeelapsed = TimeKeeper::getTick() - inputTime;
+  const float dt = timestamp - inputTimestamp;
+  inputTimestamp = timestamp;
+  if (dt > 0.0f && dt < MaxUpdateTime * 1.5f) {
+    apparentVelocity[0] = (inputPos[0] - state.pos[0]) / dt;
+    apparentVelocity[1] = (inputPos[1] - state.pos[1]) / dt;
+    apparentVelocity[2] = (inputPos[2] - state.pos[2]) / dt;
   }
 
   // set the current state
   setDeadReckoning();
 
-  // adjust for the time offset
-  if (deadReckoningState >= DRStateStable) {
-    // get predicted offset state
-    float predictedPos[3];
-    float predictedVel[3];
-    float predictedAzimuth;
-    getDeadReckoning(predictedPos, &predictedAzimuth, predictedVel, -offset);
-    move(predictedPos, predictedAzimuth);
-    setVelocity(predictedVel);
-    // set the current state, again
-    setDeadReckoning();
-  }
-
   setRelativeMotion();
-
-  return;
 }
 
 
@@ -1348,7 +1378,7 @@ void Player::setDeadReckoning()
   // the relative motion information (with respect to the physics drivers)
   calcRelativeMotion(inputRelVel, inputRelSpeed, inputRelAngVel);
 
-  // setup the turning pararmeters
+  // setup the turning parameters
   inputTurning = false;
   if (fabsf(inputRelAngVel) > 0.001f) {
     inputTurning = true;
@@ -1362,6 +1392,12 @@ void Player::setDeadReckoning()
   return;
 }
 
+void Player::setExplodePos( const float * p)
+{
+  if (tankNode)
+    tankNode->explodePos = fvec3(p[0],p[1],p[2]);
+}
+
 
 void Player::renderRadar() const
 {
@@ -1375,6 +1411,28 @@ void Player::renderRadar() const
 }
 
 
+bool Player::getIpAddress(Address& addr)
+{
+  if (!haveIpAddr) {
+    return false;
+  }
+  addr = ipAddr;
+  return true;
+}
+
+
+void Player::setIpAddress(const Address& addr)
+{
+  ipAddr = addr;
+  haveIpAddr = true;
+}
+
+
+void Player::setHandicap(float _handicap)
+{
+    handicap = _handicap;
+}
+
 // Local Variables: ***
 // mode:C++ ***
 // tab-width: 8 ***
