@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -139,7 +139,7 @@ void RadarRenderer::drawTank(const float pos[3], const Player* player)
     const float tankAngle = player->getAngle();
     glPushMatrix();
     glRotatef(float(tankAngle * 180.0 / M_PI), 0.0f, 0.0f, 1.0f);
-    if (useTankModels) {
+    if (0/*useTankModels*/) { // TODO fix the fancy drawing
       drawFancyTank(player);
     } else {
       const float* dims = player->getDimensions();
@@ -286,12 +286,14 @@ void RadarRenderer::renderFrame(SceneRenderer& renderer)
     glVertex2f(left, bottom);
   } glEnd();
 
-  glBegin(GL_POINTS); {
-    glVertex2f(left, top);
-    glVertex2f(right, top);
-    glVertex2f(right, bottom);
-    glVertex2f(left, bottom);
-  } glEnd();
+  if (0){
+	  glBegin(GL_POINTS); {
+		glVertex2f(left, top);
+		glVertex2f(right, top);
+		glVertex2f(right, bottom);
+		glVertex2f(left, bottom);
+	  } glEnd();
+	}
 
   if (BZDBCache::blend)
 	  glDisable(GL_BLEND);
@@ -349,8 +351,6 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
     return;
   }
 
-  bool rabbitMode = World::getWorld()->allowRabbit();
-
   smooth = !multiSampled && BZDBCache::smooth;
   const bool fastRadar = ((BZDBCache::radarStyle == 1) ||
 			  (BZDBCache::radarStyle == 2)) && BZDBCache::zbuffer;
@@ -497,7 +497,7 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
     } else {
       useTankDimensions = false;
     }
-    if (useTankDimensions && (renderer.useQuality() >= 3)) {
+    if (useTankDimensions && (renderer.useQuality() >= 2)) {
       useTankModels = true;
     } else {
       useTankModels = false;
@@ -603,9 +603,9 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
 
 	const float *color;
 	if (myTank->getFlag() == Flags::Colorblindness) {
-	  color = Team::getRadarColor(RogueTeam,rabbitMode);
+	  color = Team::getRadarColor(RogueTeam);
 	} else {
-	  color = Team::getRadarColor(player->getTeam(),rabbitMode);
+	  color = Team::getRadarColor(player->getTeam());
 	}
 
 	float dimmedcolor[3];
@@ -615,7 +615,7 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
 	glColor3fv(dimmedcolor);
       } else {
 	glColor3fv(Team::getRadarColor(myTank->getFlag() ==
-			     Flags::Colorblindness ? RogueTeam : player->getTeam(), rabbitMode));
+			     Flags::Colorblindness ? RogueTeam : player->getTeam()));
       }
       // If this tank is hunted flash it on the radar
       if (player->isHunted() && myTank->getFlag() != Flags::Colorblindness) {
@@ -652,9 +652,9 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
 	  const float *shotcolor;
 	  if (coloredShot) {
 	    if (myTank->getFlag() == Flags::Colorblindness)
-	      shotcolor = Team::getRadarColor(RogueTeam,rabbitMode);
+	      shotcolor = Team::getRadarColor(RogueTeam);
 	    else
-	      shotcolor = Team::getRadarColor(player->getTeam(),rabbitMode);
+	      shotcolor = Team::getRadarColor(player->getTeam());
 	    const float cs = colorScale(shot->getPosition()[2], muzzleHeight);
 	    glColor3f(shotcolor[0] * cs, shotcolor[1] * cs, shotcolor[2] * cs);
 	  } else {
@@ -678,6 +678,16 @@ void RadarRenderer::render(SceneRenderer& renderer, bool blank, bool observer)
       // don't draw normal flags if we aren't supposed to
       if (flag.type->flagTeam == NoTeam && !drawNormalFlags)
 	continue;
+      if (BZDB.isTrue(StateDatabase::BZDB_HIDETEAMFLAGSONRADAR)) {
+	if (flag.type->flagTeam != ::NoTeam) {
+	  continue;
+	}
+      }
+      if (BZDB.isTrue(StateDatabase::BZDB_HIDEFLAGSONRADAR)) {
+	if (flag.type) {
+	  continue;
+	}
+      }
       // Flags change color by height
       const float cs = colorScale(flag.position[2], muzzleHeight);
       const float *flagcolor = flag.type->getColor();
