@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -18,9 +18,9 @@
 
 
 RemotePlayer::RemotePlayer(const PlayerId& _id, TeamColor _team,
-			   const char* _name, const char* _email,
+			   const char* _name, const char* _motto,
 			   const PlayerType _type) :
-  Player(_id, _team, _name, _email, _type)
+  Player(_id, _team, _name, _motto, _type)
 {
   numShots = World::getWorld()->getMaxShots();
   shots = new RemoteShotPath*[numShots];
@@ -65,8 +65,8 @@ void			RemotePlayer::addShot(const FiringInfo& info)
   //	 (flag each shot with a 'default dimensions' state?)
   move(newpos, getAngle());
   // FIXME - timestamp of shots are handled slightly different than for
-  //         tank position updates, better so ignore them for now
-  //         detail: settick | shot: current() | move | draw | tankpos: tick()
+  //	 tank position updates, better so ignore them for now
+  //	 detail: settick | shot: current() | move | draw | tankpos: tick()
   //setDeadReckoning(info.timeSent);
   setDeadReckoning(-1.0f);
 }
