@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -37,7 +37,7 @@
 typedef std::vector<std::string> NameList;
 
 extern int curMaxPlayers;
-extern RemotePlayer** player;
+extern RemotePlayer** remotePlayers;
 extern RobotPlayer* robots[MAX_ROBOTS];
 extern int numRobots;
 
