@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* FIXME -- ugh.. the class header for this file is listed as a public header
@@ -16,39 +16,33 @@
  * may be moved elsewhere.
  */
 
-#include "common.h"
-
-/* system implementation headers */
-#include <stdlib.h>
-#include <string.h>
+// interface header
+#include "SceneRenderer.h"
 
 /* common implementation headers */
-#include "bzfgl.h"
-#include "SceneRenderer.h"
 #include "SceneDatabase.h"
-#include "SceneNode.h"
 #include "MainWindow.h"
-#include "OpenGLGState.h"
-#include "RenderNode.h"
 #include "DynamicColor.h"
 #include "TextureMatrix.h"
-#include "BzfWindow.h"
 #include "TankSceneNode.h"
 #include "StateDatabase.h"
 #include "TextUtils.h"
 #include "ParseColor.h"
 #include "BZDBCache.h"
-#include "TankGeometryMgr.h"
+#include "MeshSceneNode.h"
 
 /* FIXME - local implementation dependancies */
 #include "BackgroundRenderer.h"
-#include "HUDRenderer.h"
 #include "LocalPlayer.h"
 #include "daylight.h"
 #include "World.h"
 #include "TrackMarks.h"
 
 
+static bool mapFog;
+static bool setupMapFog();
+
+
 #ifdef GL_ABGR_EXT
 static int		strrncmp(const char* s1, const char* s2, int num)
 {
@@ -109,7 +103,6 @@ SceneRenderer::SceneRenderer() :
 				panelOpacity(0.3f),
 				radarSize(4),
 				maxMotionFactor(5),
-				useFogHack(false),
 				viewType(Normal),
 				inOrder(false),
 				depthRange(0),
@@ -121,7 +114,8 @@ SceneRenderer::SceneRenderer() :
 				lastFrame(true),
 				sameFrame(false),
 				needStyleUpdate(true),
-				rebuildTanks(true)
+				rebuildTanks(true),
+				fogActive(false)
 {
   lightsSize = 4;
   lights = new OpenGLLight*[lightsSize];
@@ -142,8 +136,8 @@ void SceneRenderer::setWindow(MainWindow* _window) {
   window->getWindow()->makeCurrent();
   GLint bits;
   if (!BZDB.isSet("zbuffer")) {
-    glGetIntegerv(GL_DEPTH_BITS, &bits);
-    BZDB.set("zbuffer", (bits > 0) ? "1" : "0");
+//	  glGetIntegerv(GL_DEPTH_BITS, &bits);
+    BZDB.set("zbuffer", "1");
   }
   glGetIntegerv(GL_STENCIL_BITS, &bits);
   useStencilOn = (bits > 0);
@@ -177,15 +171,7 @@ void SceneRenderer::setWindow(MainWindow* _window) {
   // can only do hidden line if polygon offset is available
   canUseHiddenLine = true;
 
-  // check if we're running OpenGL 1.1.  if so we'll use the fog hack
-  // to fade the screen;  otherwise fall back on a full screen blended
-  // polygon.
-  if (version != NULL && strncmp(version, "1.1", 3) == 0) {
-    useFogHack = true;
-  }
-
   // prepare context with stuff that'll never change
-  glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
   glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
   glGetIntegerv(GL_MAX_LIGHTS, &maxLights);
   reservedLights = 1;			// only one light between sun and moon
@@ -276,37 +262,67 @@ void SceneRenderer::setQuality(int value)
 
   notifyStyleChange();
 
-  if (useQualityValue >= 2) {
+  if (useQualityValue >= 1) {
     glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
     glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
-    glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
+    // GL_NICEST for polygon smoothing seems to make some drivers
+    // cause massive slowdowns and "spikes" when drawing the radar
+    glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
   } else {
     glHint(GL_LINE_SMOOTH_HINT, GL_FASTEST);
     glHint(GL_POINT_SMOOTH_HINT, GL_FASTEST);
     glHint(GL_POLYGON_SMOOTH_HINT, GL_FASTEST);
   }
 
-  if (useQualityValue >= 3)
+  if (useQualityValue >= 2)
     TankSceneNode::setMaxLOD(-1);
   else if (useQualityValue >= 1)
     TankSceneNode::setMaxLOD(3);
   else
     TankSceneNode::setMaxLOD(2);
 
-  if (useQualityValue >= 3)
+  if (useQualityValue >= 2)
     BZDB.set("flagChunks","32");
-  else if (useQualityValue >= 2)
+  else if (useQualityValue >= 1)
     BZDB.set("flagChunks","12");
   else
     BZDB.set("flagChunks","8");
 
-  if (useQualityValue >= 3)
+  if (useQualityValue >= 2)
     BZDB.set("moonSegments","64");
-  else if (useQualityValue >= 2)
+  else if (useQualityValue >= 1)
     BZDB.set("moonSegments","24");
   else
     BZDB.set("moonSegments","12");
 
+  if (useQualityValue > 0) {
+    // this can be modified by OpenGLMaterial
+    glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
+  } else {
+    // OpenGLMaterial will not modify if (quality <= 0)
+    glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
+  }
+
+  // this setting helps keep those specular highlights
+  // highlighting when applied to a dark textured surface.
+  // It was mainlined in OpenGL Version 1.2
+  // (there's also the GL_EXT_separate_specular_color extension)
+#ifdef GL_LIGHT_MODEL_COLOR_CONTROL
+  if (useQualityValue >= 1)
+    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
+  else
+    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR);
+#  else // in case someone includes <GL/glext.h> at some point
+#  ifdef GL_LIGHT_MODEL_COLOR_CONTROL_EXT
+  if (useQualityValue >= 1)
+    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL_EXT,
+		  GL_SEPARATE_SPECULAR_COLOR_EXT);
+  else
+    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL_EXT,
+		  GL_SINGLE_COLOR_EXT);
+#  endif
+#endif
+
   BZDB.set("useQuality", TextUtils::format("%d", value));
 }
 
@@ -337,6 +353,7 @@ void SceneRenderer::setRebuildTanks()
 void SceneRenderer::setupBackgroundMaterials()
 {
   if (background) {
+    background->setupSkybox();
     background->setupGroundMaterials();
   }
   return;
@@ -411,6 +428,8 @@ int SceneRenderer::getRadarSize() const
 
 void SceneRenderer::setMaxMotionFactor(int factor)
 {
+  if (factor < -11)
+    factor = -11;
   maxMotionFactor = factor;
   notifyStyleChange();
   if (window) {
@@ -443,6 +462,16 @@ void SceneRenderer::setExposed()
 }
 
 
+void SceneRenderer::clearRadar(float opacity)
+{
+  int size = window->getHeight() - window->getViewHeight();
+  float op = (opacity > 1.0f) ? 1.0f : (opacity < 0.0f) ? 0.0f : opacity;
+  glScissor(window->getOriginX(), 0, size, size);
+  glClearColor(0.0f, 0.0f, 0.0f, op);
+  glClear(GL_COLOR_BUFFER_BIT);
+}
+
+
 void SceneRenderer::setSceneDatabase(SceneDatabase* db)
 {
   // update the styles
@@ -502,7 +531,7 @@ void SceneRenderer::enableSun(bool on)
 void SceneRenderer::setupSun()
 {
   if (BZDBCache::lighting && sunOrMoonUp) {
-    theSun.execute(SunLight);
+    theSun.execute(SunLight, BZDB.isTrue("lightLists"));
   }
 }
 
@@ -567,8 +596,7 @@ void SceneRenderer::setTimeOfDay(double julianDay)
 
   getSunPosition(julianDay, latitude, longitude, sunDir);
   getMoonPosition(julianDay, latitude, longitude, moonDir);
-  ::getCelestialTransform(julianDay, latitude, longitude,
-				(float(*)[4])celestialTransform);
+  ::getCelestialTransform(julianDay, latitude, longitude, celestialTransform);
 
   // set sun position
   if (sunDir[2] >= -0.009f) {
@@ -588,7 +616,6 @@ void SceneRenderer::setTimeOfDay(double julianDay)
   }
 
   // set sun and ambient colors
-  GLfloat ambientColor[4];
   ::getSunColor(sunDir, sunColor, ambientColor, sunBrightness);
   theSun.setColor(sunColor);
   GLfloat maxComponent = sunColor[0];
@@ -655,6 +682,12 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
   lastFrame = _lastFrame;
   sameFrame = _sameFrame;
 
+  triangleCount = 0;
+  RenderNode::resetTriangleCount();
+  if (background) {
+    background->resetTriangleCount();
+  }
+
   // update the SceneNode, Background, and TrackMark styles
   if (needStyleUpdate) {
     if (scene) {
@@ -684,6 +717,20 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
     return;
   }
 
+  // setup the viewport LOD scale
+  MeshSceneNode::setLodScale(window->getWidth(), frustum.getFOVx(),
+			     window->getViewHeight(), frustum.getFOVy());
+  {
+    const int pixelsX = window->getWidth();
+    const int pixelsY = window->getViewHeight();
+    const float fovx = frustum.getFOVx();
+    const float fovy = frustum.getFOVy();
+    const float lppx = 2.0f * sinf(fovx * 0.5f) / (float)pixelsX;
+    const float lppy = 2.0f * sinf(fovy * 0.5f) / (float)pixelsY;
+    const float lpp = (lppx < lppy) ? lppx : lppy;
+    lengthPerPixel = lpp * BZDB.eval("lodScale");
+  }
+
   // get the track mark sceneNodes (only for BSP)
   if (scene) {
     TrackMarks::addSceneNodes(scene);
@@ -696,6 +743,8 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
     teleporterProximity = LocalPlayer::getMyTank()->getTeleporterProximity();
   }
 
+  // fog setup
+  mapFog = setupMapFog();
 
   mirror = (BZDB.get(StateDatabase::BZDB_MIRROR) != "none")
 	   && BZDB.isTrue("userMirror");
@@ -710,19 +759,23 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
     frustum.flipVertical();
     glFrontFace(GL_CW);
 
+    // different occluders for the mirror
+    if (scene) {
+      scene->setOccluderManager(1);
+    }
+
     // the reflected scene
     renderScene(_lastFrame, _sameFrame, fullWindow);
 
+    // different occluders for the mirror
+    if (scene) {
+      scene->setOccluderManager(0);
+    }
+
     // flip back
     frustum.flipVertical();
     glFrontFace(GL_CCW);
 
-    // darken the reflection
-    glMatrixMode(GL_PROJECTION);
-    glLoadIdentity();
-    glMatrixMode(GL_MODELVIEW);
-    glLoadIdentity();
-
     float mirrorColor[4];
     if (!parseColorString(BZDB.get(StateDatabase::BZDB_MIRROR), mirrorColor)) {
       mirrorColor[0] = mirrorColor[1] = mirrorColor[2] = 0.0f;
@@ -737,19 +790,51 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
       mirrorColor[3] = 0.2f;
     }
 
-    // if low quality then use stipple -- it's probably much faster
-    if (BZDBCache::blend && (useQualityValue >= 2)) {
-      glColor4fv(mirrorColor);
-      glEnable(GL_BLEND);
-      glRectf(-1.0f, -1.0f, +1.0f, +1.0f);
-      glDisable(GL_BLEND);
+    // darken the reflection
+    if (!mapFog) {
+      glMatrixMode(GL_PROJECTION);
+      glLoadIdentity();
+      glMatrixMode(GL_MODELVIEW);
+      glLoadIdentity();
+      // if low quality then use stipple -- it's probably much faster
+      if (BZDBCache::blend && (useQualityValue >= 1)) {
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	glEnable(GL_BLEND);
+	glColor4fv(mirrorColor);
+	glRectf(-1.0f, -1.0f, +1.0f, +1.0f);
+	glDisable(GL_BLEND);
+      } else {
+	float stipple = mirrorColor[3];
+	glColor3fv(mirrorColor);
+	OpenGLGState::setStipple(stipple);
+	glEnable(GL_POLYGON_STIPPLE);
+	glRectf(-1.0f, -1.0f, +1.0f, +1.0f);
+	glDisable(GL_POLYGON_STIPPLE);
+      }
     } else {
-      float stipple = mirrorColor[3];
-      glColor3fv(mirrorColor);
-      OpenGLGState::setStipple(stipple);
-      glEnable(GL_POLYGON_STIPPLE);
-      glRectf(-1.0f, -1.0f, +1.0f, +1.0f);
-      glDisable(GL_POLYGON_STIPPLE);
+      // need the proper matrices for fog generation
+      // if low quality then use stipple -- it's probably much faster
+      frustum.executeView();
+      frustum.executeProjection();
+      const float extent = BZDBCache::worldSize * 10.0f;
+      if (BZDBCache::blend && (useQualityValue >= 1)) {
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	glEnable(GL_BLEND);
+	glColor4fv(mirrorColor);
+	glRectf(-extent, -extent, +extent, +extent);
+	glDisable(GL_BLEND);
+      } else {
+	float stipple = mirrorColor[3];
+	glColor3fv(mirrorColor);
+	OpenGLGState::setStipple(stipple);
+	glEnable(GL_POLYGON_STIPPLE);
+	glRectf(-extent, -extent, +extent, +extent);
+	glDisable(GL_POLYGON_STIPPLE);
+      }
+      glMatrixMode(GL_PROJECTION);
+      glLoadIdentity();
+      glMatrixMode(GL_MODELVIEW);
+      glLoadIdentity();
     }
 
     clearZbuffer = false;
@@ -758,14 +843,27 @@ void SceneRenderer::render(bool _lastFrame, bool _sameFrame,
   // the real scene
   renderScene(_lastFrame, _sameFrame, fullWindow);
 
+  // finalize dimming
+  if (mapFog) {
+    glDisable(GL_FOG);
+  }
+
+  renderDimming();
+
+  triangleCount = RenderNode::getTriangleCount();
+  if (background) {
+    triangleCount += background->getTriangleCount();
+  }
+
   return;
 }
 
 
-void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
+void SceneRenderer::renderScene(bool UNUSED(_lastFrame), bool UNUSED(_sameFrame),
 				bool fullWindow)
 {
   int i;
+  const bool lightLists = BZDB.isTrue("lightLists");
 
   // avoid OpenGL calls as long as possible -- there's a good
   // chance we're waiting on the vertical retrace.
@@ -784,22 +882,18 @@ void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
 
   // turn sunlight on -- the ground needs it
   if (BZDBCache::lighting && sunOrMoonUp) {
-    theSun.execute(SunLight);
+    theSun.execute(SunLight, lightLists);
     theSun.enableLight(SunLight, true);
   }
 
-  bool reallyUseFogHack = useFogHack && (useQualityValue >= 2);
-
-  if (reallyUseFogHack) {
-    renderPreDimming();
-  }
-
   // set scissor
-
   glScissor(window->getOriginX(), window->getOriginY() + window->getHeight() - window->getViewHeight(),
       window->getWidth(), window->getViewHeight());
 
   if (useDepthComplexityOn) {
+    if (BZDBCache::stencilShadows) {
+      BZDB.set("stencilShadows", "0");
+    }
     glEnable(GL_STENCIL_TEST);
     if (!mirror || (clearZbuffer)) {
       glClear(GL_STENCIL_BUFFER_BIT);
@@ -850,7 +944,16 @@ void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
   if (background) {
     background->setBlank(blank);
     background->setInvert(invert);
-    background->renderSky(*this, fullWindow, mirror);
+
+    const bool avoidSkyFog = (mapFog && BZDB.isTrue("_fogNoSky"));
+    if (avoidSkyFog) {
+      glDisable(GL_FOG);
+      background->renderSky(*this, fullWindow, mirror);
+      glEnable(GL_FOG);
+    } else {
+      background->renderSky(*this, fullWindow, mirror);
+    }
+
     if (drawGround) {
       background->renderGround(*this, fullWindow);
     }
@@ -860,7 +963,7 @@ void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
   // we may need to turn them on when drawing the background.
   if (BZDBCache::lighting) {
     for (i = 0; i < dynamicLights; i++) {
-      lights[i]->execute(i + reservedLights);
+      lights[i]->execute(i + reservedLights, lightLists);
     }
   }
 
@@ -888,14 +991,6 @@ void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
       glEnable(GL_POLYGON_OFFSET_FILL);
     }
 
-    if (scene && BZDBCache::showCullingGrid) {
-      scene->drawCuller();
-    }
-    const World* world = World::getWorld();
-    if (scene && BZDBCache::showCollisionGrid && (world != NULL)) {
-      world->drawCollisionGrid();
-    }
-
 
     ///////////////////////
     // THE BIG RENDERING //
@@ -903,6 +998,15 @@ void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
     doRender();
 
 
+    if (scene && BZDBCache::showCullingGrid) {
+      scene->drawCuller();
+    }
+
+    const World* world = World::getWorld();
+    if (scene && BZDBCache::showCollisionGrid && (world != NULL)) {
+      world->drawCollisionGrid();
+    }
+
     if (useHiddenLineOn) {
       glDisable(GL_POLYGON_OFFSET_FILL);
       glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
@@ -938,15 +1042,6 @@ void SceneRenderer::renderScene(bool /*_lastFrame*/, bool /*_sameFrame*/,
   }
   glPopMatrix();
 
-  // finalize dimming
-  if (reallyUseFogHack) {
-    if ((teleporterProximity > 0.0f) || useDimming) {
-      glDisable(GL_FOG);
-    }
-  } else {
-    renderPostDimming();
-  }
-
   // do depth complexity
   if (useDepthComplexityOn) {
     renderDepthComplexity();
@@ -990,43 +1085,72 @@ void SceneRenderer::doRender()
 }
 
 
-void SceneRenderer::renderPreDimming()
+static bool setupMapFog()
 {
-  float worldSize = BZDBCache::worldSize;
+  RENDERER.setFogActive(false);
+  std::string fogModeStr;
+  if (BZDB.get(StateDatabase::BZDB_FOGMODE) == "none")
+  {
+    glDisable(GL_FOG);
+    glHint(GL_FOG_HINT, GL_FASTEST);
+    return false;
+  }
+  RENDERER.setFogActive(true);
+  GLenum fogMode = GL_EXP;
+  GLfloat fogDensity = 0.001f;
+  GLfloat fogStart = 0.5f * BZDBCache::worldSize;
+  GLfloat fogEnd = BZDBCache::worldSize;
+  GLfloat fogColor[4] = {0.25f, 0.25f, 0.25f, 0.25f};
 
-  if (useDimming) {
-    const float density = dimDensity;
-    glFogi(GL_FOG_MODE, GL_LINEAR);
-    glFogf(GL_FOG_START, -density * 1000.0f * worldSize);
-    glFogf(GL_FOG_END, (1.0f - density) * 1000.0f * worldSize);
-    glFogfv(GL_FOG_COLOR, dimnessColor);
-    glEnable(GL_FOG);
+  // parse the values;
+  const std::string modeStr = BZDB.get("_fogMode");
+  if (modeStr == "linear") {
+    fogMode = GL_LINEAR;
+  } else if (modeStr == "exp") {
+    fogMode = GL_EXP;
+  } else if (modeStr == "exp2") {
+    fogMode = GL_EXP2;
+  } else {
+    fogMode = GL_EXP;
   }
-  else if (teleporterProximity > 0.0f && useFogHack) {
-    const float density = (teleporterProximity > 0.75f) ? 1.0f
-			  : (teleporterProximity / 0.75f);
-    glFogi(GL_FOG_MODE, GL_LINEAR);
-    glFogf(GL_FOG_START, -density * 1000.0f * worldSize);
-    glFogf(GL_FOG_END, (1.0f - density) * 1000.0f * worldSize);
-    glFogfv(GL_FOG_COLOR, blindnessColor);
-    glEnable(GL_FOG);
+  fogDensity = BZDB.eval(StateDatabase::BZDB_FOGDENSITY);
+  fogStart = BZDB.eval(StateDatabase::BZDB_FOGSTART);
+  fogEnd = BZDB.eval(StateDatabase::BZDB_FOGEND);
+  if (!parseColorString(BZDB.get(StateDatabase::BZDB_FOGCOLOR), fogColor)) {
+    fogColor[0] = fogColor[1] = fogColor[2] = 0.1f;
+    fogColor[3] = 0.0f; // has no effect
   }
-  return;
+  if (BZDB.evalInt("fogEffect") >= 1) {
+    glHint(GL_FOG_HINT, GL_NICEST);
+  } else {
+    glHint(GL_FOG_HINT, GL_FASTEST);
+  }
+
+  // setup GL fog
+  glFogi(GL_FOG_MODE, fogMode);
+  glFogf(GL_FOG_DENSITY, fogDensity);
+  glFogf(GL_FOG_START, fogStart);
+  glFogf(GL_FOG_END, fogEnd);
+  glFogfv(GL_FOG_COLOR, fogColor);
+  glEnable(GL_FOG);
+
+  RENDERER.setFogColor(fogColor);
+  return true;
 }
 
 
-void SceneRenderer::renderPostDimming()
+void SceneRenderer::renderDimming()
 {
   float density = 0.0f;
   const GLfloat* color = NULL;
   if (useDimming) {
-    density = dimDensity;
     color = dimnessColor;
+    density = dimDensity;
   }
   else if (teleporterProximity > 0.0f) {
-    density = (teleporterProximity > 0.75f) ?
-		      1.0f : teleporterProximity / 0.75f;
     color = blindnessColor;
+    density =
+      (teleporterProximity > 0.75f) ? 1.0f : teleporterProximity / 0.75f;
   }
   if (density > 0.0f && color != NULL) {
     glMatrixMode(GL_PROJECTION);
@@ -1035,7 +1159,7 @@ void SceneRenderer::renderPostDimming()
     glColor4f(color[0], color[1], color[2], density);
 
     // if low quality then use stipple -- it's probably much faster
-    if (BZDBCache::blend && (useQualityValue >= 2)) {
+    if (BZDBCache::blend && (useQualityValue >= 1)) {
       glEnable(GL_BLEND);
       glRectf(-1.0f, -1.0f, 1.0f, 1.0f);
       glDisable(GL_BLEND);
@@ -1206,6 +1330,22 @@ const GLfloat* SceneRenderer::getSunDirection() const
 }
 
 
+const Extents* SceneRenderer::getVisualExtents() const
+{
+  if (scene) {
+    return scene->getVisualExtents();
+  } else {
+    return NULL;
+  }
+}
+
+
+int SceneRenderer::getFrameTriangleCount() const
+{
+  return triangleCount;
+}
+
+
 // Local Variables: ***
 // mode: C++ ***
 // tab-width: 8 ***
