@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -8,11 +8,8 @@
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
- *
- * $Id: ScoreboardRenderer.cxx 16723 2008-03-06 18:03:09Z timriker $
  */
 
-
 /* interface header */
 #include "ScoreboardRenderer.h"
 
@@ -23,28 +20,43 @@
 #include "BZDBCache.h"
 #include "OpenGLGState.h"
 #include "TextUtils.h"
+#include "TimeKeeper.h"
 
 /* local implementation headers */
 #include "LocalPlayer.h"
 #include "World.h"
 #include "sound.h"
 
+// because of the 'player' crap, we can't  #include "Roaming.h"  easily
+extern Player* getRoamTargetTank();
+
 #define DEBUG_SHOWRATIOS 1
 
-std::string		ScoreboardRenderer::scoreSpacingLabel("88% 8888 888-888 [88]");
-std::string		ScoreboardRenderer::scoreLabel("Score");
-std::string		ScoreboardRenderer::killSpacingLabel("888~888 Hunt->");
-std::string		ScoreboardRenderer::killLabel(" Kills");
-std::string		ScoreboardRenderer::teamScoreSpacingLabel("88 (888-888) 88");
-std::string		ScoreboardRenderer::teamCountSpacingLabel("888");
-std::string		ScoreboardRenderer::playerLabel("Player");
+std::string ScoreboardRenderer::scoreSpacingLabel("88% 8888 888-888 [88]");
+std::string ScoreboardRenderer::scoreLabel("Score");
+std::string ScoreboardRenderer::killSpacingLabel("888~888 Hunt->");
+std::string ScoreboardRenderer::killLabel(" Kills");
+std::string ScoreboardRenderer::teamScoreSpacingLabel("88 (888-888) 88");
+std::string ScoreboardRenderer::teamCountSpacingLabel("888");
+std::string ScoreboardRenderer::playerLabel("Player");
 
 // NOTE: order of sort labels must match SORT_ consts
-const char *ScoreboardRenderer::sortLabels[] = {
-  "[Score]", "[Normalized Score]", "[Callsign]", "[Team Kills]", "[TK ratio]", "[Team]", "[1on1]", NULL};
+const char* ScoreboardRenderer::sortLabels[] = {
+  "[Score]",
+  "[Normalized Score]",
+  "[Callsign]",
+  "[Team Kills]",
+  "[TK ratio]",
+  "[Team]",
+  "[1on1]",
+  NULL
+};
+
 int ScoreboardRenderer::sortMode = 0;
+
 bool ScoreboardRenderer::alwaysShowTeamScore = 0;
 
+
 ScoreboardRenderer::ScoreboardRenderer() :
 				winWidth (0.0),
 				dim(false),
@@ -94,13 +106,13 @@ const char  **ScoreboardRenderer::getSortLabels ()
 }
 
 
-void		ScoreboardRenderer::setSort (int _sortby)
+void ScoreboardRenderer::setSort (int _sortby)
 {
   sortMode = _sortby;
   BZDB.setInt ("scoreboardSort", sortMode);
 }
 
-int			ScoreboardRenderer::getSort ()
+int ScoreboardRenderer::getSort ()
 {
   return sortMode;
 }
@@ -116,7 +128,7 @@ bool    ScoreboardRenderer::getAlwaysTeamScore ()
   return alwaysShowTeamScore;
 }
 
-void		ScoreboardRenderer::setMinorFontSize(float height)
+void ScoreboardRenderer::setMinorFontSize(float height)
 {
   FontManager &fm = FontManager::instance();
   minorFontFace = fm.getFaceID(BZDB.get("consoleFont"));
@@ -154,7 +166,7 @@ void		ScoreboardRenderer::setMinorFontSize(float height)
 }
 
 
-void			ScoreboardRenderer::setLabelsFontSize(float height)
+void ScoreboardRenderer::setLabelsFontSize(float height)
 {
   const float s = height / 96.0f;
   FontManager &fm = FontManager::instance();
@@ -163,7 +175,7 @@ void			ScoreboardRenderer::setLabelsFontSize(float height)
 }
 
 
-void			ScoreboardRenderer::setDim(bool _dim)
+void ScoreboardRenderer::setDim(bool _dim)
 {
   dim = _dim;
 }
@@ -171,7 +183,7 @@ void			ScoreboardRenderer::setDim(bool _dim)
 
 static const float dimFactor = 0.2f;
 
-void			ScoreboardRenderer::hudColor3fv(const GLfloat* c)
+void ScoreboardRenderer::hudColor3fv(const GLfloat* c)
 {
   if (dim)
     glColor3f(dimFactor * c[0], dimFactor * c[1], dimFactor * c[2]);
@@ -194,7 +206,7 @@ void    ScoreboardRenderer::exitSelectState (void){
 
 
 
-void	ScoreboardRenderer::render(bool forceDisplay)
+void ScoreboardRenderer::render(bool forceDisplay)
 {
   FontManager &fm = FontManager::instance();
   if (dim) {
@@ -208,7 +220,7 @@ void	ScoreboardRenderer::render(bool forceDisplay)
     if (getHuntState() == HUNT_SELECTING){      // 'S' pressed while selecting ...
       exitSelectState ();
     }
-    if (BZDB.isTrue("alwaysShowTeamScores")){
+    if (BZDB.isTrue("alwaysShowTeamScores") && World::getWorld()->allowTeams()){
       OpenGLGState::resetState();
       renderTeamScores(winWidth, winY,
 	    FontManager::instance().getStrHeight(minorFontFace, minorFontSize, " "));
@@ -225,31 +237,31 @@ int ScoreboardRenderer::teamScoreCompare(const void* _c, const void* _d)
 {
   Team* c = World::getWorld()->getTeams()+*(int*)_c;
   Team* d = World::getWorld()->getTeams()+*(int*)_d;
-  return (d->won-d->lost) - (c->won-c->lost);
+  return (d->getWins()-d->getLosses()) - (c->getWins()-c->getLosses());
 }
 
 // invoked by playing.cxx when 'prev' is pressed
-void		ScoreboardRenderer::setHuntPrevEvent()
+void ScoreboardRenderer::setHuntPrevEvent()
 {
   huntPositionEvent = -1;
   --huntPosition;
 }
 
 // invoked by playing.cxx when 'next' is pressed
-void		ScoreboardRenderer::setHuntNextEvent()
+void ScoreboardRenderer::setHuntNextEvent()
 {
   huntPositionEvent = 1;
   ++huntPosition;
 }
 
 // invoked by playing.cxx when select (fire) is pressed
-void			ScoreboardRenderer::setHuntSelectEvent ()
+void ScoreboardRenderer::setHuntSelectEvent ()
 {
   huntSelectEvent = true;
 }
 
 // invoked by clientCommands.cxx when '7' or 'U' is pressed
-void			ScoreboardRenderer::huntKeyEvent (bool isAdd)
+void ScoreboardRenderer::huntKeyEvent (bool isAdd)
 {
   if (getHuntState() == HUNT_ENABLED) {
     if (isAdd) {
@@ -274,7 +286,7 @@ void			ScoreboardRenderer::huntKeyEvent (bool isAdd)
 }
 
 
-void		ScoreboardRenderer::setHuntState (int _huntState)
+void ScoreboardRenderer::setHuntState (int _huntState)
 {
   if (huntState == _huntState)
     return;
@@ -289,7 +301,7 @@ void		ScoreboardRenderer::setHuntState (int _huntState)
 }
 
 
-int			ScoreboardRenderer::getHuntState() const
+int ScoreboardRenderer::getHuntState() const
 {
   return huntState;
 }
@@ -302,7 +314,7 @@ void ScoreboardRenderer::huntReset()
     numHunted = 0;
 }
 
-void			ScoreboardRenderer::renderTeamScores (float x, float y, float dy){
+void ScoreboardRenderer::renderTeamScores (float x, float y, float dy){
   // print teams sorted by score
   int teams[NumTeams];
   int teamCount = 0;
@@ -333,13 +345,11 @@ void			ScoreboardRenderer::renderTeamScores (float x, float y, float dy){
   qsort(teams, teamCount, sizeof(int), teamScoreCompare);
   y -= dy;
 
-  bool rabbitMode = World::getWorld()->allowRabbit();
-
   char score[44];
   for (i = 0 ; i < teamCount; i++){
     Team& team = World::getWorld()->getTeam(teams[i]);
-    sprintf(score, "%3d (%3d-%-3d) %3d", team.won - team.lost, team.won, team.lost, team.size);
-    hudColor3fv(Team::getRadarColor((TeamColor)teams[i],rabbitMode));
+    sprintf(score, "%3d (%3d-%-3d) %3d", team.getWins() - team.getLosses(), team.getWins(), team.getLosses(), team.size);
+    hudColor3fv(Team::getRadarColor((TeamColor)teams[i]));
     fm.drawString(xn, y, 0, minorFontFace, minorFontSize, score);
     y -= dy;
   }
@@ -347,7 +357,7 @@ void			ScoreboardRenderer::renderTeamScores (float x, float y, float dy){
 
 
 // not used yet - new feature coming
-void			ScoreboardRenderer::renderCtfFlags (){
+void ScoreboardRenderer::renderCtfFlags (){
   int i;
   RemotePlayer* player;
   const int curMaxPlayers = World::getWorld()->getCurMaxPlayers();
@@ -369,8 +379,8 @@ void			ScoreboardRenderer::renderCtfFlags (){
       FlagType* flagd = player->getFlag();
       TeamColor teamIndex = player->getTeam();
       if (flagd!=Flags::Null && flagd->flagTeam != NoTeam) {   // if player has team flag ...
-	      std::string playerInfo = ColorStrings[flagd->flagTeam];
-	sprintf (flagColor, "%-12s", flagd->flagName);
+	std::string playerInfo = ColorStrings[flagd->flagTeam];
+	snprintf(flagColor, 200, "%-12s", flagd->flagName.c_str());
 	playerInfo += flagColor;
 	playerInfo += ColorStrings[teamIndex];
 	playerInfo += player->getCallSign();
@@ -384,7 +394,7 @@ void			ScoreboardRenderer::renderCtfFlags (){
 }
 
 
-void     ScoreboardRenderer::clearHuntedTanks ()
+void ScoreboardRenderer::clearHuntedTanks ()
 {
   World *world = World::getWorld();
   if (!world)
@@ -399,12 +409,11 @@ void     ScoreboardRenderer::clearHuntedTanks ()
 }
 
 
-
-void			ScoreboardRenderer::renderScoreboard(void)
+void ScoreboardRenderer::renderScoreboard(void)
 {
   int i=0;
   int numPlayers;
-  int emailLen;
+  int mottoLen;
   Player** players;
   Player*  player;
   bool haveObs = false;
@@ -479,25 +488,43 @@ void			ScoreboardRenderer::renderScoreboard(void)
     }
   }
 
-
-  emailLen = BZDB.getIntClamped ("emailDispLen", 0, 128);
+  mottoLen = BZDB.getIntClamped ("mottoDispLen", 0, 128);
   huntSelectEvent = false;
   huntPositionEvent = 0;
   numHunted = 0;
 
-  while ( (player = players[i]) != NULL){
-    if (player->isHunted())
-      ++numHunted;
-    if (player->getTeam()==ObserverTeam && !haveObs){
+  const int maxLines = BZDB.evalInt("maxScoreboardLines");
+  int lines = 0;
+  int hiddenLines = 0;
+  while ((player = players[i]) != NULL) {
+    if ((maxLines > 0) && (lines >= maxLines)) {
+      hiddenLines++;
+    }
+    else {
+      if (player->isHunted()) {
+	++numHunted;
+      }
+      if (player->getTeam()==ObserverTeam && !haveObs){
+	y -= dy;
+	haveObs = true;
+      }
+      if (huntState==HUNT_SELECTING && i==huntPosition) {
+	drawPlayerScore(player, x1, x2, x3, xs, (float)y, mottoLen, true);
+      } else {
+	drawPlayerScore(player, x1, x2, x3, xs, (float)y, mottoLen, false);
+      }
       y -= dy;
-      haveObs = true;
     }
-    if (huntState==HUNT_SELECTING && i==huntPosition)
-      drawPlayerScore(player, x1, x2, x3, xs, (float)y, emailLen, true);
-    else
-      drawPlayerScore(player, x1, x2, x3, xs, (float)y, emailLen, false);
-    y -= dy;
     ++i;
+    ++lines;
+  }
+
+  if (hiddenLines > 0) {
+    char buf[64];
+    snprintf(buf, sizeof(buf), "...%i...", hiddenLines);
+    fm.drawString(x1, y, 0, minorFontFace, minorFontSize, buf);
+    fm.drawString(x2, y, 0, minorFontFace, minorFontSize, buf);
+    fm.drawString(x3, y, 0, minorFontFace, minorFontSize, buf);
   }
 
   if (huntState==HUNT_ENABLED && !numHunted) {
@@ -507,12 +534,13 @@ void			ScoreboardRenderer::renderScoreboard(void)
   }
 
   delete[] players;
-  renderTeamScores(winWidth, y0, dy);
+
+  if (World::getWorld()->allowTeams())
+    renderTeamScores(winWidth, y0, dy);
 }
 
 
-
-void      ScoreboardRenderer::stringAppendNormalized (std::string *s, float n)
+void ScoreboardRenderer::stringAppendNormalized (std::string *s, float n)
 {
   char fmtbuf[10];
   sprintf (fmtbuf, "  [%4.2f]", n);
@@ -520,10 +548,42 @@ void      ScoreboardRenderer::stringAppendNormalized (std::string *s, float n)
 }
 
 
+void ScoreboardRenderer::drawRoamTarget(float _x0, float _y0,
+					float _x1, float UNUSED(_y1))
+{
+  static const TimeKeeper startTime = TimeKeeper::getCurrent();
 
-void			ScoreboardRenderer::drawPlayerScore(const Player* player,
-			    float x1, float x2, float x3, float xs, float y, 
-                            int emailLen, bool huntCursor)
+  const float x0 = floorf(_x0) + 0.5f;
+  const float y0 = floorf(_y0) + 0.5f;
+  const float x1 = floorf(_x1) + 0.5f;
+  const float y1 = y0 + 1.0f;
+
+  const float black[4] = { 0.0f, 0.0f, 0.0f, 1.0f };
+  const float white[4] = { 1.0f, 1.0f, 1.0f, 1.0f };
+  const float* c0 = black;
+  const float* c1 = white;
+
+  const double diff = (TimeKeeper::getCurrent() - startTime);
+  if (fmod(diff, 0.5) > 0.25) {
+    c0 = white;
+    c1 = black;
+  }
+
+  glPushAttrib(GL_ALL_ATTRIB_BITS);
+  glDisable(GL_BLEND);
+  glDisable(GL_LIGHTING);
+  glDisable(GL_TEXTURE_2D);
+  glBegin(GL_LINES);
+  glColor4fv(c0); glVertex2f(x0, y1); glVertex2f(x1, y1);
+  glColor4fv(c1); glVertex2f(x0, y0); glVertex2f(x1, y0);
+  glEnd();
+  glPopAttrib();
+}
+
+
+void ScoreboardRenderer::drawPlayerScore(const Player* player,
+			    float x1, float x2, float x3, float xs, float y,
+			    int mottoLen, bool huntCursor)
 {
   // score
   char score[40], kills[40];
@@ -541,11 +601,15 @@ void			ScoreboardRenderer::drawPlayerScore(const Player* player,
 	    player->getScore(), player->getWins(), player->getLosses(),
 	    highlightTKratio ? ColorStrings[CyanColor].c_str() : "",
 	    player->getTeamKills());
-  } else {
+  } else if (World::getWorld()->allowTeams()) {
     sprintf(score, "%4d %4d-%-4d%s[%2d]", player->getScore(),
 	    player->getWins(), player->getLosses(),
 	    highlightTKratio ? ColorStrings[CyanColor].c_str() : "",
 	    player->getTeamKills());
+  } else {
+    sprintf(score, "%4d %4d-%-4d%s", player->getScore(),
+	    player->getWins(), player->getLosses(),
+	    highlightTKratio ? ColorStrings[CyanColor].c_str() : "");
   }
 
   // kills
@@ -557,20 +621,13 @@ void			ScoreboardRenderer::drawPlayerScore(const Player* player,
 
 
   // team color
-  bool rabbitMode = World::getWorld()->allowRabbit();
   TeamColor teamIndex = player->getTeam();
   if (teamIndex < RogueTeam) {
     teamIndex = RogueTeam;
   }
   std::string teamColor;
   if (player->getId() < 200) {
-    if (rabbitMode && (teamIndex == RogueTeam)) {
-      // hunters get orange (hack)
-      teamColor = ColorStrings[OrangeColor];
-    } else {
-      // ordinary players
-      teamColor = ColorStrings[teamIndex];
-    }
+    teamColor = ColorStrings[teamIndex];
   } else {
     teamColor = ColorStrings[CyanColor]; // replay observers
   }
@@ -604,33 +661,33 @@ void			ScoreboardRenderer::drawPlayerScore(const Player* player,
     playerInfo += slot;
     playerInfo += " - ";
   }
-  if (roaming && BZDB.isTrue("showVelocities"))
-  {
-	  float vel[3] = {0};
-	  memcpy(vel,player->getVelocity(),sizeof(float)*3);
 
-	  float linSpeed = sqrt(vel[0]*vel[0]+vel[1]*vel[1]);
+  if (roaming && BZDB.isTrue("showVelocities")) {
+    float vel[3] = {0};
+    memcpy(vel,player->getVelocity(),sizeof(float)*3);
 
-	  float badFactor = 1.5f;
-	  if (linSpeed > player->getMaxSpeed()*badFactor)
-		  playerInfo += ColorStrings[RedColor];
-	  if (linSpeed > player->getMaxSpeed())
-		  playerInfo += ColorStrings[YellowColor];
-	  else if (linSpeed < 0.0001f)
-		  playerInfo += ColorStrings[GreyColor];
-	  else
-		  playerInfo += ColorStrings[WhiteColor];
-	  playerInfo += TextUtils::format("%5.2f ",linSpeed);
-	  playerInfo += teamColor;
+    float linSpeed = sqrt(vel[0]*vel[0]+vel[1]*vel[1]);
 
+    float badFactor = 1.5f;
+    if (linSpeed > player->getMaxSpeed()*badFactor)
+	    playerInfo += ColorStrings[RedColor];
+    if (linSpeed > player->getMaxSpeed())
+	    playerInfo += ColorStrings[YellowColor];
+    else if (linSpeed < 0.0001f)
+	    playerInfo += ColorStrings[GreyColor];
+    else
+	    playerInfo += ColorStrings[WhiteColor];
+    playerInfo += TextUtils::format("%5.2f ",linSpeed);
+    playerInfo += teamColor;
   }
+
   // callsign
   playerInfo += player->getCallSign();
 
-  // email in parenthesis
-  if (player->getEmailAddress()[0] != '\0' && emailLen>0) {
+  // motto in parentheses
+  if (player->getMotto()[0] != '\0' && mottoLen>0) {
     playerInfo += " (";
-    playerInfo += TextUtils::str_trunc_continued (player->getEmailAddress(), emailLen);
+    playerInfo += TextUtils::str_trunc_continued (player->getMotto(), mottoLen);
     playerInfo += ")";
   }
   // carried flag
@@ -657,6 +714,7 @@ void			ScoreboardRenderer::drawPlayerScore(const Player* player,
       playerInfo += teamColor;
     }
   }
+
   // status
   if (player->isPaused())
     playerInfo += "[p]";
@@ -677,11 +735,17 @@ void			ScoreboardRenderer::drawPlayerScore(const Player* player,
   FontManager &fm = FontManager::instance();
   fm.setDimFactor(dimFactor);
 
+  if (player == getRoamTargetTank()) {
+    const float w = fm.getStrLength(minorFontFace, minorFontSize, playerInfo);
+    const float h = fm.getStrHeight(minorFontFace, minorFontSize, playerInfo);
+    drawRoamTarget(x3, y, x3 + w, y + h);
+  }
+
   // draw
   if (player->getTeam() != ObserverTeam) {
-    hudColor3fv(Team::getRadarColor(teamIndex,rabbitMode));
+    hudColor3fv(Team::getRadarColor(teamIndex));
     fm.drawString(x1, y, 0, minorFontFace, minorFontSize, score);
-    hudColor3fv(Team::getRadarColor(teamIndex,rabbitMode));
+    hudColor3fv(Team::getRadarColor(teamIndex));
     fm.drawString(x2, y, 0, minorFontFace, minorFontSize, kills);
   }
   fm.drawString(x3, y, 0, minorFontFace, minorFontSize, playerInfo);
@@ -712,8 +776,6 @@ void			ScoreboardRenderer::drawPlayerScore(const Player* player,
     fm.drawString(xs - huntPlusesWidth, y, 0, minorFontFace, minorFontSize,
 		  huntStr.c_str());
   }
-
- 
 }
 
 
@@ -769,10 +831,11 @@ int       ScoreboardRenderer::sortCompareCp(const void* _a, const void* _b)
 
 int       ScoreboardRenderer::sortCompareI2(const void* _a, const void* _b)
 {
-  sortEntry *a = (sortEntry *)_a;
-  sortEntry *b = (sortEntry *)_b;
-  if (a->i1 != b->i1 )
+  const sortEntry *a = (sortEntry *)_a;
+  const sortEntry *b = (sortEntry *)_b;
+  if (a->i1 != b->i1 ) {
     return b->i1 - a->i1;
+  }
   return b->i2 - a->i2;
 }
 
@@ -870,6 +933,25 @@ Player **  ScoreboardRenderer::newSortedList (int sortType, bool obsLast, int *_
 }
 
 
+void ScoreboardRenderer::getPlayerList(std::vector<Player*>& players)
+{
+  players.clear();
+
+  int playerCount;
+  Player** pList = newSortedList(getSort(), true, &playerCount);
+  if (pList == NULL) {
+    return;
+  }
+  for (int i = 0; i < playerCount; i++) {
+    Player* p = pList[i];
+    if (p && (p->getTeam() != ObserverTeam)) {
+      players.push_back(p);
+    }
+  }
+  delete[] pList;
+}
+
+
 // Local Variables: ***
 // mode:C++ ***
 // tab-width: 8 ***
