@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -43,18 +43,16 @@ SegmentedShotStrategy::SegmentedShotStrategy(ShotPath* _path, bool useSuperTextu
   if (_path->getPlayer() == ServerPlayer) {
     TeamColor tmpTeam = _path->getFiringInfo().shot.team;
     team = (tmpTeam < RogueTeam) ? RogueTeam :
-	   (tmpTeam > RabbitTeam) ? RogueTeam : tmpTeam;
+	   (tmpTeam > HunterTeam) ? RogueTeam : tmpTeam;
   } else {
     Player* p = lookupPlayer(_path->getPlayer());
     team = p ? p->getTeam() : RogueTeam;
   }
 
   // initialize scene nodes
-  boltSceneNode = new BoltSceneNode(_path->getPosition());
+  boltSceneNode = new BoltSceneNode(_path->getPosition(),_path->getVelocity(),useSuperTexture);
 
-  bool rabbitMode = World::getWorld()->allowRabbit();
-
-  const float* c = Team::getRadarColor(team, rabbitMode);
+  const float* c = Team::getRadarColor(team);
   if (faint) {
     boltSceneNode->setColor(c[0], c[1], c[2], 0.2f);
     boltSceneNode->setTextureColor(1.0f, 1.0f, 1.0f, 0.3f);
@@ -296,8 +294,7 @@ void			SegmentedShotStrategy::addShot(
     boltSceneNode->setColorblind(colorblind);
     TeamColor currentTeam = colorblind ? RogueTeam : team;
 
-    bool rabbitMode = World::getWorld()->allowRabbit();
-    const float* c = Team::getRadarColor(currentTeam, rabbitMode);
+    const float* c = Team::getRadarColor(currentTeam);
     boltSceneNode->setColor(c[0], c[1], c[2]);
 
     TextureManager &tm = TextureManager::instance();
@@ -415,7 +412,7 @@ void			SegmentedShotStrategy::makeSegments(ObstacleEffect e)
     float t = timeLeft + minTime;
     int face;
     bool hitGround = getGround(r, Epsilon, t);
-    Obstacle* building = (Obstacle*)((e != Through) ? getFirstBuilding(r, Epsilon, t) : NULL);
+    Obstacle* building = (Obstacle*)((e == Through) ? NULL : getFirstBuilding(r, Epsilon, t));
     const Teleporter* teleporter = getFirstTeleporter(r, Epsilon, t, face);
     t -= minTime;
     minTime = 0.0f;
@@ -475,35 +472,42 @@ void			SegmentedShotStrategy::makeSegments(ObstacleEffect e)
     else if (building) {
       // hit building -- can bounce off or stop, buildings ignored for Through
       switch (e) {
-	case Stop:
-	  timeLeft = 0.0f;
-	break;
+	case Stop: {
+	  if(!building->canRicochet()) {
+	    timeLeft = 0.0f;
+	    break;
+	  } else {
 
-      case Reflect: {
-	// move origin to point of reflection
-	o[0] += t * d[0];
-	o[1] += t * d[1];
-	o[2] += t * d[2];
-
-	// reflect direction about normal to building
-	float normal[3];
-	building->get3DNormal(o, normal);
-	reflect(d, normal);
-	reason = ShotPathSegment::Ricochet;
+	  }
 	}
-	break;
+	case Reflect: {
+	  // move origin to point of reflection
+	  o[0] += t * d[0];
+	  o[1] += t * d[1];
+	  o[2] += t * d[2];
 
-      case Through:
-	assert(0);
+	  // reflect direction about normal to building
+	  float normal[3];
+	  building->get3DNormal(o, normal);
+	  reflect(d, normal);
+	  reason = ShotPathSegment::Ricochet;
+	  break;
+	}
+
+	case Through: {
+	  assert(0);
+	  break;
+	}
       }
     }
-    else if (hitGround)	// we hit the ground
-    {
+    else if (hitGround) { // we hit the ground
+
       switch (e) {
 	case Stop:
-	case Through:
+	case Through: {
 	  timeLeft = 0.0f;
 	  break;
+	}
 
 	case Reflect: {
 	  // move origin to point of reflection
@@ -639,6 +643,13 @@ ThiefStrategy::ThiefStrategy(ShotPath *_path) :
     thiefNodes[i] = new LaserSceneNode(ray.getOrigin(), dir);
     if (texture >= 0)
       thiefNodes[i]->setTexture(texture);
+
+    if (i == 0) {
+      thiefNodes[i]->setFirst();
+    }
+
+    thiefNodes[i]->setColor(0,1,1);
+    thiefNodes[i]->setCenterColor(0,0,0);
   }
   setCurrentSegment(numSegments - 1);
 }
@@ -804,6 +815,13 @@ LaserStrategy::LaserStrategy(ShotPath* _path) :
     laserNodes[i] = new LaserSceneNode(ray.getOrigin(), dir);
     if (texture >= 0)
       laserNodes[i]->setTexture(texture);
+
+    const float* color = Team::getRadarColor(tmpTeam);
+    laserNodes[i]->setColor(color[0], color[1], color[2]);
+
+    if (i == 0) {
+      laserNodes[i]->setFirst();
+    }
   }
   setCurrentSegment(numSegments - 1);
 }
