@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -61,9 +61,6 @@ private:
     CountDown,
     SuperKill,
     Shutdown,
-    Register,
-    Identify,
-    Setpass,
     Grouplist,
     Groupperms,
     Vote,
@@ -76,7 +73,7 @@ private:
   int startIndex;
   const int numModes;
   const int numNonAdminModes;
-  static const Mode nonAdminModes[8];
+  static const Mode nonAdminModes[6];
 
 
 };
