@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -15,6 +15,7 @@
 
 // interface header
 #include "ServerLink.h"
+// #include "MsgStrings.h"
 
 #if defined(DEBUG)
 #define NETWORK_STATS
@@ -127,17 +128,20 @@ ServerLink::ServerLink(const Address& serverAddress, int port) :
   // for UDP, used later
   memcpy((unsigned char *)&usendaddr,(unsigned char *)&addr, sizeof(addr));
 
-#if !defined(_WIN32)
-  const bool okay = true;
+  bool okay = true;
   int fdMax = query;
-  if (BzfNetwork::setNonBlocking(query) < 0) {
-    close(query);
-    return;
-  }
   struct timeval timeout;
   fd_set write_set;
   fd_set read_set;
   int nfound;
+
+#if !defined(_WIN32)
+  okay = true;
+  fdMax = query;
+  if (BzfNetwork::setNonBlocking(query) < 0) {
+    close(query);
+    return;
+  }
   if (connect(query, (CNCTType*)&addr, sizeof(addr)) < 0) {
     if (getErrno() != EINPROGRESS) {
       close(query);
@@ -175,7 +179,7 @@ ServerLink::ServerLink(const Address& serverAddress, int port) :
   hConnected = CreateEvent(NULL, FALSE, FALSE, "Connected Event");
 
   hThread = CreateThread(NULL, 0, ThreadConnect, &conn, 0, &ThreadID);
-  const bool okay = (WaitForSingleObject(hConnected, 5000) == WAIT_OBJECT_0);
+  okay = (WaitForSingleObject(hConnected, 5000) == WAIT_OBJECT_0);
   if(!okay)
     TerminateThread(hThread ,1);
 
@@ -185,30 +189,92 @@ ServerLink::ServerLink(const Address& serverAddress, int port) :
 
 #endif // !defined(_WIN32)
   if (!okay)
-    goto done;
-
-  // get server version and verify
-#if !defined(_WIN32)
-  FD_ZERO(&read_set);
-  FD_SET((unsigned int)query, &read_set);
-  timeout.tv_sec = long(5);
-  timeout.tv_usec = 0;
-  nfound = select(fdMax + 1, (fd_set*)&read_set, NULL, NULL, &timeout);
-  if (nfound <= 0) {
+  {
     close(query);
     return;
   }
-#endif // !defined(_WIN32)
-  i = recv(query, (char*)version, 8, 0);
-  if (i < 8)
-    goto done;
 
-  if (debugLevel >= 1) {
-    char cServerVersion[128];
-    sprintf(cServerVersion,"Server version: '%8s'",version);
-    printError(cServerVersion);
+  // send out the connect header
+  // this will let the server know we are BZFS protocol.
+  // after the server gets this it will send back a version for us to check
+  int sendRepply = ::send(query,BZ_CONNECT_HEADER,(int)strlen(BZ_CONNECT_HEADER),0);
+
+  logDebugMessage(2,"CONNECT:send in connect returned %d\n",sendRepply);
+
+  // wait to get data back. we are still blocking so these
+  // calls should be sync.
+
+  FD_ZERO(&read_set);
+  FD_ZERO(&write_set);
+  FD_SET((unsigned int)query, &read_set);
+  FD_SET((unsigned int)query, &write_set);
+
+  timeout.tv_sec = long(10);
+  timeout.tv_usec = 0;
+
+  // pick some limit to time out on ( in seconds )
+  double thisStartTime = TimeKeeper::getCurrent().getSeconds();
+  double connectTimeout = 30.0;
+  if (BZDB.isSet("connectionTimeout"))
+    connectTimeout = BZDB.eval("connectionTimeout")  ;
+
+  bool gotNetData = false;
+
+  // loop calling select untill we read some data back.
+  // its only 8 bytes so it better come back in one packet.
+  int loopCount = 0;
+  while(!gotNetData) {
+    loopCount++;
+    nfound = select(fdMax + 1, (fd_set*)&read_set, (fd_set*)&write_set, NULL, &timeout);
+
+    // there has to be at least one socket active, or we are screwed
+    if (nfound <= 0) {
+      logDebugMessage(1,"CONNECT:select in connect failed, nfound = %d\n",nfound);
+      close(query);
+      return;
+    }
+
+    // try and get data back from the server
+    i = recv(query, (char*)version, 8, 0);
+
+    // if we got some, then we are done
+    if (i > 0) {
+      logDebugMessage(2,"CONNECT:got net data in connect, bytes read = %d\n",i);
+      logDebugMessage(2,"CONNECT:Time To Connect = %f\n",(TimeKeeper::getCurrent().getSeconds() - thisStartTime));
+      gotNetData = true;
+    } else {
+      // if we have waited too long, then bail
+      if ((TimeKeeper::getCurrent().getSeconds() - thisStartTime) > connectTimeout) {
+	logDebugMessage(1,"CONNECT:connect time out failed\n");
+	logDebugMessage(2,"CONNECT:connect loop count = %d\n",loopCount);
+	close(query);
+	return;
+      }
+
+      TimeKeeper::sleep(0.25f);
+    }
   }
 
+  logDebugMessage(2,"CONNECT:connect loop count = %d\n",loopCount);
+
+  // if we got back less then the expected connect responce (BZFSXXXX)
+  // then something went bad, and we are done.
+  if (i < 8) {
+    close(query);
+    return;
+  }
+
+  // since we are connected, we can go non blocking
+  // on BSD sockets systems
+  // all other messages after this are handled via the normal
+  // message system
+#if !defined(_WIN32)
+  if (BzfNetwork::setNonBlocking(query) < 0) {
+    close(query);
+    return;
+  }
+#endif
+
   // FIXME is it ok to try UDP always?
   server_abilities |= CanDoUDP;
   if (strcmp(version, getServerVersion()) != 0) {
@@ -226,7 +292,8 @@ ServerLink::ServerLink(const Address& serverAddress, int port) :
       rejectionMessage = message;
     }
 
-    goto done;
+    close(query);
+    return;
   }
 
   // read local player's id
@@ -246,7 +313,8 @@ ServerLink::ServerLink(const Address& serverAddress, int port) :
     return;
   if (id == 0xff) {
     state = Rejected;
-    goto done;
+    close(query);
+    return;
   }
 
 #if !defined(_WIN32)
@@ -278,8 +346,6 @@ ServerLink::ServerLink(const Address& serverAddress, int port) :
   }
   return;
 
-done:
-  close(query);
 }
 
 ServerLink::~ServerLink()
@@ -332,6 +398,8 @@ void			ServerLink::send(uint16_t code, uint16_t len,
 {
   bool needForSpeed=false;
   if (state != Okay) return;
+//  if (code != MsgPlayerUpdateSmall && code != MsgPlayerUpdate)
+//    logDebugMessage(1,"send %s len %d\n",MsgStrings::strMsgCode(code),len);
   char msgbuf[MaxPacketLen];
   void* buf = msgbuf;
   buf = nboPackUShort(buf, len);
@@ -425,6 +493,8 @@ int			ServerLink::read(uint16_t& code, uint16_t& len,
       }
       udpBufferPtr = (char *)nboUnpackUShort(udpBufferPtr, len);
       udpBufferPtr = (char *)nboUnpackUShort(udpBufferPtr, code);
+//      if (code != MsgPlayerUpdateSmall && code != MsgPlayerUpdate && code != MsgGameTime)
+//	logDebugMessage(1,"rcvd %s len %d\n",MsgStrings::strMsgCode(code),len);
       UDEBUG("<** UDP Packet Code %x Len %x\n",code, len);
       if (len > udpLength) {
 	udpLength = 0;
@@ -491,7 +561,7 @@ int			ServerLink::read(uint16_t& code, uint16_t& len,
   buf = nboUnpackUShort(buf, len);
   buf = nboUnpackUShort(buf, code);
 
-  //printError("Code is %02x",code);
+//  logDebugMessage(1,"rcvd %s len %d\n",MsgStrings::strMsgCode(code),len);
   if (len > MaxPacketLen)
     return -1;
   if (len > 0)
@@ -501,25 +571,26 @@ int			ServerLink::read(uint16_t& code, uint16_t& len,
 #if defined(NETWORK_STATS)
   if (rlen >= 0) bytesReceived += rlen;
 #endif
-  if (rlen == int(len)) goto success;	// got whole thing
-
-  // keep reading until we get the whole message
-  tlen = rlen;
-  while (rlen >= 1 && tlen < int(len)) {
-    FD_ZERO(&read_set);
-    FD_SET((unsigned int)fd, &read_set);
-    nfound = select(fd+1, (fd_set*)&read_set, 0, 0, NULL);
-    if (nfound == 0) continue;
-    if (nfound < 0) return -1;
-    rlen = recv(fd, (char*)msg + tlen, int(len) - tlen, 0);
-    if (rlen >= 0) tlen += rlen;
-#if defined(NETWORK_STATS)
-  if (rlen >= 0) bytesReceived += rlen;
-#endif
+  if (rlen != int(len))
+  {
+    // keep reading until we get the whole message
+    tlen = rlen;
+    while (rlen >= 1 && tlen < int(len))
+    {
+	FD_ZERO(&read_set);
+	FD_SET((unsigned int)fd, &read_set);
+	nfound = select(fd+1, (fd_set*)&read_set, 0, 0, NULL);
+	if (nfound == 0) continue;
+	if (nfound < 0) return -1;
+	rlen = recv(fd, (char*)msg + tlen, int(len) - tlen, 0);
+	if (rlen >= 0) tlen += rlen;
+    #if defined(NETWORK_STATS)
+      if (rlen >= 0) bytesReceived += rlen;
+    #endif
+    }
+    if (tlen < int(len)) return -1;
   }
-  if (tlen < int(len)) return -1;
 
-success:
 // FIXME -- packet recording
 if (packetStream) {
   long dt = (long)((TimeKeeper::getCurrent() - packetStartTime) * 10000.0f);
@@ -534,19 +605,19 @@ if (packetStream) {
 void			ServerLink::sendEnter(PlayerType type,
 						TeamColor team,
 						const char* name,
-						const char* email,
+						const char* motto,
 						const char* token)
 {
   if (state != Okay) return;
-  char msg[PlayerIdPLen + 4 + CallSignLen + EmailLen + TokenLen + VersionLen];
+  char msg[PlayerIdPLen + 4 + CallSignLen + MottoLen + TokenLen + VersionLen];
   ::memset(msg, 0, sizeof(msg));
   void* buf = msg;
   buf = nboPackUShort(buf, uint16_t(type));
   buf = nboPackUShort(buf, uint16_t(team));
   ::memcpy(buf, name, ::strlen(name));
   buf = (void*)((char*)buf + CallSignLen);
-  ::memcpy(buf, email, ::strlen(email));
-  buf = (void*)((char*)buf + EmailLen);
+  ::memcpy(buf, motto, ::strlen(motto));
+  buf = (void*)((char*)buf + MottoLen);
   ::memcpy(buf, token, ::strlen(token));
   buf = (void*)((char*)buf + TokenLen);
   ::memcpy(buf, getAppVersion(), ::strlen(getAppVersion()) + 1);
@@ -792,16 +863,6 @@ void			ServerLink::confirmIncomingUDP()
   send(MsgUDPLinkEstablished, 0, NULL);
 }
 
-#ifdef HAVE_KRB5
-void ServerLink::sendKerberosTicket(const char      *principal,
-				    const krb5_data *ticket)
-{
-  logDebugMessage(3,"Sent authentication ticket to server : \n");
-  send(MsgKrbPrincipal, strlen(principal), principal);
-  send(MsgKrbTicket, ticket->length, ticket->data);
-}
-#endif
-
 // Local Variables: ***
 // mode:C++ ***
 // tab-width: 8 ***
