@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -19,7 +19,6 @@
 /* common implementation headers */
 #include "FontManager.h"
 #include "TextUtils.h"
-#include "bzglob.h"
 #include "AnsiCodes.h"
 
 /* local implementation headers */
@@ -27,86 +26,97 @@
 #include "HUDDialogStack.h"
 #include "playing.h"
 #include "HUDui.h"
+#include "ServerListFilter.h"
 
 const int ServerMenu::NumReadouts = 24;
 const int ServerMenu::NumItems = 10;
 
+
+static std::string colorizeSearch(const std::string& s);
+
+
 bool ServerMenuDefaultKey::keyPress(const BzfKeyEvent& key)
 {
-  if (key.ascii == 0) switch (key.button) {
-    case BzfKeyEvent::Up:
-      if (HUDui::getFocus()) {
-	if (!menu->getFind())
-	  menu->setSelected(menu->getSelected() - 1);
-	else
-	  menu->setFind(false);
+  if (key.ascii == 0) {
+    switch (key.button) {
+      case BzfKeyEvent::Up: {
+	if (HUDui::getFocus()) {
+	  if (!menu->getFind()) {
+	    menu->setSelected(menu->getSelected() - 1);
+	  } else {
+	    menu->setFind(false);
+	  }
+	}
+	return true;
       }
-      return true;
-
-    case BzfKeyEvent::Down:
-      if (HUDui::getFocus()) {
-	if (!menu->getFind())
-	  menu->setSelected(menu->getSelected() + 1);
-	else
-	  menu->setFind(false);
+      case BzfKeyEvent::Down: {
+	if (HUDui::getFocus()) {
+	  if (!menu->getFind()) {
+	    menu->setSelected(menu->getSelected() + 1);
+	  } else {
+	    menu->setFind(false);
+	  }
+	}
+	return true;
       }
-      return true;
-
-    case BzfKeyEvent::PageUp:
-      if (HUDui::getFocus()) {
-	if (!menu->getFind())
-	  menu->setSelected(menu->getSelected() - ServerMenu::NumItems);
-	else
-	  menu->setFind(false);
+      case BzfKeyEvent::PageUp: {
+	if (HUDui::getFocus()) {
+	  if (!menu->getFind()) {
+	    menu->setSelected(menu->getSelected() - ServerMenu::NumItems);
+	  } else {
+	    menu->setFind(false);
+	  }
+	}
+	return true;
       }
-      return true;
-
-    case BzfKeyEvent::PageDown:
-      if (HUDui::getFocus()) {
-	if (!menu->getFind())
-	  menu->setSelected(menu->getSelected() + ServerMenu::NumItems);
-	else
-	  menu->setFind(false);
+      case BzfKeyEvent::PageDown: {
+	if (HUDui::getFocus()) {
+	  if (!menu->getFind()) {
+	    menu->setSelected(menu->getSelected() + ServerMenu::NumItems);
+	  } else {
+	    menu->setFind(false);
+	  }
+	}
+	return true;
       }
-      return true;
-
+    }
   }
-
   else if (key.ascii == '\t') {
     if (HUDui::getFocus()) {
       menu->setSelected(menu->getSelected() + 1);
     }
     return true;
   }
-
   else if (key.ascii == '/') {
     if (HUDui::getFocus() && !menu->getFind()) {
-      menu->setFind(true);
+      menu->setFind(true, (key.shift & BzfKeyEvent::AltKey) != 0);
       return true;
     }
   }
-
   else if (key.ascii == 'f') {
     if (HUDui::getFocus() && !menu->getFind()) {
       menu->toggleFavView();
       return true;
     }
   }
-
   else if (key.ascii == '+') {
     if (HUDui::getFocus() && !menu->getFind()) {
       menu->setFav(true);
       return true;
     }
   }
-
   else if (key.ascii == '-') {
     if (HUDui::getFocus() && !menu->getFind()) {
       menu->setFav(false);
       return true;
     }
   }
-
+  else if ((key.ascii >= '0') && (key.ascii <= '9')) {
+    if (HUDui::getFocus() && !menu->getFind()) {
+      menu->setFindIndex(key.ascii - '0');
+      return true;
+    }
+  }
   else if (key.ascii == 27) {
     if (HUDui::getFocus()) {
       // escape drops out of find mode
@@ -121,32 +131,43 @@ bool ServerMenuDefaultKey::keyPress(const BzfKeyEvent& key)
   return MenuDefaultKey::keyPress(key);
 }
 
+
 bool ServerMenuDefaultKey::keyRelease(const BzfKeyEvent& key)
 {
   switch (key.button) {
     case BzfKeyEvent::Up:
     case BzfKeyEvent::Down:
     case BzfKeyEvent::PageUp:
-    case BzfKeyEvent::PageDown:
+    case BzfKeyEvent::PageDown: {
       return true;
+    }
   }
   switch (key.ascii) {
-    case 27:	// escape
-    case 13:	// return
+    case 27:   // escape
+    case 13: { // return
       return true;
+    }
   }
   return false;
 }
 
-ServerMenu::ServerMenu() : defaultKey(this),
-			   selectedIndex(0),
-			   serversFound(0),
-			   realServersFound(0),
-			   findMode(false),
-			   filter("*"),
-			   favView(false),
-			   newfilter(false)
+
+ServerMenu::ServerMenu()
+: defaultKey(this)
+, selectedIndex(0)
+, serversFound(0)
+, realServersFound(0)
+, findMode(false)
+, favView(false)
+, newfilter(true)
+, listFilter(BZDB.get("listFilter"))
+, lastWidth(0)
+, lastHeight(0)
 {
+  if (debugLevel > 0) {
+    listFilter.print();
+  }
+
   // add controls
   addLabel("Servers", "");
   addLabel("Players", "");
@@ -182,7 +203,9 @@ ServerMenu::ServerMenu() : defaultKey(this),
   // find server
   search = new HUDuiTypeIn;
   search->setFontFace(MainMenu::getFontFace());
-  search->setMaxLength(30);
+  search->setMaxLength(42);
+  search->setString(listFilter.getSource());
+  search->setColorFunc(colorizeSearch);
   getControls().push_back(search);
   setFind(false);
 
@@ -197,7 +220,6 @@ ServerMenu::ServerMenu() : defaultKey(this),
 }
 
 
-
 void ServerMenu::addLabel(const char* msg, const char* _label)
 {
   HUDuiLabel* label = new HUDuiLabel;
@@ -207,38 +229,67 @@ void ServerMenu::addLabel(const char* msg, const char* _label)
   getControls().push_back(label);
 }
 
-void ServerMenu::setFind(bool mode)
+
+void ServerMenu::setFindLabel(const std::string& label)
 {
-  std::string oldfilter = filter;
-  if (mode) {
-    search->setLabel("Find Servers:");
+  search->setLabel(label);
+  if (lastWidth && lastHeight) {
+    resize(lastWidth, lastHeight); // overkill...
+  }
+}
+
+
+void ServerMenu::setFind(bool mode, bool clear)
+{
+  const std::string oldFilterSource = listFilter.getSource();;
+
+  if (clear) {
+    search->setString("");
+  }
+
+  if (mode) { // filter is being typed in
+    setFindLabel(ANSI_STR_FG_GREEN "Edit Filter:");
     search->setFocus();
-  } else {
-    if (search->getString() == "" || search->getString() == "*") {
-      search->setLabel("Press '/' to search");
-      search->setString("");
-      filter = "*";
+  }
+  else {
+    listFilter.parse(search->getString());
+    BZDB.set("listFilter", search->getString());
+    if (listFilter.getSource().empty()) {
+      setFindLabel("Press '/' to search");
     } else {
-      if (search->getString().find("*") == std::string::npos
-	&& search->getString().find("?") == std::string::npos)
-	search->setString("*" + search->getString() + "*");
-      search->setLabel("Using filter:");
-      // filter is set in lower case
-      filter = TextUtils::tolower(search->getString());
+      setFindLabel(ANSI_STR_FG_RED "Using filter:");
     }
     // select the first item in the list
     setSelected(0);
   }
+
+  if (debugLevel > 0) {
+    listFilter.print();
+  }
+
   findMode = mode;
-  
-  newfilter = (filter != oldfilter);
+
+  newfilter = (listFilter.getSource() != oldFilterSource);
 }
 
+
+void ServerMenu::setFindIndex(int index)
+{
+  if ((index >= 0) && (index <= 9)) {
+    std::string name = "listFilter";
+    name += (index + '0');
+    search->setString(BZDB.get(name));
+    setFind(false);
+  }
+}
+
+
 bool ServerMenu::getFind() const
 {
   return findMode;
 }
 
+
 void ServerMenu::toggleFavView()
 {
   favView = !favView;
@@ -246,13 +297,14 @@ void ServerMenu::toggleFavView()
   updateStatus();
 }
 
+
 void ServerMenu::setFav(bool fav)
 {
   const ServerItem& item = serverList.getServers()[selectedIndex];
   std::string addrname = item.getAddrName();
   ServerListCache *cache = ServerListCache::get();
   ServerListCache::SRV_STR_MAP::iterator i = cache->find(addrname);
-  if (i!= cache->end()) {
+  if (i != cache->end()) {
     i->second.favorite = fav;
   } else {
     // FIXME  should not ever come here, but what to do?
@@ -263,11 +315,13 @@ void ServerMenu::setFav(bool fav)
   setSelected(getSelected()+1, true);
 }
 
+
 int ServerMenu::getSelected() const
 {
   return selectedIndex;
 }
 
+
 void ServerMenu::setSelected(int index, bool forcerefresh)
 {
   // clamp index
@@ -277,8 +331,9 @@ void ServerMenu::setSelected(int index, bool forcerefresh)
     index = 0;
 
   // ignore if no change
-  if (!forcerefresh && selectedIndex == index)
+  if (!forcerefresh && selectedIndex == index) {
     return;
+  }
 
   // update selected index and get old and new page numbers
   const int oldPage = (selectedIndex < 0) ? -1 : (selectedIndex / NumItems);
@@ -294,34 +349,35 @@ void ServerMenu::setSelected(int index, bool forcerefresh)
       HUDuiLabel* label = (HUDuiLabel*)listHUD[i + NumReadouts];
       if (base + i < (int)serverList.size()) {
 	const ServerItem &server = serverList.getServers()[base + i];
-	const short gameStyle = server.ping.gameStyle;
+	const short gameType = server.ping.gameType;
+  const short gameOptions = server.ping.gameOptions;
 	std::string fullLabel;
 	if (BZDB.isTrue("listIcons")) {
 	  // game mode
 	  if ((server.ping.observerMax == 16) &&
 	      (server.ping.maxPlayers == 200)) {
 	    fullLabel += ANSI_STR_FG_CYAN "*  "; // replay
-	  } else if (gameStyle & TeamFlagGameStyle) {
+	  } else if (gameType == ClassicCTF) {
 	    fullLabel += ANSI_STR_FG_RED "*  "; // ctf
-	  } else if (gameStyle & RabbitChaseGameStyle) {
+	  } else if (gameType == RabbitChase) {
 	    fullLabel += ANSI_STR_FG_WHITE "*  "; // white rabbit
 	  } else {
 	    fullLabel += ANSI_STR_FG_YELLOW "*  "; // free-for-all
 	  }
 	  // jumping?
-	  if (gameStyle & JumpingGameStyle) {
+	  if (gameOptions & JumpingGameStyle) {
 	    fullLabel += ANSI_STR_BRIGHT ANSI_STR_FG_MAGENTA "J ";
 	  } else {
 	    fullLabel += ANSI_STR_DIM ANSI_STR_FG_WHITE "J ";
 	  }
 	  // superflags ?
-	  if (gameStyle & SuperFlagGameStyle) {
+	  if (gameOptions & SuperFlagGameStyle) {
 	    fullLabel += ANSI_STR_BRIGHT ANSI_STR_FG_BLUE "F ";
 	  } else {
 	    fullLabel += ANSI_STR_DIM ANSI_STR_FG_WHITE "F ";
 	  }
 	  // ricochet?
-	  if (gameStyle & RicochetGameStyle) {
+	  if (gameOptions & RicochetGameStyle) {
 	    fullLabel += ANSI_STR_BRIGHT ANSI_STR_FG_GREEN "R";
 	  } else {
 	    fullLabel += ANSI_STR_DIM ANSI_STR_FG_WHITE "R";
@@ -354,26 +410,23 @@ void ServerMenu::setSelected(int index, bool forcerefresh)
 	}
 	else {
 	  // colorize servers: many shots->red, jumping->green, CTF->blue
-	  const float rf = std::min(1.0f, logf(server.ping.maxShots) / logf(20.0f));
-	  const float gf = gameStyle & JumpingGameStyle ? 1.0f : 0.0f;
-	  const float bf = gameStyle & TeamFlagGameStyle ? 1.0f : 0.0f;
+	  const float rf = std::min(1.0f, logf((float)server.ping.maxShots) / logf(20.0f));
+	  const float gf = gameOptions & JumpingGameStyle ? 1.0f : 0.0f;
+	  const float bf = (gameType == ClassicCTF) ? 1.0f : 0.0f;
 	  label->setColor(0.5f + rf * 0.5f, 0.5f + gf * 0.5f, 0.5f + bf * 0.5f);
 	}
 
-	std::string addr = stripAnsiCodes(server.description);
-	std::string desc;
-	std::string::size_type pos = addr.find_first_of(';');
-	if (pos != std::string::npos) {
-	  desc = addr.substr(pos > 0 ? pos+1 : pos);
-	  addr.resize(pos);
-        }
-        if (server.favorite)
-          fullLabel += ANSI_STR_FG_ORANGE;
-        else
+	std::string addr, desc;
+	server.splitAddrTitle(addr, desc);
+	if (server.favorite)
+	  fullLabel += ANSI_STR_FG_ORANGE;
+	else
 	  fullLabel += ANSI_STR_FG_WHITE;
 	fullLabel += addr;
-	fullLabel += ANSI_STR_RESET " ";
-	fullLabel += desc;
+	if (!desc.empty()) {
+	  fullLabel += ANSI_STR_RESET "  ";
+	  fullLabel += desc;
+	}
 	label->setString(fullLabel);
 	label->setDarker(server.cached);
       }
@@ -404,10 +457,12 @@ void ServerMenu::setSelected(int index, bool forcerefresh)
   pick();
 }
 
+
 void ServerMenu::pick()
 {
-  if (serverList.size() == 0)
+  if (serverList.size() == 0) {
     return;
+  }
 
   // get server info
   const ServerItem& item = serverList.getServers()[selectedIndex];
@@ -426,7 +481,8 @@ void ServerMenu::pick()
       sprintf(buf, "?/%d", maxes[i-1]);
       ((HUDuiLabel*)listHUD[i])->setLabel(buf);
     }
-  } else {  // not an old item, set players #s to info we have
+  }
+  else {  // not an old item, set players #s to info we have
     sprintf(buf, "%d/%d", ping.rogueCount + ping.redCount + ping.greenCount +
 	ping.blueCount + ping.purpleCount + ping.observerCount, ping.maxPlayers);
     ((HUDuiLabel*)listHUD[1])->setLabel(buf);
@@ -489,24 +545,26 @@ void ServerMenu::pick()
   else
     ((HUDuiLabel*)listHUD[8])->setString("{1} Shots", &args );
 
-  if (ping.gameStyle & TeamFlagGameStyle)
-    ((HUDuiLabel*)listHUD[9])->setString("Capture-the-Flag");
-  else if (ping.gameStyle & RabbitChaseGameStyle)
+  if (ping.gameType == ClassicCTF)
+    ((HUDuiLabel*)listHUD[9])->setString("Classic Capture-the-Flag");
+  else if (ping.gameType == RabbitChase)
     ((HUDuiLabel*)listHUD[9])->setString("Rabbit Chase");
+  else if (ping.gameType == OpenFFA)
+    ((HUDuiLabel*)listHUD[9])->setString("Open (Teamless) Free-For-All");
   else
     ((HUDuiLabel*)listHUD[9])->setString("Free-style");
 
-  if (ping.gameStyle & SuperFlagGameStyle)
+  if (ping.gameOptions & SuperFlagGameStyle)
     ((HUDuiLabel*)listHUD[10])->setString("Super Flags");
   else
     ((HUDuiLabel*)listHUD[10])->setString("");
 
-  if (ping.gameStyle & AntidoteGameStyle)
+  if (ping.gameOptions & AntidoteGameStyle)
     ((HUDuiLabel*)listHUD[11])->setString("Antidote Flags");
   else
     ((HUDuiLabel*)listHUD[11])->setString("");
 
-  if ((ping.gameStyle & ShakableGameStyle) && ping.shakeTimeout != 0) {
+  if ((ping.gameOptions & ShakableGameStyle) && ping.shakeTimeout != 0) {
     std::vector<std::string> dropArgs;
     sprintf(buf, "%.1f", 0.1f * float(ping.shakeTimeout));
     dropArgs.push_back(buf);
@@ -517,10 +575,11 @@ void ServerMenu::pick()
       ((HUDuiLabel*)listHUD[12])->setString("{1} secs To Drop Bad Flag",
 					    &dropArgs);
   }
-  else
+  else {
     ((HUDuiLabel*)listHUD[13])->setString("");
+  }
 
-  if ((ping.gameStyle & ShakableGameStyle) && ping.shakeWins != 0) {
+  if ((ping.gameOptions & ShakableGameStyle) && ping.shakeWins != 0) {
     std::vector<std::string> dropArgs;
     sprintf(buf, "%d", ping.shakeWins);
     dropArgs.push_back(buf);
@@ -532,23 +591,27 @@ void ServerMenu::pick()
       ((HUDuiLabel*)listHUD[12])->setString("{1} Wins Drops Bad Flag",
 					    &dropArgs);
   }
-  else
+  else {
     ((HUDuiLabel*)listHUD[13])->setString("");
+  }
 
-  if (ping.gameStyle & JumpingGameStyle)
+  if (ping.gameOptions & JumpingGameStyle) {
     ((HUDuiLabel*)listHUD[14])->setString("Jumping");
-  else
+  } else {
     ((HUDuiLabel*)listHUD[14])->setString("");
+  }
 
-  if (ping.gameStyle & RicochetGameStyle)
+  if (ping.gameOptions & RicochetGameStyle) {
     ((HUDuiLabel*)listHUD[15])->setString("Ricochet");
-  else
+  } else {
     ((HUDuiLabel*)listHUD[15])->setString("");
+  }
 
-  if (ping.gameStyle & HandicapGameStyle)
+  if (ping.gameOptions & HandicapGameStyle) {
     ((HUDuiLabel*)listHUD[16])->setString("Handicap");
-  else
+  } else {
     ((HUDuiLabel*)listHUD[16])->setString("");
+  }
 
   if (ping.maxTime != 0) {
     std::vector<std::string> pingArgs;
@@ -570,8 +633,9 @@ void ServerMenu::pick()
     scoreArgs.push_back(buf);
     ((HUDuiLabel*)listHUD[18])->setString("Max team score: {1}", &scoreArgs);
   }
-  else
+  else {
     ((HUDuiLabel*)listHUD[18])->setString("");
+  }
 
 
   if (ping.maxPlayerScore != 0) {
@@ -583,7 +647,7 @@ void ServerMenu::pick()
     ((HUDuiLabel*)listHUD[19])->setString("");
   }
 
-  if (item.cached){
+  if (item.cached) {
     ((HUDuiLabel*)listHUD[20])->setString("Cached");
     ((HUDuiLabel*)listHUD[21])->setString(item.getAgeString());
   }
@@ -593,7 +657,8 @@ void ServerMenu::pick()
   }
 }
 
-void			ServerMenu::show()
+
+void ServerMenu::show()
 {
   // clear server readouts
   std::vector<HUDuiControl*>& listHUD = getControls();
@@ -621,8 +686,7 @@ void			ServerMenu::show()
 
   // add cache items w/o re-caching them
   serversFound = 0;
-  int numItemsAdded;
-  numItemsAdded = realServerList.updateFromCache();
+  realServerList.updateFromCache();
 
   // update the status
   updateStatus();
@@ -634,31 +698,34 @@ void			ServerMenu::show()
   // listen for echos
   addPlayingCallback(&playingCB, this);
   realServerList.startServerPings(getStartupInfo());
-
 }
 
-void			ServerMenu::execute()
+
+void ServerMenu::execute()
 {
-  HUDuiControl* _focus = HUDui::getFocus();
-  if (_focus == search) {
+  const bool endFind = (HUDui::getFocus() == search);
+  if (endFind) {
     setFind(false);
+    status->setFocus();
     return;
   }
 
-  if (selectedIndex < 0 || selectedIndex >= (int)serverList.size())
+  if (selectedIndex < 0 || selectedIndex >= (int)serverList.size()) {
     return;
+  }
 
   // update startup info
   StartupInfo* info = getStartupInfo();
   strcpy(info->serverName, serverList.getServers()[selectedIndex].name.c_str());
   info->serverPort = ntohs((unsigned short)
-				serverList.getServers()[selectedIndex].ping.serverId.port);
+			   serverList.getServers()[selectedIndex].ping.serverId.port);
 
   // all done
   HUDDialogStack::get()->pop();
 }
 
-void			ServerMenu::dismiss()
+
+void ServerMenu::dismiss()
 {
   // no more callbacks
   removePlayingCallback(&playingCB, this);
@@ -666,8 +733,12 @@ void			ServerMenu::dismiss()
   // FIXME myTank.token = serverList.token;
 }
 
-void			ServerMenu::resize(int _width, int _height)
+
+void ServerMenu::resize(int _width, int _height)
 {
+  lastWidth = _width;
+  lastHeight = _height;
+
   // remember size
   HUDDialog::resize(_width, _height);
 
@@ -715,7 +786,7 @@ void			ServerMenu::resize(int _width, int _height)
     status->setFontSize(fontSize);
     const float statusWidth = fm.getStrLength(status->getFontFace(), fontSize, status->getString());
     x = 0.5f * ((float)_width - statusWidth);
-    y -= 0.8f * fontHt;
+    y -= 1.5f * fontHt;
     status->setPosition(x, y);
   }
 
@@ -724,8 +795,8 @@ void			ServerMenu::resize(int _width, int _height)
     fontSize = (float)_height / 36.0f;
     float fontHt = fm.getStrHeight(MainMenu::getFontFace(), fontSize, " ");
     search->setFontSize(fontSize);
-    const float searchWidth = fm.getStrLength(search->getFontFace(), fontSize, search->getString());
-    x = 0.5f * ((float)_width - searchWidth);
+    const float searchWidth = fm.getStrLength(search->getFontFace(), fontSize, search->getLabel());
+    x = (0.1f * (float)_width) + searchWidth;
     search->setPosition(x, fontHt * 2 /* near bottom of screen */);
   }
 
@@ -757,7 +828,8 @@ void			ServerMenu::resize(int _width, int _height)
   }
 }
 
-void			ServerMenu::setStatus(const char* msg, const std::vector<std::string> *parms)
+
+void ServerMenu::setStatus(const char* msg, const std::vector<std::string> *parms)
 {
   status->setString(msg, parms);
   FontManager &fm = FontManager::instance();
@@ -765,7 +837,8 @@ void			ServerMenu::setStatus(const char* msg, const std::vector<std::string> *pa
   status->setPosition(0.5f * ((float)width - statusWidth), status->getY());
 }
 
-void			ServerMenu::updateStatus() {
+
+void ServerMenu::updateStatus() {
   // nothing here to see
   if (!realServerList.serverFound()) {
     setStatus("Searching");
@@ -773,18 +846,25 @@ void			ServerMenu::updateStatus() {
   }
 
   // don't run unnecessarily
-  if (realServersFound == realServerList.size() && !newfilter)
+  if (realServersFound == realServerList.size() && !newfilter) {
     return;
+  }
 
-  // do filtering
+  // do filtering and counting
+  int playerCount = 0;
+  int observerCount = 0;
   serverList.clear();
   for (unsigned int i = 0; i < realServerList.size(); ++i) {
     const ServerItem &item = realServerList.getServers()[i];
+    const PingPacket& ping = item.ping;
+    playerCount += ping.rogueCount;
+    playerCount += ping.redCount;
+    playerCount += ping.greenCount;
+    playerCount += ping.blueCount;
+    playerCount += ping.purpleCount;
+    observerCount += ping.observerCount;
     // filter is already lower case.  do case insensitive matching.
-    if ((glob_match(filter, TextUtils::tolower(item.description)) ||
-         glob_match(filter, TextUtils::tolower(item.name))) &&
-        (!favView || item.favorite)
-       ) {
+    if (listFilter.check(item) && (!favView || item.favorite)) {
       serverList.addToList(item);
     }
   }
@@ -797,10 +877,15 @@ void			ServerMenu::updateStatus() {
   args.push_back(buffer);
   sprintf(buffer, "%d", (unsigned int)realServerList.size());
   args.push_back(buffer);
-  if (favView)
-    setStatus("Favorite servers: {1}/{2}", &args);
-  else
-    setStatus("Servers found: {1}/{2}", &args);
+  sprintf(buffer, "%d", playerCount);
+  args.push_back(buffer);
+  sprintf(buffer, "%d", observerCount);
+  args.push_back(buffer);
+  if (favView) {
+    setStatus("Favorite servers: {1}/{2}  ({3} players, {4} observers)", &args);
+  } else {
+    setStatus("Servers found: {1}/{2}  ({3} players, {4} observers)", &args);
+  }
   pageLabel->setString("");
   selectedIndex = -1;
   setSelected(0);
@@ -810,13 +895,77 @@ void			ServerMenu::updateStatus() {
 }
 
 
-void			ServerMenu::playingCB(void* _self)
+void ServerMenu::playingCB(void* _self)
 {
   ((ServerMenu*)_self)->realServerList.checkEchos(getStartupInfo());
 
   ((ServerMenu*)_self)->updateStatus();
 }
 
+
+static std::string colorizeSearch(const std::string& in)
+{
+  std::string out;
+  std::vector<std::string> filters;
+  std::vector<char> separators;
+  const char *c, *s, *s0 = in.c_str();
+  for (c = s = s0; true; c++) {
+    if (*c == 0) {
+      filters.push_back(std::string(s, c - s));
+      s = c + 1;
+      break;
+    }
+    else if ((*c == '/') || (*c == ',')) {
+      filters.push_back(std::string(s, c - s));
+      separators.push_back(*c);
+      s = c + 1;
+    }
+  }
+
+  static const std::string controlColor = ANSI_STR_FG_CYAN;
+  static const std::string badColor     = ANSI_STR_FG_ORANGE;
+  static const std::string unknownColor = ANSI_STR_FG_RED;
+  static const std::string commentColor = ANSI_STR_FG_BLACK;
+  static const std::string boolColor    = ANSI_STR_FG_YELLOW;
+  static const std::string rangeColor   = ANSI_STR_FG_GREEN;
+  static const std::string globColor    = ANSI_STR_FG_BLUE;
+  static const std::string regexColor   = ANSI_STR_FG_MAGENTA;
+
+  out += filters[0];
+  for (size_t i = 1; i < filters.size(); i++) {
+    out += controlColor;
+    out += separators[i - 1];
+    char op;
+    std::string lbl, param;
+    const char type =
+      ServerListFilter::parseFilterType(filters[i], op, lbl, param);
+    switch (type) {
+      case 'p': {
+	if (ServerListFilter::isPatternLabel(lbl)) {
+	  out += (op == ')') ? globColor : regexColor;
+	} else {
+	  out += unknownColor;
+	}
+	break;
+      }
+      case 'b': {
+	out += ServerListFilter::isBoolLabel(lbl) ? boolColor : unknownColor;
+	break;
+      }
+      case 'r': {
+	out += ServerListFilter::isRangeLabel(lbl) ? rangeColor : unknownColor;
+	break;
+      }
+      case '#': { out += commentColor; break; }
+      default:  { out += badColor;     break; }
+    }
+    out += filters[i];
+  }
+
+  return out;
+}
+
+
 // Local Variables: ***
 // mode: C++ ***
 // tab-width: 8 ***
