@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -20,6 +20,7 @@
 /* system interface headers */
 #include <string>
 #include <vector>
+#include <deque>
 
 /* common interface headers */
 #include "BzfEvent.h"
@@ -30,9 +31,12 @@
 #include "HUDuiDefaultKey.h"
 #include "HUDuiLabel.h"
 #include "HUDuiTypeIn.h"
+#include "ServerListFilter.h"
+
 
 class ServerMenu;
 
+
 class ServerMenuDefaultKey : public MenuDefaultKey {
 public:
   ServerMenuDefaultKey(ServerMenu* _menu) :
@@ -58,15 +62,18 @@ public:
   void execute();
   void dismiss();
   void resize(int width, int height);
-  static void playingCB(void*);
   void updateStatus();
+  void setFindLabel(const std::string& label);
 
   bool getFind() const;
-  void setFind(bool mode);
+  void setFind(bool mode, bool clear = false);
+  void setFindIndex(int index);
 
   void toggleFavView();
   void setFav(bool);
 
+  static void playingCB(void*);
+
   static const int NumItems;
 
 private:
@@ -90,10 +97,13 @@ private:
 
   HUDuiTypeIn* search;
   bool findMode;
-  std::string filter;
   bool favView;
   bool newfilter;
 
+  ServerListFilter listFilter;
+
+  int lastWidth, lastHeight;
+
   static const int NumReadouts;
 };
 
