@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -319,14 +319,6 @@ HUDuiDefaultKey* ServerStartMenu::getDefaultKey()
 
 void ServerStartMenu::setSettings(const char* _settings)
 {
-  // FIXME -- temporary to automatically upgrade old configurations
-  if (strlen(_settings) == 14) {
-    strcpy(settings, _settings);
-    settings[12] = settings[13];
-    settings[13] = settings[14];
-    return;
-  }
-
   if (strlen(_settings) != strlen(settings)) return;
   strcpy(settings, _settings);
 }
@@ -523,22 +515,22 @@ void ServerStartMenu::execute()
     if (pid != -1) {
       int i;
       for (i = 0; i < 12; i++) {
-        if (kill(pid, SIGTERM) == 0) {
-          TimeKeeper::sleep(0.25); // be gracious for max 3 seconds
-        } else {
-          if (errno == ESRCH) {
-            break; // the pid doesn't exist
-          }
-        }
+	if (kill(pid, SIGTERM) == 0) {
+	  TimeKeeper::sleep(0.25); // be gracious for max 3 seconds
+	} else {
+	  if (errno == ESRCH) {
+	    break; // the pid doesn't exist
+	  }
+	}
       }
       if (i == 12) {
-        kill(pid, SIGKILL); // be brutal
+	kill(pid, SIGKILL); // be brutal
       }
     }
 
     // fork
     pid = fork();
-    
+
     if (pid == -1) {
       setStatus("Failed... cannot fork.");
     }
@@ -563,13 +555,13 @@ void ServerStartMenu::execute()
       if (waitpid(pid, &pStatus, WNOHANG) != 0) {
 	pid = -1;
 	char failBuf[64];
-        if (WIFEXITED(pStatus)) {
-          snprintf(failBuf, 64, "Failed (exit = %i).", WEXITSTATUS(pStatus));
-        } else if (WIFSIGNALED(pStatus)) {
-          snprintf(failBuf, 64, "Failed (signal = %i).", WTERMSIG(pStatus));
-        } else {
-          strcpy(failBuf, "Failed.");
-        }
+	if (WIFEXITED(pStatus)) {
+	  snprintf(failBuf, 64, "Failed (exit = %i).", WEXITSTATUS(pStatus));
+	} else if (WIFSIGNALED(pStatus)) {
+	  snprintf(failBuf, 64, "Failed (signal = %i).", WTERMSIG(pStatus));
+	} else {
+	  strcpy(failBuf, "Failed.");
+	}
 	setStatus(failBuf);
       } else {
 	setStatus("Server started.");
