@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -46,7 +46,8 @@ int			World::flagTexture(-1);
 
 
 World::World() :
-  gameStyle(PlainGameStyle),
+  gameType(TeamFFA),
+  gameOptions(0),
   linearAcceleration(0.0f),
   angularAcceleration(0.0f),
   maxPlayers(0),
@@ -171,7 +172,7 @@ const Teleporter* World::getTeleporter(int source, int& face) const
 
 TeamColor		World::whoseBase(const float* pos) const
 {
-  if (!(gameStyle & TeamFlagGameStyle))
+   if (gameType != ClassicCTF)
     return NoTeam;
 
   for (int i = 1; i < NumTeams; i++) {
@@ -563,7 +564,7 @@ void			World::freeMeshDrawMgrs()
 }
 
 
-void			World::updateAnimations(float /*dt*/)
+void			World::updateAnimations(float UNUSED(dt))
 {
   const double gameTime = GameTime::getStepTime();
   for (int i = 0; i < drawInfoCount; i++) {
@@ -685,7 +686,7 @@ void			World::initFlag(int index)
   }
 }
 
-void			World::updateWind(float /*dt*/)
+void			World::updateWind(float UNUSED(dt))
 {
   const float minWindSpeed = 0.0f; // FIXME - BZDB
   const float maxWindSpeed = 10.0f; // FIXME - BZDB
@@ -1226,6 +1227,73 @@ static void drawLines (int count, float (*vertices)[3], int color)
   return;
 }
 
+
+static void drawInsideOutsidePoints()
+{
+  std::vector<const float*> insides;
+  std::vector<const float*> outsides;
+
+  const ObstacleList& meshes = OBSTACLEMGR.getMeshes();
+  for (unsigned int i = 0; i < meshes.size(); i++) {
+    const MeshObstacle* mesh = (const MeshObstacle*) meshes[i];
+    const int    checkCount  = mesh->getCheckCount();
+    const char*  checkTypes  = mesh->getCheckTypes();
+    const afvec3* checkPoints = mesh->getCheckPoints();
+    for (int c = 0; c < checkCount; c++) {
+      switch (checkTypes[c]) {
+	case MeshObstacle::CheckInside: {
+	  insides.push_back(checkPoints[c]);
+	  break;
+	}
+	case MeshObstacle::CheckOutside: {
+	  outsides.push_back(checkPoints[c]);
+	  break;
+	}
+	default: {
+	  break;
+	}
+      }
+    }
+  }
+
+  glPushAttrib(GL_DEPTH_BUFFER_BIT | GL_POINT_BIT | GL_LINE_BIT);
+
+  glDisable(GL_DEPTH_TEST);
+  glEnable(GL_POINT_SMOOTH);
+  glEnable(GL_LINE_SMOOTH);
+  glLineWidth(1.49f);
+  glPointSize(4.49f);
+
+  glBegin(GL_POINTS); {
+    glColor4f(0.0f, 1.0f, 0.0f, 0.8f);
+    for (size_t i = 0; i < insides.size(); i++) {
+      glVertex3fv(insides[i]);
+    }
+    glColor4f(1.0f, 0.0f, 0.0f, 0.8f);
+    for (size_t i = 0; i < outsides.size(); i++) {
+      glVertex3fv(outsides[i]);
+    }
+  }
+  glEnd();
+
+  glBegin(GL_LINES); {
+    glColor4f(0.0f, 1.0f, 0.0f, 0.2f);
+    for (size_t i = 0; i < insides.size(); i++) {
+      glVertex3f(insides[i][0], insides[i][1], 0.0f);
+      glVertex3fv(insides[i]);
+    }
+    glColor4f(1.0f, 0.0f, 0.0f, 0.2f);
+    for (size_t i = 0; i < outsides.size(); i++) {
+      glVertex3f(outsides[i][0], outsides[i][1], 0.0f);
+      glVertex3fv(outsides[i]);
+    }
+  }
+  glEnd();
+
+  glPopAttrib();
+}
+
+
 void World::drawCollisionGrid() const
 {
   GLboolean usingTextures;
@@ -1235,6 +1303,8 @@ void World::drawCollisionGrid() const
 
   COLLISIONMGR.draw (&drawLines);
 
+  drawInsideOutsidePoints();
+
   if (usingTextures) {
     glEnable (GL_TEXTURE_2D);
   }
