@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -37,6 +37,7 @@
 
 /* common headers */
 #include "Address.h"
+#include "AresHandler.h"
 #include "BZDBCache.h"
 #include "BZDBLocal.h"
 #include "BundleMgr.h"
@@ -89,8 +90,6 @@
 int beginendCount = 0;
 
 const char*		argv0;
-static bool		anonymous = false;
-static std::string	anonymousName("anonymous");
 std::string		alternateConfig;
 static bool		noAudio = false;
 struct tm		userTime;
@@ -155,8 +154,11 @@ static void		setVisual(BzfVisual* visual)
   }
 
   // optional
+  visual->setStencil(4);
 #if defined(DEBUG_RENDERING)
   visual->setStencil(4);
+#else
+  visual->setStencil(1);
 #endif
   if (BZDB.isTrue("multisample"))
     visual->setMultisample(4);
@@ -169,8 +171,6 @@ static void		setVisual(BzfVisual* visual)
 static void		usage()
 {
   printFatalError("usage: %s"
-	" [-3dfx] [-no3dfx]"
-	" [-anonymous]"
 	" [-badwords <filterfile>]"
 	" [-config <configfile>]"
 	" [-configdir <config dir name>]"
@@ -179,7 +179,6 @@ static void		usage()
 	" [{-dir | -directory} <data-directory>]"
 	" [-e | -echo]"
 	" [-ea | -echoAnsi]"
-	" [{-g | -geometry} <geometry-spec>]"
 	" [-h | -help | --help]"
 	" [-latitude <latitude>] [-longitude <longitude>]"
 	" [-list <list-server-url>] [-nolist]"
@@ -194,7 +193,7 @@ static void		usage()
 	" [-time hh:mm:ss] [-notime]"
 	" [-v | -version | --version]"
 	" [-view {normal|stereo|stacked|three|anaglyph|interlaced}]"
-	" [-window]"
+	" [-window <geometry-spec>]"
 	" [-zoom <zoom-factor>]"
 	" [callsign[:password]@]server[:port]\n\nExiting.", argv0);
   if (display != NULL) {
@@ -217,50 +216,36 @@ static void		parse(int argc, char** argv)
 {
 // = 9;
   for (int i = 1; i < argc; i++) {
-    if (strcmp(argv[i], "-a") == 0 ||
-		strcmp(argv[i], "-anonymous") == 0) {
-      anonymous = true;
-    } else if (strcmp(argv[i], "-config") == 0) {
+    if (strcmp(argv[i], "-config") == 0) {
       checkArgc(i, argc, argv[i]);
       // the setting has already been done in parseConfigName()
-	} else if (strcmp(argv[i], "-configdir") == 0) {
-		checkArgc(i, argc, argv[i]);
-		// the setting has already been done in parseConfigName()
-	} else if ((strcmp(argv[i], "-d") == 0) ||
-	       (strcmp(argv[i], "-debug") == 0)) {
-      debugLevel++;
-    } else if ((strcmp(argv[i], "-dir") == 0) ||
-	       (strcmp(argv[i], "-directory") == 0)) {
+    }
+    else if (strcmp(argv[i], "-configdir") == 0) {
+      checkArgc(i, argc, argv[i]);
+      // the setting has already been done in parseConfigName()
+    }
+    else if ((strcmp(argv[i], "-dir") == 0) ||
+	     (strcmp(argv[i], "-directory") == 0)) {
       checkArgc(i, argc, argv[i]);
       if (strlen(argv[i]) == 0) {
 	BZDB.unset("directory");
       } else {
 	BZDB.set("directory", argv[i]);
       }
-    } else if (strcmp(argv[i], "-e") == 0 || strcmp(argv[i], "-echo") == 0) {
+    }
+    else if (strcmp(argv[i], "-e") == 0 || strcmp(argv[i], "-echo") == 0) {
       echoToConsole = true;
-    } else if (strcmp(argv[i], "-ea") == 0 || strcmp(argv[i], "-echoAnsi") == 0) {
+    }
+    else if (strcmp(argv[i], "-ea") == 0 || strcmp(argv[i], "-echoAnsi") == 0) {
       echoToConsole = true;
       echoAnsi = true;
-    } else if (strcmp(argv[i], "-h") == 0 ||
+    }
+    else if (strcmp(argv[i], "-h") == 0 ||
 	     strcmp(argv[i], "-help") == 0 ||
 	     strcmp(argv[i], "--help") == 0) {
       usage();
-    } else if (strcmp(argv[i], "-g") == 0 || strcmp(argv[i], "-geometry") == 0) {
-      checkArgc(i, argc, argv[i]);
-      int w, h, x, y, count;
-      char xs = '+', ys = '+';
-      if (strcmp(argv[i], "default") != 0 &&
-	  (((count = sscanf(argv[i], "%dx%d%c%d%c%d",
-		&w, &h, &xs, &x, &ys, &y)) != 6 && count != 2) ||
-	  (xs != '-' && xs != '+') || (ys != '-' && ys != '+'))) {
-	printFatalError("Invalid argument for %s.\n"
-			"Correct format is <width>x<height>[+|-]<x>[+|-]<y>.",
-			argv[i-1]);
-	usage();
-      }
-      BZDB.set("geometry", argv[i]);
-    } else if (strcmp(argv[i], "-latitude") == 0) {
+    }
+    else if (strcmp(argv[i], "-latitude") == 0) {
       checkArgc(i, argc, argv[i]);
       double latitude = atof(argv[i]);
       if (latitude < -90.0 || latitude > 90.0) {
@@ -268,7 +253,8 @@ static void		parse(int argc, char** argv)
 	usage();
       }
       BZDB.set("latitude", argv[i]);
-    } else if (strcmp(argv[i], "-longitude") == 0) {
+    }
+    else if (strcmp(argv[i], "-longitude") == 0) {
       checkArgc(i, argc, argv[i]);
       double longitude = atof(argv[i]);
       if (longitude < -180.0 || longitude > 180.0) {
@@ -276,7 +262,8 @@ static void		parse(int argc, char** argv)
 	usage();
       }
       BZDB.set("longitude", argv[i]);
-    } else if (strcmp(argv[i], "-list") == 0) {
+    }
+    else if (strcmp(argv[i], "-list") == 0) {
       checkArgc(i, argc, argv[i]);
       if (strcmp(argv[i], "default") == 0) {
 	BZDB.set("list", BZDB.getDefault("list"));
@@ -284,7 +271,8 @@ static void		parse(int argc, char** argv)
 	startupInfo.listServerURL = argv[i];
 	BZDB.set("list", argv[i]);
       }
-    } else if (strcmp(argv[i], "-locale") == 0) {
+    }
+    else if (strcmp(argv[i], "-locale") == 0) {
       checkArgc(i, argc, argv[i]);
       BZDB.set("locale", argv[i]);
     } else if (strcmp(argv[i], "-motd") == 0) {
@@ -295,18 +283,23 @@ static void		parse(int argc, char** argv)
 	BZDB.set("motdServer", argv[i]);
       }
       BZDB.unset("disableMOTD");
-    } else if (strcmp(argv[i], "-nomotd") == 0) {
+    }
+    else if (strcmp(argv[i], "-nomotd") == 0) {
       BZDB.set("disableMOTD", "1");
-    } else if (strcmp(argv[i], "-nolist") == 0) {
+    }
+    else if (strcmp(argv[i], "-nolist") == 0) {
       startupInfo.listServerURL = "";
       BZDB.set("list", "");
-    } else if (strcmp(argv[i], "-m") == 0 ||
+    }
+    else if (strcmp(argv[i], "-m") == 0 ||
 		strcmp(argv[i], "-mute") == 0) {
       noAudio = true;
-    } else if (strcmp(argv[i], "-multisample") == 0) {
+    }
+    else if (strcmp(argv[i], "-multisample") == 0) {
       BZDB.set("_multisample", "1");
 #ifdef ROBOT
-    } else if (strcmp(argv[i], "-solo") == 0) {
+    }
+    else if (strcmp(argv[i], "-solo") == 0) {
       checkArgc(i, argc, argv[i]);
       numRobotTanks = atoi(argv[i]);
       if (numRobotTanks < 1 || numRobotTanks > MAX_ROBOTS) {
@@ -314,7 +307,8 @@ static void		parse(int argc, char** argv)
 	usage();
       }
 #endif
-    } else if (strcmp(argv[i], "-team") == 0) {
+    }
+    else if (strcmp(argv[i], "-team") == 0) {
       checkArgc(i, argc, argv[i]);
       if ((strcmp(argv[i], "a") == 0) ||
 	  (strcmp(argv[i], "auto") == 0) ||
@@ -345,22 +339,24 @@ static void		parse(int argc, char** argv)
 		bzfcopyright);
       bail(0);
       exit(0);
-    } else if (strcmp(argv[i], "-window") == 0) {
+    }
+    else if (strcmp(argv[i], "-window") == 0) {
       BZDB.set("_window", "1");
-    } else if (strcmp(argv[i], "-3dfx") == 0 || strcmp(argv[i], "-3Dfx") == 0) {
-#if !defined(__linux__)
-      putenv("MESA_GLX_FX=fullscreen");
-#else
-      setenv("MESA_GLX_FX", "fullscreen", 1);
-#endif
-    } else if (strcmp(argv[i], "-no3dfx") == 0 || strcmp(argv[i], "-no3Dfx") == 0) {
-#if !defined(__linux__)
-      putenv("MESA_GLX_FX=");
-#else
-      unsetenv("MESA_GLX_FX");
-#endif
+      checkArgc(i, argc, argv[i]);
+      int w, h, x, y, count;
+      char xs ='+', ys='+';
+      if (strcmp(argv[i], "default") != 0
+	 && (((count = sscanf(argv[i], "%dx%d%c%d%c%d", &w, &h, &xs, &x, &ys, &y)) != 6 && count != 2)
+	 || (xs != '-' && xs != '+')
+	 || (ys != '-' && ys != '+')))
+      {
+	printFatalError("Invalid argument for %s. \nCorrect format is <width>x<height>[+|-]<x>[+|-].", argv[i-1]);
+	usage();
+      }
+      BZDB.set("geometry", argv[i]);
 #ifdef DEBUG
-    } else if (strcmp(argv[i], "-date") == 0) {
+    }
+    else if (strcmp(argv[i], "-date") == 0) {
       checkArgc(i, argc, argv[i]);
       int month, day, year;
       // FIXME: should use iso yyyy.mm.dd format
@@ -377,15 +373,19 @@ static void		parse(int argc, char** argv)
       userTime.tm_mon = month - 1;
       userTime.tm_year = year;
 #endif
-    } else if (strcmp(argv[i], "-time") == 0) {
+    }
+    else if (strcmp(argv[i], "-time") == 0) {
       checkArgc(i, argc, argv[i]);
       BZDB.set("fixedTime", argv[i]);
-    } else if (strcmp(argv[i], "-notime") == 0) {
+    }
+    else if (strcmp(argv[i], "-notime") == 0) {
       BZDB.unset("fixedTime");
-    } else if (strcmp(argv[i], "-view") == 0) {
+    }
+    else if (strcmp(argv[i], "-view") == 0) {
       checkArgc(i, argc, argv[i]);
       BZDB.set("view", argv[i]);
-    } else if (strcmp(argv[i], "-zoom") == 0) {
+    }
+    else if (strcmp(argv[i], "-zoom") == 0) {
       checkArgc(i, argc, argv[i]);
       const int zoom = atoi(argv[i]);
       if (zoom < 1 || zoom > 8) {
@@ -393,7 +393,8 @@ static void		parse(int argc, char** argv)
 	usage();
       }
       BZDB.set("displayZoom", argv[i]);
-    } else if (strcmp(argv[i], "-zbuffer") == 0) {
+    }
+    else if (strcmp(argv[i], "-zbuffer") == 0) {
       checkArgc(i, argc, argv[i]);
       if (strcmp(argv[i], "on") == 0) {
 	BZDB.set("zbuffer", "1");
@@ -403,83 +404,117 @@ static void		parse(int argc, char** argv)
 	printFatalError("Invalid argument for %s.", argv[i-1]);
 	usage();
       }
-    } else if (strcmp(argv[i], "-eyesep") == 0) {
+    }
+    else if (strcmp(argv[i], "-eyesep") == 0) {
       checkArgc(i, argc, argv[i]);
       BZDB.set("eyesep", argv[i]);
-    } else if (strcmp(argv[i], "-focal") == 0) {
+    }
+    else if (strcmp(argv[i], "-focal") == 0) {
       checkArgc(i, argc, argv[i]);
       BZDB.set("focal", argv[i]);
-    } else if (strncmp(argv[i], "-psn", 4) == 0) {
+    }
+    else if (strncmp(argv[i], "-psn", 4) == 0) {
       std::vector<std::string> args;
       args.push_back(argv[i]);
       printError("Ignoring Finder argument \"{1}\"", &args);
       // ignore process serial number argument (-psn_x_xxxx for MacOS X
-    } else if (strcmp(argv[i], "-badwords") == 0) {
+    }
+    else if (strcmp(argv[i], "-badwords") == 0) {
       checkArgc(i, argc, argv[i], "Missing bad word filter file");
       BZDB.set("filterFilename", argv[i], StateDatabase::ReadOnly);
-    } else if (argv[i][0] != '-') {
-      if (i == argc-1) {
-
+    }
+    else if (argv[i][0] != '-') {
+      if (i == (argc - 1)) {
 	// find the beginning of the server name, parse the callsign
-	char* serverName;
+	char *serverName;
 	if ((serverName = strchr(argv[i], '@')) != NULL) {
-    char* password;
+	  char *password;
 	  *serverName = '\0';
-	  if (strlen(argv[i]) >= sizeof(startupInfo.callsign))
+	  if (strlen(argv[i]) >= sizeof(startupInfo.callsign)) {
 	    printFatalError("Callsign truncated.");
+	  }
 	  strncpy(startupInfo.callsign, argv[i],
 		  sizeof(startupInfo.callsign) - 1);
 	  startupInfo.callsign[sizeof(startupInfo.callsign) - 1] = '\0';
 	  if ((password = strchr(startupInfo.callsign, ':')) != NULL) {
-      *(strchr(startupInfo.callsign, ':')) = '\0';
+	    *(strchr(startupInfo.callsign, ':')) = '\0';
 	    *password = '\0', ++password;
-	    if (strlen(argv[i]) >= sizeof(startupInfo.password))
+	    if (strlen(argv[i]) >= sizeof(startupInfo.password)) {
 	      printFatalError("Password truncated.");
-	    strncpy(startupInfo.password, password, sizeof(startupInfo.password) - 1);
+	    }
+	    strncpy(startupInfo.password, password,
+		    sizeof(startupInfo.password) - 1);
 	    startupInfo.password[sizeof(startupInfo.password) - 1] = '\0';
-    }
+	  }
 	  ++serverName;
-	} else {
+	}
+	else {
 	  serverName = argv[i];
 	}
 
 	// find the beginning of the port number, parse it
-	char* portNumber;
-	if ((portNumber = strchr(serverName, ':')) != NULL) {
+	char *portNumber;
+	if ((portNumber = strchr(serverName, ':')) == NULL) {
+	  startupInfo.serverPort = ServerPort;  // use the default
+	}
+	else {
 	  *portNumber = '\0';
 	  ++portNumber;
 	  startupInfo.serverPort = atoi(portNumber);
 	  if (startupInfo.serverPort < 1 || startupInfo.serverPort > 65535) {
 	    startupInfo.serverPort = ServerPort;
-	    printFatalError("Bad port, using default %d.",
-			    startupInfo.serverPort);
+	    printFatalError("Bad port, using default %d.", startupInfo.serverPort);
 	  }
 	}
 	if (strlen(serverName) >= sizeof(startupInfo.serverName)) {
 	  printFatalError("Server name too long.  Ignoring.");
-	} else {
+	}
+	else {
 	  strcpy(startupInfo.serverName, serverName);
 	  startupInfo.autoConnect = true;
 	}
-      } else {
+      }
+      else {
 	printFatalError("Unexpected: %s. Server must go after all options.", argv[i]);
       }
-    } else {
+    }
+    else if (strcmp(argv[i], "-debug") == 0) {
+      debugLevel++;
+    }
+    // has to be the last option that starts with -d
+    else if (strncmp(argv[i], "-d", 2) == 0) {
+      const char num = argv[i][2];
+      if ((num >= '0') && (num <= '9') && (argv[i][3] == 0)) {
+	debugLevel = num - '0';
+      }
+      else {
+	const char* c = argv[i] + 2;
+	while (*c != 0) {
+	  if (*c != 'd') {
+	    printFatalError("Unknown option %s.", argv[i]);
+	    usage();
+	  }
+	  c++;
+	}
+	debugLevel += (int)((c - argv[i]) - 1);
+      }
+    }
+    else {
       printFatalError("Unknown option %s.", argv[i]);
       usage();
     }
   }
 }
 
-static void		parseConfigName(int argc, char** argv)
+static void parseConfigName(int argc, char** argv)
 {
-	for (int i = 1; i < argc; i++) {
-		if (strcmp(argv[i], "-configdir") == 0) {
-			checkArgc(i, argc, argv[i]);
-			setCustomConfigDir(argv[i]);
-			alternateConfig += argv[i];
-		}
-	}
+  for (int i = 1; i < argc; i++) {
+    if (strcmp(argv[i], "-configdir") == 0) {
+      checkArgc(i, argc, argv[i]);
+      setCustomConfigDir(argv[i]);
+      alternateConfig += argv[i];
+    }
+  }
   for (int i = 1; i < argc; i++) {
     if (strcmp(argv[i], "-config") == 0) {
       checkArgc(i, argc, argv[i]);
@@ -495,7 +530,7 @@ static void		parseConfigName(int argc, char** argv)
 // (so user won't have to wait through performance testing again).
 //
 
-void			dumpResources()
+void dumpResources()
 {
   // collect new configuration
 
@@ -560,7 +595,7 @@ void			dumpResources()
     BZDB.set(TextUtils::format("silencedPerson%d", i), list[i]);
   }
 
-  BZDB.set("email", startupInfo.email); // note email of zero length does not stick
+  BZDB.set("motto", startupInfo.motto);
 
   BZDB.set("serverCacheAge", TextUtils::format("%1d", (long)(ServerListCache::get())->getMaxCacheAge()));
 
@@ -639,6 +674,8 @@ int			main(int argc, char** argv)
   }
 #endif
 
+  initGlobalAres();
+
   WordFilter *filter = (WordFilter *)NULL;
 
   argv0 = argv[0];
@@ -674,8 +711,8 @@ int			main(int argc, char** argv)
 
     // write the registry key in question
     HKEY key = NULL;
-    if (RegCreateKeyEx(HKEY_CURRENT_USER, "Software\\BZFlag", 
-        0, NULL, REG_OPTION_VOLATILE, KEY_ALL_ACCESS, NULL, 
+    if (RegCreateKeyEx(HKEY_CURRENT_USER, "Software\\BZFlag",
+	0, NULL, REG_OPTION_VOLATILE, KEY_ALL_ACCESS, NULL,
 	&key, NULL) == ERROR_SUCCESS) {
       RegSetValueEx(key, "CurrentRunningPath", 0, REG_SZ, (LPBYTE)temppath,
 		   (DWORD)strlen(temppath));
@@ -686,7 +723,7 @@ int			main(int argc, char** argv)
   createCacheSignature();
 
   // initialize global objects and classes
-  bzfsrand(time(0));
+  bzfsrand((unsigned int)time(0));
 
     // set default DB entries
   for (unsigned int gi = 0; gi < numGlobalDBItems; ++gi) {
@@ -698,10 +735,10 @@ int			main(int argc, char** argv)
     BZDB.setPersistent(globalDBItems[gi].name, globalDBItems[gi].persistent);
     BZDB.setPermission(globalDBItems[gi].name, globalDBItems[gi].permission);
   }
-  
+
   BZDBCache::init();
   BZDBLOCAL.init();
-  
+
   Flags::init();
 
   if (getenv("BZFLAGID")) {
@@ -811,7 +848,7 @@ int			main(int argc, char** argv)
     // however, if the "__window" setting is enabled, let it through
     if (BZDB.isSet("__window"))
       if (BZDB.isTrue("__window"))
-        BZDB.set("_window", "1");
+	BZDB.set("_window", "1");
   }
 
   // use UDP? yes
@@ -901,36 +938,10 @@ int			main(int argc, char** argv)
     }
   }
 
-  // get email address if not anonymous
-  std::string email = "default";
-  if (!anonymous) {
-    if (BZDB.isSet("email")) {
-      email = BZDB.get("email");
-    }
-
-    if (email == "default") {
-      email = anonymousName;
-      std::string hostname = Address::getHostName();
-#if defined(_WIN32)
-      char username[256];
-      DWORD usernameLen = sizeof(username);
-      GetUserName(username, &usernameLen);
-#else
-      struct passwd* pwent = getpwuid(getuid());
-      const char* username = pwent ? pwent->pw_name : NULL;
-#endif
-      if (hostname == "") {
-	hostname = "unknown";
-      }
-      if (username) {
-	email = username;
-	email += "@";
-	email += hostname;
-      }
-    }
-  }
-  email = email.substr(0, sizeof(startupInfo.email) - 1);
-  strcpy(startupInfo.email, email.c_str());
+  // use empty motto string unless previously set
+  std::string motto = BZDB.get("motto");
+  motto = motto.substr(0, sizeof(startupInfo.motto) - 1);
+  strcpy(startupInfo.motto, motto.c_str());
 
   // make platform factory
   PlatformFactory* platformFactory = PlatformFactory::getInstance();
@@ -975,14 +986,37 @@ int			main(int argc, char** argv)
 
 #if (defined(_WIN32) || defined(WIN32))
     char exePath[MAX_PATH];
+    char dataPath[MAX_PATH];
     GetModuleFileName(NULL,exePath,MAX_PATH);
     char* last = strrchr(exePath,'\\');
     if (last)
       *last = '\0';
-    strcat(exePath,"\\data");
+
+    strcpy(dataPath,exePath);
+
+    char temp[MAX_PATH];
+    getcwd(temp,MAX_PATH);
+
+    // find the data dir
+    strcat(dataPath,"\\data");
+
+    if (chdir(dataPath) != 0)
+    {
+      strcpy(dataPath,exePath);
+
+      char* last = strrchr(dataPath,'\\');
+      if (last)
+	*last = '\0';
+
+      strcat(dataPath,"\\data");
+    }
+
+    chdir(temp);
     BzfMedia *media = PlatformFactory::getMedia();
     if (media)
-      media->setMediaDirectory(exePath);
+      media->setMediaDirectory(dataPath);
+
+    FileManager::instance().setDataPath(std::string(dataPath));
 #else
     // It's only checking existence of l10n directory
     std::string mediadir = DEFAULT_MEDIA_DIR;
@@ -1053,8 +1087,7 @@ int			main(int argc, char** argv)
   }
 
   // set window size (we do it here because the OpenGL context isn't yet
-  // bound and 3Dfx passthrough cards use the window size to determine
-  // the resolution to use)
+  // bound.)
   const bool useFullscreen = needsFullscreen();
   if (useFullscreen) {
 #ifndef HAVE_SDL
@@ -1192,13 +1225,12 @@ int			main(int argc, char** argv)
   // add the zbuffer callback here, after the OpenGL context is initialized
   BZDB.addCallback("zbuffer", setDepthBuffer, NULL);
 
-  // if we're running on 3Dfx fullscreen add a fake cursor.
-  // let the defaults file override this, though.
+  //add a fake cursor. Let the defaults file override this, though.
   if (!BZDB.isSet("fakecursor")) {
     // check that the glrenderer is Mesa Glide
-    if ((glRenderer != NULL) && (strncmp(glRenderer, "Mesa Glide", 10) == 0 ||
-	strncmp(glRenderer, "3Dfx", 4) == 0))
+    if ((glRenderer != NULL) && (strncmp(glRenderer, "Mesa Glide", 10) == 0)){
       BZDB.set("fakecursor", "1");
+    }
   }
 
   // set gamma if set in resources and we have gamma control
@@ -1330,6 +1362,11 @@ int			main(int argc, char** argv)
   }
 
   // shut down
+
+
+  killGlobalAres();
+  AresHandler::globalShutdown();
+
   if (filter != NULL)
     delete filter;
   filter = NULL;
@@ -1346,10 +1383,10 @@ int			main(int argc, char** argv)
 
 #if defined(_WIN32)
   {
-    /* clear HKEY_CURRENT_USER\Software\BZFlag\CurrentRunningPath if it 
+    /* clear HKEY_CURRENT_USER\Software\BZFlag\CurrentRunningPath if it
      * exists */
     HKEY key = NULL;
-    if (RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\BZFlag", 
+    if (RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\BZFlag",
 	0, KEY_ALL_ACCESS, &key) == ERROR_SUCCESS) {
       RegSetValueEx(key, "CurrentRunningPath", 0, REG_SZ, (LPBYTE)"\0", 1);
     }
