@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,78 +7,176 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
 #include "commands.h"
 
 /* system implementation headers */
-#include <fstream>
+#ifndef _WIN32
+#  include <sys/types.h>
+#  include <dirent.h>
+#endif
 #include <string>
-#include "../zlib/zconf.h"
-#include "../zlib/zlib.h"
+#include <vector>
 
 /* common implementation headers */
-#include "global.h"
-#include "StateDatabase.h"
+#include "BZDBCache.h"
 #include "TextUtils.h"
 #include "FileManager.h"
 #include "DirectoryNames.h"
 #include "version.h"
+#include "SceneRenderer.h"
+#include "bzglob.h"
+#include "BzPNG.h"
 
 /* local implementation headers */
 #include "LocalPlayer.h"
 #include "sound.h"
 #include "ComposeDefaultKey.h"
-#include "Roster.h"
 #include "SilenceDefaultKey.h"
 #include "ServerCommandKey.h"
-
-/* FIXME -- from playing.cxx */
-extern LocalPlayer* myTank;
-extern bool fireButton;
-#include "ServerLink.h"
-extern ServerLink* serverLink;
-void handleFlagDropped(Player* tank);
-void setTarget();
-extern bool gameOver;
-extern float destructCountdown;
+#include "Roaming.h"
+#include "playing.h"
 #include "HUDRenderer.h"
-extern HUDRenderer* hud;
-extern bool pausedByUnmap;
-#include "ControlPanel.h"
-extern ControlPanel* controlPanel;
-extern int savedVolume;
-bool shouldGrabMouse();
-#include "MainWindow.h"
-extern MainWindow* mainWindow;
-extern float pauseCountdown;
-#include "Address.h"
-extern char messageMessage[PlayerIdPLen + MessageLen];
-extern float clockAdjust;
-extern bool roaming;
-extern enum roamingView {
-  roamViewFree = 0,
-  roamViewTrack,
-  roamViewFollow,
-  roamViewFP,
-  roamViewFlag,
-  roamViewCount
-} roamView;
-extern int roamTrackTank, roamTrackWinner, roamTrackFlag;
-extern float roamPos[3], roamDPos[3];
-extern float roamTheta, roamDTheta;
-extern float roamPhi, roamDPhi;
-extern float roamZoom, roamDZoom;
-extern bool roamButton;
-#include "World.h"
-extern World* world;
-void setRoamingLabel(bool force);
-extern void warnAboutMainFlags();
-extern void warnAboutRadarFlags();
-extern void warnAboutRadar();
-extern void warnAboutConsole();
+#include "HUDui.h"
+
+/** jump
+ */
+static std::string cmdJump(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** fire weapon
+ */
+static std::string cmdFire(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** drop a flag
+ */
+static std::string cmdDrop(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** toggle radar visibility
+ */
+static std::string cmdToggleRadar(const std::string&,
+				  const CommandManager::ArgList& args, bool*);
+
+/** toggle console visibility
+ */
+static std::string cmdToggleConsole(const std::string&,
+				    const CommandManager::ArgList& args, bool*);
+
+/** toggle flags
+ */
+static std::string cmdToggleFlags(const std::string&,
+				  const CommandManager::ArgList& args, bool*);
+
+/** identify to a server
+ */
+static std::string cmdIdentify(const std::string&,
+			       const CommandManager::ArgList& args, bool*);
+
+/** restart/respawn
+ */
+static std::string cmdRestart(const std::string&,
+			      const CommandManager::ArgList& args, bool*);
+
+/** self-destruct
+ */
+static std::string cmdDestruct(const std::string&,
+			       const CommandManager::ArgList& args, bool*);
+
+/** pause
+ */
+static std::string cmdPause(const std::string&,
+			    const CommandManager::ArgList& args, bool*);
+
+/** select tabbed message
+ */
+static std::string cmdMessagePanel(const std::string&,
+				   const CommandManager::ArgList& args, bool*);
+
+/** toggle auto-pilot
+ */
+static std::string cmdAutoPilot(const std::string&,
+				const CommandManager::ArgList& args, bool*);
+
+/** change radar scale
+ */
+static std::string cmdRadarZoom(const std::string&,
+				const CommandManager::ArgList& args, bool*);
+
+/** change view angle
+ */
+static std::string cmdViewZoom(const std::string&,
+			       const CommandManager::ArgList& args, bool*);
+
+/** send
+ */
+static std::string cmdSend(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** capture a screenshot
+ */
+static std::string cmdScreenshot(const std::string&,
+				 const CommandManager::ArgList& args, bool*);
+
+/** time
+ */
+static std::string cmdTime(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** roam
+ */
+static std::string cmdRoam(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** silence another player
+ */
+static std::string cmdSilence(const std::string&,
+			      const CommandManager::ArgList& args, bool*);
+
+/** perform a server command
+ */
+static std::string cmdServerCommand(const std::string&,
+				    const CommandManager::ArgList& args,
+				    bool*);
+
+/** scroll the chat panel
+ */
+static std::string cmdScrollPanel(const std::string&,
+				  const CommandManager::ArgList& args, bool*);
+
+/** hunt a player
+ */
+static std::string cmdHunt(const std::string&,
+			   const CommandManager::ArgList& args, bool*);
+
+/** hunt another player
+ */
+static std::string cmdAddHunt(const std::string&,
+			      const CommandManager::ArgList& args, bool*);
+
+/** iconify window
+ */
+static std::string cmdIconify(const std::string&,
+			      const CommandManager::ArgList& args, bool*);
+
+/** mouse box size flags
+ */
+static std::string cmdMouseBox(const std::string&,
+			       const CommandManager::ArgList& args, bool*);
+
+/** toggle mouse capture
+ */
+static std::string cmdMouseGrab(const std::string&,
+				const CommandManager::ArgList& args, bool*);
+
+/** toggle Full Screen
+ */
+static std::string cmdToggleFS(const std::string&,
+			       const CommandManager::ArgList& args, bool*);
 
 
 const struct CommandListItem commandList[] = {
@@ -90,16 +188,17 @@ const struct CommandListItem commandList[] = {
   { "destruct", &cmdDestruct,	"destruct:  self destruct" },
   { "pause",	&cmdPause,	"pause:  pause/resume" },
   { "send",	&cmdSend,	"send {all|team|nemesis|recipient|admin}:  start composing a message" },
-#ifdef SNAPPING
   { "screenshot", &cmdScreenshot, "screenshot:  take a screenshot" },
-#endif
   { "time",	&cmdTime,	"time {forward|backward}:  adjust the current time" },
   { "roam",	&cmdRoam,	"roam {zoom|cycle} <args>:  roam around" },
   { "silence",	&cmdSilence,	"silence:  silence/unsilence a player" },
   { "servercommand",	&cmdServerCommand,	"servercommand:  quick admin" },
   { "scrollpanel",	&cmdScrollPanel,	"scrollpanel {up|down}:  scroll message panel" },
   { "hunt",	&cmdHunt,	"hunt:  hunt a specific player" },
+  { "addhunt",	&cmdAddHunt,	"addhunt:  add/modify hunted player(s)" },
   { "iconify",  &cmdIconify,	"iconify: iconify & pause bzflag" },
+  { "mousebox", &cmdMouseBox, "mousebox <size>:  change the mousebox size"},
+  { "mousegrab", &cmdMouseGrab, "mousegrab: toggle exclusive mouse mode" },
   { "fullscreen", &cmdToggleFS, "fullscreen: toggle fullscreen mode" },
   { "autopilot",&cmdAutoPilot,	"autopilot:  set/unset autopilot bot code" },
   { "radarZoom", &cmdRadarZoom, "radarZoom {in/out}: change maxRadar range"},
@@ -112,8 +211,8 @@ const struct CommandListItem commandList[] = {
 };
 
 
-std::string cmdToggleFS(const std::string&,
-			const CommandManager::ArgList& args)
+static std::string cmdToggleFS(const std::string&,
+			       const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: fullscreen";
@@ -122,25 +221,57 @@ std::string cmdToggleFS(const std::string&,
   return std::string();
 }
 
-std::string cmdIconify(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdMouseBox(const std::string&,
+			       const CommandManager::ArgList& args, bool*)
 {
-  if (args.size() != 0)
-    return "usage: iconify";
-  mainWindow->iconify();
+  if (args.size() != 1) {
+    return "usage: mousebox <size>";
+  }
+  const char* start = args[0].c_str();
+  char* end;
+  const int value = (int) strtol(args[0].c_str(), &end, 10);
+  if (start == end) {
+    return "bad number";
+  }
+  RENDERER.setMaxMotionFactor(value);
   return std::string();
 }
 
-std::string cmdJump(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdMouseGrab(const std::string&,
+				const CommandManager::ArgList& args, bool*)
+{
+  if (args.size() != 0)
+    return "usage: mousegrab";
+  const bool grabbing = !(BZDB.isTrue("mousegrab"));
+  BZDB.set("mousegrab", grabbing ? "true" : "false");
+  mainWindow->enableGrabMouse(grabbing);
+  return std::string();
+}
+
+static std::string cmdIconify(const std::string&,
+			      const CommandManager::ArgList& args, bool*)
+{
+  if (args.size() != 0)
+    return "usage: iconify";
+
+  if (!BZDB.isTrue("Win32NoMin"))
+	 mainWindow->iconify();
+  return std::string();
+}
+
+static std::string cmdJump(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: jump";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank != NULL)
     myTank->setJump();
   return std::string();
 }
 
-std::string cmdToggleFlags (const std::string&, const
-CommandManager::ArgList& args)
+static std::string cmdToggleFlags(const std::string&, const
+				  CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 1)
     return "usage: main|radar";
@@ -157,8 +288,8 @@ CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdToggleRadar (const std::string&, const
-CommandManager::ArgList& args)
+static std::string cmdToggleRadar(const std::string&,
+				  const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0) {
     return "usage: toggleRadar";
@@ -171,8 +302,8 @@ CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdToggleConsole (const std::string&, const
-CommandManager::ArgList& args)
+static std::string cmdToggleConsole(const std::string&,
+				    const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0) {
     return "usage: toggleConsole";
@@ -186,20 +317,24 @@ CommandManager::ArgList& args)
 }
 
 
-std::string cmdFire(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdFire(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: fire";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (fireButton && myTank != NULL && myTank->isAlive()
       && myTank->getTeam() != ObserverTeam)
     myTank->fireShot();
   return std::string();
 }
 
-std::string cmdDrop(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdDrop(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: drop";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank != NULL) {
     FlagType* flag = myTank->getFlag();
     if ((flag != Flags::Null) && !myTank->isPaused() &&
@@ -215,33 +350,43 @@ std::string cmdDrop(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdIdentify(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdIdentify(const std::string&,
+			       const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: identify";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank != NULL)
     if (myTank->isAlive() && !myTank->isPaused())
       setTarget();
   return std::string();
 }
 
-std::string cmdRestart(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdRestart(const std::string&,
+			      const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: restart";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank != NULL)
     if (!gameOver && !myTank->isSpawning() && (myTank->getTeam() != ObserverTeam) && !myTank->isAlive() && !myTank->isExploding()) {
       serverLink->sendAlive();
       myTank->setSpawning(true);
+      CommandManager::ArgList zoomArgs;
+      std::string resetArg = "reset";
+      zoomArgs.push_back(resetArg);
+      cmdViewZoom("", zoomArgs,NULL);
     }
 
   return std::string();
 }
 
-std::string cmdDestruct(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdDestruct(const std::string&,
+			       const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: destruct";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (myTank != NULL) {
     if (destructCountdown > 0.0f) {
       destructCountdown = 0.0f;
@@ -256,11 +401,13 @@ std::string cmdDestruct(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdPause(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdPause(const std::string&,
+			    const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: pause";
 
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (!pausedByUnmap && myTank && myTank->isAlive() && !myTank->isAutoPilot()) {
     if (myTank->isPaused()) {
       // already paused, so unpause
@@ -307,12 +454,23 @@ std::string cmdPause(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdAutoPilot(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdAutoPilot(const std::string&,
+				const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: autopilot";
 
-  if (BZDB.isTrue(StateDatabase::BZDB_DISABLEBOTS)) {
+  // don't enable autopilot until we've fully joined and checked the value
+  // of the server-side _disableBots
+  if (! BZDB.isSet(StateDatabase::BZDB_DISABLEBOTS))
+    return std::string();
+
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
+
+  if (!BZDB.isTrue(StateDatabase::BZDB_TANKWIDTH))
+    return std::string();
+
+  if (BZDB.isTrue(StateDatabase::BZDB_DISABLEBOTS) && ! myTank->isAutoPilot()) {
     hud->setAlert(0, "autopilot not allowed on this server", 1.0f, true);
     return std::string();
   }
@@ -351,8 +509,8 @@ std::string cmdAutoPilot(const std::string&, const CommandManager::ArgList& args
   return std::string();
 }
 
-std::string cmdRadarZoom(const std::string&,
-			 const CommandManager::ArgList& args)
+static std::string cmdRadarZoom(const std::string&,
+				const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 1)
     return "usage: radarZoom {in|out}";
@@ -376,18 +534,19 @@ std::string cmdRadarZoom(const std::string&,
   return std::string();
 }
 
-std::string cmdViewZoom(const std::string&,
-			const CommandManager::ArgList& args)
+static std::string cmdViewZoom(const std::string&,
+			       const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 1)
     return "usage: viewZoom {in|out|toggle}";
 
   float fov = BZDB.eval("displayFOV");
+  float defFov = BZDB.eval("defaultFOV");
 
   if (args[0] == "out") {
     fov += 1.0f;
-    if (fov > 60.0f)
-      fov = 60.0f;
+    if (fov > defFov)
+      fov = defFov;
     BZDB.setFloat("displayFOV", fov);
   } else if (args[0] == "in") {
     fov -= 1.0f;
@@ -395,20 +554,31 @@ std::string cmdViewZoom(const std::string&,
       fov = 15.0f;
     BZDB.setFloat("displayFOV", fov);
   } else if (args[0] == "toggle") {
-    if (fov < 15.5f)
-      fov = 60.0f;
-    else
+    if (fov < 15.5f) {
+      fov = defFov;
+    } else {
       fov = 15.0f;
+    }
+    BZDB.setFloat("displayFOV", fov);
+    // also toggle the observer fov
+    if (ROAM.getZoom() != defFov) {
+      ROAM.setZoom(defFov);
+    } else {
+      ROAM.setZoom(15.0f);
+    }
+  } else if (args[0] == "reset") {
+    fov = defFov;
+    ROAM.setZoom(defFov);
     BZDB.setFloat("displayFOV", fov);
   } else {
-    return "usage: viewZoom {in|out|toggle}";
+    return "usage: viewZoom {in|out|toggle|reset}";
   }
 
   return std::string();
 }
 
-std::string cmdMessagePanel(const std::string&,
-			    const CommandManager::ArgList& args)
+static std::string cmdMessagePanel(const std::string&,
+				   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 1)
     return "usage: messagepanel {all|chat|server|misc}";
@@ -433,11 +603,13 @@ std::string cmdMessagePanel(const std::string&,
   return std::string();
 }
 
-std::string cmdSend(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdSend(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   static ComposeDefaultKey composeKeyHandler;
   if (args.size() != 1)
     return "usage: send {all|team|nemesis|recipient|admin}";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (!myTank)
     return "use send only when connected";
   std::string composePrompt;
@@ -446,9 +618,17 @@ std::string cmdSend(const std::string&, const CommandManager::ArgList& args)
     buf = nboPackUByte(buf, AllPlayers);
     composePrompt = "Send to all: ";
   } else if (args[0] == "team") {
-    void* buf = messageMessage;
-    buf = nboPackUByte(buf, TeamToPlayerId(myTank->getTeam()));
-    composePrompt = "Send to teammates: ";
+    if (World::getWorld()->allowTeams())
+    {
+      void* buf = messageMessage;
+      buf = nboPackUByte(buf, TeamToPlayerId(myTank->getTeam()));
+      composePrompt = "Send to teammates: ";
+    }
+    else {
+      void* buf = messageMessage;
+      buf = nboPackUByte(buf, AllPlayers);
+      composePrompt = "Send to all: ";
+    }
   } else if (args[0] == "nemesis") {
     const Player* nemesis = myTank->getNemesis();
     if (!nemesis) return std::string();
@@ -462,8 +642,8 @@ std::string cmdSend(const std::string&, const CommandManager::ArgList& args)
     const Player* recipient = myTank->getRecipient();
     if (!recipient) {
       for (int i = 0; i < curMaxPlayers; i++) {
-	if (player[i]) {
-	  myTank->setRecipient(player[i]);
+	if (remotePlayers[i]) {
+	  myTank->setRecipient(remotePlayers[i]);
 	  break;
 	}
       }
@@ -490,144 +670,163 @@ std::string cmdSend(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-#ifdef SNAPPING
-std::string cmdScreenshot(const std::string&, const CommandManager::ArgList& args)
+
+struct ScreenshotData
 {
-  static int snap = 0;
-  if (args.size() != 0)
-    return "usage: screenshot";
+  std::string renderer;
+  unsigned char* pixels;
+  int xsize;
+  int ysize;
+  int channels;
+};
 
-  std::string filename = getScreenShotDirName();
-  filename += TextUtils::format("bzfi%04d.png", snap++);
+#ifdef _WIN32
+static DWORD WINAPI writeScreenshot(void* data)
+#else
+static void* writeScreenshot(void* data)
+#endif
+{
+  ScreenshotData* ssdata = (ScreenshotData*)data;
 
-  std::ostream* f = FILEMGR.createDataOutStream (filename.c_str(), true, true);
+  const std::string dirname = getScreenShotDirName();
+  const std::string prefix  = "bzfi";
+  const std::string ext     = ".png";
 
-  if (f != NULL) {
-    int w = mainWindow->getWidth(), h = mainWindow->getHeight();
-    const unsigned long blength = h * w * 3 + h;    //size of b[] and br[]
-    unsigned char* b = new unsigned char[blength];  //screen of pixels + column for filter type required by PNG - filtered data
+  // scan the directory and start numbering with the filename
+  // that follows the existing filename with the highest snap number
+  int snap = 0;
 
-    //Prepare gamma table
-    const bool gammaAdjust = BZDB.isSet("gamma");
-    unsigned char gammaTable[256];
-    if (gammaAdjust) {
-      float gamma = (float) atof(BZDB.get("gamma").c_str());
-      for (int i = 0; i < 256; i++) {
-	float lum = ((float) i) / 256.0f;
-	float lumadj = pow(lum, 1.0f / gamma);
-	gammaTable[i] = (unsigned char) (lumadj * 256);
+#ifdef _WIN32
+  const std::string pattern = dirname + prefix + "*" + ext;
+  WIN32_FIND_DATA findData;
+  HANDLE h = FindFirstFile(pattern.c_str(), &findData);
+  if (h != INVALID_HANDLE_VALUE) {
+    std::string file = findData.cFileName;
+    snap = atoi((file.substr(file.length() - 8, 4)).c_str());
+    while (FindNextFile(h, &findData)) {
+      file = findData.cFileName;
+      const int number = atoi((file.substr(file.length() - 8, 4)).c_str());
+      if (snap < number) {
+	snap = number;
       }
     }
-
-    /* Write a PNG stream.
-       FIXME: Note that there are several issues with this code, altho it produces perfectly fine PNGs.
-       1. We do no filtering.  Sub filters increase screenshot size, other filters would require a lot of effort to implement.
-       2. Gamma-correction is preapplied by BZFlag's gamma table.  This ignores the PNG gAMA chunk, but so do many viewers (including Mozilla)
-       3. Timestamp (tIME) chunk is not added to the file, but would be a good idea.
-    */
-    int temp = 0; //temporary values for binary file writing
-    char tempByte = 0;
-    int crc = 0;  //used for running CRC values
-
-    // Write PNG headers
-    (*f) << "\211PNG\r\n\032\n";
-#define PNGTAG(t_) ((((int)t_[0]) << 24) | \
-		   (((int)t_[1]) << 16) | \
-		   (((int)t_[2]) <<  8) | \
-		   (int)t_[3])
-
-    // IHDR chunk
-    temp = htonl((int) 13);       //(length) IHDR is always 13 bytes long
-    f->write((char*) &temp, 4);
-    temp = htonl(PNGTAG("IHDR")); //(tag) IHDR
-    f->write((char*) &temp, 4);
-    crc = crc32(crc, (unsigned char*) &temp, 4);
-    temp = htonl(w);	      //(data) Image width
-    f->write((char*) &temp, 4);
-    crc = crc32(crc, (unsigned char*) &temp, 4);
-    temp = htonl(h);	      //(data) Image height
-    f->write((char*) &temp, 4);
-    crc = crc32(crc, (unsigned char*) &temp, 4);
-    tempByte = 8;		 //(data) Image bitdepth (8 bits/sample = 24 bits/pixel)
-    f->write(&tempByte, 1);
-    crc = crc32(crc, (unsigned char*) &tempByte, 1);
-    tempByte = 2;		 //(data) Color type: RGB = 2
-    f->write(&tempByte, 1);
-    crc = crc32(crc, (unsigned char*) &tempByte, 1);
-    tempByte = 0;
-    int i;
-    for (i = 0; i < 3; i++) { //(data) Last three tags are compression (only 0 allowed), filtering (only 0 allowed), and interlacing (we don't use it, so it's 0)
-      f->write(&tempByte, 1);
-      crc = crc32(crc, (unsigned char*) &tempByte, 1);
+  }
+#else
+  const std::string pattern = prefix + "*" + ext;
+  DIR* directory = opendir(dirname.c_str());
+  if (directory) {
+    struct dirent* contents;
+    std::string file;
+    while ((contents = readdir(directory))) {
+      file = contents->d_name;
+      if (glob_match(pattern, file)) {
+	const int number = atoi((file.substr(file.length() - 8, 4)).c_str());
+	if (snap < number) {
+	  snap = number;
+	}
+      }
     }
-    crc = htonl(crc);
-    f->write((char*) &crc, 4);    //(crc) write crc
+    closedir(directory);
+  }
+#endif // _WIN32
 
-    // IDAT chunk
-    for (i = h - 1; i >= 0; i--) {
-      const unsigned long line = (h - (i + 1)) * (w * 3 + 1); //beginning of this line
-      b[line] = 0;  //filter type byte at the beginning of each scanline (0 = no filter, 1 = sub filter)
-      glReadPixels(0, i, w, 1, GL_RGB, GL_UNSIGNED_BYTE, b + line + 1); //capture line
-      // apply gamma correction if necessary
-      if (gammaAdjust) {
-	unsigned char *ptr = b + line + 1;
-	for (int i = 1; i < w * 3 + 1; i++) {
-	  *ptr = gammaTable[*ptr];
-	  ptr++;
+  snap++;
+  std::string filename = dirname + prefix + TextUtils::format("%04d", snap) + ext;
+
+  std::ostream* f = FILEMGR.createDataOutStream(filename.c_str(), true, true);
+
+  if (f != NULL) {
+    delete(f);
+
+    const std::string& renderer = ssdata->renderer;
+    unsigned char* pixels       = ssdata->pixels;
+    const int xsize	     = ssdata->xsize;
+    const int ysize	     = ssdata->ysize;
+    const int channels	  = ssdata->channels;
+
+    // Gamma-correction is preapplied by BZFlag's gamma table
+    // This ignores the PNG gAMA chunk, but so do many viewers (including Mozilla)
+    if (BZDB.isSet("gamma")) {
+      const float gamma = BZDB.eval("gamma");
+      if (gamma != 1.0f) {
+	unsigned char gammaTable[256];
+	for (int i = 0; i < 256; i++) {
+	  const float lum    = float(i) / 256.0f;
+	  const float lumadj = pow(lum, 1.0f / gamma);
+	  gammaTable[i] = (unsigned char) (lumadj * 256);
+	}
+	const int pixelCount = (xsize * ysize * channels);
+	for (int i = 0; i < pixelCount; i++) {
+	  pixels[i] = gammaTable[pixels[i]];
 	}
       }
     }
 
-    unsigned long zlength = blength + 15;	    //length of bz[], will be changed by zlib to the length of the compressed string contained therein
-    unsigned char* bz = new unsigned char[zlength]; //just like b, but compressed; might get bigger, so give it room
-    // compress b into bz
-    compress2(bz, &zlength, b, blength, 5);
-    temp = htonl(zlength);			  //(length) IDAT length after compression
-    f->write((char*) &temp, 4);
-    temp = htonl(PNGTAG("IDAT"));		   //(tag) IDAT
-    f->write((char*) &temp, 4);
-    crc = crc32(crc = 0, (unsigned char*) &temp, 4);
-    f->write(reinterpret_cast<char*>(bz), zlength);  //(data) This line of pixels, compressed
-    crc = htonl(crc32(crc, bz, zlength));
-    f->write((char*) &crc, 4);		       //(crc) write crc
+    const std::string versionStr = std::string("BZFlag") + getAppVersion();
+    std::vector<BzPNG::Chunk> chunks;
+    chunks.push_back(BzPNG::Chunk("tEXt", "Software", versionStr));
+    chunks.push_back(BzPNG::Chunk("tEXt", "GL Renderer", renderer));
 
-    // tEXt chunk containing bzflag build/version
-    temp = htonl((int) 9 + strlen(getAppVersion()));//(length) tEXt is 9 + strlen(getAppVersion())
-    f->write((char*) &temp, 4);
-    temp = htonl(PNGTAG("tEXt"));		   //(tag) tEXt
-    f->write((char*) &temp, 4);
-    crc = crc32(crc = 0, (unsigned char*) &temp, 4);
-    strcpy(reinterpret_cast<char*>(b), "Software"); //(data) Keyword
-    f->write(reinterpret_cast<char*>(b), strlen(reinterpret_cast<const char*>(b)));
-    crc = crc32(crc, b, strlen(reinterpret_cast<const char*>(b)));
-    tempByte = 0;				    //(data) Null character separator
-    f->write(&tempByte, 1);
-    crc = crc32(crc, (unsigned char*) &tempByte, 1);
-    strcpy((char*) b, getAppVersion());	     //(data) Text contents (build/version)
-    f->write(reinterpret_cast<char*>(b), strlen(reinterpret_cast<const char*>(b)));
-    crc = htonl(crc32(crc, b, strlen(reinterpret_cast<const char*>(b))));
-    f->write((char*) &crc, 4);		       //(crc) write crc
-
-    // IEND chunk
-    temp = htonl((int) 0);	//(length) IEND is always 0 bytes long
-    f->write((char*) &temp, 4);
-    temp = htonl(PNGTAG("IEND")); //(tag) IEND
-    f->write((char*) &temp, 4);
-    crc = htonl(crc32(crc = 0, (unsigned char*) &temp, 4));
-    //(data) IEND has no data field
-    f->write((char*) &crc, 4);     //(crc) write crc
-    delete [] bz;
-    delete [] b;
-    delete f;
-    char notify[128];
-    snprintf(notify, 128, "%s: %dx%d", filename.c_str(), w, h);
-    controlPanel->addMessage(notify);
+    char buf[128];
+    if (BzPNG::save(filename, chunks, xsize, ysize, channels, pixels)) {
+      snprintf(buf, sizeof(buf), "%s: %dx%d", filename.c_str(), xsize, ysize);
+    } else {
+      snprintf(buf, sizeof(buf), "%s: failed to save", filename.c_str());
+    }
+    ControlPanel::addMutexMessage(buf);
   }
-  return std::string();
+
+  delete[] ssdata->pixels;
+  delete ssdata;
+
+  return NULL;
 }
+
+static std::string cmdScreenshot(const std::string&, const CommandManager::ArgList& args, bool*)
+{
+  if (args.size() != 0) {
+    return "usage: screenshot";
+  }
+
+  ScreenshotData* ssdata = new ScreenshotData;
+  ssdata->renderer += (const char*)(glGetString(GL_VENDOR));
+  ssdata->renderer += ": ";
+  ssdata->renderer += (const char*)(glGetString(GL_RENDERER));
+  ssdata->renderer += " (OpenGL ";
+  ssdata->renderer += (const char*)(glGetString(GL_VERSION));
+  ssdata->renderer += ")";
+  int w = mainWindow->getWidth();
+  int h = mainWindow->getHeight();
+  ssdata->xsize = w;
+  ssdata->ysize = h;
+  ssdata->channels = 3; // GL_RGB
+  ssdata->pixels = new unsigned char[h * w * 3];
+  glPushClientAttrib(GL_CLIENT_PIXEL_STORE_BIT);
+  glPixelStorei(GL_PACK_ALIGNMENT, 1);
+  glReadPixels(0, 0, w, h, GL_RGB, GL_UNSIGNED_BYTE, ssdata->pixels);
+  glPopClientAttrib();
+
+#if defined(HAVE_PTHREADS)
+  pthread_t thread;
+  pthread_create(&thread, NULL, writeScreenshot, (void *) ssdata);
+#elif defined(_WIN32)
+  CreateThread(
+	    NULL, // Security attributes
+	    0, // Stack size (0 -> default)
+	    writeScreenshot,
+	    ssdata,
+	    0, // creation flags (0 -> run immediately)
+	    NULL); // thread id return value (NULL -> don't care)
+#else
+  // no threads?  sucks to be you, but we'll still write the screenshot
+  writeScreenshot(ssdata);
 #endif
 
-std::string cmdTime(const std::string&, const CommandManager::ArgList& args)
+  return std::string();
+}
+
+static std::string cmdTime(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   // FIXME - time should be moved into BZDB
   if (args.size() != 1)
@@ -642,11 +841,12 @@ std::string cmdTime(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdRoam(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdRoam(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() == 0)
     return "usage: roam {zoom|cycle} <args>";
-  if (!roaming)
+  if (!ROAM.isRoaming())
     return std::string();
   if (args[0] == "zoom") {
     if (args.size() != 2)
@@ -654,11 +854,11 @@ std::string cmdRoam(const std::string&, const CommandManager::ArgList& args)
     if (!roamButton || args[1] == "stop") {
       roamDZoom = 0.0f;
     } else if (args[1] == "out") {
-      roamDZoom = 50.0f;
+      roamDZoom = +2.0f * BZDBCache::tankSpeed;
     } else if (args[1] == "in") {
-      roamDZoom = -50.0f;
+      roamDZoom = -2.0f * BZDBCache::tankSpeed;
     } else if (args[1] == "normal") {
-      roamZoom = 60.0f;
+      ROAM.setZoom(60.0f);
     } else {
       return "usage: roam zoom {in|out|normal|stop}";
     }
@@ -667,88 +867,19 @@ std::string cmdRoam(const std::string&, const CommandManager::ArgList& args)
       return "usage: roam cycle {type|subject} {forward|backward}";
     if (args[1] == "type") {
       if (args[2] == "forward") {
-	roamView = roamingView((roamView + 1) % roamViewCount);
-	if (roamView == roamViewFlag) {
-	  const int maxFlags = world->getMaxFlags();
-	  bool found = false;
-	  for (int i = 0; i < maxFlags; i++) {
-	    const Flag& flag = world->getFlag(i);
-	    if (flag.type->flagTeam != NoTeam) {
-	      roamTrackFlag = i;
-	      found = true;
-	      break;
-	    }
-	  }
-	  if (!found)
-	    roamView = roamViewFree;
-	} else if ((roamTrackTank != -1) && (roamView == roamViewTrack || roamView == roamViewFollow || roamView == roamViewFP)) {
-	  if ((player[roamTrackTank] != NULL) && (!player[roamTrackTank]->isAlive())) {
-	    bool found = false;
-	    for (int i = 0; i < curMaxPlayers; i++) {
-	      if (player[i] && player[i]->isAlive()) {
-		roamTrackTank = roamTrackWinner = i;
-		found = true;
-		break;
-	      }
-	    }
-	    if (!found)
-	      roamTrackTank = -1;
-	  }
-	}
-	setRoamingLabel(true);
+	ROAM.setMode(Roaming::RoamingView((ROAM.getMode() + 1) % Roaming::roamViewCount));
       } else if (args[2] == "backward") {
-	// FIXME
+	int setto = (ROAM.getMode() - 1) % Roaming::roamViewCount;
+	if (setto < 0) setto += Roaming::roamViewCount;
+	ROAM.setMode(Roaming::RoamingView(setto));
       } else {
 	return "usage: roam cycle {type|subject} {forward|backward}";
       }
     } else if (args[1] == "subject") {
       if (args[2] == "forward") {
-	if (roamView == roamViewFree) {
-	  // do nothing
-	} else if (roamView == roamViewFlag) {
-	  const int maxFlags = world->getMaxFlags();
-	  for (int i = 1; i < maxFlags; i++) {
-	    int j = (roamTrackFlag + i) % maxFlags;
-	    const Flag& flag = world->getFlag(j);
-	    if (flag.type->flagTeam != NoTeam) {
-	      roamTrackFlag = j;
-	      break;
-	    }
-	  }
-	} else {
-	  int i, j;
-	  for (i = 0; i < curMaxPlayers; i++) {
-	    j = (roamTrackTank + i + 2) % (curMaxPlayers + 1) - 1;
-	    if ((j == -1) || (player[j] && player[j]->isAlive())) {
-	      roamTrackTank = roamTrackWinner = j;
-	      break;
-	    }
-	  }
-	}
-	setRoamingLabel(true);
+	ROAM.changeTarget(Roaming::next);
       } else if (args[2] == "backward") {
-	if (roamView == roamViewFree) {
-	  // do nothing
-	} else if (roamView == roamViewFlag) {
-	  const int maxFlags = world->getMaxFlags();
-	  for (int i = 1; i < maxFlags; i++) {
-	    int j = (roamTrackFlag - i + maxFlags) % maxFlags;
-	    const Flag& flag = world->getFlag(j);
-	    if (flag.type->flagTeam != NoTeam) {
-	      roamTrackFlag = j;
-	      break;
-	    }
-	  }
-	} else {
-	  for (int i = 0; i < curMaxPlayers; i++) {
-	    int j = (roamTrackTank - i + curMaxPlayers + 1) % (curMaxPlayers + 1) - 1;
-	    if ((j == -1) || (player[j] && player[j]->isAlive())) {
-	      roamTrackTank = roamTrackWinner = j;
-	      break;
-	    }
-	  }
-	}
-	setRoamingLabel(true);
+	ROAM.changeTarget(Roaming::previous);
       } else {
 	return "usage: roam cycle {type|subject} {forward|backward}";
       }
@@ -761,7 +892,8 @@ std::string cmdRoam(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdSilence(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdSilence(const std::string&,
+			      const CommandManager::ArgList& args, bool*)
 {
   static SilenceDefaultKey silenceKeyHandler;
   if (args.size() != 0)
@@ -772,11 +904,13 @@ std::string cmdSilence(const std::string&, const CommandManager::ArgList& args)
   return std::string();
 }
 
-std::string cmdServerCommand(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdServerCommand(const std::string&,
+				    const CommandManager::ArgList& args, bool*)
 {
   static ServerCommandKey serverCommandKeyHandler;
   if (args.size() != 0)
     return "usage: servercommand";
+  LocalPlayer *myTank = LocalPlayer::getMyTank();
   if (!myTank)
     return "use only when connected";
   static bool prevAdmin = myTank->isAdmin();
@@ -790,33 +924,51 @@ std::string cmdServerCommand(const std::string&, const CommandManager::ArgList&
   return std::string();
 }
 
-std::string cmdScrollPanel(const std::string&, const CommandManager::ArgList& args)
+static std::string cmdScrollPanel(const std::string&,
+				  const CommandManager::ArgList& args, bool*)
 {
-  if (args.size() != 1)
-    return "usage: scrollpanel {up|down}\n";
+  if ((args.size() < 1) || (args.size() > 2)) {
+    return "usage: scrollpanel {up|up_page|down|down_page|top|bottom} [count]\n";
+  }
+  int count = 1;
+  int linecount = 2;
+  if (args.size() == 2) {
+    count = atoi(args[1].c_str());
+    linecount = count;
+  }
+  // whence - (0 = set, 1 = cur, 2 = end)
   if (args[0] == "up") {
-    controlPanel->setMessagesOffset(2,1);
+    controlPanel->setMessagesOffset(+linecount, 1 /* current */, false);
   } else if (args[0] == "down") {
-    controlPanel->setMessagesOffset(-2,1);
-  } else {
-    return "usage: scrollpanel {up|down}\n";
+    controlPanel->setMessagesOffset(-linecount, 1 /* current */, false);
+  } else if (args[0] == "up_page") {
+    controlPanel->setMessagesOffset(+count, 1 /* current */, true);
+  } else if (args[0] == "down_page") {
+    controlPanel->setMessagesOffset(-count, 1 /* current */, true);
+  } else if (args[0] == "top") {
+    controlPanel->setMessagesOffset(123456789, 0 /* set */, false);
+  } else if (args[0] == "bottom") {
+    controlPanel->setMessagesOffset(0, 0 /* set */, false);
   }
   return std::string();
 }
 
-std::string cmdHunt(const std::string&, const CommandManager::ArgList& args)
+
+static std::string cmdHunt(const std::string&,
+			   const CommandManager::ArgList& args, bool*)
 {
   if (args.size() != 0)
     return "usage: hunt";
-  if (hud->getHunting()) {
-    hud->setHunting(false);
-  } else {
-    playLocalSound(SFX_HUNT);
-    hud->setHunt(!hud->getHunt());
-    hud->setHuntPosition(0);
-    if (!BZDB.isTrue("displayScore"))
-      BZDB.set("displayScore", "1");
-  }
+   hud->getScoreboard()->huntKeyEvent (false);
+  return std::string();
+}
+
+static std::string cmdAddHunt(const std::string&,
+			      const CommandManager::ArgList& args, bool*)
+{
+  if (args.size() != 0)
+    return "usage: addhunt";
+   hud->getScoreboard()->huntKeyEvent (true);
   return std::string();
 }
 
