@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993-2010 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -28,11 +28,7 @@ typedef struct CommandListItem {
 
 /* FIXME -- gcc is not liking array type with sizeof() without size
  * for some reason */
-#ifdef SNAPPING
-extern const CmdListItem commandList[26];
-#else
-extern const CmdListItem commandList[25];
-#endif
+extern const CmdListItem commandList[27];
 
 #endif /* __COMMANDS_H__ */
 
