@@ -1,14 +1,14 @@
 /* bzflag
-* Copyright (c) 1993-2010 Tim Riker
-*
-* This package is free software;  you can redistribute it and/or
-* modify it under the terms of the license found in the file
-* named COPYING that should have accompanied this file.
-*
-* THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
-* IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
-* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-*/
+ * Copyright (c) 1993-2011 Tim Riker
+ *
+ * This package is free software;  you can redistribute it and/or
+ * modify it under the terms of the license found in the file
+ * named COPYING that should have accompanied this file.
+ *
+ * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ */
 
 
 // BZFlag common header
@@ -26,7 +26,7 @@ DefaultDBItem	defaultDBItems[] = {
   { "saveSettings",		"1",			true,	StateDatabase::ReadWrite,	NULL },
   { "udpnet",			"1",			true,	StateDatabase::ReadWrite,	NULL },
   { "timedate",			"0",			true,	StateDatabase::ReadWrite,	NULL },
-  { "email",			"default",		true,	StateDatabase::ReadWrite,	NULL },
+  { "motto",			"",			true,	StateDatabase::ReadWrite,	NULL },
   { "team",			"Rogue",		true,	StateDatabase::ReadWrite,	NULL },
   { "list",			DefaultListServerURL,	true,	StateDatabase::ReadWrite,	NULL },
   { "motdServer",		DefaultMOTDServer,	true,	StateDatabase::ReadWrite,	NULL },
@@ -81,6 +81,7 @@ DefaultDBItem	defaultDBItems[] = {
   { "userMirror",		"1",			true,	StateDatabase::ReadWrite,	NULL },
   { "showTreads",		"0",			true,	StateDatabase::ReadWrite,	NULL },
   { "animatedTreads",		"1",			true,	StateDatabase::ReadWrite,	NULL },
+  { "shotLength",		"1",			true,	StateDatabase::ReadWrite,	NULL },
   { "treadStyle",		"1",			true,	StateDatabase::ReadWrite,	NULL },
   { "userTrackFade",		"1.0",			true,	StateDatabase::ReadWrite,	NULL },
   { "trackMarkCulling",		"3",			true,	StateDatabase::ReadWrite,	NULL },
@@ -106,6 +107,14 @@ DefaultDBItem	defaultDBItems[] = {
   { "shadowAlpha",		"0.5",			true,	StateDatabase::ReadWrite,	NULL },
   { "aniso",			"1",			true,	StateDatabase::ReadWrite,	NULL },
 
+  // roam smooth follow settings
+  { "followDist",		"32.0",			true,	StateDatabase::ReadWrite,	NULL },
+  { "followHeight",		"8.0",			true,	StateDatabase::ReadWrite,	NULL },
+  { "followOffsetZ",		"1.0",			true,	StateDatabase::ReadWrite,	NULL },
+  { "followSpeedX",		"2.5",			true,	StateDatabase::ReadWrite,	NULL },
+  { "followSpeedY",		"1.0",			true,	StateDatabase::ReadWrite,	NULL },
+  { "followSpeedZ",		"1.0",			true,	StateDatabase::ReadWrite,	NULL },
+
   // hidden graphics rendering params
   { "useDrawInfo",		"1",			true,	StateDatabase::ReadWrite,	NULL },
   { "f2bsort",			"1",			true,	StateDatabase::ReadWrite,	NULL },
@@ -157,7 +166,7 @@ DefaultDBItem	defaultDBItems[] = {
   { "deathEffect",		"1",			true,	StateDatabase::ReadWrite,	NULL },
   { "useVelOnShotEffects",	"1",			true,	StateDatabase::ReadWrite,	NULL },
   { "landEffect",		"1",			true,	StateDatabase::ReadWrite,	NULL },
-  { "gmPuffEffect",		"2",			true,	StateDatabase::ReadWrite,	NULL },
+  { "gmPuffEffect",		"3",			true,	StateDatabase::ReadWrite,	NULL },
   { "gmPuffTime",		"1/8",			true,	StateDatabase::ReadWrite,	NULL },
   { "ricoEffect",		"1",			true,	StateDatabase::ReadWrite,	NULL },
   { "tpEffect",			"1",			true,	StateDatabase::ReadWrite,	NULL },
@@ -167,12 +176,31 @@ DefaultDBItem	defaultDBItems[] = {
   { "httpTimeout",		"15",			true,	StateDatabase::ReadWrite,	NULL },
 
   // hud drawing
-  { "hudGUIBorderOpacityFactor","0.75",			true,	StateDatabase::ReadWrite,	NULL },
-  { "hideEmails",		"0",			true,	StateDatabase::ReadWrite,	NULL },
-  { "emailDispLen",		"32",			true,	StateDatabase::ReadWrite,	NULL },
+  { "hudGUIBorderOpacityFactor",	"0.75",		true,  StateDatabase::ReadWrite, NULL },
+  { "hudWayPMarkerSize",		"15",		true,  StateDatabase::ReadWrite, NULL },
 
-  // last screenshot
-  { "lastScreenshot",		"0",			true,	StateDatabase::ReadWrite,	NULL }
+
+  // hud drawing
+  { "hudGUIBorderOpacityFactor","0.75",			true,	StateDatabase::ReadWrite,	NULL },
+  { "hideMottos",		"0",			true,	StateDatabase::ReadWrite,	NULL },
+  { "mottoDispLen",		"32",			true,	StateDatabase::ReadWrite,	NULL },
+  { "scoreboardSort",	"0",			true,	StateDatabase::ReadWrite,	NULL },
+
+  // listFilters
+  { "listFilter",  "",					true, StateDatabase::ReadWrite, NULL },
+  { "listFilter0", "",					true, StateDatabase::ReadWrite, NULL },
+  { "listFilter1", "/p>1, # busy",			    true, StateDatabase::ReadWrite, NULL },
+  { "listFilter2", "/p>1,s>1,s<4, # busy - 2 or 3 shots",     true, StateDatabase::ReadWrite, NULL },
+  { "listFilter3", "/p>1,+rabbit, # busy rabbit",	     true, StateDatabase::ReadWrite, NULL },
+  { "listFilter4", "/-j,+r,-f,s=2,+t,vt=2, # ducati",	 true, StateDatabase::ReadWrite, NULL },
+  { "listFilter5", "/vt=3,+t,Rm<1, # three-way",	      true, StateDatabase::ReadWrite, NULL },
+  { "listFilter6", "/d)replay, # replay servers",	     true, StateDatabase::ReadWrite, NULL },
+  { "listFilter7", "/a]\\.bz(:[0-9]+)?$, # .bz servers",      true, StateDatabase::ReadWrite, NULL },
+  { "listFilter8", "/d)louman/d)ahs3/d)spazzy, # fancy maps", true, StateDatabase::ReadWrite, NULL },
+  { "listFilter9", "/d]louman|ahs3|spazzy, # fancy maps2",    true, StateDatabase::ReadWrite, NULL },
+
+  // We don't want to keep the geometry settings
+  { "geometry",		"",			false,	StateDatabase::ReadWrite, NULL }
 };
 
 
