@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /*
@@ -24,14 +24,15 @@
 #include <vector>
 
 /* common headers */
-#include "BzfDisplay.h"
-#include "Player.h"
-#include "SceneRenderer.h"
 #include "StartupInfo.h"
+#include "AutoCompleter.h"
 
 /* local headers */
 #include "MainWindow.h"
+#include "ControlPanel.h"
+#include "HUDRenderer.h"
 
+#include "Protocol.h"
 
 #define MAX_MESSAGE_HISTORY (20)
 
@@ -65,13 +66,16 @@ bool			addExplosion(const float* pos,
 				float size, float duration);
 void			addTankExplosion(const float* pos);
 void			addShotExplosion(const float* pos);
-void			addShotPuff(const float* pos);
+void			addShotPuff(const float* pos, float azimuth, float elevation);
 void			warnAboutMainFlags();
 void			warnAboutRadarFlags();
+void		    warnAboutRadar();
+void		    warnAboutConsole();
 void			addPlayingCallback(PlayingCallback, void* data);
 void			removePlayingCallback(PlayingCallback, void* data);
 
 void			joinGame(JoinGameCallback, void* userData);
+void		    leaveGame();
 std::vector<std::string>& getSilenceList();
 void			updateEvents();
 void			addMessage(const Player* player,
@@ -80,18 +84,54 @@ void			addMessage(const Player* player,
 				   bool highlight = false,
 				   const char* oldColor = NULL);
 
+int			curlProgressFunc(void* clientp,
+					 double dltotal, double dlnow,
+					 double ultotal, double ulnow);
+
 void selectNextRecipient (bool forward, bool robotIn);
 void handleFlagDropped(Player* tank);
 void setTarget();
 bool shouldGrabMouse();
-void setRoamingLabel(bool force);
+void setRoamingLabel();
+void drawFrame(const float dt);
+
+void initGlobalAres();
+void killGlobalAres();
+
+extern void joinGame();
 
 extern HUDRenderer	*hud;
 extern char		messageMessage[PlayerIdPLen + MessageLen];
 extern ServerLink*	serverLink;
 extern int		numFlags;
 extern StartupInfo	startupInfo;
+extern DefaultCompleter completer;
+extern bool	     gameOver;
+extern ControlPanel    *controlPanel;
+extern bool	     fireButton;
+extern float	    destructCountdown;
+extern bool	     pausedByUnmap;
+extern int	      savedVolume;
+extern MainWindow      *mainWindow;
+extern float	    pauseCountdown;
+extern float	    clockAdjust;
+extern float	    roamDZoom;
+extern bool	     roamButton;
 
+#define MAX_DT_LIMIT 0.1f
+#define MIN_DT_LIMIT 0.001f
+
+enum BlowedUpReason {
+	GotKilledMsg,
+	GotShot,
+	GotRunOver,
+	GotCaptured,
+	GenocideEffect,
+	SelfDestruct,
+	WaterDeath,
+	LastReason,
+	DeathTouch = PhysicsDriverDeath
+};
 
 #endif // BZF_PLAYING_H
 
