@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -198,39 +198,15 @@ bool AccessControlList::validate(const in_addr &ipAddr, BanInfo *info)
 }
 
 
-bool AccessControlList::does_match(const char *targ, int targlen, const char *pat, int patlen)
-{
-  if (!targlen)
-    return patlen == 0;
-  if (!patlen)
-    return targlen == 0;
-
-  while (*pat != '*') {
-    if (*pat != *targ)
-      return false;
-
-    pat++; patlen--;
-    targ++; targlen--;
-    if (!targlen)
-      return patlen == 0;
-    if (!patlen)
-      return targlen == 0;
-  }
-
-  // found a *, search for matches in the rest of the string
-  for (int pos = 0; pos <= targlen; pos++)
-    if (does_match(targ+pos, targlen-pos, pat+1, patlen-1))
-      return true;
-  return false;
-}
-
-
 bool AccessControlList::hostValidate(const char *hostname, HostBanInfo *info)
 {
   expire();
 
+  const std::string upperHost = TextUtils::toupper(hostname);
+
   for (hostBanList_t::iterator it = hostBanList.begin(); it != hostBanList.end(); ++it) {
-    if (does_match(hostname, strlen(hostname), it->hostpat.c_str(), it->hostpat.length())) {
+    const std::string upperPattern = TextUtils::toupper(it->hostpat);
+    if (glob_match(upperPattern, upperHost)) {
       if (info)
 	*info = *it;
       return false;
@@ -278,7 +254,7 @@ static std::string makeGlobPattern(const char* str)
 }
 
 
-static std::string getBanMaskString(in_addr mask)
+std::string AccessControlList::getBanMaskString(in_addr mask)
 {
   std::ostringstream os;
   os << (ntohl(mask.s_addr) >> 24) << '.';
@@ -376,23 +352,30 @@ void AccessControlList::sendHostBans(PlayerId id, const char* pattern)
     }
 
     char *pMsg = banlistmessage;
-    sprintf(pMsg, "%s", bi.hostpat.c_str());
+    snprintf(pMsg, MessageLen, "%s", bi.hostpat.c_str());
 
     // print duration when < 1 year
     double duration = bi.banEnd - TimeKeeper::getCurrent();
-    if (duration < 365.0f * 24 * 3600)
-      sprintf(pMsg + strlen(pMsg)," (%.1f minutes)", duration / 60);
-    if (bi.bannedBy.length())
-      sprintf(pMsg + strlen(pMsg), " banned by: %s", bi.bannedBy.c_str());
-    if (bi.fromMaster)
-      sprintf(pMsg + strlen(pMsg), "(m)");
+    int remaining;
+    remaining = MessageLen - strlen(pMsg);
+    if (duration < 365.0f * 24 * 3600) {
+      snprintf(pMsg + strlen(pMsg), remaining, " (%.1f minutes)", duration / 60);
+    }
+    remaining = MessageLen - strlen(pMsg);
+    if (bi.bannedBy.length()) {
+      snprintf(pMsg + strlen(pMsg), remaining, " banned by: %s", bi.bannedBy.c_str());
+    }
+    remaining = MessageLen - strlen(pMsg);
+    if (bi.fromMaster) {
+      snprintf(pMsg + strlen(pMsg), remaining, "(m)");
+    }
 
     sendMessage(ServerPlayer, id, banlistmessage);
 
     // add reason, if any
     if (bi.reason.size()) {
       pMsg = banlistmessage;
-      sprintf(pMsg, "   reason: %s", bi.reason.c_str());
+      snprintf(pMsg, MessageLen, "   reason: %s", bi.reason.c_str());
       sendMessage(ServerPlayer, id, banlistmessage);
     }
   }
@@ -421,26 +404,33 @@ void AccessControlList::sendIdBans(PlayerId id, const char* pattern)
 
     bool useQuotes = (bi.idpat.find_first_of(" \t") != std::string::npos);
     if (useQuotes) {
-      sprintf(pMsg, "\"%s\"", bi.idpat.c_str());
+      snprintf(pMsg, MessageLen, "\"%s\"", bi.idpat.c_str());
     } else {
-      sprintf(pMsg, "%s", bi.idpat.c_str());
+      snprintf(pMsg, MessageLen, "%s", bi.idpat.c_str());
     }
 
     // print duration when < 1 year
     double duration = bi.banEnd - TimeKeeper::getCurrent();
-    if (duration < 365.0f * 24 * 3600)
-      sprintf(pMsg + strlen(pMsg)," (%.1f minutes)", duration / 60);
-    if (bi.bannedBy.length())
-      sprintf(pMsg + strlen(pMsg), " banned by: %s", bi.bannedBy.c_str());
-    if (bi.fromMaster)
-      sprintf(pMsg + strlen(pMsg), "(m)");
+    int remaining;
+    remaining = MessageLen - strlen(pMsg);
+    if (duration < 365.0f * 24 * 3600) {
+      snprintf(pMsg + strlen(pMsg), remaining, " (%.1f minutes)", duration / 60);
+    }
+    remaining = MessageLen - strlen(pMsg);
+    if (bi.bannedBy.length()) {
+      snprintf(pMsg + strlen(pMsg), remaining, " banned by: %s", bi.bannedBy.c_str());
+    }
+    remaining = MessageLen - strlen(pMsg);
+    if (bi.fromMaster) {
+      snprintf(pMsg + strlen(pMsg), remaining, "(m)");
+    }
 
     sendMessage(ServerPlayer, id, banlistmessage);
 
     // add reason, if any
     if (bi.reason.size()) {
       pMsg = banlistmessage;
-      sprintf(pMsg, "   reason: %s", bi.reason.c_str());
+      snprintf(pMsg, MessageLen, "   reason: %s", bi.reason.c_str());
       sendMessage(ServerPlayer, id, banlistmessage);
     }
   }
