@@ -1,9 +1,9 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
@@ -244,10 +244,10 @@ bool BZWReader::readWorldStream(std::vector<WorldFileObject*>& wlist,
 	  }
 	} else if (customObject.size())	{
 	  bz_CustomMapObjectInfo data;
-	  data.name = bzApiString(customObject);
+	  data.name = bz_ApiString(customObject);
 	  for (unsigned int i = 0; i < customLines.size(); i++)
 	    data.data.push_back(customLines[i]);
-	  customObjectMap[customObject]->handle(bzApiString(customObject),&data);
+	  customObjectMap[customObject]->MapObject(bz_ApiString(customObject),&data);
 	  object = NULL;
 	}
 	object = NULL;
@@ -429,7 +429,7 @@ WorldInfo* BZWReader::defineWorldFromFile()
     return NULL;
   }
 
-  if (!BZDB.isTrue("noWalls")) 
+  if (!BZDB.isTrue("noWalls"))
     makeWalls();
 
   // generate group instances
