@@ -1,9 +1,9 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
@@ -139,6 +139,14 @@ public:
 			   GameKeeper::Player *playerData);
 };
 
+class MuteListCommand : ServerCommand {
+public:
+  MuteListCommand();
+
+  virtual bool operator() (const char    *commandLine,
+			   GameKeeper::Player *playerData);
+};
+
 class MasterBanCommand : ServerCommand {
 public:
   MasterBanCommand();
@@ -161,6 +169,7 @@ static IdUnbanCommand     idUnbanCommand;
 static IdBanListCommand   idBanListCommand;
 static MuteCommand	  muteCommand;
 static UnmuteCommand	  unmuteCommand;
+static MuteListCommand	  muteListCommand;
 static MasterBanCommand   masterBanCommand;
 
 KickCommand::KickCommand()		 : ServerCommand("/kick",
@@ -191,6 +200,8 @@ MuteCommand::MuteCommand()		 : ServerCommand("/mute",
   "<#slot|PlayerName|\"Player Name\"> - remove the ability for a player to communicate with other players") {}
 UnmuteCommand::UnmuteCommand()		 : ServerCommand("/unmute",
   "<#slot|PlayerName|\"Player Name\"> - restore the TALK permission to a previously muted player") {}
+MuteListCommand::MuteListCommand()	 : ServerCommand("/mutelist",
+  "list the players current muted") {}
 MasterBanCommand::MasterBanCommand()	 : ServerCommand("/masterban",
   "<flush|reload|list> - manage the masterban list") {}
 
@@ -282,6 +293,30 @@ bool UnmuteCommand::operator() (const char	 *message,
   return true;
 }
 
+bool MuteListCommand::operator() (const char * UNUSED(message),
+				  GameKeeper::Player *playerData)
+{
+  int t = playerData->getIndex();
+  if (!playerData->accessInfo.hasPerm(PlayerAccessInfo::mute)) {
+    sendMessage(ServerPlayer, t,
+		"You do not have permission to run the mutelist command");
+    return true;
+  }
+
+  sendMessage(ServerPlayer, t, "Muted Players");
+  sendMessage(ServerPlayer, t, "-------------");
+
+  for (int i = 0; i < curMaxPlayers; i++) {
+    GameKeeper::Player *iPlayerData = GameKeeper::Player::getPlayerByIndex(i);
+    if (iPlayerData) {
+      if (!iPlayerData->accessInfo.hasPerm(PlayerAccessInfo::talk)) {
+	sendMessage(ServerPlayer, t, iPlayerData->player.getCallSign());
+      }
+    }
+  }
+  return true;
+}
+
 bool KickCommand::operator() (const char	 *message,
 			      GameKeeper::Player *playerData)
 {
@@ -309,7 +344,7 @@ bool KickCommand::operator() (const char	 *message,
   if (i >= 0) {
 
     // call any plugin events registered for /kick
-    bz_KickEventData kickEvent;
+    bz_KickEventData_V1 kickEvent;
     kickEvent.kickerID = t;
     kickEvent.kickedID = i;
     kickEvent.reason = argv[2].c_str();
@@ -400,7 +435,7 @@ bool KillCommand::operator() (const char	 *message,
 
   if (i >= 0) {
     // call any plugin events registered for /kill
-    bz_KillEventData killEvent;
+    bz_KillEventData_V1 killEvent;
     killEvent.killerID = t;
     killEvent.killedID = i;
     if (argv.size() > 2)
@@ -669,7 +704,7 @@ bool BanCommand::operator() (const char	 *message,
   }
 
   // call any plugin events registered for /ban
-  bz_BanEventData banEvent;
+  bz_BanEventData_V1 banEvent;
   banEvent.bannerID = t;
   banEvent.ipAddress = ip.c_str();
   banEvent.reason = reason.c_str();
@@ -772,7 +807,7 @@ bool HostbanCommand::operator() (const char* message,
 		  "1w2d1m");
       return true;
     }
-    if (specifiedDuration >= 0)
+    if (specifiedDuration >= 0) {
       if ((durationInt > 0) &&
 	  ((specifiedDuration > durationInt) || (specifiedDuration <= 0)) &&
 	  !playerData->accessInfo.hasPerm(PlayerAccessInfo::ban)) {
@@ -781,6 +816,7 @@ bool HostbanCommand::operator() (const char* message,
       } else {
 	durationInt = specifiedDuration;
       }
+    }
   }
 
   // set the ban reason
@@ -790,13 +826,13 @@ bool HostbanCommand::operator() (const char* message,
   }
 
   // call any plugin events registered for /hostban
-  bz_HostBanEventData hostBanEvent;
+  bz_HostBanEventData_V1 hostBanEvent;
   hostBanEvent.bannerID = t;
   hostBanEvent.hostPattern = hostpat.c_str();
   hostBanEvent.reason = reason.c_str();
   hostBanEvent.duration = durationInt;
 
-  worldEventManager.callEvents(bz_eHostBanEvent,&hostBanEvent);
+  worldEventManager.callEvents(bz_eHostBanModifyEvent,&hostBanEvent);
 
   // a plugin might have changed bannerID
   if (t != hostBanEvent.bannerID) {
