@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -50,9 +50,12 @@ void CustomGate::writeToGroupDef(GroupDefinition *groupdef) const
   Teleporter* tele =
     new Teleporter(pos, rotation,
 		   fabsf(size[0]), fabsf(size[1]), fabsf(size[2]),
-		   border, horizontal, driveThrough, shootThrough);
+		   border, horizontal, driveThrough, shootThrough, ricochet);
 
-  tele->setName(telename);
+  if (!telename.size() && name.size())
+    tele->setName(name);
+  else
+    tele->setName(telename);
 
   groupdef->addObstacle(tele);
 }
