@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -36,6 +36,7 @@ CustomMeshFace::CustomMeshFace(const BzMaterial& _material, int physics,
   shootThrough = shoot;
   driveThrough = drive;
   material = _material;
+  ricochet = false;
   return;
 }
 
@@ -109,6 +110,9 @@ bool CustomMeshFace::read(const char *cmd, std::istream& input)
   else if (strcasecmp(cmd, "passable") == 0) {
     driveThrough = shootThrough = true;
   }
+  else if (strcasecmp(cmd, "ricochet") == 0) {
+    ricochet = true;
+  }
   else if (parseMaterials(cmd, input, &material, 1, materror)) {
     if (materror) {
       return false;
@@ -127,7 +131,7 @@ void CustomMeshFace::write(MeshObstacle *mesh) const
 {
   const BzMaterial* matref = MATERIALMGR.addMaterial(&material);
   mesh->addFace(vertices, normals, texcoords, matref, phydrv,
-		noclusters, smoothBounce, driveThrough, shootThrough,
+		noclusters, smoothBounce, driveThrough, shootThrough, ricochet,
 		true /* triangulate if required */);
   return;
 }
