@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -44,7 +44,7 @@ const char* CustomPyramid::faceNames[FaceCount] = {
 CustomPyramid::CustomPyramid()
 {
   isOldPyramid = true;
-  
+
   flipz = false;
 
   size[0] = size[1] = BZDB.eval(StateDatabase::BZDB_PYRBASE);
@@ -64,6 +64,7 @@ CustomPyramid::CustomPyramid()
     phydrv[i] = -1;
     drivethrough[i] = false;
     shootthrough[i] = false;
+    ricochets[i] = false;
   }
 
   return;
@@ -85,9 +86,9 @@ bool CustomPyramid::read(const char *cmd, std::istream& input)
   std::istringstream parms(line);
   std::string tmpCmd;
   input.putback('\n');
-      
+
   std::vector<int> faceList;
-      
+
   // see if a face has been specified
   for (int i = 0; i < FaceCount; i++) {
     if (strcasecmp(cmd, faceNames[i]) == 0) {
@@ -101,7 +102,7 @@ bool CustomPyramid::read(const char *cmd, std::istream& input)
       faceList.push_back(ZN);
     }
     else if ((strcasecmp(cmd, "edge") == 0) || // meshpyr keyword
-             (strcasecmp(cmd, "sides") == 0)) {
+	     (strcasecmp(cmd, "sides") == 0)) {
       faceList.push_back(XP);
       faceList.push_back(XN);
       faceList.push_back(YP);
@@ -123,10 +124,10 @@ bool CustomPyramid::read(const char *cmd, std::istream& input)
       faceList.push_back(i);
     }
   }
-  const int faceCount = (int)faceList.size(); 
-  
-  
-  
+  const int faceCount = (int)faceList.size();
+
+
+
   // parse the command
   if (strcasecmp(cmd, "flipz") == 0) {
     flipz = true;
@@ -165,9 +166,9 @@ bool CustomPyramid::read(const char *cmd, std::istream& input)
       return false;
     } else {
       for (int i = 0; i < (int)faceList.size(); i++) {
-        const int f = faceList[i];
-        texsize[f][0] = tmp[0];
-        texsize[f][1] = tmp[1];
+	const int f = faceList[i];
+	texsize[f][0] = tmp[0];
+	texsize[f][1] = tmp[1];
       }
     }
     return true;
@@ -179,9 +180,9 @@ bool CustomPyramid::read(const char *cmd, std::istream& input)
       return false;
     } else {
       for (int i = 0; i < (int)faceList.size(); i++) {
-        const int f = faceList[i];
-        texoffset[f][0] = tmp[0];
-        texoffset[f][1] = tmp[1];
+	const int f = faceList[i];
+	texoffset[f][0] = tmp[0];
+	texoffset[f][1] = tmp[1];
       }
     }
     return true;
@@ -199,8 +200,8 @@ bool CustomPyramid::read(const char *cmd, std::istream& input)
       return false;
     } else {
       for (int i = 0; i < (int)faceList.size(); i++) {
-        const int f = faceList[i];
-        phydrv[f] = pd;
+	const int f = faceList[i];
+	phydrv[f] = pd;
       }
       return true;
     }
@@ -222,7 +223,7 @@ bool CustomPyramid::read(const char *cmd, std::istream& input)
     else {
       gotMaterial = true;
       if (materror) {
-        gotMatError = true;
+	gotMatError = true;
       }
     }
   }
@@ -235,7 +236,7 @@ bool CustomPyramid::read(const char *cmd, std::istream& input)
   if (WorldFileObstacle::read(cmd, parms)) {
     return true;
   }
-  
+
   return false;
 }
 
@@ -281,8 +282,8 @@ void CustomPyramid::writeToGroupDef(GroupDefinition *groupdef) const
   if (isOldPyramid && transform.isEmpty()) {
     PyramidBuilding* pyr =
       new PyramidBuilding(pos, rotation,
-                          fabsf(size[0]), fabsf(size[1]), fabsf(size[2]),
-                          driveThrough, shootThrough);
+			  fabsf(size[0]), fabsf(size[1]), fabsf(size[2]),
+			  driveThrough, shootThrough, ricochet);
     if (flipz || (size[2] < 0.0f)) {
       pyr->setZFlip();
     }
@@ -291,7 +292,7 @@ void CustomPyramid::writeToGroupDef(GroupDefinition *groupdef) const
   }
 
   int i;
-  
+
   // setup the transform
   MeshTransform xform;
   if (flipz || (size[2] < 0.0f)) {
@@ -300,7 +301,7 @@ void CustomPyramid::writeToGroupDef(GroupDefinition *groupdef) const
     xform.addScale(flipScale);
     xform.addShift(flipShift);
   }
-  
+
   const float zAxis[3] = {0.0f, 0.0f, 1.0f};
   xform.addScale(size);
   xform.addSpin((float)(rotation * (180.0 / M_PI)), zAxis);
@@ -311,8 +312,8 @@ void CustomPyramid::writeToGroupDef(GroupDefinition *groupdef) const
   // (X, Y, and 4 sheared Z values for pyramids)
   float edgeLengths[6];
   getEdgeLengths(xform, edgeLengths);
-  
-  
+
+
   std::vector<char> checkTypes;
   std::vector<cfvec3> checkPoints;
   std::vector<cfvec3> verts;
@@ -327,7 +328,7 @@ void CustomPyramid::writeToGroupDef(GroupDefinition *groupdef) const
   // add the vertex coordinates
   const float vertsData[5][3] = {
     {-1.0f, -1.0f, 0.0f}, {+1.0f, -1.0f, 0.0f},
-    {+1.0f, +1.0f, 0.0f}, {-1.0f, +1.0f, 0.0f}, 
+    {+1.0f, +1.0f, 0.0f}, {-1.0f, +1.0f, 0.0f},
     {+0.0f, +0.0f, 1.0f}
   };
   for (i = 0; i < 5; i++) {
@@ -359,24 +360,24 @@ void CustomPyramid::writeToGroupDef(GroupDefinition *groupdef) const
     for (int corner = 0; corner < cornerCount; corner++) {
       float txcd[2];
       for (int a = 0; a < 2; a++) {
-        float scale;
-        if (texsize[face][a] >= 0.0f) {
-          scale = texsize[face][a];
-        } else {
-          const int axis = txcdAxis[face][a];
-          scale = (edgeLengths[axis] / -texsize[face][a]);
-        }
-        const int realCorner = corner + cornerOffset;
-        txcd[a] = (txcdData[realCorner][a] - texoffset[face][a]) * scale;
+	float scale;
+	if (texsize[face][a] >= 0.0f) {
+	  scale = texsize[face][a];
+	} else {
+	  const int axis = txcdAxis[face][a];
+	  scale = (edgeLengths[axis] / -texsize[face][a]);
+	}
+	const int realCorner = corner + cornerOffset;
+	txcd[a] = (txcdData[realCorner][a] - texoffset[face][a]) * scale;
       }
       txcds.push_back(txcd);
     }
   }
-  
-  
+
+
   MeshObstacle* mesh = new MeshObstacle(xform, checkTypes, checkPoints,
-                                        verts, norms, txcds, FaceCount,
-                                        false, false, false, false);
+					verts, norms, txcds, FaceCount,
+					false, false, false, false, false);
 
   // get the material refs
   const BzMaterial* mats[FaceCount];
@@ -394,35 +395,35 @@ void CustomPyramid::writeToGroupDef(GroupDefinition *groupdef) const
   iv.push_back(1); iv.push_back(2); iv.push_back(4);
   it.push_back(0); it.push_back(1); it.push_back(2);
   mesh->addFace(iv, in, it, mats[XP], phydrv[XP], false, false,
-                drivethrough[XP], shootthrough[XP], false);
-                
+		drivethrough[XP], shootthrough[XP], ricochets[XP],  false);
+
   // XN
   iv.clear(); it.clear();
   iv.push_back(3); iv.push_back(0); iv.push_back(4);
   it.push_back(3); it.push_back(4); it.push_back(5);
   mesh->addFace(iv, in, it, mats[XN], phydrv[XN], false, false,
-                drivethrough[XN], shootthrough[XN], false);
-                
+		drivethrough[XN], shootthrough[XN], ricochets[XN], false);
+
   // YP
   iv.clear(); it.clear();
   iv.push_back(2); iv.push_back(3); iv.push_back(4);
   it.push_back(6); it.push_back(7); it.push_back(8);
   mesh->addFace(iv, in, it, mats[YP], phydrv[YP], false, false,
-                drivethrough[YP], shootthrough[YP], false);
-                
+		drivethrough[YP], shootthrough[YP], ricochets[XN], false);
+
   // YN
   iv.clear(); it.clear();
   iv.push_back(0); iv.push_back(1); iv.push_back(4);
   it.push_back(9); it.push_back(10); it.push_back(11);
   mesh->addFace(iv, in, it, mats[YN], phydrv[YN], false, false,
-                drivethrough[YN], shootthrough[YN], false);
-                
+		drivethrough[YN], shootthrough[YN], ricochets[YN], false);
+
   // ZN
   iv.clear(); it.clear();
   iv.push_back(1); iv.push_back(0); iv.push_back(3); iv.push_back(2);
   it.push_back(12); it.push_back(13); it.push_back(14); it.push_back(15);
   mesh->addFace(iv, in, it, mats[ZN], phydrv[ZN], false, false,
-                drivethrough[ZN], shootthrough[ZN], false);
+		drivethrough[ZN], shootthrough[ZN], ricochets[ZN], false);
 
   // to be or not to be...
   if (mesh->isValid()) {
