@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -17,6 +17,7 @@
 /* system headers */
 #include <sstream>
 #include <string>
+#include <string.h>
 #include <math.h>
 
 /* local implementation headers */
@@ -105,7 +106,7 @@ bool CustomWeapon::read(const char *cmd, std::istream& input) {
       std::cout << "weapon trigger type:" << triggerName << " unknown" << std::endl;
       return true;
     }
-    DEBUG4("Adding world weapon triggered '%s'\n", triggerName.c_str());
+    logDebugMessage(4,"Adding world weapon triggered '%s'\n", triggerName.c_str());
   }
   else if (strcmp(cmd, "eventteam") == 0) {
     input >> eventTeam;
