@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
@@ -18,8 +18,8 @@
 
 /* common implementation headers */
 #include "StateDatabase.h"
-#include "TextUtils.h"
 #include "BZDBCache.h"
+#include "TextUtils.h"
 
 
 CustomWorld::CustomWorld()
@@ -32,13 +32,17 @@ CustomWorld::CustomWorld()
 
 bool CustomWorld::read(const char *cmd, std::istream& input)
 {
-  if (strcmp(cmd, "size") == 0) {
+  if (strcasecmp(cmd, "size") == 0) {
     input >> _size;
     _size *= 2.0;
     BZDB.set(StateDatabase::BZDB_WORLDSIZE, TextUtils::format("%f", _size));
-  } else if (strcmp(cmd, "flagHeight") == 0) {
+  } else if (strcasecmp(cmd, "flagHeight") == 0) {
     input >> _fHeight;
     BZDB.set(StateDatabase::BZDB_FLAGHEIGHT, TextUtils::format("%f", _fHeight));
+  } else if (strcasecmp(cmd, "noWalls") == 0) {
+    BZDB.setBool("noWalls", true);
+  } else if (strcasecmp(cmd, "freeCtfSpawns") == 0) {
+    BZDB.setBool("freeCtfSpawns", true);
   } else {
     return WorldFileObject::read(cmd, input);
   }
@@ -50,6 +54,23 @@ void CustomWorld::writeToWorld(WorldInfo*) const
 {
 }
 
+std::map<std::string,bz_CustomMapObjectHandler*>	customObjectMap;
+
+void registerCustomMapObject ( const char* object, bz_CustomMapObjectHandler *handler )
+{
+	std::string objectName = object;
+
+	customObjectMap[TextUtils::toupper(objectName)] = handler;
+}
+
+void removeCustomMapObject ( const char* object )
+{
+	std::string objectName = object;
+
+	if ( customObjectMap.find(TextUtils::toupper(objectName)) != customObjectMap.end() )
+		customObjectMap.erase(customObjectMap.find(TextUtils::toupper(objectName)));
+}
+
 // Local variables: ***
 // mode: C++ ***
 // tab-width: 8 ***
