@@ -1,9 +1,9 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
@@ -36,7 +36,7 @@ public:
   void dropFlag(float pos[3], float landingPos[3], bool vanish);
   void resetFlag(float position[3], bool teamIsEmpty);
   void grab(int playerIndex);
-  int  teamIndex() const;
+  TeamColor teamIndex() const;
   int  getIndex() const;
   bool landing(const TimeKeeper &tm);
   void getTextualInfo(char *message);
