@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -139,12 +139,8 @@ bool PlayerAccessInfo::isRegistered() const {
   return userExists(regName);
 }
 
-bool PlayerAccessInfo::isIdentifyRequired() {
-  return getUserInfo(regName).hasPerm(requireIdentify);
-}
-
 bool PlayerAccessInfo::isAllowedToEnter() {
-  return verified || !isRegistered() || !isIdentifyRequired();
+  return verified || !isRegistered();
 }
 
 bool PlayerAccessInfo::isVerified() const{
@@ -244,7 +240,7 @@ bool PlayerAccessInfo::hasPerm(PlayerAccessInfo::AccessPerm right) const
 	isAllowed = true;
     }
   }
-  if (publiclyDisconected)
+  if (publiclyDisconnected)
   {
 	  PlayerAccessMap::iterator group = groupAccess.find(std::string("DISCONNECTED"));
 		if (group != groupAccess.end())
@@ -293,7 +289,7 @@ bool	PlayerAccessInfo::hasCustomPerm(const char* right) const
 		}
 	}
 
-	if (publiclyDisconected)
+	if (publiclyDisconnected)
 	{
 		PlayerAccessMap::iterator group = groupAccess.find(std::string("DISCONNECTED"));
 		if (group != groupAccess.end())
@@ -375,7 +371,6 @@ std::string nameFromPerm(PlayerAccessInfo::AccessPerm perm)
     case PlayerAccessInfo::adminMessageReceive: return "adminMessageReceive";
     case PlayerAccessInfo::adminMessageSend: return "adminMessageSend";
     case PlayerAccessInfo::antiban : return "antiban";
-    case PlayerAccessInfo::antideregister : return "antideregister";
     case PlayerAccessInfo::antikick : return "antikick";
     case PlayerAccessInfo::antikill : return "antikill";
     case PlayerAccessInfo::antipoll : return "antipoll";
@@ -415,7 +410,6 @@ std::string nameFromPerm(PlayerAccessInfo::AccessPerm perm)
     case PlayerAccessInfo::rejoin: return "rejoin";
     case PlayerAccessInfo::removePerms: return "removePerms";
     case PlayerAccessInfo::replay: return "replay";
-    case PlayerAccessInfo::requireIdentify: return "requireIdentify";
     case PlayerAccessInfo::say: return "say";
     case PlayerAccessInfo::sendHelp : return "sendHelp";
     case PlayerAccessInfo::setAll: return "setAll";
@@ -443,7 +437,6 @@ PlayerAccessInfo::AccessPerm permFromName(const std::string &name)
   if (name == "ADMINMESSAGERECEIVE") return PlayerAccessInfo::adminMessageReceive;
   if (name == "ADMINMESSAGESEND") return PlayerAccessInfo::adminMessageSend;
   if (name == "ANTIBAN") return PlayerAccessInfo::antiban;
-  if (name == "ANTIDEREGISTER") return PlayerAccessInfo::antideregister;
   if (name == "ANTIKICK") return PlayerAccessInfo::antikick;
   if (name == "ANTIKILL") return PlayerAccessInfo::antikill;
   if (name == "ANTIPOLL") return PlayerAccessInfo::antipoll;
@@ -483,7 +476,6 @@ PlayerAccessInfo::AccessPerm permFromName(const std::string &name)
   if (name == "REJOIN") return PlayerAccessInfo::rejoin;
   if (name == "REMOVEPERMS") return PlayerAccessInfo::removePerms;
   if (name == "REPLAY") return PlayerAccessInfo::replay;
-  if (name == "REQUIREIDENTIFY") return PlayerAccessInfo::requireIdentify;
   if (name == "SAY") return PlayerAccessInfo::say;
   if (name == "SENDHELP") return PlayerAccessInfo::sendHelp;
   if (name == "SETALL") return PlayerAccessInfo::setAll;
@@ -504,13 +496,17 @@ PlayerAccessInfo::AccessPerm permFromName(const std::string &name)
   return PlayerAccessInfo::lastPerm;
 }
 
-void parsePermissionString(const std::string &permissionString, PlayerAccessInfo &info)
+// Parse a list of permissions &permissionString and set the corrosponding Permissions
+// in &info. Return true if a group is referenced but not defined yet, else false.
+// return value is only needed for groupdb parsing, not for userdb.
+bool parsePermissionString(const std::string &permissionString, PlayerAccessInfo &info)
 {
   if (permissionString.length() < 1)
-    return;
+    return false;
 
   std::istringstream permStream(permissionString);
   std::string word;
+  bool invokeGroupdbRecursion = false;
 
   while (permStream >> word) {
     makeupper(word);
@@ -525,7 +521,7 @@ void parsePermissionString(const std::string &permissionString, PlayerAccessInfo
     // Operators are not allowed for userdb
     if (!info.groupState.test(PlayerAccessInfo::isGroup) && first != '\0'){
       logDebugMessage(1,"userdb: illegal permission string, operators are not allowed in userdb\n");
-      return;
+      return false;
     }
 
     // if we have an operator, lets handle it
@@ -540,9 +536,8 @@ void parsePermissionString(const std::string &permissionString, PlayerAccessInfo
 	  if (refgroup != groupAccess.end()) {
 	    info.explicitAllows |= refgroup->second.explicitAllows;
 	    info.explicitDenys |= refgroup->second.explicitDenys;
-	    refgroup->second.groupState.set(PlayerAccessInfo::isReferenced);
 	  } else {
-	    logDebugMessage(1,"groupdb: unknown group \"%s\" was referenced\n", word.c_str());
+	    invokeGroupdbRecursion = true;
 	  }
 
 	  continue;
@@ -554,14 +549,14 @@ void parsePermissionString(const std::string &permissionString, PlayerAccessInfo
 	if (perm != PlayerAccessInfo::lastPerm) {
 	  info.explicitDenys.set(perm);
 	} else {
-          if (word == "ALL") {
-            info.explicitDenys.set();
-            info.explicitDenys[PlayerAccessInfo::lastPerm] = false;
+	  if (word == "ALL") {
+	    info.explicitDenys.set();
+	    info.explicitDenys[PlayerAccessInfo::lastPerm] = false;
 	    info.hasALLPerm = false;
-          } else {
-            logDebugMessage(1,"groupdb: Cannot forbid unknown permission %s\n", word.c_str());
-          }
-        }
+	  } else {
+	    logDebugMessage(1,"groupdb: Cannot forbid unknown permission %s\n", word.c_str());
+	  }
+	}
 
 	continue;
       }
@@ -607,101 +602,66 @@ void parsePermissionString(const std::string &permissionString, PlayerAccessInfo
       }
     }
   }
+  return invokeGroupdbRecursion;
 }
 
 
-bool readPassFile(const std::string &filename)
-{
-  std::ifstream in(filename.c_str());
-  if (!in)
-    return false;
-
-  std::string line;
-  while (std::getline(in, line)) {
-    // Should look at an unescaped ':'
-    int colonpos = TextUtils::unescape_lookup(line, '\\', ':');
-    if (colonpos == -1)
-      continue;
-    {
-      std::string name = TextUtils::unescape(line.substr(0, colonpos), '\\');
-      std::string pass = line.substr(colonpos + 1);
-      makeupper(name);
-      setUserPassword(name.c_str(), pass.c_str());
-    }
-  }
-
-  return (passwordDatabase.size() > 0);
-}
-
-bool writePassFile(const std::string &filename)
-{
-  std::ofstream out(filename.c_str());
-  if (!out)
-    return false;
-  PasswordMap::iterator itr = passwordDatabase.begin();
-  while (itr != passwordDatabase.end()) {
-    if (itr->second != "*") {
-      out << TextUtils::escape(itr->first, '\\') << ':' << itr->second << std::endl;
-    }
-    itr++;
-  }
-  out.close();
-  return true;
-}
-
 bool PlayerAccessInfo::readGroupsFile(const std::string &filename)
 {
   std::ifstream in(filename.c_str());
   if (!in)
     return false;
 
-  int linenum = 0;
-  std::string line;
-  while (std::getline(in, line)) {
-    linenum++;
+  int recursionNeeded = 1;
+  bool initialRun = true;
 
-    // check for a comment string
-    bool skip = true;
-    for (std::string::size_type pos = 0; pos < line.size(); pos++) {
-      const char c = line[pos];
-      if (!TextUtils::isWhitespace(c)) {
-	if (c != '#') {
-	  skip = false;
-	}
-	break;
-      }
-    }
-    if (skip) continue;
+   // read groupdb and check for unsolved group references, read the file again
+  // for each of them. One more recursion should make sure everything is fine.
+  do {
+    in.clear();
+    in.seekg(0, std::ios::beg);
+    int linenum = 0;
 
-    makeupper(line);
+    std::string line;
+    while (std::getline(in, line)) {
+      linenum++;
 
-    std::string::size_type colonpos = line.find(':');
-    if (colonpos != std::string::npos) {
-      std::string name = line.substr(0, colonpos);
-      std::string perm = line.substr(colonpos + 1);
+      // strip leading whitespace
+      line.erase(0 , line.find_first_not_of(" \t"));
 
-      // check if we already have this group, else make a new
-      PlayerAccessInfo info;
-      PlayerAccessMap::iterator oldgroup = groupAccess.find(name);
-      if (oldgroup != groupAccess.end())
-	info = oldgroup->second;
-      else
-	info.groupState[PlayerAccessInfo::isGroup] = true;
-
-      // don't allow changing permissions for a group
-      // that was used as a reference before
-      if (info.groupState.test(isReferenced)) {
-	logDebugMessage(1,"groupdb: skipping groupdb line (%i), group was used as reference before\n", linenum);
+      // check for a comment string or empty line
+      if(line.empty() || line[0] == '#')
 	continue;
-      }
-      parsePermissionString(perm, info);
-      info.verified = true;
-      groupAccess[name] = info;
-    } else {
-      logDebugMessage(1,"WARNING: bad groupdb line (%i)\n", linenum);
-    }
 
-  }
+      makeupper(line);
+
+      std::string::size_type colonpos = line.find(':');
+      if (colonpos == std::string::npos)
+	logDebugMessage(1,"WARNING: bad groupdb line (%i)\n", linenum);
+      else {
+	std::string name = line.substr(0, colonpos);
+	std::string perm = line.substr(colonpos + 1);
+
+	// check if we already have this group, else make a new
+	PlayerAccessInfo accessInfo;
+	PlayerAccessMap::iterator oldgroup = groupAccess.find(name);
+	if (oldgroup != groupAccess.end())
+	  accessInfo = oldgroup->second;
+	else
+	  accessInfo.groupState[PlayerAccessInfo::isGroup] = true;
+
+	// Parse the permission string. If it contains a yet undefined group
+	// add a recursion
+	if(parsePermissionString(perm, accessInfo) && initialRun){
+	  recursionNeeded++;
+	}
+
+	accessInfo.verified = true;
+	groupAccess[name] = accessInfo;
+      }
+    }
+    initialRun = false;
+  } while(recursionNeeded--);
 
   return true;
 }
@@ -718,7 +678,7 @@ bool PlayerAccessInfo::readPermsFile(const std::string &filename)
     if (!std::getline(in, name))
       break;
 
-    PlayerAccessInfo info;
+    PlayerAccessInfo accessInfo;
 
     // 2nd line - groups
     std::string groupline;
@@ -726,22 +686,22 @@ bool PlayerAccessInfo::readPermsFile(const std::string &filename)
     std::istringstream groupstream(groupline);
     std::string group;
     while (groupstream >> group) {
-      info.addGroup(group);
+      accessInfo.addGroup(group);
     }
 
     // 3rd line - allows
     std::string perms;
     std::getline(in, perms);
-    parsePermissionString(perms, info);
+    parsePermissionString(perms, accessInfo);
 
     // 4th line - denies
     // FIXME: not nice ... any ideas how to make it better?
     std::getline(in, perms);
     PlayerAccessInfo dummy;
     parsePermissionString(perms, dummy);
-    info.explicitDenys = dummy.explicitAllows;
+    accessInfo.explicitDenys = dummy.explicitAllows;
 
-    userDatabase[name] = info;
+    userDatabase[name] = accessInfo;
   }
 
   return true;
@@ -780,13 +740,10 @@ bool PlayerAccessInfo::writePermsFile(const std::string &filename)
 }
 
 std::string		groupsFile;
-std::string		passFile;
 std::string		userDatabaseFile;
 
 void PlayerAccessInfo::updateDatabases()
 {
-  if (passFile.size())
-    writePassFile(passFile);
   if (userDatabaseFile.size())
     writePermsFile(userDatabaseFile);
 }
