@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -47,7 +47,6 @@ public:
       adminMessageReceive,
       adminMessageSend,
       antiban,
-      antideregister,
       antikick,
       antikill,
       antipoll,
@@ -87,7 +86,6 @@ public:
       rejoin,
       removePerms,
       replay,
-      requireIdentify,
       say,
       sendHelp,
       setAll,
@@ -115,7 +113,6 @@ public:
 	{
 	  isGroup,  // we can check if this is a group or a player
 	  isDefault,   // mark default groups
-	  isReferenced, // do not allow to alter group perms once it's referenced
 	  lastState
 	};
 
@@ -155,7 +152,6 @@ public:
 	bool	hasCustomPerm(const char* right) const;
 
   bool	isRegistered() const;
-  bool	isIdentifyRequired();
   bool	isAllowedToEnter();
   bool	isVerified() const;
   uint8_t     getPlayerProperties();
@@ -196,7 +192,6 @@ extern PlayerAccessMap	userDatabase;
 extern PasswordMap	passwordDatabase;
 
 extern std::string		groupsFile;
-extern std::string		passFile;
 extern std::string		userDatabaseFile;
 
 inline void makeupper(std::string& str)
@@ -209,9 +204,7 @@ bool checkPasswordExistence(const std::string &nick);
 bool verifyUserPassword(const std::string &nick, const std::string &pass);
 std::string nameFromPerm(PlayerAccessInfo::AccessPerm perm);
 PlayerAccessInfo::AccessPerm permFromName(const std::string &name);
-void parsePermissionString(const std::string &permissionString, std::bitset<PlayerAccessInfo::lastPerm> &perms);
-bool readPassFile(const std::string &filename);
-bool writePassFile(const std::string &filename);
+bool parsePermissionString(const std::string &permissionString, std::bitset<PlayerAccessInfo::lastPerm> &perms);
 
 uint8_t GetPlayerProperties( bool registered, bool verified, bool admin );
 
