@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -33,7 +33,7 @@ RandomSpawnPolicy::~RandomSpawnPolicy()
 {
 }
 
-void RandomSpawnPolicy::getPosition(float pos[3], int playerId, bool onGroundOnly, bool /*notNearEdges*/)
+void RandomSpawnPolicy::getPosition(float pos[3], int playerId, bool onGroundOnly, bool UNUSED(notNearEdges))
 {
   /* the player is coming to life, depending on who they are an what
    * style map/configuration is being played determines how they will
