@@ -1,9 +1,9 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
@@ -111,8 +111,8 @@ typedef struct {
   u32 flagsSize;		// size of the flags data
   u32 worldSize;		// size of world database
   char callSign[CallSignLen];   // player's callsign
-  char email[EmailLen];	 // player's email
-  char serverVersion[8];	// BZFS protocol version
+  char motto[MottoLen];	 // player's motto
+  char ServerVersion[8];	// BZFS protocol version
   char appVersion[MessageLen];  // BZFS application version
   char realHash[64];	    // hash of worldDatabase
   char worldSettings[4 + WorldSettingsSize]; // the game settings
@@ -124,7 +124,7 @@ typedef struct {
 static const unsigned int ReplayHeaderSize =
   HEADER_SIZE_STUFFING +
   (sizeof(u32) * 6) + sizeof(RRtime) +
-  CallSignLen + EmailLen + 8 + MessageLen + 64 + 4 + WorldSettingsSize;
+  CallSignLen + MottoLen + 8 + MessageLen + 64 + 4 + WorldSettingsSize;
 
 typedef struct {
   std::string file;
@@ -841,9 +841,9 @@ bool Replay::loadFile(int playerIndex, const char *filename)
   snprintf(buffer, MessageLen, "Loaded file:  %s", name.c_str());
   sendMessage(ServerPlayer, playerIndex, buffer);
   snprintf(buffer, MessageLen, "  author:     %s (%s)",
-	    header.callSign, header.email);
+	    header.callSign, header.motto);
   sendMessage(ServerPlayer, playerIndex, buffer);
-  snprintf(buffer, MessageLen, "  protocol:   %.8s", header.serverVersion);
+  snprintf(buffer, MessageLen, "  protocol:   %.8s", header.ServerVersion);
   sendMessage(ServerPlayer, playerIndex, buffer);
   snprintf(buffer, MessageLen, "  server:     %s", header.appVersion);
   sendMessage(ServerPlayer, playerIndex, buffer);
@@ -1622,7 +1622,7 @@ static bool saveFlagsState()
 
 static bool saveRabbitState()
 {
-  if (clOptions->gameStyle & int(RabbitChaseGameStyle)) {
+  if (clOptions->gameType == RabbitChase) {
     char bufStart[MaxPacketLen];
     void *buf;
     buf = nboPackUByte(bufStart, rabbitIndex);
@@ -2021,9 +2021,9 @@ static bool saveHeader(int p, RRtime filetime, FILE *f)
     return false;
   }
 
-  // player callsign and email
+  // player callsign and motto
   const char* callsign = "SERVER";
-  const char* email = "";
+  const char* motto = "";
   if (p != ServerPlayer) {
     GameKeeper::Player *gkPlayer = GameKeeper::Player::getPlayerByIndex(p);
     if (gkPlayer == NULL) {
@@ -2031,15 +2031,15 @@ static bool saveHeader(int p, RRtime filetime, FILE *f)
     } else {
       PlayerInfo *pi = &gkPlayer->player;
       callsign = pi->getCallSign();
-      email = pi->getEMail();
+      motto = pi->getMotto();
     }
   }
 
   // setup the data
   memset(&hdr, 0, sizeof(hdr));
   strncpy(hdr.callSign, callsign, sizeof(hdr.callSign));
-  strncpy(hdr.email, email, sizeof(hdr.email));
-  strncpy(hdr.serverVersion, getServerVersion(), sizeof(hdr.serverVersion));
+  strncpy(hdr.motto, motto, sizeof(hdr.motto));
+  strncpy(hdr.ServerVersion, getServerVersion(), sizeof(hdr.ServerVersion));
   strncpy(hdr.appVersion, getAppVersion(), sizeof(hdr.appVersion));
   strncpy(hdr.realHash, hexDigest, sizeof(hdr.realHash));
   packFlagTypes(flagsBuf, &hdr.flagsSize);
@@ -2056,8 +2056,8 @@ static bool saveHeader(int p, RRtime filetime, FILE *f)
   buf = nboPackUInt(buf, hdr.flagsSize);
   buf = nboPackUInt(buf, worldDatabaseSize);
   buf = nboPackString(buf, hdr.callSign, sizeof(hdr.callSign));
-  buf = nboPackString(buf, hdr.email, sizeof(hdr.email));
-  buf = nboPackString(buf, hdr.serverVersion, sizeof(hdr.serverVersion));
+  buf = nboPackString(buf, hdr.motto, sizeof(hdr.motto));
+  buf = nboPackString(buf, hdr.ServerVersion, sizeof(hdr.ServerVersion));
   buf = nboPackString(buf, hdr.appVersion, sizeof(hdr.appVersion));
   buf = nboPackString(buf, hdr.realHash, sizeof(hdr.realHash));
   buf = nboPackString(buf, worldSettings, sizeof(worldSettings));
@@ -2098,8 +2098,8 @@ static bool loadHeader(ReplayHeader *h, FILE *f)
   buf = nboUnpackUInt(buf, h->flagsSize);
   buf = nboUnpackUInt(buf, h->worldSize);
   buf = nboUnpackString(buf, h->callSign, sizeof(h->callSign));
-  buf = nboUnpackString(buf, h->email, sizeof(h->email));
-  buf = nboUnpackString(buf, h->serverVersion, sizeof(h->serverVersion));
+  buf = nboUnpackString(buf, h->motto, sizeof(h->motto));
+  buf = nboUnpackString(buf, h->ServerVersion, sizeof(h->ServerVersion));
   buf = nboUnpackString(buf, h->appVersion, sizeof(h->appVersion));
   buf = nboUnpackString(buf, h->realHash, sizeof(h->realHash));
   buf = nboUnpackString(buf, h->worldSettings, sizeof(h->worldSettings));
