@@ -1,13 +1,13 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #ifndef __SCORE_H__
@@ -19,6 +19,8 @@
 class Score {
  public:
   Score();
+  int playerID;
+
   void  dump();
   /** Take into account the quality of player wins/(wins+loss)
       Try to penalize winning casuality
@@ -30,10 +32,22 @@ class Score {
   void  kill();
   void *pack(void *buf);
   bool  reached();
+  int	getWins(){return wins;}
+  int	getLosses(){return losses;}
+  int	getTKs(){return tks;}
+
+  void	setWins(int v){wins = v;}
+  void	setLosses(int v){losses = v;}
+  void	setTKs(int v){tks = v;}
+
+  static bool KeepPlayerScores;
+  static bool KeepTeamScores;
 
   static void setTeamKillRatio(int _tkKickRatio);
   static void setWinLimit(int _score);
   static void setRandomRanking();
+
+  int   getHandicap();
  private:
   // player's score
   int wins, losses, tks;
