@@ -1,9 +1,9 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
@@ -45,10 +45,47 @@
 #include "WorldInfo.h"
 #include "VotingArbiter.h"
 
+#include <list>
+class PendingChatMessages
+{
+public:
+	int to;
+	int from;
+	std::string text;
+	MessageType type;
+
+	PendingChatMessages ( int t, int f, const std::string &m, MessageType y )
+	{
+		to = t;
+		from = f;
+		text = m;
+		type = y;
+	}
+
+	PendingChatMessages ( int t, int f, const char* m, MessageType y )
+	{
+		to = t;
+		from = f;
+		if (m)
+			text = m;
+		type = y;
+	}
+
+	PendingChatMessages ( const PendingChatMessages &m )
+	{
+		to = m.to;
+		from = m.from;
+		text = m.text;
+		type = m.type;
+	}
+};
+
+extern std::list<PendingChatMessages> pendingChatMessages;
 
 extern void sendMessage(int	 playerIndex,
 			PlayerId    dstPlayer,
-			const char *message);
+			const char *message,
+			MessageType	   type = ChatMessage);
 extern void removePlayer(int	 playerIndex,
 			 const char *reason,
 			 bool	notify = true);
@@ -59,6 +96,7 @@ extern void playerKilled(int	     victimIndex,
 			 const FlagType *flagType,
 			 int	     phydrv,
 			 bool	    respawnOnBase = false);
+extern void doSpawns();
 extern void sendPlayerMessage(GameKeeper::Player *playerData,
 			      PlayerId dstPlayer,
 			      const char *message);
@@ -84,7 +122,12 @@ extern TeamColor  whoseBase(float x, float y, float z);
 bool defineWorld ( void );
 bool saveWorldCache( const char* file = NULL );
 
-void rescanForBans ( bool isOperator = true, const char* callsign = NULL, int playerID = -1 );
+void playerAlive(int playerIndex);
+bool allowTeams ( void );
+extern const std::string& getPublicOwner();
+extern void	       setPublicOwner(const std::string& owner);
+
+void rescanForBans ( bool isOperator = false, const char* callsign = NULL, int playerID = -1 );
 
 // initialize permission groups
 extern void initGroups();
@@ -96,9 +139,11 @@ extern bool	    done;
 extern bool	    gameOver;
 extern TeamInfo	team[NumTeams];
 extern int	     numFlags;
+extern TimeKeeper	gameStartTime;
 extern bool	    countdownActive;
 extern int	     countdownDelay;
 extern TimeKeeper      countdownPauseStart;
+extern int	     countdownResumeDelay;
 extern char	    hexDigest[50];
 extern WorldInfo      *world;
 extern char	   *worldDatabase;
@@ -111,9 +156,8 @@ extern uint16_t	maxPlayers;
 extern uint16_t	maxRealPlayers;
 extern float	   pluginWorldSize;
 extern float	   pluginWorldHeight;
-extern float	   pluginMaxWait;
 
-extern bool		  publiclyDisconected;
+extern bool		  publiclyDisconnected;
 
 extern VotingArbiter *votingarbiter;
 
@@ -122,7 +166,45 @@ void resumeCountdown ( const char *resumedBy );
 void resetTeamScores ( void );
 void startCountdown ( int delay, float limit, const char *buyWho );
 
+void dropPlayerFlag(GameKeeper::Player &playerData, const float dropPos[3]);
+void playerAlive(int playerIndex);
+void sendChatMessage(PlayerId srcPlayer, PlayerId dstPlayer, const char *message, MessageType type);
+
 void makeWalls ( void );
+
+PlayerId getNewPlayerID();
+void checkGameOn();
+void checkTeamScore(int playerIndex, int teamIndex);
+void sendClosestFlagMessage(int playerIndex,FlagType *type, float pos[3] );
+
+// peer list
+struct NetConnectedPeer {
+	int socket;
+	int player;
+
+	NetHandler*		    netHandler;
+	bz_NonPlayerConnectionHandler* apiHandler;
+
+	std::list<std::string> sendChunks;
+	std::string bufferedInput;
+
+	TimeKeeper startTime;
+	TimeKeeper lastActivity;
+
+	bool   sent;
+	bool   deleteMe;
+};
+
+extern std::map<int, NetConnectedPeer> netConnectedPeers;
+
+extern unsigned int maxNonPlayerDataChunk;
+extern void sendBufferedNetDataForPeer(NetConnectedPeer &peer);
+
+// utils
+void playerStateToAPIState(bz_PlayerUpdateState &apiState, const PlayerState &playerState);
+void APIStateToplayerState(PlayerState &playerState, const bz_PlayerUpdateState &apiState);
+
+
 #endif
 
 // Local Variables: ***
