@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -36,6 +36,7 @@ BZDBCache::Int   BZDBCache::radarStyle;
 BZDBCache::Float BZDBCache::radarTankPixels;
 BZDBCache::Float BZDBCache::linedRadarShots;
 BZDBCache::Float BZDBCache::sizedRadarShots;
+BZDBCache::Float BZDBCache::shotLength;
 BZDBCache::Int   BZDBCache::flagChunks;
 BZDBCache::Float BZDBCache::pulseRate;
 BZDBCache::Float BZDBCache::pulseDepth;
@@ -62,6 +63,8 @@ BZDBCache::Float BZDBCache::flagPoleSize;
 BZDBCache::Float BZDBCache::flagPoleWidth;
 BZDBCache::Float BZDBCache::maxLOD;
 
+BZDBCache::Float BZDBCache::gmSize;
+
 BZDBCache::Float BZDBCache::hudGUIBorderOpacityFactor;
 
 
@@ -106,6 +109,7 @@ void BZDBCache::init()
   BZDB.addCallback("smooth", clientCallback, NULL);
   BZDB.addCallback("colorful", clientCallback, NULL);
   BZDB.addCallback("animatedTreads", clientCallback, NULL);
+  BZDB.addCallback("shotLength", clientCallback, NULL);
   BZDB.addCallback("leadingShotLine", clientCallback, NULL);
   BZDB.addCallback("flagChunks", clientCallback, NULL);
   BZDB.addCallback("pulseRate", clientCallback, NULL);
@@ -132,6 +136,7 @@ void BZDBCache::init()
   BZDB.addCallback(StateDatabase::BZDB_FLAGRADIUS, serverCallback, NULL);
   BZDB.addCallback(StateDatabase::BZDB_FLAGPOLESIZE, serverCallback, NULL);
   BZDB.addCallback(StateDatabase::BZDB_FLAGPOLEWIDTH, serverCallback, NULL);
+  BZDB.addCallback(StateDatabase::BZDB_GMSIZE, serverCallback, NULL);
 
   drawCelestial = BZDB.isTrue(StateDatabase::BZDB_DRAWCELESTIAL);
   drawClouds = BZDB.isTrue(StateDatabase::BZDB_DRAWCLOUDS);
@@ -153,6 +158,8 @@ void BZDBCache::init()
   flagPoleSize = getGoodPosValue(flagPoleSize,StateDatabase::BZDB_FLAGPOLESIZE);
   flagPoleWidth = getGoodPosValue(flagPoleWidth,StateDatabase::BZDB_FLAGPOLEWIDTH);
 
+  gmSize = getGoodPosValue(flagPoleWidth,StateDatabase::BZDB_GMSIZE);
+
   update();
 }
 
@@ -187,6 +194,8 @@ void BZDBCache::clientCallback(const std::string& name, void *)
     radarTankPixels = BZDB.eval("radarTankPixels");
   else if (name == "animatedTreads")
     animatedTreads = BZDB.isTrue("animatedTreads");
+  else if (name == "shotLength")
+    shotLength = BZDB.eval("shotLength");
   else if (name == "leadingShotLine")
     leadingShotLine = BZDB.isTrue("leadingShotLine");
   else if (name == "flagChunks")
@@ -258,6 +267,9 @@ void BZDBCache::serverCallback(const std::string& name, void *)
   else if (name == StateDatabase::BZDB_FLAGPOLEWIDTH) {
     flagPoleWidth = getGoodPosValue(flagPoleWidth,StateDatabase::BZDB_FLAGPOLEWIDTH);
   }
+  else if (name == StateDatabase::BZDB_GMSIZE) {
+    gmSize = getGoodPosValue(flagPoleWidth,StateDatabase::BZDB_GMSIZE);
+  }
 }
 
 
