@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -18,6 +18,8 @@
 /* system implementation headers */
 #include <math.h>
 #include <string>
+#include <assert.h>
+#include <string.h>
 
 /* common implementation headers */
 #include "Team.h"
@@ -25,11 +27,9 @@
 #include "TextUtils.h"
 
 
-int FlagType::flagCount = 0;
 FlagSet *FlagType::flagSets = NULL;
 const int FlagType::packSize = FlagPackSize;
-
-static const char NullString[2] = { '\0', '\0' };
+FlagSet FlagType::customFlags;
 
 // Initialize flag description singletons in our Flags namespace
 namespace Flags {
@@ -83,7 +83,7 @@ namespace Flags {
 
   void init()
   {
-    Null	= new FlagType( "", NullString, FlagNormal, NormalShot, FlagGood, NoTeam, NULL );
+    Null	= new FlagType( "", "", FlagNormal, NormalShot, FlagGood, NoTeam, "" );
 
     RedTeam	= new FlagType( "Red Team", "R*", FlagNormal, NormalShot, FlagGood, ::RedTeam,
 					    "If it's yours, prevent other teams from taking it.  If it's not take it to your base to capture it!" );
@@ -181,8 +181,8 @@ namespace Flags {
 
   void kill()
   {
-    delete Null;
-    delete RedTeam;
+    clearCustomFlags();
+
     delete GreenTeam;
     delete BlueTeam;
     delete PurpleTeam;
@@ -231,12 +231,31 @@ namespace Flags {
 
     delete[] FlagType::flagSets;
   }
+
+  void clearCustomFlags()
+  {
+    FlagSet::iterator itr, nitr;
+    for (int q = 0; q < (int)NumQualities; ++q) {
+      for (itr = FlagType::flagSets[q].begin(); itr != FlagType::flagSets[q].end(); ++itr) {
+	if ((*itr)->custom) {
+	  FlagType::getFlagMap().erase((*itr)->flagAbbv);
+	  nitr = itr;
+	  ++nitr;
+	  delete (*itr);
+	  FlagType::flagSets[q].erase(itr);
+	  itr = nitr;
+	  if (itr == FlagType::flagSets[q].end()) break;
+	}
+      }
+    }
+    FlagType::customFlags.clear();
+  }
 }
 
 void* FlagType::pack(void* buf) const
 {
-  buf = nboPackUByte(buf, flagAbbv[0]);
-  buf = nboPackUByte(buf, flagAbbv[1]);
+  buf = nboPackUByte(buf, (flagAbbv.size() > 0) ? flagAbbv[0] : 0);
+  buf = nboPackUByte(buf, (flagAbbv.size() > 1) ? flagAbbv[1] : 0);
   return buf;
 }
 
@@ -249,13 +268,51 @@ void* FlagType::fakePack(void* buf) const
 
 void* FlagType::unpack(void* buf, FlagType* &type)
 {
-  unsigned char abbv[3] = {0,0,0};
+  unsigned char abbv[3] = {0, 0, 0};
   buf = nboUnpackUByte(buf, abbv[0]);
   buf = nboUnpackUByte(buf, abbv[1]);
   type = Flag::getDescFromAbbreviation((const char *)abbv);
   return buf;
 }
 
+void* FlagType::packCustom(void* buf) const
+{
+  buf = pack(buf);
+  buf = nboPackUByte(buf, uint8_t(flagQuality));
+  buf = nboPackUByte(buf, uint8_t(flagShot));
+  buf = nboPackStdString(buf, flagName);
+  buf = nboPackStdString(buf, flagHelp);
+  return buf;
+}
+
+void* FlagType::unpackCustom(void* buf, FlagType* &type)
+{
+  uint8_t *abbv = new uint8_t[3];
+  abbv[0]=abbv[1]=abbv[2]=0;
+  buf = nboUnpackUByte(buf, abbv[0]);
+  buf = nboUnpackUByte(buf, abbv[1]);
+
+  uint8_t quality, shot;
+  buf = nboUnpackUByte(buf, quality);
+  buf = nboUnpackUByte(buf, shot);
+
+  // make copies to keep - note that these will need to be deleted.
+  std::string sName, sHelp;
+  buf = nboUnpackStdString(buf, sName);
+  buf = nboUnpackStdString(buf, sHelp);
+
+  FlagEndurance e = FlagUnstable;
+  switch((FlagQuality)quality) {
+    case FlagGood: e = FlagUnstable; break;
+    case FlagBad: e = FlagSticky; break;
+    default: assert(false); // shouldn't happen
+  }
+
+  type = new FlagType(sName, reinterpret_cast<const char*>(&abbv[0]),
+		      e, (ShotType)shot, (FlagQuality)quality, NoTeam, sHelp, true);
+  return buf;
+}
+
 FlagTypeMap& FlagType::getFlagMap() {
   static FlagTypeMap flagMap;
   return flagMap;
@@ -354,13 +411,13 @@ const std::string FlagType::label() const
 
   /* convert to lowercase so we can uppercase the abbreviation later */
   std::string caseName = "";
-  for (i = 0; i < strlen(flagName); i++) {
-    caseName += tolower(flagName[i]);
+  for (i = 0; i < flagName.size(); i++) {
+    caseName += tolower(flagName.c_str()[i]);
   }
 
   /* modify the flag name to exemplify the abbreviation */
   int charPosition;
-  for (i = 0; i < strlen(flagAbbv); i++) {
+  for (i = 0; i < flagAbbv.size(); i++) {
 
     charPosition = caseName.find_first_of(tolower(flagAbbv[i]), 0);
 
@@ -387,8 +444,8 @@ const std::string FlagType::label() const
   } else {
     /* non-team flags */
     caseName += TextUtils::format(" (%c%s)",
-				    flagQuality==FlagGood?'+':'-',
-				    flagAbbv);
+				  flagQuality==FlagGood?'+':'-',
+				  flagAbbv.c_str());
   }
 
   return caseName;
@@ -398,8 +455,8 @@ const std::string FlagType::label() const
 const std::string FlagType::information() const
 {
   return TextUtils::format("%s: %s",
-			     label().c_str(),
-			     flagHelp);
+			   label().c_str(),
+			   flagHelp.c_str());
 }
 
 // Local Variables: ***
