@@ -1,9 +1,9 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
@@ -25,6 +25,7 @@
 #include <iostream>
 #include <math.h>
 #include <string>
+#include <string.h>
 
 // local implementation headers
 #include "ErrorHandler.h"
@@ -80,134 +81,139 @@ template <>
 StateDatabase* Singleton<StateDatabase>::_instance = (StateDatabase*)0;
 
 
-const std::string StateDatabase::BZDB_AGILITYADVEL	= std::string("_agilityAdVel");
-const std::string StateDatabase::BZDB_AGILITYTIMEWINDOW	= std::string("_agilityTimeWindow");
-const std::string StateDatabase::BZDB_AGILITYVELDELTA	= std::string("_agilityVelDelta");
-const std::string StateDatabase::BZDB_AMBIENTLIGHT	= std::string("_ambientLight");
-const std::string StateDatabase::BZDB_ANGLETOLERANCE    = std::string("_angleTolerance");
-const std::string StateDatabase::BZDB_ANGULARAD		= std::string("_angularAd");
-const std::string StateDatabase::BZDB_AVENUESIZE	= std::string("_avenueSize");
-const std::string StateDatabase::BZDB_BASESIZE		= std::string("_baseSize");
-const std::string StateDatabase::BZDB_BOXBASE		= std::string("_boxBase");
-const std::string StateDatabase::BZDB_BOXHEIGHT		= std::string("_boxHeight");
-const std::string StateDatabase::BZDB_BURROWDEPTH       = std::string("_burrowDepth");
-const std::string StateDatabase::BZDB_BURROWSPEEDAD     = std::string("_burrowSpeedAd");
-const std::string StateDatabase::BZDB_BURROWANGULARAD   = std::string("_burrowAngularAd");
-const std::string StateDatabase::BZDB_COLDETDEPTH       = std::string("_coldetDepth");
-const std::string StateDatabase::BZDB_COLDETELEMENTS    = std::string("_coldetElements");
-const std::string StateDatabase::BZDB_CULLDEPTH		= std::string("_cullDepth");
-const std::string StateDatabase::BZDB_CULLELEMENTS      = std::string("_cullElements");
-const std::string StateDatabase::BZDB_CULLOCCLUDERS     = std::string("_cullOccluders");
-const std::string StateDatabase::BZDB_DISABLEBOTS       = std::string("_disableBots");
-const std::string StateDatabase::BZDB_DRAWCELESTIAL	= std::string("_drawCelestial");
-const std::string StateDatabase::BZDB_DRAWCLOUDS	= std::string("_drawClouds");
-const std::string StateDatabase::BZDB_DRAWGROUND	= std::string("_drawGround");
-const std::string StateDatabase::BZDB_DRAWGROUNDLIGHTS	= std::string("_drawGroundLights");
-const std::string StateDatabase::BZDB_DRAWMOUNTAINS	= std::string("_drawMountains");
-const std::string StateDatabase::BZDB_DRAWSKY		= std::string("_drawSky");
-const std::string StateDatabase::BZDB_ENDSHOTDETECTION  = std::string("_endShotDetection");
-const std::string StateDatabase::BZDB_EXPLODETIME       = std::string("_explodeTime");
-const std::string StateDatabase::BZDB_FLAGALTITUDE      = std::string("_flagAltitude");
-const std::string StateDatabase::BZDB_FLAGEFFECTTIME    = std::string("_flagEffectTime");
-const std::string StateDatabase::BZDB_FLAGHEIGHT	= std::string("_flagHeight");
-const std::string StateDatabase::BZDB_FLAGPOLEWIDTH     = std::string("_flagPoleWidth");
-const std::string StateDatabase::BZDB_FLAGPOLESIZE      = std::string("_flagPoleSize");
-const std::string StateDatabase::BZDB_FLAGRADIUS	= std::string("_flagRadius");
-const std::string StateDatabase::BZDB_FOGMODE		= std::string("_fogMode");
-const std::string StateDatabase::BZDB_FOGDENSITY	= std::string("_fogDensity");
-const std::string StateDatabase::BZDB_FOGSTART		= std::string("_fogStart");
-const std::string StateDatabase::BZDB_FOGEND		= std::string("_fogEnd");
-const std::string StateDatabase::BZDB_FOGCOLOR		= std::string("_fogColor");
-const std::string StateDatabase::BZDB_FRICTION		= std::string("_friction");
-const std::string StateDatabase::BZDB_GMACTIVATIONTIME	= std::string("_gmActivationTime");
-const std::string StateDatabase::BZDB_GMADLIFE		= std::string("_gmAdLife");
-const std::string StateDatabase::BZDB_GMTURNANGLE	= std::string("_gmTurnAngle");
-const std::string StateDatabase::BZDB_GRAVITY		= std::string("_gravity");
-const std::string StateDatabase::BZDB_HANDICAPSCOREDIFF = std::string("_handicapScoreDiff");
-const std::string StateDatabase::BZDB_HANDICAPVELAD	= std::string("_handicapVelAd");
-const std::string StateDatabase::BZDB_HANDICAPANGAD	= std::string("_handicapAngAd");
-const std::string StateDatabase::BZDB_HANDICAPSHOTAD	= std::string("_handicapShotAd");
-const std::string StateDatabase::BZDB_IDENTIFYRANGE	= std::string("_identifyRange");
-const std::string StateDatabase::BZDB_JUMPVELOCITY	= std::string("_jumpVelocity");
-const std::string StateDatabase::BZDB_LASERADVEL	= std::string("_laserAdVel");
-const std::string StateDatabase::BZDB_LASERADRATE	= std::string("_laserAdRate");
-const std::string StateDatabase::BZDB_LASERADLIFE	= std::string("_laserAdLife");
-const std::string StateDatabase::BZDB_LATITUDE		= std::string("_latitude");
-const std::string StateDatabase::BZDB_LOCKONANGLE	= std::string("_lockOnAngle");
-const std::string StateDatabase::BZDB_LONGITUDE		= std::string("_longitude");
-const std::string StateDatabase::BZDB_LRADRATE		= std::string("_lRAdRate");
-const std::string StateDatabase::BZDB_MAXBUMPHEIGHT     = std::string("_maxBumpHeight");
-const std::string StateDatabase::BZDB_MAXFLAGGRABS      = std::string("_maxFlagGrabs");
-const std::string StateDatabase::BZDB_MAXLOD		= std::string("_maxLOD");
-const std::string StateDatabase::BZDB_MIRROR		= std::string("_mirror");
-const std::string StateDatabase::BZDB_MOMENTUMLINACC	= std::string("_momentumLinAcc");
-const std::string StateDatabase::BZDB_MOMENTUMANGACC	= std::string("_momentumAngAcc");
-const std::string StateDatabase::BZDB_MOMENTUMFRICTION	= std::string("_momentumFriction");
-const std::string StateDatabase::BZDB_MGUNADVEL		= std::string("_mGunAdVel");
-const std::string StateDatabase::BZDB_MGUNADRATE	= std::string("_mGunAdRate");
-const std::string StateDatabase::BZDB_MGUNADLIFE	= std::string("_mGunAdLife");
-const std::string StateDatabase::BZDB_MUZZLEFRONT	= std::string("_muzzleFront");
-const std::string StateDatabase::BZDB_MUZZLEHEIGHT	= std::string("_muzzleHeight");
-const std::string StateDatabase::BZDB_NOCLIMB		= std::string("_noClimb");
-const std::string StateDatabase::BZDB_NOSHADOWS		= std::string("_noShadows");
-const std::string StateDatabase::BZDB_NOSMALLPACKETS    = std::string("_noSmallPackets");
-const std::string StateDatabase::BZDB_NOTRESPONDINGTIME = std::string("_notRespondingTime");
-const std::string StateDatabase::BZDB_OBESEFACTOR       = std::string("_obeseFactor");
-const std::string StateDatabase::BZDB_PAUSEDROPTIME	= std::string("_pauseDropTime");
-const std::string StateDatabase::BZDB_POSITIONTOLERANCE = std::string("_positionTolerance");
-const std::string StateDatabase::BZDB_PYRBASE		= std::string("_pyrBase");
-const std::string StateDatabase::BZDB_PYRHEIGHT		= std::string("_pyrHeight");
-const std::string StateDatabase::BZDB_RADARLIMIT	= std::string("_radarLimit");
-const std::string StateDatabase::BZDB_REJOINTIME	= std::string("_rejoinTime");
-const std::string StateDatabase::BZDB_RELOADTIME	= std::string("_reloadTime");
-const std::string StateDatabase::BZDB_RFIREADVEL	= std::string("_rFireAdVel");
-const std::string StateDatabase::BZDB_RFIREADRATE       = std::string("_rFireAdRate");
-const std::string StateDatabase::BZDB_RFIREADLIFE       = std::string("_rFireAdLife");
-const std::string StateDatabase::BZDB_SHIELDFLIGHT      = std::string("_shieldFlight");
-const std::string StateDatabase::BZDB_SHOCKADLIFE       = std::string("_shockAdLife");
-const std::string StateDatabase::BZDB_SHOCKINRADIUS     = std::string("_shockInRadius");
-const std::string StateDatabase::BZDB_SHOCKOUTRADIUS    = std::string("_shockOutRadius");
-const std::string StateDatabase::BZDB_SHOTRADIUS	= std::string("_shotRadius");
-const std::string StateDatabase::BZDB_SHOTRANGE		= std::string("_shotRange");
-const std::string StateDatabase::BZDB_SHOTSPEED		= std::string("_shotSpeed");
-const std::string StateDatabase::BZDB_SHOTTAILLENGTH	= std::string("_shotTailLength");
-const std::string StateDatabase::BZDB_SHOTSKEEPVERTICALV= std::string("_shotsKeepVerticalVelocity");
-const std::string StateDatabase::BZDB_SPEEDCHECKSLOGONLY= std::string("_speedChecksLogOnly");
-const std::string StateDatabase::BZDB_SRRADIUSMULT	= std::string("_srRadiusMult");
-const std::string StateDatabase::BZDB_SQUISHFACTOR	= std::string("_squishFactor");
-const std::string StateDatabase::BZDB_SQUISHTIME	= std::string("_squishTime");
-const std::string StateDatabase::BZDB_SYNCTIME		= std::string("_syncTime");
-const std::string StateDatabase::BZDB_SYNCLOCATION      = std::string("_syncLocation");
-const std::string StateDatabase::BZDB_TANKANGVEL	= std::string("_tankAngVel");
-const std::string StateDatabase::BZDB_TANKEXPLOSIONSIZE = std::string("_tankExplosionSize");
-const std::string StateDatabase::BZDB_TANKHEIGHT	= std::string("_tankHeight");
-const std::string StateDatabase::BZDB_TANKLENGTH	= std::string("_tankLength");
-const std::string StateDatabase::BZDB_TANKRADIUS	= std::string("_tankRadius");
-const std::string StateDatabase::BZDB_TANKSPEED		= std::string("_tankSpeed");
-const std::string StateDatabase::BZDB_TANKWIDTH		= std::string("_tankWidth");
-const std::string StateDatabase::BZDB_TARGETINGANGLE	= std::string("_targetingAngle");
-const std::string StateDatabase::BZDB_TELEBREADTH       = std::string("_teleportBreadth");
-const std::string StateDatabase::BZDB_TELEHEIGHT	= std::string("_teleportHeight");
-const std::string StateDatabase::BZDB_TELEPORTTIME      = std::string("_teleportTime");
-const std::string StateDatabase::BZDB_TELEWIDTH		= std::string("_teleportWidth");
-const std::string StateDatabase::BZDB_THIEFADLIFE	= std::string("_thiefAdLife");
-const std::string StateDatabase::BZDB_THIEFADRATE       = std::string("_thiefAdRate");
-const std::string StateDatabase::BZDB_THIEFADSHOTVEL    = std::string("_thiefAdShotVel");
-const std::string StateDatabase::BZDB_THIEFTINYFACTOR   = std::string("_thiefTinyFactor");
-const std::string StateDatabase::BZDB_THIEFVELAD	= std::string("_thiefVelAd");
-const std::string StateDatabase::BZDB_THIEFDROPTIME     = std::string("_thiefDropTime");
-const std::string StateDatabase::BZDB_TINYFACTOR	= std::string("_tinyFactor");
-const std::string StateDatabase::BZDB_TRACKFADE		= std::string("_trackFade");
-const std::string StateDatabase::BZDB_UPDATETHROTTLERATE= std::string("_updateThrottleRate");
-const std::string StateDatabase::BZDB_VELOCITYAD	= std::string("_velocityAd");
-const std::string StateDatabase::BZDB_WALLHEIGHT	= std::string("_wallHeight");
-const std::string StateDatabase::BZDB_WEAPONS		= std::string("_weapons");
-const std::string StateDatabase::BZDB_WIDEANGLEANG      = std::string("_wideAngleAng");
-const std::string StateDatabase::BZDB_WINGSGRAVITY      = std::string("_wingsGravity");
-const std::string StateDatabase::BZDB_WINGSJUMPCOUNT    = std::string("_wingsJumpCount");
-const std::string StateDatabase::BZDB_WINGSJUMPVELOCITY = std::string("_wingsJumpVelocity");
-const std::string StateDatabase::BZDB_WINGSSLIDETIME    = std::string("_wingsSlideTime");
-const std::string StateDatabase::BZDB_WORLDSIZE		= std::string("_worldSize");
+const std::string StateDatabase::BZDB_AGILITYADVEL	   = std::string("_agilityAdVel");
+const std::string StateDatabase::BZDB_AGILITYTIMEWINDOW	   = std::string("_agilityTimeWindow");
+const std::string StateDatabase::BZDB_AGILITYVELDELTA	   = std::string("_agilityVelDelta");
+const std::string StateDatabase::BZDB_AMBIENTLIGHT	   = std::string("_ambientLight");
+const std::string StateDatabase::BZDB_ANGLETOLERANCE       = std::string("_angleTolerance");
+const std::string StateDatabase::BZDB_ANGULARAD		   = std::string("_angularAd");
+const std::string StateDatabase::BZDB_AVENUESIZE	   = std::string("_avenueSize");
+const std::string StateDatabase::BZDB_BASESIZE		   = std::string("_baseSize");
+const std::string StateDatabase::BZDB_BOXBASE		   = std::string("_boxBase");
+const std::string StateDatabase::BZDB_BOXHEIGHT		   = std::string("_boxHeight");
+const std::string StateDatabase::BZDB_BURROWDEPTH	  = std::string("_burrowDepth");
+const std::string StateDatabase::BZDB_BURROWSPEEDAD	= std::string("_burrowSpeedAd");
+const std::string StateDatabase::BZDB_BURROWANGULARAD      = std::string("_burrowAngularAd");
+const std::string StateDatabase::BZDB_COLDETDEPTH	  = std::string("_coldetDepth");
+const std::string StateDatabase::BZDB_COLDETELEMENTS       = std::string("_coldetElements");
+const std::string StateDatabase::BZDB_COUNTDOWNRESDELAY	   = std::string("_countdownResumeDelay");
+const std::string StateDatabase::BZDB_CULLDEPTH		   = std::string("_cullDepth");
+const std::string StateDatabase::BZDB_CULLELEMENTS	 = std::string("_cullElements");
+const std::string StateDatabase::BZDB_CULLOCCLUDERS	= std::string("_cullOccluders");
+const std::string StateDatabase::BZDB_DISABLEBOTS	  = std::string("_disableBots");
+const std::string StateDatabase::BZDB_DRAWCELESTIAL	   = std::string("_drawCelestial");
+const std::string StateDatabase::BZDB_DRAWCLOUDS	   = std::string("_drawClouds");
+const std::string StateDatabase::BZDB_DRAWGROUND	   = std::string("_drawGround");
+const std::string StateDatabase::BZDB_DRAWGROUNDLIGHTS	   = std::string("_drawGroundLights");
+const std::string StateDatabase::BZDB_DRAWMOUNTAINS	   = std::string("_drawMountains");
+const std::string StateDatabase::BZDB_DRAWSKY		   = std::string("_drawSky");
+const std::string StateDatabase::BZDB_ENDSHOTDETECTION     = std::string("_endShotDetection");
+const std::string StateDatabase::BZDB_EXPLODETIME	  = std::string("_explodeTime");
+const std::string StateDatabase::BZDB_FLAGALTITUDE	 = std::string("_flagAltitude");
+const std::string StateDatabase::BZDB_FLAGEFFECTTIME       = std::string("_flagEffectTime");
+const std::string StateDatabase::BZDB_FLAGHEIGHT	   = std::string("_flagHeight");
+const std::string StateDatabase::BZDB_FLAGPOLEWIDTH	= std::string("_flagPoleWidth");
+const std::string StateDatabase::BZDB_FLAGPOLESIZE	 = std::string("_flagPoleSize");
+const std::string StateDatabase::BZDB_FLAGRADIUS	   = std::string("_flagRadius");
+const std::string StateDatabase::BZDB_FOGMODE		   = std::string("_fogMode");
+const std::string StateDatabase::BZDB_FOGDENSITY	   = std::string("_fogDensity");
+const std::string StateDatabase::BZDB_FOGSTART		   = std::string("_fogStart");
+const std::string StateDatabase::BZDB_FOGEND		   = std::string("_fogEnd");
+const std::string StateDatabase::BZDB_FOGCOLOR		   = std::string("_fogColor");
+const std::string StateDatabase::BZDB_FRICTION		   = std::string("_friction");
+const std::string StateDatabase::BZDB_GMACTIVATIONTIME	   = std::string("_gmActivationTime");
+const std::string StateDatabase::BZDB_GMADLIFE		   = std::string("_gmAdLife");
+const std::string StateDatabase::BZDB_GMSIZE		   = std::string("_gmSize");
+const std::string StateDatabase::BZDB_GMTURNANGLE	   = std::string("_gmTurnAngle");
+const std::string StateDatabase::BZDB_GRAVITY		   = std::string("_gravity");
+const std::string StateDatabase::BZDB_HANDICAPSCOREDIFF    = std::string("_handicapScoreDiff");
+const std::string StateDatabase::BZDB_HANDICAPVELAD	   = std::string("_handicapVelAd");
+const std::string StateDatabase::BZDB_HANDICAPANGAD	   = std::string("_handicapAngAd");
+const std::string StateDatabase::BZDB_HANDICAPSHOTAD	   = std::string("_handicapShotAd");
+const std::string StateDatabase::BZDB_HIDEFLAGSONRADAR     = std::string("_hideFlagsOnRadar");
+const std::string StateDatabase::BZDB_HIDETEAMFLAGSONRADAR = std::string("_hideTeamFlagsOnRadar");
+const std::string StateDatabase::BZDB_IDENTIFYRANGE	   = std::string("_identifyRange");
+const std::string StateDatabase::BZDB_JUMPVELOCITY	   = std::string("_jumpVelocity");
+const std::string StateDatabase::BZDB_LASERADVEL	   = std::string("_laserAdVel");
+const std::string StateDatabase::BZDB_LASERADRATE	   = std::string("_laserAdRate");
+const std::string StateDatabase::BZDB_LASERADLIFE	   = std::string("_laserAdLife");
+const std::string StateDatabase::BZDB_LATITUDE		   = std::string("_latitude");
+const std::string StateDatabase::BZDB_LOCKONANGLE	   = std::string("_lockOnAngle");
+const std::string StateDatabase::BZDB_LONGITUDE		   = std::string("_longitude");
+const std::string StateDatabase::BZDB_LRADRATE		   = std::string("_lRAdRate");
+const std::string StateDatabase::BZDB_MAXBUMPHEIGHT	= std::string("_maxBumpHeight");
+const std::string StateDatabase::BZDB_MAXFLAGGRABS	 = std::string("_maxFlagGrabs");
+const std::string StateDatabase::BZDB_MAXLOD		   = std::string("_maxLOD");
+const std::string StateDatabase::BZDB_MIRROR		   = std::string("_mirror");
+const std::string StateDatabase::BZDB_MOMENTUMLINACC	   = std::string("_momentumLinAcc");
+const std::string StateDatabase::BZDB_MOMENTUMANGACC	   = std::string("_momentumAngAcc");
+const std::string StateDatabase::BZDB_MOMENTUMFRICTION	   = std::string("_momentumFriction");
+const std::string StateDatabase::BZDB_MGUNADVEL		   = std::string("_mGunAdVel");
+const std::string StateDatabase::BZDB_MGUNADRATE	   = std::string("_mGunAdRate");
+const std::string StateDatabase::BZDB_MGUNADLIFE	   = std::string("_mGunAdLife");
+const std::string StateDatabase::BZDB_MUZZLEFRONT	   = std::string("_muzzleFront");
+const std::string StateDatabase::BZDB_MUZZLEHEIGHT	   = std::string("_muzzleHeight");
+const std::string StateDatabase::BZDB_NOCLIMB		   = std::string("_noClimb");
+const std::string StateDatabase::BZDB_NOSHADOWS		   = std::string("_noShadows");
+const std::string StateDatabase::BZDB_NOSMALLPACKETS       = std::string("_noSmallPackets");
+const std::string StateDatabase::BZDB_NOTRESPONDINGTIME    = std::string("_notRespondingTime");
+const std::string StateDatabase::BZDB_OBESEFACTOR	  = std::string("_obeseFactor");
+const std::string StateDatabase::BZDB_PAUSEDROPTIME	   = std::string("_pauseDropTime");
+const std::string StateDatabase::BZDB_POSITIONTOLERANCE    = std::string("_positionTolerance");
+const std::string StateDatabase::BZDB_PYRBASE		   = std::string("_pyrBase");
+const std::string StateDatabase::BZDB_PYRHEIGHT		   = std::string("_pyrHeight");
+const std::string StateDatabase::BZDB_RADARLIMIT	   = std::string("_radarLimit");
+const std::string StateDatabase::BZDB_REJOINTIME	   = std::string("_rejoinTime");
+const std::string StateDatabase::BZDB_RELOADTIME	   = std::string("_reloadTime");
+const std::string StateDatabase::BZDB_RFIREADVEL	   = std::string("_rFireAdVel");
+const std::string StateDatabase::BZDB_RFIREADRATE	  = std::string("_rFireAdRate");
+const std::string StateDatabase::BZDB_RFIREADLIFE	  = std::string("_rFireAdLife");
+const std::string StateDatabase::BZDB_SHIELDFLIGHT	 = std::string("_shieldFlight");
+const std::string StateDatabase::BZDB_SHOCKADLIFE	  = std::string("_shockAdLife");
+const std::string StateDatabase::BZDB_SHOCKINRADIUS	= std::string("_shockInRadius");
+const std::string StateDatabase::BZDB_SHOCKOUTRADIUS       = std::string("_shockOutRadius");
+const std::string StateDatabase::BZDB_SHOTRADIUS	   = std::string("_shotRadius");
+const std::string StateDatabase::BZDB_SHOTRANGE		   = std::string("_shotRange");
+const std::string StateDatabase::BZDB_SHOTSPEED		   = std::string("_shotSpeed");
+const std::string StateDatabase::BZDB_SHOTTAILLENGTH	   = std::string("_shotTailLength");
+const std::string StateDatabase::BZDB_SHOTSKEEPVERTICALV   = std::string("_shotsKeepVerticalVelocity");
+const std::string StateDatabase::BZDB_SPEEDCHECKSLOGONLY   = std::string("_speedChecksLogOnly");
+const std::string StateDatabase::BZDB_SRRADIUSMULT	   = std::string("_srRadiusMult");
+const std::string StateDatabase::BZDB_SQUISHFACTOR	   = std::string("_squishFactor");
+const std::string StateDatabase::BZDB_SQUISHTIME	   = std::string("_squishTime");
+const std::string StateDatabase::BZDB_SYNCTIME		   = std::string("_syncTime");
+const std::string StateDatabase::BZDB_SYNCLOCATION	 = std::string("_syncLocation");
+const std::string StateDatabase::BZDB_TANKANGVEL	   = std::string("_tankAngVel");
+const std::string StateDatabase::BZDB_TANKEXPLOSIONSIZE    = std::string("_tankExplosionSize");
+const std::string StateDatabase::BZDB_TANKHEIGHT	   = std::string("_tankHeight");
+const std::string StateDatabase::BZDB_TANKLENGTH	   = std::string("_tankLength");
+const std::string StateDatabase::BZDB_TANKRADIUS	   = std::string("_tankRadius");
+const std::string StateDatabase::BZDB_TANKSPEED		   = std::string("_tankSpeed");
+const std::string StateDatabase::BZDB_TANKWIDTH		   = std::string("_tankWidth");
+const std::string StateDatabase::BZDB_TARGETINGANGLE	   = std::string("_targetingAngle");
+const std::string StateDatabase::BZDB_TARGETINGDISTANCE	   = std::string("_targetingDistance");
+const std::string StateDatabase::BZDB_TELEBREADTH	  = std::string("_teleportBreadth");
+const std::string StateDatabase::BZDB_TELEHEIGHT	   = std::string("_teleportHeight");
+const std::string StateDatabase::BZDB_TELEPORTTIME	 = std::string("_teleportTime");
+const std::string StateDatabase::BZDB_TELEWIDTH		   = std::string("_teleportWidth");
+const std::string StateDatabase::BZDB_THIEFADLIFE	   = std::string("_thiefAdLife");
+const std::string StateDatabase::BZDB_THIEFADRATE	  = std::string("_thiefAdRate");
+const std::string StateDatabase::BZDB_THIEFADSHOTVEL       = std::string("_thiefAdShotVel");
+const std::string StateDatabase::BZDB_THIEFTINYFACTOR      = std::string("_thiefTinyFactor");
+const std::string StateDatabase::BZDB_THIEFVELAD	   = std::string("_thiefVelAd");
+const std::string StateDatabase::BZDB_THIEFDROPTIME	= std::string("_thiefDropTime");
+const std::string StateDatabase::BZDB_TINYFACTOR	   = std::string("_tinyFactor");
+const std::string StateDatabase::BZDB_TRACKFADE		   = std::string("_trackFade");
+const std::string StateDatabase::BZDB_UPDATETHROTTLERATE   = std::string("_updateThrottleRate");
+const std::string StateDatabase::BZDB_VELOCITYAD	   = std::string("_velocityAd");
+const std::string StateDatabase::BZDB_WALLHEIGHT	   = std::string("_wallHeight");
+const std::string StateDatabase::BZDB_WEAPONS		   = std::string("_weapons");
+const std::string StateDatabase::BZDB_WIDEANGLEANG	 = std::string("_wideAngleAng");
+const std::string StateDatabase::BZDB_WINGSGRAVITY	 = std::string("_wingsGravity");
+const std::string StateDatabase::BZDB_WINGSJUMPCOUNT       = std::string("_wingsJumpCount");
+const std::string StateDatabase::BZDB_WINGSJUMPVELOCITY    = std::string("_wingsJumpVelocity");
+const std::string StateDatabase::BZDB_WINGSSLIDETIME       = std::string("_wingsSlideTime");
+const std::string StateDatabase::BZDB_WORLDSIZE		   = std::string("_worldSize");
 
 //
 // StateDatabase::Item
@@ -353,6 +359,17 @@ void			StateDatabase::removeCallback(
   index->second.callbacks.remove(callback, userData);
 }
 
+void StateDatabase::addGlobalCallback(Callback callback, void* userData)
+{
+  globalCallbacks.add(callback, userData);
+}
+
+void StateDatabase::removeGlobalCallback(Callback callback, void* userData)
+{
+  globalCallbacks.remove(callback, userData);
+}
+
+
 bool			StateDatabase::isSet(const std::string& name) const
 {
   debugLookups(name);
@@ -430,7 +447,8 @@ float			StateDatabase::eval(const std::string& name)
   float retn = evaluate(pre);
   variables.erase(ins_it);
 
-  evalCache[name] = retn;
+  if (retn != NaN)
+    evalCache[name] = retn;
   return retn;
 }
 
@@ -515,8 +533,14 @@ StateDatabase::Map::iterator
 
 void			StateDatabase::notify(Map::iterator index)
 {
+  const std::string& name = index->first;
+  const Item& item = index->second;
+
   evalCache.erase(index->first);
-  index->second.callbacks.iterate(&onCallback, const_cast<void*>(static_cast<const void*>(&index->first)));
+
+  void* namePtr = const_cast<void*>(static_cast<const void*>(&name));
+  globalCallbacks.iterate(&onCallback, namePtr);
+  item.callbacks.iterate(&onCallback, namePtr);
 }
 
 bool			StateDatabase::onCallback(Callback callback,
@@ -928,6 +952,10 @@ float StateDatabase::evaluate(Expression e) const
 	}
 	if (evaluationStack.size() == 0) {
 	  // syntax error
+	  // ugly hack, since gcc 2.95 doesn't have <limits>
+	  float NaN;
+	  memset(&NaN, 0xff, sizeof(NaN));
+	  return NaN;
 	}
 	// rvalue and lvalue are switched, since we're reversed
 	rvalue = evaluationStack.top(); evaluationStack.pop();
