@@ -1,20 +1,23 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 // this classes interface
 #include "WordFilter.h"
 
-// implementation-specific headers
+// system headers
+#include <ctype.h>
 #include <string>
+
+// implementation-specific headers
 #ifdef DEBUG
 #  include "TimeKeeper.h"
 #endif
@@ -45,6 +48,9 @@ bool WordFilter::simpleFilter(char *input) const
       endPosition = line.length();
 
     word = line.substr(startPosition, endPosition-startPosition);
+    // transform to lowercase
+    std::transform (word.begin(), word.end(), word.begin(), tolower);
+
     findWord.word = word;
     firstchar = (unsigned char)word[0];
     if (filters[firstchar].find(findWord) != \
@@ -177,18 +183,18 @@ bool WordFilter::aggressiveFilter(char *input) const
 
 	    /* we are in the middle of a word.. see if we can match a prefix before this */
 	    bool foundit =  false;
-	    for (ExpCompareSet::const_iterator j = prefixes.begin();
-		 j != prefixes.end(); ++j) {
-	      if (regexec(j->compiled, sInput.c_str(), 1, match, 0) == 0) {
+	    for (ExpCompareSet::const_iterator k = prefixes.begin();
+		 k != prefixes.end(); ++k) {
+	      if (regexec(k->compiled, sInput.c_str(), 1, match, 0) == 0) {
 
-//std::cout << "checking prefix: " << j->word << std::endl;
+//std::cout << "checking prefix: " << k->word << std::endl;
 
 		if ( (match[0].rm_so > 1) && (isalpha(sInput[match[0].rm_so - 1])) ) {
 		  /* we matched, but we are still in the middle of a word */
 		  continue;
 		}
 
-//std::cout << "matched a prefix! " << j->word << std::endl;
+//std::cout << "matched a prefix! " << k->word << std::endl;
 		if (match[0].rm_eo == startOffset) {
 		  /* perfect prefix match */
 		  startOffset = match[0].rm_so;
@@ -212,11 +218,11 @@ bool WordFilter::aggressiveFilter(char *input) const
 
 	    /* we are at the start of a word, but not at the end, try to get to the end */
 	    bool foundit = false;
-	    for (ExpCompareSet::const_iterator j = suffixes.begin();
-		 j != suffixes.end(); ++j) {
-//std::cout << "checking " << j->word << " against [" << input + endOffset << "]" << std::endl;
+	    for (ExpCompareSet::const_iterator k = suffixes.begin();
+		 k != suffixes.end(); ++k) {
+//std::cout << "checking " << k->word << " against [" << input + endOffset << "]" << std::endl;
 
-	      if (regexec(j->compiled, sInput.c_str() + endOffset, 1, match, 0) == 0) {
+	      if (regexec(k->compiled, sInput.c_str() + endOffset, 1, match, 0) == 0) {
 
 //std::cout << "is " << match[0].rm_eo << " less than " << inputLength - endOffset << std::endl;
 //std::cout << "is alpha =?= " << input[endOffset + match[0].rm_eo + 1] << std::endl;
@@ -228,7 +234,7 @@ bool WordFilter::aggressiveFilter(char *input) const
 		  continue;
 		}
 
-//std::cout << "matched a suffix! " << j->word << std::endl;
+//std::cout << "matched a suffix! " << k->word << std::endl;
 		if (match[0].rm_so == 0) {
 		  /* push the end forward a little since we matched */
 		  endOffset += match[0].rm_eo;
@@ -623,14 +629,14 @@ WordFilter::WordFilter()
 }
 
 /** default copy constructor */
-WordFilter::WordFilter(const WordFilter& filter)
-  : alphabet(filter.alphabet),
-    filterChars(filter.filterChars),
-    suffixes(filter.suffixes),
-    prefixes(filter.prefixes)
+WordFilter::WordFilter(const WordFilter& _filter)
+  : alphabet(_filter.alphabet),
+    filterChars(_filter.filterChars),
+    suffixes(_filter.suffixes),
+    prefixes(_filter.prefixes)
 {
   for (int i=0; i < MAX_FILTER_SETS; i++) {
-    filters[i] = filter.filters[i];
+    filters[i] = _filter.filters[i];
   }
 }
 
@@ -771,6 +777,9 @@ unsigned int WordFilter::loadFromFile(const std::string &fileName, bool verbose)
       }
     }
 
+    // convert the word to lowercase
+    std::transform (filterWord.begin(),filterWord.end(), filterWord.begin(), tolower);
+
     bool added = addToFilter(filterWord, std::string(""));
     if ((!added) && (verbose)) {
 	std::cout << std::endl << "Word is already added: " << filterWord << std::endl;
@@ -791,7 +800,7 @@ unsigned int WordFilter::loadFromFile(const std::string &fileName, bool verbose)
  * pattern match (default) catching hundreds of variations per filter
  * word or using a simple exact word match technique (original).
  */
-bool WordFilter::filter(char *input, bool simple) const
+bool WordFilter::filter(char *input, const bool simple) const
 {
 #ifdef DEBUG
   TimeKeeper before = TimeKeeper::getCurrent();
@@ -809,7 +818,7 @@ bool WordFilter::filter(char *input, bool simple) const
   return filtered;
 }
 
-bool WordFilter::filter(std::string &input, bool simple) const
+bool WordFilter::filter(std::string &input, const bool simple) const
 {
   char input2[512] = {0};
   bool filtered = false;
