@@ -1,14 +1,15 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
+
 #if defined(_MSC_VER)
 // turn off bogus `this used in base member initialization list'
 #  pragma warning(disable: 4786)
@@ -21,10 +22,11 @@ const struct GlobalDBItem globalDBItems[] = {
   { "_agilityAdVel",		"2.25",				false, StateDatabase::Locked},
   { "_agilityTimeWindow",	"1.0",				false, StateDatabase::Locked},
   { "_agilityVelDelta",		"0.3",				false, StateDatabase::Locked},
+  { "_ambientLight",		"none",				false, StateDatabase::Locked},
   { "_angleTolerance",		"0.05",				false, StateDatabase::Locked},
   { "_angularAd",		"1.5",				false, StateDatabase::Locked},
   { "_avenueSize",		"2.0*_boxBase",			false, StateDatabase::Locked},
-  { "_baseSize",		"60.0",			 false, StateDatabase::Locked},
+  { "_baseSize",		"60.0",				false, StateDatabase::Locked},
   { "_boxBase",			"30.0",				false, StateDatabase::Locked},
   { "_boxHeight",		"6.0*_muzzleHeight",		false, StateDatabase::Locked},
   { "_burrowAngularAd",		"0.55",				false, StateDatabase::Locked},
@@ -32,9 +34,22 @@ const struct GlobalDBItem globalDBItems[] = {
   { "_burrowSpeedAd",		"0.80",				false, StateDatabase::Locked},
   { "_coldetDepth",		"6",				false, StateDatabase::Locked},
   { "_coldetElements",		"4",				false, StateDatabase::Locked},
+  { "_countdownResumeDelay",	"5",				false, StateDatabase::Locked},
   { "_cullDepth",		"6",				false, StateDatabase::Locked},
+  { "_cullDist",		"fog",				false, StateDatabase::Locked},
   { "_cullElements",		"16",				false, StateDatabase::Locked},
   { "_cullOccluders",		"0",				false, StateDatabase::Locked},
+  { "_disableBots",		"0",				false, StateDatabase::Locked},
+  { "_disableSpeedChecks",	"0",				false, StateDatabase::Locked},
+  { "_disableHeightChecks",	"0",				false, StateDatabase::Locked},
+  { "_drawCelestial",		"1",				false, StateDatabase::Locked},
+  { "_drawClouds",		"1",				false, StateDatabase::Locked},
+  { "_drawGround",		"1",				false, StateDatabase::Locked},
+  { "_drawGroundLights",	"1",				false, StateDatabase::Locked},
+  { "_drawMountains",		"1",				false, StateDatabase::Locked},
+  { "_drawSky",			"1",				false, StateDatabase::Locked},
+  { "_enableDistanceCheck",	"0",				false, StateDatabase::Locked},
+  { "_endShotDetection",	"5",				false, StateDatabase::Locked},
   { "_explodeTime",		"5.0",				false, StateDatabase::Locked},
   { "_flagAltitude",		"11.0",				false, StateDatabase::Locked},
   { "_flagEffectTime",		"0.64",				false, StateDatabase::Locked},
@@ -43,14 +58,27 @@ const struct GlobalDBItem globalDBItems[] = {
   { "_flagPoleSize",		"0.8",				false, StateDatabase::Locked},
   { "_flagRadius",		"2.5",				false, StateDatabase::Locked},
   { "_friction",		"0",				false, StateDatabase::Locked},
+  { "_forbidMarkers",		"0",				false, StateDatabase::Locked},
+  { "_fogMode",			"none",				false, StateDatabase::Locked},
+  { "_fogDensity",		"0.001",			false, StateDatabase::Locked},
+  { "_fogStart",		"0.5*_worldSize",		false, StateDatabase::Locked},
+  { "_fogEnd",			"_worldSize",			false, StateDatabase::Locked},
+  { "_fogColor",		"0.25 0.25 0.25",		false, StateDatabase::Locked},
+  { "_fogNoSky",		"0",				false, StateDatabase::Locked},
+  { "_forbidIdentify",		"0",				false, StateDatabase::Locked},
+  { "_forbidHunting",		"0",				false, StateDatabase::Locked},
   { "_gmActivationTime",	"0.5",				false, StateDatabase::Locked},
   { "_gmAdLife",		"0.95",				false, StateDatabase::Locked},
+  { "_gmSize",			"1.5",				false, StateDatabase::Locked},
   { "_gmTurnAngle",		"0.628319",			false, StateDatabase::Locked},
   { "_gravity",			"-9.8",				false, StateDatabase::Locked},
   { "_handicapScoreDiff",	"50.0",				false, StateDatabase::Locked},
   { "_handicapVelAd",		"2.0",				false, StateDatabase::Locked},
   { "_handicapAngAd",		"1.5",				false, StateDatabase::Locked},
   { "_handicapShotAd",		"1.75",				false, StateDatabase::Locked},
+  { "_hideFlagsOnRadar",	"0",			    false, StateDatabase::Locked},
+  { "_hideTeamFlagsOnRadar",    "0",			    false, StateDatabase::Locked},
+  { "_HTTPIndexResourceDir",    "",				false, StateDatabase::Locked},
   { "_identifyRange",		"50.0",				false, StateDatabase::Locked},
   { "_jumpVelocity",		"19.0",				false, StateDatabase::Locked},
   { "_laserAdLife",		"0.1",				false, StateDatabase::Locked},
@@ -113,6 +141,11 @@ const struct GlobalDBItem globalDBItems[] = {
   { "_shotSpeed",		"100.0",			false, StateDatabase::Locked},
   { "_shotsKeepVerticalVelocity",	"0",			false, StateDatabase::Locked},
   { "_skyColor",		"white",			false, StateDatabase::Locked},
+  { "_spawnMaxCompTime",	"0.01",				false, StateDatabase::Locked},
+  { "_spawnSafeRadMod",		"20",				false, StateDatabase::Locked},
+  { "_spawnSafeSRMod",		"3",				false, StateDatabase::Locked},
+  { "_spawnSafeSWMod",		"1.5",				false, StateDatabase::Locked},
+  { "_speedChecksLogOnly",	"0",				false, StateDatabase::Locked},
   { "_srRadiusMult",		"2.0",				false, StateDatabase::Locked},
   { "_squishFactor",		"1.0",				false, StateDatabase::Locked},
   { "_squishTime",		"1.0",				false, StateDatabase::Locked},
@@ -126,6 +159,7 @@ const struct GlobalDBItem globalDBItems[] = {
   { "_tankSpeed",		"25.0",				false, StateDatabase::Locked},
   { "_tankWidth",		"2.8",				false, StateDatabase::Locked},
   { "_targetingAngle",		"0.3",				false, StateDatabase::Locked},
+  { "_targetingDistance",	"300.00",			false, StateDatabase::Locked},
   { "_teleportBreadth",		"4.48",				false, StateDatabase::Locked},
   { "_teleportHeight",		"10.08",			false, StateDatabase::Locked},
   { "_teleportTime",		"1.0",				false, StateDatabase::Locked},
