@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <string.h>
@@ -53,6 +53,10 @@ const BzMaterial* BzMaterialManager::addMaterial(const BzMaterial* material)
 {
   for (unsigned int i = 0; i < materials.size(); i++) {
     if (*material == *(materials[i])) {
+      const std::string& name = material->getName();
+      if (name.size() > 0) {
+	materials[i]->addAlias(name);
+      }
       return materials[i];
     }
   }
@@ -80,8 +84,17 @@ const BzMaterial* BzMaterialManager::findMaterial(const std::string& target) con
   }
   else {
     for (unsigned int i = 0; i < materials.size(); i++) {
-      if (target == materials[i]->getName()) {
-	return materials[i];
+      const BzMaterial* mat = materials[i];
+      // check the base name
+      if (target == mat->getName()) {
+	return mat;
+      }
+      // check the aliases
+      const std::vector<std::string>& aliases = mat->getAliases();
+      for (unsigned int j = 0; j < aliases.size(); j++) {
+	if (target == aliases[j]) {
+	  return mat;
+	}
       }
     }
     return NULL;
@@ -115,6 +128,7 @@ void* BzMaterialManager::pack(void* buf)
   for (unsigned int i = 0; i < materials.size(); i++) {
     buf = materials[i]->pack(buf);
   }
+
   return buf;
 }
 
@@ -152,6 +166,15 @@ void BzMaterialManager::print(std::ostream& out, const std::string& indent) cons
 }
 
 
+void BzMaterialManager::printMTL(std::ostream& out, const std::string& indent) const
+{
+  for (unsigned int i = 0; i < materials.size(); i++) {
+    materials[i]->printMTL(out, indent);
+  }
+  return;
+}
+
+
 void BzMaterialManager::printReference(std::ostream& out,
 				       const BzMaterial* mat) const
 {
@@ -174,6 +197,36 @@ void BzMaterialManager::printReference(std::ostream& out,
 }
 
 
+void BzMaterialManager::makeTextureList(TextureSet& set, bool referenced) const
+{
+  set.clear();
+  for (unsigned int i = 0; i < materials.size(); i++) {
+    const BzMaterial* mat = materials[i];
+    for (int j = 0; j < mat->getTextureCount(); j++) {
+      if (mat->getReference() || !referenced) {
+	set.insert(mat->getTexture(j));
+      }
+    }
+  }
+  return;
+}
+
+
+void BzMaterialManager::setTextureLocal(const std::string& url,
+					const std::string& local)
+{
+  for (unsigned int i = 0; i < materials.size(); i++) {
+    BzMaterial* mat = materials[i];
+    for (int j = 0; j < mat->getTextureCount(); j++) {
+      if (mat->getTexture(j) == url) {
+	mat->setTextureLocal(j, local);
+      }
+    }
+  }
+  return;
+}
+
+
 /****************************************************************************/
 //
 // BzMaterial
@@ -196,8 +249,13 @@ void BzMaterial::reset()
   memcpy (emission, defEmission, sizeof(emission));
   shininess = 0.0f;
   alphaThreshold = 0.0f;
+  occluder = false;
+  groupAlpha = false;
+  noRadar = false;
+  noShadow = false;
   noCulling = false;
   noSorting = false;
+  noLighting = false;
 
   delete[] textures;
   textures = NULL;
@@ -207,6 +265,8 @@ void BzMaterial::reset()
   shaders = NULL;
   shaderCount = 0;
 
+  referenced = false;
+
   return;
 }
 
@@ -241,7 +301,10 @@ BzMaterial& BzMaterial::operator=(const BzMaterial& m)
 {
   int i;
 
+  referenced = false;
+
   name = m.name;
+  aliases = m.aliases;
 
   dynamicColor = m.dynamicColor;
   memcpy (ambient, m.ambient, sizeof(ambient));
@@ -250,8 +313,13 @@ BzMaterial& BzMaterial::operator=(const BzMaterial& m)
   memcpy (emission, m.emission, sizeof(emission));
   shininess = m.shininess;
   alphaThreshold = m.alphaThreshold;
+  occluder = m.occluder;
+  groupAlpha = m.groupAlpha;
+  noRadar = m.noRadar;
+  noShadow = m.noShadow;
   noCulling = m.noCulling;
   noSorting = m.noSorting;
+  noLighting = m.noLighting;
 
   delete[] textures;
   textureCount = m.textureCount;
@@ -288,9 +356,11 @@ bool BzMaterial::operator==(const BzMaterial& m) const
       (memcmp (diffuse, m.diffuse, sizeof(float[4])) != 0) ||
       (memcmp (specular, m.specular, sizeof(float[4])) != 0) ||
       (memcmp (emission, m.emission, sizeof(float[4])) != 0) ||
-      (shininess != m.shininess) ||
-      (alphaThreshold != m.alphaThreshold) ||
-      (noCulling != m.noCulling) || (noSorting != m.noSorting)) {
+      (shininess != m.shininess) || (alphaThreshold != m.alphaThreshold) ||
+      (occluder != m.occluder) || (groupAlpha != m.groupAlpha) ||
+      (noRadar != m.noRadar) || (noShadow != m.noShadow) ||
+      (noCulling != m.noCulling) || (noSorting != m.noSorting) ||
+      (noLighting != m.noLighting)) {
     return false;
   }
 
@@ -346,10 +416,15 @@ void* BzMaterial::pack(void* buf) const
   int i;
 
   buf = nboPackStdString(buf, name);
-  
+
   uint8_t modeByte = 0;
-  if (noCulling) modeByte |= (1 << 0);
-  if (noSorting) modeByte |= (1 << 1);
+  if (noCulling)	modeByte |= (1 << 0);
+  if (noSorting)	modeByte |= (1 << 1);
+  if (noRadar)		modeByte |= (1 << 2);
+  if (noShadow)		modeByte |= (1 << 3);
+  if (occluder)	 modeByte |= (1 << 4);
+  if (groupAlpha)       modeByte |= (1 << 5);
+  if (noLighting)       modeByte |= (1 << 6);
   buf = nboPackUByte(buf, modeByte);
 
   buf = nboPackInt(buf, dynamicColor);
@@ -363,6 +438,7 @@ void* BzMaterial::pack(void* buf) const
   buf = nboPackUByte(buf, textureCount);
   for (i = 0; i < textureCount; i++) {
     const TextureInfo* texinfo = &textures[i];
+
     buf = nboPackStdString(buf, texinfo->name);
     buf = nboPackInt(buf, texinfo->matrix);
     buf = nboPackInt(buf, texinfo->combineMode);
@@ -397,8 +473,13 @@ void* BzMaterial::unpack(void* buf)
 
   uint8_t modeByte;
   buf = nboUnpackUByte(buf, modeByte);
-  noCulling = (modeByte & (1 << 0)) != 0;
-  noSorting = (modeByte & (1 << 1)) != 0;
+  noCulling	= (modeByte & (1 << 0)) != 0;
+  noSorting	= (modeByte & (1 << 1)) != 0;
+  noRadar	= (modeByte & (1 << 2)) != 0;
+  noShadow	= (modeByte & (1 << 3)) != 0;
+  occluder	= (modeByte & (1 << 4)) != 0;
+  groupAlpha    = (modeByte & (1 << 5)) != 0;
+  noLighting    = (modeByte & (1 << 6)) != 0;
 
   buf = nboUnpackInt(buf, inTmp);
   dynamicColor = int(inTmp);
@@ -416,6 +497,7 @@ void* BzMaterial::unpack(void* buf)
   for (i = 0; i < textureCount; i++) {
     TextureInfo* texinfo = &textures[i];
     buf = nboUnpackStdString(buf, texinfo->name);
+    texinfo->localname = texinfo->name;
     buf = nboUnpackInt(buf, inTmp);
     texinfo->matrix = int(inTmp);
     buf = nboUnpackInt(buf, inTmp);
@@ -451,12 +533,12 @@ void* BzMaterial::unpack(void* buf)
 int BzMaterial::packSize() const
 {
   int i;
-  
+
   const int modeSize = sizeof(uint8_t);
 
   const int colorSize = sizeof(int32_t) + (4 * sizeof(float[4])) +
-                        sizeof(float) + sizeof(float);
-  
+			sizeof(float) + sizeof(float);
+
   int textureSize = sizeof(unsigned char);
   for (i = 0; i < textureCount; i++) {
     textureSize += nboStdStringPackSize(textures[i].name);
@@ -485,18 +567,18 @@ static void printColor(std::ostream& out, const char *name,
 }
 
 
-void BzMaterial::print(std::ostream& out, const std::string& /*indent*/) const
+void BzMaterial::print(std::ostream& out, const std::string& indent) const
 {
   int i;
 
-  out << "material" << std::endl;
+  out << indent << "material" << std::endl;
 
   if (name.size() > 0) {
-    out << "  name " << name << std::endl;
+    out << indent << "  name " << name << std::endl;
   }
 
   if (dynamicColor != defaultMaterial.dynamicColor) {
-    out << "  dyncol ";
+    out << indent << "  dyncol ";
     const DynamicColor* dyncol = DYNCOLORMGR.getColor(dynamicColor);
     if ((dyncol != NULL) && (dyncol->getName().size() > 0)) {
       out << dyncol->getName();
@@ -510,23 +592,38 @@ void BzMaterial::print(std::ostream& out, const std::string& /*indent*/) const
   printColor(out, "  specular ", specular, defaultMaterial.specular);
   printColor(out, "  emission ", emission, defaultMaterial.emission);
   if (shininess != defaultMaterial.shininess) {
-    out << "  shininess " << shininess << std::endl;
+    out << indent << "  shininess " << shininess << std::endl;
   }
   if (alphaThreshold != defaultMaterial.alphaThreshold) {
-    out << "  alphathresh " << alphaThreshold << std::endl;
+    out << indent << "  alphathresh " << alphaThreshold << std::endl;
+  }
+  if (occluder) {
+    out << indent << "  occluder" << std::endl;
+  }
+  if (groupAlpha) {
+    out << indent << "  groupAlpha" << std::endl;
+  }
+  if (noRadar) {
+    out << indent << "  noradar" << std::endl;
+  }
+  if (noShadow) {
+    out << indent << "  noshadow" << std::endl;
   }
   if (noCulling) {
-    out << "  noculling" << std::endl;
+    out << indent << "  noculling" << std::endl;
   }
   if (noSorting) {
-    out << "  nosorting" << std::endl;
+    out << indent << "  nosorting" << std::endl;
+  }
+  if (noLighting) {
+    out << indent << "  nolighting" << std::endl;
   }
 
   for (i = 0; i < textureCount; i++) {
     const TextureInfo* texinfo = &textures[i];
-    out << "  addtexture " << texinfo->name << std::endl;
+    out << indent << "  addtexture " << texinfo->name << std::endl;
     if (texinfo->matrix != -1) {
-      out << "    texmat ";
+      out << indent << "    texmat ";
       const TextureMatrix* texmat = TEXMATRIXMGR.getMatrix(texinfo->matrix);
       if ((texmat != NULL) && (texmat->getName().size() > 0)) {
 	out << texmat->getName();
@@ -537,27 +634,68 @@ void BzMaterial::print(std::ostream& out, const std::string& /*indent*/) const
     }
 
     if (!texinfo->useAlpha) {
-      out << "    notexalpha" << std::endl;
+      out << indent << "    notexalpha" << std::endl;
     }
     if (!texinfo->useColor) {
-      out << "    notexcolor" << std::endl;
+      out << indent << "    notexcolor" << std::endl;
     }
     if (texinfo->useSphereMap) {
-      out << "    spheremap" << std::endl;
+      out << indent << "    spheremap" << std::endl;
     }
   }
 
   for (i = 0; i < shaderCount; i++) {
     const ShaderInfo* shdinfo = &shaders[i];
-    out << "  addshader " << shdinfo->name << std::endl;
+    out << indent << "  addshader " << shdinfo->name << std::endl;
   }
 
-  out << "end" << std::endl << std::endl;
+  out << indent << "end" << std::endl << std::endl;
 
   return;
 }
 
 
+void BzMaterial::printMTL(std::ostream& out, const std::string& UNUSED(indent)) const
+{
+  out << "newmtl ";
+  if (name.size() > 0) {
+    out << name << std::endl;
+  } else {
+    out << MATERIALMGR.getIndex(this) << std::endl;
+  }
+  if (noLighting) {
+    out << "illum 0" << std::endl;
+  } else {
+    out << "illum 2" << std::endl;
+  }
+  out << "d " << diffuse[3] << std::endl;
+  const float* c;
+  c = ambient; // not really used
+  out << "#Ka " << c[0] << " " << c[1] << " " << c[2] << std::endl;
+  c = diffuse;
+  out << "Kd " << c[0] << " " << c[1] << " " << c[2] << std::endl;
+  c = emission;
+  out << "Ke " << c[0] << " " << c[1] << " " << c[2] << std::endl;
+  c = specular;
+  out << "Ks " << c[0] << " " << c[1] << " " << c[2] << std::endl;
+  out << "Ns " << (1000.0f * (shininess / 128.0f)) << std::endl;
+  if (textureCount > 0) {
+    const TextureInfo* ti = &textures[0];
+    const unsigned int nlen = (unsigned int)ti->name.size();
+    if (nlen > 0) {
+      std::string texname = ti->name;
+      const char* cname = texname.c_str();
+      if ((nlen < 4) || (strcasecmp(cname + (nlen - 4), ".png") != 0)) {
+	texname += ".png";
+      }
+      out << "map_Kd " << texname << std::endl;
+    }
+  }
+  out << std::endl;
+  return;
+}
+
+
 /****************************************************************************/
 //
 // Parameter setting
@@ -579,6 +717,27 @@ bool BzMaterial::setName(const std::string& matname)
   return true;
 }
 
+bool BzMaterial::addAlias(const std::string& alias)
+{
+  if (alias.size() <= 0) {
+    name = "";
+    return false;
+  }
+  else if ((alias[0] >= '0') && (alias[0] <= '9')) {
+    name = "";
+    return false;
+  }
+  else {
+	for ( unsigned int i = 0; i < (unsigned int)aliases.size(); i++)
+	{
+		if ( aliases[i] == alias )
+			return true;
+	}
+	aliases.push_back(alias); // only add it if it's new
+  }
+  return true;
+}
+
 void BzMaterial::setDynamicColor(int dyncol)
 {
   dynamicColor = dyncol;
@@ -621,6 +780,30 @@ void BzMaterial::setAlphaThreshold(const float thresh)
   return;
 }
 
+void BzMaterial::setOccluder(bool value)
+{
+  occluder = value;
+  return;
+}
+
+void BzMaterial::setGroupAlpha(bool value)
+{
+  groupAlpha = value;
+  return;
+}
+
+void BzMaterial::setNoRadar(bool value)
+{
+  noRadar = value;
+  return;
+}
+
+void BzMaterial::setNoShadow(bool value)
+{
+  noShadow = value;
+  return;
+}
+
 void BzMaterial::setNoCulling(bool value)
 {
   noCulling = value;
@@ -633,6 +816,12 @@ void BzMaterial::setNoSorting(bool value)
   return;
 }
 
+void BzMaterial::setNoLighting(bool value)
+{
+  noLighting = value;
+  return;
+}
+
 
 void BzMaterial::addTexture(const std::string& texname)
 {
@@ -646,6 +835,7 @@ void BzMaterial::addTexture(const std::string& texname)
 
   TextureInfo* texinfo = &textures[textureCount - 1];
   texinfo->name = texname;
+  texinfo->localname = texname;
   texinfo->matrix = -1;
   texinfo->combineMode = decal;
   texinfo->useAlpha = true;
@@ -666,6 +856,14 @@ void BzMaterial::setTexture(const std::string& texname)
   return;
 }
 
+void BzMaterial::setTextureLocal(int texid, const std::string& localname)
+{
+  if ((texid >= 0) && (texid < textureCount)) {
+    textures[texid].localname = localname;
+  }
+  return;
+}
+
 void BzMaterial::setTextureMatrix(int matrix)
 {
   if (textureCount > 0) {
@@ -760,6 +958,11 @@ const std::string& BzMaterial::getName() const
   return name;
 }
 
+const std::vector<std::string>& BzMaterial::getAliases() const
+{
+  return aliases;
+}
+
 int BzMaterial::getDynamicColor() const
 {
   return dynamicColor;
@@ -795,6 +998,26 @@ float BzMaterial::getAlphaThreshold() const
   return alphaThreshold;
 }
 
+bool BzMaterial::getOccluder() const
+{
+  return occluder;
+}
+
+bool BzMaterial::getGroupAlpha() const
+{
+  return groupAlpha;
+}
+
+bool BzMaterial::getNoRadar() const
+{
+  return noRadar;
+}
+
+bool BzMaterial::getNoShadow() const
+{
+  return noShadow;
+}
+
 bool BzMaterial::getNoCulling() const
 {
   return noCulling;
@@ -805,6 +1028,11 @@ bool BzMaterial::getNoSorting() const
   return noSorting;
 }
 
+bool BzMaterial::getNoLighting() const
+{
+  return noLighting;
+}
+
 
 int BzMaterial::getTextureCount() const
 {
@@ -820,6 +1048,15 @@ const std::string& BzMaterial::getTexture(int texid) const
   }
 }
 
+const std::string& BzMaterial::getTextureLocal(int texid) const
+{
+  if ((texid >= 0) && (texid < textureCount)) {
+    return textures[texid].localname;
+  } else {
+    return nullString;
+  }
+}
+
 int BzMaterial::getTextureMatrix(int texid) const
 {
   if ((texid >= 0) && (texid < textureCount)) {
@@ -881,6 +1118,17 @@ const std::string& BzMaterial::getShader(int shdid) const
 }
 
 
+bool BzMaterial::isInvisible() const
+{
+  const DynamicColor* dyncol = DYNCOLORMGR.getColor(dynamicColor);
+  if ((diffuse[3] == 0.0f) && (dyncol == NULL) &&
+      !((textureCount > 0) && !textures[0].useColor)) {
+    return true;
+  }
+  return false;
+}
+
+
 // Local Variables: ***
 // mode:C++ ***
 // tab-width: 8 ***
