@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -293,13 +293,6 @@ MeshTransform::Tool::Tool(const MeshTransform& xform)
     inverted = false;
   }
 
-  // FIXME - remove this check when protocol changes from "0026"
-  const float badcheck_2_0_0 = vm[0][0] * vm[1][1] * vm[2][2];
-  if ((determinant * badcheck_2_0_0) < 0.0f) {
-    printf ("WARNING:  MeshTransform::Tool::Tool()  2.0.0 inversion bug\n");
-    printf ("	  The most likely cause is a 'spin' transformation\n");
-  }
-
   return;
 }
 
