@@ -1,13 +1,13 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 
@@ -53,7 +53,7 @@ static FlagListType   FlagList;
 static PlayerListType PlayerList;
 static bool UseDNS = true;
 static bool Colorize = true;
-static bool ShowEmail = true;
+static bool ShowMotto = true;
 static bool TrackState = true;
 
 static MsgStringList listMsgBasics (PacketInfo *pi);
@@ -66,11 +66,6 @@ static std::string strVector (float *vector);
 static std::string strKillReason (int16_t reason);
 static std::string strAddress (Address& address);
 
-/*
-static std::string strEmail ();
-static std::string strHostName (); // ew, ADNS ?
-*/
-
 static MsgStringList handleMsgNull(PacketInfo *pi); // fake packet type
 static MsgStringList handleMsgAccept(PacketInfo *pi);
 static MsgStringList handleMsgAlive(PacketInfo *pi);
@@ -81,11 +76,14 @@ static MsgStringList handleMsgDropFlag(PacketInfo *pi);
 static MsgStringList handleMsgEnter(PacketInfo *pi);
 static MsgStringList handleMsgExit(PacketInfo *pi);
 static MsgStringList handleMsgFlagUpdate(PacketInfo *pi);
+static MsgStringList handleMsgGameTime(PacketInfo *pi);
 static MsgStringList handleMsgGrabFlag(PacketInfo *pi);
 static MsgStringList handleMsgGMUpdate(PacketInfo *pi);
 static MsgStringList handleMsgGetWorld(PacketInfo *pi);
+static MsgStringList handleMsgGameSettings(PacketInfo *pi);
 static MsgStringList handleMsgKilled(PacketInfo *pi);
 static MsgStringList handleMsgMessage(PacketInfo *pi);
+static MsgStringList handleMsgNearFlag(PacketInfo *pi);
 static MsgStringList handleMsgNewRabbit(PacketInfo *pi);
 static MsgStringList handleMsgNegotiateFlags(PacketInfo *pi);
 static MsgStringList handleMsgPause(PacketInfo *pi);
@@ -108,10 +106,13 @@ static MsgStringList handleMsgTeleport(PacketInfo *pi);
 static MsgStringList handleMsgTransferFlag(PacketInfo *pi);
 static MsgStringList handleMsgTeamUpdate(PacketInfo *pi);
 static MsgStringList handleMsgWantWHash(PacketInfo *pi);
+static MsgStringList handleMsgWantSettings(PacketInfo *pi);
 static MsgStringList handleMsgUDPLinkRequest(PacketInfo *pi);
 static MsgStringList handleMsgUDPLinkEstablished(PacketInfo *pi);
 static MsgStringList handleMsgServerControl(PacketInfo *pi);
 static MsgStringList handleMsgLagPing(PacketInfo *pi);
+static MsgStringList handleMsgPingCodeReply(PacketInfo *pi);
+static MsgStringList handleMsgPingCodeRequest(PacketInfo *pi);
 
 
 typedef struct {
@@ -133,11 +134,14 @@ static PacketListEntry PacketList[] = {
   PACKET_LIST_ENTRY (MsgEnter),
   PACKET_LIST_ENTRY (MsgExit),
   PACKET_LIST_ENTRY (MsgFlagUpdate),
+  PACKET_LIST_ENTRY (MsgGameTime),
   PACKET_LIST_ENTRY (MsgGrabFlag),
   PACKET_LIST_ENTRY (MsgGMUpdate),
   PACKET_LIST_ENTRY (MsgGetWorld),
+  PACKET_LIST_ENTRY (MsgGameSettings),
   PACKET_LIST_ENTRY (MsgKilled),
   PACKET_LIST_ENTRY (MsgMessage),
+  PACKET_LIST_ENTRY (MsgNearFlag),
   PACKET_LIST_ENTRY (MsgNewRabbit),
   PACKET_LIST_ENTRY (MsgNegotiateFlags),
   PACKET_LIST_ENTRY (MsgPause),
@@ -160,10 +164,13 @@ static PacketListEntry PacketList[] = {
   PACKET_LIST_ENTRY (MsgTransferFlag),
   PACKET_LIST_ENTRY (MsgTeamUpdate),
   PACKET_LIST_ENTRY (MsgWantWHash),
+  PACKET_LIST_ENTRY (MsgWantSettings),
   PACKET_LIST_ENTRY (MsgUDPLinkRequest),
   PACKET_LIST_ENTRY (MsgUDPLinkEstablished),
   PACKET_LIST_ENTRY (MsgServerControl),
-  PACKET_LIST_ENTRY (MsgLagPing)
+  PACKET_LIST_ENTRY (MsgLagPing),
+  PACKET_LIST_ENTRY (MsgPingCodeReply),
+  PACKET_LIST_ENTRY (MsgPingCodeRequest)
 };
 static const int PacketListCount = sizeof (PacketList) / sizeof (PacketList[0]);
 
@@ -184,7 +191,7 @@ void MsgStrings::reset ()
 
   UseDNS = true;
   Colorize = true;
-  ShowEmail = true;
+  ShowMotto = true;
   TrackState = true;
 
   FlagList.clear();
@@ -195,12 +202,12 @@ void MsgStrings::reset ()
   PlayerList[AllPlayers] = "ALL";
   PlayerList[AdminPlayers] = "ADMIN";
   PlayerList[NoPlayer] = "NOPLAYER";
-  for (i=244 ; i<=250; i++) {
-    PlayerList[i] = Team::getName (TeamColor(250 - i));
+  for (i = LastRealPlayer + 1 ; i <= FirstTeam; i++) {
+    PlayerList[i] = Team::getName (TeamColor(FirstTeam - i));
   }
 
   // set default DB entries
-  for (unsigned int i = 0; i < numGlobalDBItems; ++i) {
+  for (i = 0; i < numGlobalDBItems; ++i) {
     assert(globalDBItems[i].name != NULL);
     if (globalDBItems[i].value != NULL) {
       BZDB.set(globalDBItems[i].name, globalDBItems[i].value);
@@ -221,9 +228,9 @@ void MsgStrings::useDNS (bool value)
 }
 
 
-void MsgStrings::showEmail (bool value)
+void MsgStrings::showMotto (bool value)
 {
-  ShowEmail = value;
+  ShowMotto = value;
   return;
 }
 
@@ -509,7 +516,7 @@ static MsgStringList handleMsgAddPlayer (PacketInfo *pi)
   u8 index;
   u16 type, team, wins, losses, tks;
   char callsign[CallSignLen];
-  char email[EmailLen];
+  char motto[MottoLen];
 
   d = nboUnpackUByte (d, index);
   d = nboUnpackUShort (d, type);
@@ -518,13 +525,13 @@ static MsgStringList handleMsgAddPlayer (PacketInfo *pi)
   d = nboUnpackUShort (d, losses);
   d = nboUnpackUShort (d, tks);
   d = nboUnpackString (d, callsign, CallSignLen);
-  d = nboUnpackString (d, email, EmailLen);
+  d = nboUnpackString (d, motto, MottoLen);
 
   if (TrackState) {
     PlayerList[index] = callsign;
   }
   listPush (list, 1, "player: %s", strPlayer(index).c_str());
-  listPush (list, 1, "email:  %s", email);
+  listPush (list, 1, "motto:  %s", motto);
   listPush (list, 1, "team:   %s", strTeam(team).c_str());
   listPush (list, 1, "type:   %i", type);
   listPush (list, 2, "wins:   %i", wins);
@@ -654,6 +661,14 @@ static MsgStringList handleMsgGetWorld (PacketInfo *pi)
 }
 
 
+static MsgStringList handleMsgGameSettings (PacketInfo *pi)
+{
+  // not recorded
+  MsgStringList list = listMsgBasics (pi);
+  return list;
+}
+
+
 static MsgStringList handleMsgKilled (PacketInfo *pi)
 {
   MsgStringList list = listMsgBasics (pi);
@@ -677,7 +692,7 @@ static MsgStringList handleMsgKilled (PacketInfo *pi)
   listPush (list, 1, "shotid: %i", shot);
 
   if (flagType != Flags::Null) {
-    listPush (list, 1, "flag:   %s", flagType->flagAbbv);
+    listPush (list, 1, "flag:   %s", flagType->flagAbbv.c_str());
   } else {
     listPush (list, 1, "flag:   Null");
   }
@@ -705,6 +720,14 @@ static MsgStringList handleMsgMessage (PacketInfo *pi)
 }
 
 
+static MsgStringList handleMsgNearFlag (PacketInfo *pi)
+{
+  // not recorded
+  MsgStringList list = listMsgBasics (pi);
+  return list;
+}
+
+
 static MsgStringList handleMsgNewRabbit (PacketInfo *pi)
 {
   MsgStringList list = listMsgBasics (pi);
@@ -863,7 +886,7 @@ static MsgStringList handleMsgShotBegin (PacketInfo *pi)
   d = finfo.unpack (d);
   const ShotUpdate& shot = finfo.shot;
   listPush (list, 1, "player:   %s", strPlayer(shot.player).c_str());
-  listPush (list, 1, "type:     %.2s", finfo.flagType->flagAbbv); // FIXME ?
+  listPush (list, 1, "type:     %.2s", finfo.flagType->flagAbbv.c_str()); // FIXME ?
   listPush (list, 2, "id:       %i", shot.id);
   listPush (list, 2, "team:     %s", strTeam(shot.team).c_str());
   listPush (list, 2, "pos:      %s", strVector((float*)shot.pos).c_str());
@@ -1052,6 +1075,14 @@ static MsgStringList handleMsgWantWHash (PacketInfo *pi)
 }
 
 
+static MsgStringList handleMsgWantSettings (PacketInfo *pi)
+{
+  // not recorded
+  MsgStringList list = listMsgBasics (pi);
+  return list;
+}
+
+
 static MsgStringList handleMsgUDPLinkRequest (PacketInfo *pi)
 {
   // not recorded
@@ -1083,6 +1114,31 @@ static MsgStringList handleMsgLagPing (PacketInfo *pi)
   return list;
 }
 
+
+static MsgStringList handleMsgGameTime (PacketInfo *pi)
+{
+  // not recorded
+  MsgStringList list = listMsgBasics (pi);
+  return list;
+}
+
+
+static MsgStringList handleMsgPingCodeReply (PacketInfo *pi)
+{
+  // not recorded
+  MsgStringList list = listMsgBasics (pi);
+  return list;
+}
+
+
+static MsgStringList handleMsgPingCodeRequest (PacketInfo *pi)
+{
+  // not recorded
+  MsgStringList list = listMsgBasics (pi);
+  return list;
+}
+
+
 /******************************************************************************/
 
 // Local Variables: ***
