@@ -1,23 +1,52 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* interface header */
 #include "NetHandler.h"
+// #include "MsgStrings.h"
 
 // system headers
 #include <errno.h>
 
+#include "bzfsAPI.h"
+
 const int udpBufSize = 128000;
 
+std::vector<NetworkDataLogCallback*> logCallbacks;
+
+void addNetworkLogCallback(NetworkDataLogCallback * cb )
+{
+  if (cb)
+    logCallbacks.push_back(cb);
+}
+
+void removeNetworkLogCallback(NetworkDataLogCallback * cb )
+{
+  for ( unsigned int i = 0; i < (unsigned int)logCallbacks.size(); i++)
+  {
+    if ( logCallbacks[i] == cb )
+    {
+      logCallbacks.erase(logCallbacks.begin()+i);
+      return;
+    }
+  }
+}
+
+void callNetworkDataLog ( bool send, bool udp,  const unsigned char *data, unsigned int size, void *param = NULL )
+{
+  for ( unsigned int i = 0; i < (unsigned int)logCallbacks.size(); i++)
+    logCallbacks[i]->networkDataLog(send,udp,data,size,param);
+}
+
 bool NetHandler::pendingUDP = false;
 bool NetHandler::initHandlers(struct sockaddr_in addr) {
   // udp socket
@@ -51,10 +80,6 @@ bool NetHandler::initHandlers(struct sockaddr_in addr) {
   // don't buffer info, send it immediately
   BzfNetwork::setNonBlocking(udpSocket);
 
-#ifdef HAVE_ADNS_H
-  AdnsHandler::startupResolver();
-#endif
-
   return true;
 }
 
@@ -69,18 +94,29 @@ void NetHandler::setFd(fd_set *read_set, fd_set *write_set, int &maxFile) {
   for (int i = 0; i < maxHandlers; i++) {
     NetHandler *player = netPlayer[i];
     if (player && !player->closed) {
-#if !defined(USE_THREADS) || !defined(HAVE_SDL)
-      _FD_SET(player->fd, read_set);
-#endif
-      if (player->outmsgSize > 0)
-	_FD_SET(player->fd, write_set);
-      if (player->fd > maxFile)
+      FD_SET((unsigned int)player->fd, read_set);
+      if (player->outmsgSize > 0) {
+	FD_SET((unsigned int)player->fd, write_set);
+      }
+      if (player->fd > maxFile) {
 	maxFile = player->fd;
+      }
     }
   }
-  _FD_SET(udpSocket, read_set);
-  if (udpSocket > maxFile)
+
+  FD_SET((unsigned int)udpSocket, read_set);
+  if (udpSocket > maxFile) {
     maxFile = udpSocket;
+  }
+
+  for (int i = 0; i < maxHandlers; i++) {
+    NetHandler *player = netPlayer[i];
+    if (player)
+    {
+      if (player->ares)
+	player->ares->setFd(read_set, write_set, maxFile);
+    }
+  }
 }
 
 int NetHandler::getUdpSocket() {
@@ -108,6 +144,8 @@ int NetHandler::udpReceive(char *buffer, struct sockaddr_in *uaddr,
   void *buf = buffer;
   buf = nboUnpackUShort(buf, len);
   buf = nboUnpackUShort(buf, code);
+//  if (code != MsgPlayerUpdateSmall && code != MsgPlayerUpdate)
+//    logDebugMessage(1,"rcvd %s len %d\n",MsgStrings::strMsgCode(code),len);
   if (n == 6 && len == 2 && code == MsgPingCodeRequest)
     // Ping code request
     return -2;
@@ -126,7 +164,7 @@ int NetHandler::udpReceive(char *buffer, struct sockaddr_in *uaddr,
     uint8_t index;
     buf = nboUnpackUByte(buf, index);
     if ((index < maxHandlers) && netPlayer[index] && !netPlayer[index]->closed
-	&& !netPlayer[index]->udpin)
+	&& !netPlayer[index]->udpin) {
       if (!memcmp(&netPlayer[index]->uaddr.sin_addr, &uaddr->sin_addr,
 		  sizeof(uaddr->sin_addr))) {
 	id = index;
@@ -134,36 +172,36 @@ int NetHandler::udpReceive(char *buffer, struct sockaddr_in *uaddr,
 	  netPlayer[index]->uaddr.sin_port = uaddr->sin_port;
 	netPlayer[index]->udpin = true;
 	udpLinkRequest = true;
-	DEBUG2("Player %s [%d] inbound UDP up %s:%d actual %d\n",
+	logDebugMessage(2,"Player %s [%d] inbound UDP up %s:%d actual %d\n",
 	       netPlayer[index]->info->getCallSign(), index,
 	       inet_ntoa(uaddr->sin_addr),
 	       ntohs(netPlayer[index]->uaddr.sin_port),
 	       ntohs(uaddr->sin_port));
       } else {
-	DEBUG2
-	  ("Player %s [%d] inbound UDP rejected %s:%d different IP \
+	logDebugMessage(2,"Player %s [%d] inbound UDP rejected %s:%d different IP \
 than %s:%d\n",
 	   netPlayer[index]->info->getCallSign(), index,
 	   inet_ntoa(netPlayer[index]->uaddr.sin_addr),
 	   ntohs(netPlayer[index]->uaddr.sin_port),
 	   inet_ntoa(uaddr->sin_addr), ntohs(uaddr->sin_port));
       }
+    }
   }
 
   if (id == -1) {
     // no match, discard packet
-    DEBUG2("uread() discard packet! %s:%d choices p(l) h:p",
+    logDebugMessage(2,"uread() discard packet! %s:%d choices p(l) h:p",
 	   inet_ntoa(uaddr->sin_addr), ntohs(uaddr->sin_port));
     for (pi = 0; pi < maxHandlers; pi++) {
       if (netPlayer[pi] && !netPlayer[pi]->closed)
-	DEBUG3(" %d(%d-%d) %s:%d", pi, netPlayer[pi]->udpin,
+	logDebugMessage(3," %d(%d-%d) %s:%d", pi, netPlayer[pi]->udpin,
 	       netPlayer[pi]->udpout,
 	       inet_ntoa(netPlayer[pi]->uaddr.sin_addr),
 	       ntohs(netPlayer[pi]->uaddr.sin_port));
     }
-    DEBUG2("\n");
+    logDebugMessage(2,"\n");
   } else {
-    DEBUG4("Player %s [%d] uread() %s:%d len %d from %s:%d on %i\n",
+    logDebugMessage(4,"Player %s [%d] uread() %s:%d len %d from %s:%d on %i\n",
 	   netPlayer[id]->info->getCallSign(), id,
 	   inet_ntoa(netPlayer[id]->uaddr.sin_addr),
 	   ntohs(netPlayer[id]->uaddr.sin_port), n,
@@ -172,10 +210,13 @@ than %s:%d\n",
 #ifdef NETWORK_STATS
     netPlayer[id]->countMessage(code, len, 0);
 #endif
+
+    callNetworkDataLog (false, true, (const unsigned char*)buf,len,netPlayer[id]);
+
     if (code == MsgUDPLinkEstablished) {
       netPlayer[id]->udpout = true;
-      DEBUG2("Player %s [%d] outbound UDP up\n",
-	     netPlayer[id]->info->getCallSign(), id);
+      logDebugMessage(2,"Player %s [%d] outbound UDP up\n",
+	      netPlayer[id]->info->getCallSign(), id);
     }
   }
   return id;
@@ -188,14 +229,29 @@ bool NetHandler::isUdpFdSet(fd_set *read_set) {
   return false;
 }
 
+void NetHandler::checkDNS(fd_set *read_set, fd_set *write_set) {
+  for (int i = 0; i < maxHandlers; i++) {
+    NetHandler *player = netPlayer[i];
+    if (player)
+    {
+      if (player->ares)
+	player->ares->process(read_set, write_set);
+    }
+  }
+}
+
 int NetHandler::udpSocket = -1;
 NetHandler *NetHandler::netPlayer[maxHandlers] = {NULL};
 
 NetHandler::NetHandler(PlayerInfo* _info, const struct sockaddr_in &clientAddr,
 		       int _playerIndex, int _fd)
-  : info(_info), playerIndex(_playerIndex), fd(_fd), tcplen(0), closed(false),
+  : info(_info), playerIndex(_playerIndex), fd(_fd),
+    tcplen(0), closed(false),
     outmsgOffset(0), outmsgSize(0), outmsgCapacity(0), outmsg(NULL),
     udpOutputLen(0), udpin(false), udpout(false), toBeKicked(false) {
+
+  ares = new AresHandler(_playerIndex);
+
   // store address information for player
   AddrLen addr_len = sizeof(clientAddr);
   memcpy(&uaddr, &clientAddr, addr_len);
@@ -223,16 +279,64 @@ NetHandler::NetHandler(PlayerInfo* _info, const struct sockaddr_in &clientAddr,
 #endif
   if (!netPlayer[playerIndex])
     netPlayer[playerIndex] = this;
-#ifdef HAVE_ADNS_H
-  adns = new AdnsHandler(_playerIndex, (struct sockaddr *) &clientAddr);
+  ares->queryHostname((struct sockaddr *) &clientAddr);
+}
+
+NetHandler::NetHandler(const struct sockaddr_in &_clientAddr, int _fd)
+:fd(_fd),
+tcplen(0), closed(false),
+outmsgOffset(0), outmsgSize(0), outmsgCapacity(0), outmsg(NULL),
+udpOutputLen(0), udpin(false), udpout(false), toBeKicked(false)
+{
+  ares = NULL;
+  info = NULL;
+  playerIndex = -1;
+  // store address information for player
+  AddrLen addr_len = sizeof(_clientAddr);
+  memcpy(&uaddr, &_clientAddr, addr_len);
+  peer = Address(uaddr);
+
+  // update player state
+  time = info->now;
+#ifdef NETWORK_STATS
+
+  // initialize the inbound/outbound counters to zero
+  msgBytes[0] = 0;
+  perSecondTime[0] = time;
+  perSecondCurrentMsg[0] = 0;
+  perSecondMaxMsg[0] = 0;
+  perSecondCurrentBytes[0] = 0;
+  perSecondMaxBytes[0] = 0;
+
+  msgBytes[1] = 0;
+  perSecondTime[1] = time;
+  perSecondCurrentMsg[1] = 0;
+  perSecondMaxMsg[1] = 0;
+  perSecondCurrentBytes[1] = 0;
+  perSecondMaxBytes[1] = 0;
+
 #endif
 }
 
+void NetHandler::setPlayer ( PlayerInfo* p, int index )
+{
+  ares = new AresHandler(index);
+
+  playerIndex = index;
+  info = p;
+
+  if (!netPlayer[playerIndex])
+    netPlayer[playerIndex] = this;
+  ares->queryHostname((struct sockaddr *) &uaddr);
+}
+
 NetHandler::~NetHandler() {
 #ifdef NETWORK_STATS
-  if (info->isPlaying())
+  if (info && info->isPlaying())
     dumpMessageStats();
 #endif
+  if (ares)
+    delete(ares);
   // shutdown TCP socket
   shutdown(fd, 2);
   close(fd);
@@ -241,10 +345,6 @@ NetHandler::~NetHandler() {
 
   if (netPlayer[playerIndex] == this)
     netPlayer[playerIndex] = NULL;
-
-#ifdef HAVE_ADNS_H
-  delete adns;
-#endif
 }
 
 bool NetHandler::isFdSet(fd_set *set) {
@@ -320,8 +420,10 @@ int NetHandler::bufferedSend(const void *buffer, size_t length) {
       // are the network is down or too unreliable to that player.
       // FIXME -- is 20kB too big?  too small?
       if (newCapacity >= 20 * 1024) {
-	DEBUG2("Player %s [%d] drop, unresponsive with %d bytes queued\n",
-	       info->getCallSign(), playerIndex, outmsgSize + length);
+	if (info != NULL && playerIndex >= 0) {
+	  logDebugMessage(2,"Player %s [%d] drop, unresponsive with %d bytes queued\n",
+		info->getCallSign(), playerIndex, outmsgSize + length);
+	}
 	toBeKicked = true;
 	toBeKickedReason = "send queue too big";
 	return 0;
@@ -373,10 +475,14 @@ int NetHandler::pwrite(const void *b, int l) {
   uint16_t len, code;
   buf = nboUnpackUShort(buf, len);
   buf = nboUnpackUShort(buf, code);
+//  if (code != MsgPlayerUpdateSmall && code != MsgPlayerUpdate && code != MsgGameTime)
+//    logDebugMessage(1,"send %s len %d\n",MsgStrings::strMsgCode(code),len);
 #ifdef NETWORK_STATS
   countMessage(code, len, 1);
 #endif
 
+  bool useUDP = false;
+
   // Check if UDP Link is used instead of TCP, if so jump into udpSend
   if (udpout) {
     // only send bulk messages by UDP
@@ -387,13 +493,16 @@ int NetHandler::pwrite(const void *b, int l) {
     case MsgPlayerUpdateSmall:
     case MsgGMUpdate:
     case MsgLagPing:
-      udpSend(b, l);
-      return 0;
+    case MsgGameTime:
+		useUDP = true;
+		break;
     }
   }
 
-  // always sent MsgUDPLinkRequest over udp with udpSend
-  if (code == MsgUDPLinkRequest) {
+  callNetworkDataLog (true, useUDP, (const unsigned char*)b,len,this);
+
+  // always sent UDP messages and MsgUDPLinkRequest over udp with udpSend
+  if (useUDP || code == MsgUDPLinkRequest) {
     udpSend(b, l);
     return 0;
   }
@@ -409,14 +518,6 @@ int NetHandler::pflush(fd_set *set) {
 }
 
 RxStatus NetHandler::tcpReceive() {
-  // read more data into player's message buffer
-#if defined(USE_THREADS) && defined(HAVE_SDL)
-  fd_set read_set;
-  FD_ZERO(&read_set);
-  _FD_SET(fd, &read_set);
-  select(fd + 1, (fd_set*)&read_set, 0, 0, 0);
-#endif
-
   // read header if we don't have it yet
   RxStatus e = receive(4);
   if (e != ReadAll)
@@ -428,8 +529,9 @@ RxStatus NetHandler::tcpReceive() {
   void *buf = tcpmsg;
   buf = nboUnpackUShort(buf, len);
   buf = nboUnpackUShort(buf, code);
+//  logDebugMessage(1,"rcvd %s len %d\n",MsgStrings::strMsgCode(code),len);
   if (len > MaxPacketLen) {
-    DEBUG1("Player [%d] sent huge packet length (len=%d), possible attack\n",
+    logDebugMessage(1,"Player [%d] sent huge packet length (len=%d), possible attack\n",
 	   playerIndex, len);
     return ReadHuge;
   }
@@ -443,16 +545,21 @@ RxStatus NetHandler::tcpReceive() {
 #ifdef NETWORK_STATS
   countMessage(code, len, 0);
 #endif
+
+  callNetworkDataLog (false, false, (const unsigned char*)buf,len,this);
+
   if (code == MsgUDPLinkEstablished) {
     udpout = true;
-    DEBUG2("Player %s [%d] outbound UDP up\n", info->getCallSign(),
+    logDebugMessage(2,"Player %s [%d] outbound UDP up\n", info->getCallSign(),
 	   playerIndex);
   }
   return ReadAll;
 }
 
-RxStatus NetHandler::receive(size_t length) {
+RxStatus NetHandler::receive(size_t length, bool *retry) {
   RxStatus returnValue;
+  if (retry)
+    *retry = false;
   if ((int)length <= tcplen)
     return ReadAll;
   int size = recv(fd, tcpmsg + tcplen, (int)length - tcplen, 0);
@@ -468,9 +575,11 @@ RxStatus NetHandler::receive(size_t length) {
     const int err = getErrno();
 
     // ignore if it's one of these errors
-    if (err == EAGAIN || err == EINTR)
+    if (err == EAGAIN || err == EINTR){
+      if (retry)
+	*retry = true;
       returnValue = ReadPart;
-    else if (err == ECONNRESET || err == EPIPE) {
+    }else if (err == ECONNRESET || err == EPIPE) {
       // if socket is closed then give up
       returnValue = ReadReset;
     } else {
@@ -480,11 +589,11 @@ RxStatus NetHandler::receive(size_t length) {
     returnValue = ReadDiscon;
   }
   return returnValue;
-};
+}
 
 void *NetHandler::getTcpBuffer() {
   return tcpmsg;
-};
+}
 
 void NetHandler::flushUDP()
 {
@@ -556,21 +665,21 @@ void NetHandler::dumpMessageStats() {
   int total;
   int direction;
 
-  DEBUG1("Player connect time: %f\n", info->now - time);
+  logDebugMessage(1,"Player connect time: %f\n", info->now - time);
 
   for (direction = 0; direction <= 1; direction++) {
     total = 0;
-    DEBUG1("Player messages %s:", direction ? "out" : "in");
+    logDebugMessage(1,"Player messages %s:", direction ? "out" : "in");
 
     for (MessageCountMap::iterator i = msg[direction].begin();
 	 i != msg[direction].end(); i++) {
-      DEBUG1(" %c%c:%u(%u)", i->first >> 8, i->first & 0xff,
+      logDebugMessage(1," %c%c:%u(%u)", i->first >> 8, i->first & 0xff,
 	     i->second.count, i->second.maxSize);
       total += i->second.count;
     }
 
-    DEBUG1(" total:%u(%u) ", total, msgBytes[direction]);
-    DEBUG1("max msgs/bytes per second: %u/%u\n",
+    logDebugMessage(1," total:%u(%u) ", total, msgBytes[direction]);
+    logDebugMessage(1,"max msgs/bytes per second: %u/%u\n",
 	perSecondMaxMsg[direction],
 	perSecondMaxBytes[direction]);
   }
@@ -581,7 +690,7 @@ void NetHandler::dumpMessageStats() {
 void NetHandler::udpSend(const void *b, size_t l) {
 #ifdef TESTLINK
   if ((random()%LINKQUALITY) == 0) {
-    DEBUG1("Drop Packet due to Test\n");
+    logDebugMessage(1,"Drop Packet due to Test\n");
     return;
   }
 #endif
@@ -620,22 +729,23 @@ bool NetHandler::isMyUdpAddrPort(struct sockaddr_in _uaddr) {
     (memcmp(&uaddr.sin_addr, &_uaddr.sin_addr, sizeof(uaddr.sin_addr)) == 0);
 }
 
-void NetHandler::getPlayerList(char *list) {
-  sprintf(list, "[%d]%-16s: %s%s%s%s%s%s", playerIndex, info->getCallSign(),
+const std::string NetHandler::getPlayerHostInfo() {
+  return TextUtils::format("%s%s%s%s%s%s",
 	  peer.getDotNotation().c_str(),
-#ifdef HAVE_ADNS_H
-	  adns->getHostname() ? " (" : "",
-	  adns->getHostname() ? adns->getHostname() : "",
-	  adns->getHostname() ? ")" : "",
-#else
-	  "", "", "",
-#endif
+	  getHostname() ? " (" : "",
+	  getHostname() ? getHostname() : "",
+	  getHostname() ? ")" : "",
 	  udpin ? " udp" : "",
 	  udpout ? "+" : "");
-};
+}
 
-const char *NetHandler::getTargetIP() {
-  return peer.getDotNotation().c_str();
+const char* NetHandler::getTargetIP() {
+  /* peer->getDotNotation returns a temp variable that is not safe
+   * to pass around.  we keep a copy in allocated memory for safety.
+   */
+  if (dotNotation.size() == 0)
+    dotNotation = peer.getDotNotation();
+  return dotNotation.c_str();
 }
 
 int NetHandler::sizeOfIP() {
@@ -670,12 +780,19 @@ in_addr NetHandler::getIPAddress() {
 }
 
 const char *NetHandler::getHostname() {
-#ifdef HAVE_ADNS_H
-  adns->checkDNSResolution();
-  return adns->getHostname();
-#else
-  return NULL;
-#endif
+  if (!ares)
+    return NULL;
+  return ares->getHostname();
+}
+
+bool NetHandler::reverseDNSDone()
+{
+  if (!ares)
+    return false;
+
+  AresHandler::ResolutionStatus status = ares->getStatus();
+  return (status == AresHandler::Failed)
+    || (status == AresHandler::HbASucceeded);
 }
 
 // Local Variables: ***
