@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -21,6 +21,7 @@
 
 /* common implementation headers */
 #include "TextUtils.h"
+#include "AnsiCodes.h"
 
 /* local implementation headers */
 #include "ServerListCache.h"
@@ -230,6 +231,22 @@ unsigned int ServerItem::getSortFactor() const
 }
 
 
+void ServerItem::splitAddrTitle(std::string& addr, std::string& title) const
+{
+  addr = stripAnsiCodes(description);
+  title = "";
+  const std::string::size_type pos = addr.find_first_of(';');
+  if (pos == std::string::npos) {
+    return;
+  }
+  const std::string::size_type tpos = pos + 2; // skip the ';' and ' '
+  if (addr.size() > tpos) {
+    title = addr.substr(tpos);
+  }
+  addr.resize(pos);
+}
+
+
 // Local Variables: ***
 // mode: C++ ***
 // tab-width: 8 ***
