@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -519,8 +519,7 @@ void			SphereBspSceneNode::addRenderNodes(
   renderer.addRenderNode(&renderNode, &gstate);
 }
 
-void			SphereBspSceneNode::addShadowNodes(
-				SceneRenderer& /*renderer*/)
+void			SphereBspSceneNode::addShadowNodes(SceneRenderer& UNUSED(renderer))
 {
   return;
 /*
@@ -741,7 +740,7 @@ void			SphereFragmentSceneNode::addRenderNodes
 }
 
 void			SphereFragmentSceneNode::addShadowNodes(
-				SceneRenderer& /*renderer*/)
+				SceneRenderer& UNUSED(renderer))
 {
   return;
 /*
