@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -20,6 +20,7 @@
 
 // system headers
 #include <math.h>
+#include <string.h>
 
 // common implementation headers
 #include "StateDatabase.h"
@@ -29,6 +30,8 @@
 // local implementation headers
 #include "ViewFrustum.h"
 
+#include "TextureManager.h"
+
 using namespace TankGeometryEnums;
 
 static const float MuzzleMaxX = 4.94f;
@@ -46,13 +49,14 @@ TankSceneNode::TankSceneNode(const GLfloat pos[3], const GLfloat forward[3]) :
 				leftWheelOffset(0.0f), rightWheelOffset(0.0f),
 				useDimensions(false), useOverride(false),
 				onlyShadows(false), clip(false),
-				inTheCockpit(false), tankRenderNode(this),
+				inTheCockpit(false), tankRenderNode(this), treadsRenderNode(this),
 				shadowRenderNode(this),
 				tankSize(TankGeometryEnums::Normal)
 {
   // setup style factors (BZDB isn't set up at global init time
-
 			// prepare geometry
+  deathOverride = NULL;
+
   move(pos, forward);
   float length = BZDBCache::tankLength;
   length = 0.5f * (length + MuzzleMaxX);
@@ -192,6 +196,9 @@ void TankSceneNode::notifyStyleChange()
   }
   gstate = builder.getState();
 
+  builder.setTexture(TextureManager::instance().getTextureID("treads"));
+  treadState = builder.getState();
+
   OpenGLGStateBuilder builder2(lightsGState);
   if (BZDBCache::smooth) {
     builder2.setBlending(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -202,6 +209,7 @@ void TankSceneNode::notifyStyleChange()
   }
   lightsGState = builder2.getState();
 
+
   OpenGLGStateBuilder builder3(jumpJetsGState);
   builder3.setCulling(GL_NONE);
   builder3.setBlending(GL_SRC_ALPHA, GL_ONE);
@@ -236,9 +244,11 @@ void TankSceneNode::addRenderNodes(SceneRenderer& renderer)
     mode = LowTankLOD;
   }
   tankRenderNode.setTankLOD(mode);
+  treadsRenderNode.setTankLOD(mode);
 
   // set the tank's scaling size
   tankRenderNode.setTankSize(tankSize);
+  treadsRenderNode.setTankSize(tankSize);
 
   bool narrow = false;
   if ((tankSize == Narrow) &&
@@ -247,6 +257,7 @@ void TankSceneNode::addRenderNodes(SceneRenderer& renderer)
     narrow = true;
   }
   tankRenderNode.setNarrowWithDepth(narrow);
+  treadsRenderNode.setNarrowWithDepth(narrow);
 
   // if drawing in sorted order then decide which order
   if (sort || transparent || narrow) {
@@ -268,7 +279,10 @@ void TankSceneNode::addRenderNodes(SceneRenderer& renderer)
     tankRenderNode.sortOrder(above, towards, left);
   }
 
+  treadsRenderNode.setTreads(true);
   renderer.addRenderNode(&tankRenderNode, &gstate);
+  renderer.addRenderNode(&treadsRenderNode, &treadState);
+
 }
 
 
@@ -350,7 +364,7 @@ void TankSceneNode::setThief()
 void TankSceneNode::setDimensions(const float dims[3])
 {
   tankSize = Normal;
-  memcpy (dimensions, dims, sizeof(float[3]));
+  memcpy(dimensions, dims, sizeof(float[3]));
   useDimensions = true;
   return;
 }
@@ -493,13 +507,20 @@ void TankSceneNode::renderRadar()
   setCenter(tankPos);
   azimuth = 0.0f;
 
-  gstate.setState();
-
   float oldAlpha = color[3];
   if (color[3] < 0.15f) {
-    color[3] = 0.15f;
+	  color[3] = 0.15f;
   }
 
+  gstate.setState();
+//   if (BZDBCache::animatedTreads) {
+// //	  treadState.setState();
+// 	  treadsRenderNode.setRadar(true);
+// 	  treadsRenderNode.sortOrder(true /* above */, false, false);
+// 	  treadsRenderNode.render();
+// 	  treadsRenderNode.setRadar(false);
+//   }
+
   tankRenderNode.setRadar(true);
   tankRenderNode.sortOrder(true /* above */, false, false);
   tankRenderNode.render();
@@ -818,6 +839,7 @@ TankSceneNode::TankRenderNode::TankRenderNode(const TankSceneNode* _sceneNode) :
   drawLOD = LowTankLOD;
   drawSize = Normal;
   isRadar = false;
+  isTreads = false;
   return;
 }
 
@@ -841,6 +863,10 @@ void TankSceneNode::TankRenderNode::setShadow()
   return;
 }
 
+void TankSceneNode::TankRenderNode::setTreads(bool treads )
+{
+	isTreads = treads;
+}
 
 void TankSceneNode::TankRenderNode::sortOrder(
 				bool _above, bool _towards, bool _left)
@@ -919,13 +945,10 @@ void TankSceneNode::TankRenderNode::render()
   }
 
   if (isRadar && !isExploding) {
-    if (BZDBCache::animatedTreads) {
-      renderPart(LeftTread);
-      renderPart(RightTread);
-    } else {
-      renderPart(LeftCasing);
-      renderPart(RightCasing);
-    }
+	renderPart(LeftCasing);
+	renderPart(RightCasing);
+	renderPart(LeftTread);
+	renderPart(RightTread);
     renderPart(Body);
     renderPart(Turret);
     renderPart(Barrel);
@@ -1046,6 +1069,7 @@ void TankSceneNode::TankRenderNode::renderRightParts()
 
 void TankSceneNode::TankRenderNode::renderNarrowWithDepth()
 {
+  glDepthFunc(GL_LEQUAL);
   // render the middle stuff
   renderPart(Body);
   renderPart(Turret);
@@ -1065,8 +1089,6 @@ void TankSceneNode::TankRenderNode::renderNarrowWithDepth()
   const float offsetFactor = -0.1f;
   const float offsetDepth = -1.0f;
 
-  glDepthFunc(GL_LEQUAL);
-
   glPolygonOffset(1.0f * offsetFactor, 1.0f * offsetDepth);
   if (left) {
     renderPart(LeftCasing);
@@ -1093,13 +1115,12 @@ void TankSceneNode::TankRenderNode::renderNarrowWithDepth()
     renderPart(RightTread);
   }
 
-  glDepthFunc(GL_LESS);
-
   if (usingPolyOffset) {
     glPolygonOffset(factor, units);
   } else {
     glDisable(GL_POLYGON_OFFSET_FILL);
   }
+  glDepthFunc(GL_LESS);
 
   return;
 }
@@ -1155,17 +1176,54 @@ void TankSceneNode::TankRenderNode::renderParts()
 
 void TankSceneNode::TankRenderNode::renderPart(TankPart part)
 {
+	if (isTreads)
+	{
+		if ( part != RightTread && part != LeftTread )
+		return;
+	}
+	else
+	{
+		if ( part == RightTread || part == LeftTread )
+		return;
+	}
+
   // apply explosion transform
+	bool overide = false;
+	TankDeathOverride::DeathParams params(explodeFraction,fvec4(color[0],color[1],color[2],color[3]));
+
   if (isExploding) {
     glPushMatrix();
-    const float* vel = sceneNode->vel[part];
-    const float* spin = sceneNode->spin[part];
-    const float* cog = centerOfGravity[part];
-    glTranslatef(cog[0] + (explodeFraction * vel[0]),
-		 cog[1] + (explodeFraction * vel[1]),
-		 cog[2] + (explodeFraction * vel[2]));
-    glRotatef(spin[3] * explodeFraction, spin[0], spin[1], spin[2]);
-    glTranslatef(-cog[0], -cog[1], -cog[2]);
+	const float* cog = centerOfGravity[part];
+	if (sceneNode->deathOverride)
+		sceneNode->deathOverride->SetDeathRenderParams(params);
+	if (!overide)
+	{
+		const float* vel = sceneNode->vel[part];
+		const float* spin = sceneNode->spin[part];
+		glTranslatef(cog[0] + (explodeFraction * vel[0]),
+			 cog[1] + (explodeFraction * vel[1]),
+			 cog[2] + (explodeFraction * vel[2]));
+		glRotatef(spin[3] * explodeFraction, spin[0], spin[1], spin[2]);
+		glTranslatef(-cog[0], -cog[1], -cog[2]);
+	}
+	else
+	{
+		//const GLfloat *sphere = sceneNode->getSphere();
+		//glTranslatef(-(sphere[0]-sceneNode->explodePos.x),-(sphere[1]-sceneNode->explodePos.y), -(sphere[3]-sceneNode->explodePos.z)); // the death thingy will take care of any bounce
+		//glLoadIdentity();
+		//glTranslatef(sceneNode->explodePos.x,sceneNode->explodePos.y,sceneNode->explodePos.z);
+		//glRotatef(sceneNode->azimuth, 0.0f, 0.0f, 1.0f);
+		//glRotatef(sceneNode->elevation, 0.0f, 1.0f, 0.0f);
+
+		glTranslatef(cog[0] + params.pos.x,
+			cog[1] +  params.pos.y,
+			cog[2] +  params.pos.z);
+		glRotatef( params.rot.x, 1, 0,0);
+		glRotatef(params.rot.y, 0, 1,0);
+		glRotatef(params.rot.z, 0, 0,1);
+		glTranslatef(-cog[0], -cog[1], -cog[2]);
+		glScalef(params.scale.x,params.scale.y,params.scale.z);
+	}
   }
 
   // setup the animation texture matrix
@@ -1177,6 +1235,8 @@ void TankSceneNode::TankRenderNode::renderPart(TankPart part)
   // set color
   if (!isShadow) {
     setupPartColor(part);
+	if (overide)
+		myColor4fv(params.color);
   }
 
   // get the list
@@ -1184,12 +1244,15 @@ void TankSceneNode::TankRenderNode::renderPart(TankPart part)
   TankShadow shadow = isShadow ? ShadowOn : ShadowOff;
   list = TankGeometryMgr::getPartList(shadow, part, drawSize, drawLOD);
 
-  // draw the part
-  glCallList(list);
+  if (!overide || params.draw)
+  {
+		// draw the part
+	  glCallList(list);
 
-  // add to the triangle count
-  addTriangleCount(TankGeometryMgr::getPartTriangleCount(
-				      shadow, part, drawSize, drawLOD));
+	  // add to the triangle count
+	  addTriangleCount(TankGeometryMgr::getPartTriangleCount(
+						  shadow, part, drawSize, drawLOD));
+  }
 
   // draw the lights on the turret
   if ((part == Turret) && !isExploding && !isShadow) {
