@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 // bzflag common header
@@ -21,6 +21,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
+#include <string.h>
 
 // common implementation headers
 #include "StateDatabase.h"
@@ -89,44 +90,59 @@ bool			WallSceneNode::cull(const ViewFrustum& frustum) const
 {
   // cull if eye is behind (or on) plane
   const GLfloat* eye = frustum.getEye();
-  if (((eye[0] * plane[0]) + (eye[1] * plane[1]) + (eye[2] * plane[2]) +
-       plane[3]) <= 0.0f) {
+  const float eyedot = (eye[0] * plane[0]) +
+		       (eye[1] * plane[1]) +
+		       (eye[2] * plane[2]) + plane[3];
+  if (eyedot <= 0.0f) {
     return true;
   }
 
   // if the Visibility culler tells us that we're
   // fully visible, then skip the rest of these tests
-  if (octreeState == OctreeVisible)
+  if (octreeState == OctreeVisible) {
     return false;
+  }
 
   // get signed distance of wall center to each frustum side.
   // if more than radius outside then cull
+  const int planeCount = frustum.getPlaneCount();
   int i;
-  float d[5], d2[5];
-  const GLfloat* sphere = getSphere();
-  for (i = 0; i < 5; i++) {
+  float d[6], d2[6];
+  const GLfloat* mySphere = getSphere();
+  bool inside = true;
+  for (i = 0; i < planeCount; i++) {
     const GLfloat* norm = frustum.getSide(i);
-    d[i] = sphere[0] * norm[0] + sphere[1] * norm[1] +
-		sphere[2] * norm[2] + norm[3];
-    if (d[i] < 0.0f && (d2[i] = d[i]*d[i]) > sphere[3])
-      return true;
+    d[i] = (mySphere[0] * norm[0]) +
+	   (mySphere[1] * norm[1]) +
+	   (mySphere[2] * norm[2]) + norm[3];
+    if (d[i] < 0.0f) {
+      d2[i] = d[i] * d[i];
+      if (d2[i] > mySphere[3]) {
+	return true;
+      }
+      inside = false;
+    }
   }
 
   // see if center of wall is inside each frustum side
-  if (d[0] >= 0.0f && d[1] >= 0.0f && d[2] >= 0.0f &&
-      d[3] >= 0.0f && d[4] >= 0.0f)
+  if (inside) {
     return false;
+  }
 
   // most complicated test:  for sides sphere is behind, see if
   // center is beyond radius times the sine of the angle between
   // the normals, or equivalently:
   //	distance^2 > radius^2 * (1 - cos^2)
   // if so the wall is outside the view frustum
-  for (i = 0; i < 5; i++) {
-    if (d[i] >= 0.0f) continue;
+  for (i = 0; i < planeCount; i++) {
+    if (d[i] >= 0.0f) {
+      continue;
+    }
     const GLfloat* norm = frustum.getSide(i);
     const GLfloat c = norm[0]*plane[0] + norm[1]*plane[1] + norm[2]*plane[2];
-    if (d2[i] > sphere[3] * (1.0f - c*c)) return true;
+    if (d2[i] > mySphere[3] * (1.0f - c*c)) {
+      return true;
+    }
   }
 
   // probably visible
@@ -142,7 +158,7 @@ int			WallSceneNode::pickLevelOfDetail(
 
   int bestLOD = 0;
 
-  const GLfloat* sphere = getSphere();
+  const GLfloat* mySphere = getSphere();
   const int numLights = renderer.getNumLights();
   for (int i = 0; i < numLights; i++) {
     const GLfloat* pos = renderer.getLight(i).getPosition();
@@ -155,12 +171,12 @@ int			WallSceneNode::pickLevelOfDetail(
     if (pd < 0.0f) continue;
 
     // get squared distance from center of wall
-    GLfloat ld = (pos[0] - sphere[0]) * (pos[0] - sphere[0]) +
-		(pos[1] - sphere[1]) * (pos[1] - sphere[1]) +
-		(pos[2] - sphere[2]) * (pos[2] - sphere[2]);
+    GLfloat ld = (pos[0] - mySphere[0]) * (pos[0] - mySphere[0]) +
+		(pos[1] - mySphere[1]) * (pos[1] - mySphere[1]) +
+		(pos[2] - mySphere[2]) * (pos[2] - mySphere[2]);
 
     // pick representative distance
-    GLfloat d = (ld > 1.5f * sphere[3]) ? ld : pd * pd;
+    GLfloat d = (ld > 1.5f * mySphere[3]) ? ld : pd * pd;
 
     // choose lod based on distance and element areas;
     int j;
@@ -293,11 +309,6 @@ void			WallSceneNode::setNoSorting(bool value)
   noSorting = value;
 }
 
-bool			WallSceneNode::isTranslucent() const
-{
-  return isBlended;
-}
-
 void			WallSceneNode::setMaterial(const OpenGLMaterial& mat)
 {
   OpenGLGStateBuilder builder(gstate);
@@ -312,7 +323,7 @@ void			WallSceneNode::setTexture(const int tex)
   gstate = builder.getState();
 }
 
-void			WallSceneNode::setTextureMatrix(const int texmat)
+void			WallSceneNode::setTextureMatrix(const GLfloat* texmat)
 {
   OpenGLGStateBuilder builder(gstate);
   builder.setTextureMatrix(texmat);
@@ -343,7 +354,11 @@ void			WallSceneNode::notifyStyleChange()
     builder.enableTextureMatrix(false);
     alpha = lighted ? lightedColor[3] : color[3];
   }
-  builder.enableTextureReplace(BZDB.isTrue("_texturereplace"));
+  if (BZDB.isTrue("texturereplace")) {
+    builder.setTextureEnvMode(GL_REPLACE);
+  } else {
+    builder.setTextureEnvMode(GL_MODULATE);
+  }
   builder.enableMaterial(lighted);
   if (BZDBCache::blend && (wantBlending || (alpha != 1.0f))) {
     builder.setBlending(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -386,7 +401,7 @@ void			WallSceneNode::copyStyle(WallSceneNode* node)
 void			WallSceneNode::setColor()
 {
   if (BZDBCache::texture && useColorTexture) {
-    glColor4f(1,1,1,1);
+    myColor4f(1,1,1,1);
   }
   else if (dynamicColor != NULL) {
     myColor4fv(dynamicColor);
@@ -592,7 +607,7 @@ void WallSceneNode::splitEdge(float d1, float d2,
 }
 
 
-bool WallSceneNode::inAxisBox (const Extents& /*exts*/) const
+bool WallSceneNode::inAxisBox (const Extents& UNUSED(exts)) const
 {
   // this should never happen, only the TriWallSceneNode
   // and QuadWallSceneNode version of this function will
