@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -37,7 +37,7 @@ ArcObstacle::ArcObstacle(const MeshTransform& xform,
 			 float _rotation, float _sweepAngle, float _ratio,
 			 const float _texsize[4], bool _useNormals,
 			 int _divisions, const BzMaterial* mats[MaterialCount],
-			 int physics, bool bounce, bool drive, bool shoot)
+			 int physics, bool bounce, bool drive, bool shoot, bool rico)
 {
   // common obstace parameters
   memcpy(pos, _pos, sizeof(pos));
@@ -46,6 +46,7 @@ ArcObstacle::ArcObstacle(const MeshTransform& xform,
   ZFlip = false;
   driveThrough = drive;
   shootThrough = shoot;
+  ricochet = rico;
 
   // arc specific parameters
   transform = xform;
@@ -79,7 +80,7 @@ Obstacle* ArcObstacle::copyWithTransform(const MeshTransform& xform) const
     new ArcObstacle(tmpXform, pos, size, angle, sweepAngle, ratio,
 		    texsize, useNormals, divisions,
 		    (const BzMaterial**)materials, phydrv,
-		    smoothBounce, driveThrough, shootThrough);
+		    smoothBounce, driveThrough, shootThrough, ricochet);
   return copy;
 }
 
@@ -318,7 +319,7 @@ MeshObstacle* ArcObstacle::makePie(bool isCircle, float a, float r,
 
   mesh = new MeshObstacle(transform, checkTypes, checkPoints,
 			  vertices, normals, texcoords, fcount,
-			  false, smoothBounce, driveThrough, shootThrough);
+			  false, smoothBounce, driveThrough, shootThrough, ricochet);
 
   // now make the faces
   int vlen, nlen;
@@ -499,7 +500,7 @@ MeshObstacle* ArcObstacle::makeRing(bool isCircle, float a, float r,
 
   mesh = new MeshObstacle(transform, checkTypes, checkPoints,
 			  vertices, normals, texcoords, fcount,
-			  false, smoothBounce, driveThrough, shootThrough);
+			  false, smoothBounce, driveThrough, shootThrough, ricochet);
 
   // now make the faces
   int vlen, nlen;
@@ -615,9 +616,9 @@ bool ArcObstacle::inMovingBox(const float*, float, const float*, float,
 }
 
 
-bool ArcObstacle::isCrossing(const float* /*p*/, float /*angle*/,
-			  float /*dx*/, float /*dy*/, float /*height*/,
-			  float* /*_plane*/) const
+bool ArcObstacle::isCrossing(const float* UNUSED(p), float UNUSED(angle),
+			     float UNUSED(dx), float UNUSED(dy), float UNUSED(height),
+			     float* UNUSED(_plane)) const
 {
   assert(false);
   return false;
@@ -650,6 +651,7 @@ void* ArcObstacle::pack(void* buf) const
   stateByte |= isShootThrough() ? (1 << 1) : 0;
   stateByte |= smoothBounce     ? (1 << 2) : 0;
   stateByte |= useNormals       ? (1 << 3) : 0;
+  stateByte |= canRicochet()    ? (1 << 4) : 0;
   buf = nboPackUByte(buf, stateByte);
 
   return buf;
@@ -688,6 +690,7 @@ void* ArcObstacle::unpack(void* buf)
   shootThrough = (stateByte & (1 << 1)) != 0;
   smoothBounce = (stateByte & (1 << 2)) != 0;
   useNormals   = (stateByte & (1 << 3)) != 0;
+  ricochet     = (stateByte & (1 << 4)) != 0;
 
   finalize();
 
@@ -761,6 +764,9 @@ void ArcObstacle::print(std::ostream& out, const std::string& indent) const
   if (shootThrough) {
     out << indent << "  shootThrough" << std::endl;
   }
+  if (ricochet) {
+    out << indent << "  ricochet" << std::endl;
+  }
   if (!useNormals) {
     out << indent << "  flatshading" << std::endl;
   }
