@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -50,7 +50,7 @@ MeshFace::MeshFace(MeshObstacle* _mesh)
 MeshFace::MeshFace(MeshObstacle* _mesh, int _vertexCount,
 		   float** _vertices, float** _normals, float** _texcoords,
 		   const BzMaterial* _bzMaterial, int physics,
-		   bool _noclusters, bool bounce, bool drive, bool shoot)
+		   bool _noclusters, bool bounce, bool drive, bool shoot, bool rico)
 {
   mesh = _mesh;
   vertexCount = _vertexCount;
@@ -63,6 +63,7 @@ MeshFace::MeshFace(MeshObstacle* _mesh, int _vertexCount,
   smoothBounce = bounce;
   driveThrough = drive;
   shootThrough = shoot;
+  ricochet = rico;
   edges = NULL;
   edgePlanes = NULL;
   specialData = NULL;
@@ -107,7 +108,7 @@ void MeshFace::finalize()
     if ((debugLevel >= 3) && (mesh != NULL)) {
       logDebugMessage(0,":");
       for (i = 0; i < vertexCount; i++) {
-	logDebugMessage(0," %i", (int)((fvec3*)vertices[i] - mesh->getVertices()));
+	logDebugMessage(0," %i", (int)((afvec3*)vertices[i] - mesh->getVertices()));
       }
       print(std::cerr, "");
     }
@@ -139,7 +140,7 @@ void MeshFace::finalize()
       if ((debugLevel >= 3) && (mesh != NULL)) {
 	logDebugMessage(0,":");
 	for (i = 0; i < vertexCount; i++) {
-	  logDebugMessage(0," %i", (int)((fvec3*)vertices[i] - mesh->getVertices()));
+	  logDebugMessage(0," %i", (int)((afvec3*)vertices[i] - mesh->getVertices()));
 	}
 	print(std::cerr, "");
       }
@@ -158,7 +159,7 @@ void MeshFace::finalize()
       if ((debugLevel >= 3) && (mesh != NULL)) {
 	logDebugMessage(0,":");
 	for (i = 0; i < vertexCount; i++) {
-	  logDebugMessage(0," %i", (int)((fvec3*)vertices[i] - mesh->getVertices()));
+	  logDebugMessage(0," %i", (int)((afvec3*)vertices[i] - mesh->getVertices()));
 	}
 	print(std::cerr, "");
       }
@@ -185,7 +186,7 @@ void MeshFace::finalize()
   ZFlip = false;
 
   // make the edge planes
-  edgePlanes = new fvec4[vertexCount];
+  edgePlanes = new afvec4[vertexCount];
   for (v = 0; v < vertexCount; v++) {
     float edge[3];
     const int next = (v + 1) % vertexCount;
@@ -402,7 +403,7 @@ void MeshFace::get3DNormal(const float* p, float* n) const
 }
 
 
-void MeshFace::getNormal(const float* /*p*/, float* n) const
+void MeshFace::getNormal(const float* UNUSED(p), float* n) const
 {
   if (n) {
     memcpy (n, plane, sizeof(float[3]));
@@ -416,9 +417,9 @@ void MeshFace::getNormal(const float* /*p*/, float* n) const
 ///////////////////////////////////////////////////////////////
 
 
-bool MeshFace::getHitNormal(const float* /*oldPos*/, float /*oldAngle*/,
-			    const float* /*newPos*/, float /*newAngle*/,
-			    float /*dx*/, float /*dy*/, float /*height*/,
+bool MeshFace::getHitNormal(const float* UNUSED(oldPos), float UNUSED(oldAngle),
+			    const float* UNUSED(newPos), float UNUSED(newAngle),
+			    float UNUSED(dx), float UNUSED(dy), float UNUSED(height),
 			    float* normal) const
 {
   if (normal) {
@@ -450,7 +451,7 @@ bool MeshFace::inBox(const float* p, float _angle,
   // probably be the dominant polygon types).
 
   float pln[4]; // translated plane
-  fvec3* v = new fvec3[vertexCount]; // translated vertices
+  afvec3* v = new afvec3[vertexCount]; // translated vertices
   const float cos_val = cosf(-_angle);
   const float sin_val = sinf(-_angle);
   for (i = 0; i < vertexCount; i++) {
@@ -523,7 +524,7 @@ bool MeshFace::inBox(const float* p, float _angle,
 }
 
 
-bool MeshFace::inMovingBox(const float* oldPos, float /*oldAngle*/,
+bool MeshFace::inMovingBox(const float* oldPos, float UNUSED(oldAngle),
 			   const float* newPos, float newAngle,
 			   float dx, float dy, float height) const
 {
@@ -542,8 +543,8 @@ bool MeshFace::inMovingBox(const float* oldPos, float /*oldAngle*/,
 }
 
 
-bool MeshFace::isCrossing(const float* /*p*/, float /*angle*/,
-			  float /*dx*/, float /*dy*/, float /*height*/,
+bool MeshFace::isCrossing(const float* UNUSED(p), float UNUSED(angle),
+			  float UNUSED(dx), float UNUSED(dy), float UNUSED(height),
 			  float* _plane) const
 {
   if (_plane != NULL) {
@@ -563,19 +564,20 @@ void *MeshFace::pack(void *buf) const
   stateByte |= isShootThrough() ? (1 << 3) : 0;
   stateByte |= smoothBounce     ? (1 << 4) : 0;
   stateByte |= noclusters       ? (1 << 5) : 0;
+  stateByte |= canRicochet()    ? (1 << 6) : 0;
   buf = nboPackUByte(buf, stateByte);
 
   // vertices
   buf = nboPackInt(buf, vertexCount);
   for (int i = 0; i < vertexCount; i++) {
-    int32_t index = (fvec3*)vertices[i] - mesh->getVertices();
+    int32_t index = (afvec3*)vertices[i] - mesh->getVertices();
     buf = nboPackInt(buf, index);
   }
 
   // normals
   if (useNormals()) {
     for (int i = 0; i < vertexCount; i++) {
-      int32_t index = (fvec3*)normals[i] - mesh->getNormals();
+      int32_t index = (afvec3*)normals[i] - mesh->getNormals();
       buf = nboPackInt(buf, index);
     }
   }
@@ -583,7 +585,7 @@ void *MeshFace::pack(void *buf) const
   // texcoords
   if (useTexcoords()) {
     for (int i = 0; i < vertexCount; i++) {
-      int32_t index = (fvec2*)texcoords[i] - mesh->getTexcoords();
+      int32_t index = (afvec2*)texcoords[i] - mesh->getTexcoords();
       buf = nboPackInt(buf, index);
     }
   }
@@ -613,6 +615,7 @@ void *MeshFace::unpack(void *buf)
   shootThrough = (stateByte & (1 << 3)) != 0;
   smoothBounce = (stateByte & (1 << 4)) != 0;
   noclusters   = (stateByte & (1 << 5)) != 0;
+  ricochet     = (stateByte & (1 << 6)) != 0;
 
   // vertices
   buf = nboUnpackInt(buf, inTmp);
@@ -693,7 +696,7 @@ void MeshFace::print(std::ostream& out, const std::string& indent) const
 
   out << indent << "    vertices";
   for (i = 0; i < vertexCount; i++) {
-    int index = (fvec3*)vertices[i] - mesh->getVertices();
+    int index = (afvec3*)vertices[i] - mesh->getVertices();
     out << " " << index;
   }
   if (debugLevel >= 3) {
@@ -707,7 +710,7 @@ void MeshFace::print(std::ostream& out, const std::string& indent) const
   if (normals != NULL) {
     out << indent << "    normals";
     for (i = 0; i < vertexCount; i++) {
-      int index = (fvec3*)normals[i] - mesh->getNormals();
+      int index = (afvec3*)normals[i] - mesh->getNormals();
       out << " " << index;
     }
     if (debugLevel >= 3) {
@@ -722,7 +725,7 @@ void MeshFace::print(std::ostream& out, const std::string& indent) const
   if (texcoords != NULL) {
     out << indent << "    texcoords";
     for (i = 0; i < vertexCount; i++) {
-      int index = (fvec2*)texcoords[i] - mesh->getTexcoords();
+      int index = (afvec2*)texcoords[i] - mesh->getTexcoords();
       out << " " << index;
     }
     if (debugLevel >= 3) {
@@ -766,6 +769,9 @@ void MeshFace::print(std::ostream& out, const std::string& indent) const
       out << indent << "    shootThrough" << std::endl;
     }
   }
+  if (ricochet &&  !mesh->canRicochet()) {
+    out << indent << "  ricochet" << std::endl;
+  }
 
   out << indent << "  endface" << std::endl;
 
