@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -26,8 +26,8 @@ PyramidBuilding::PyramidBuilding()
 }
 
 PyramidBuilding::PyramidBuilding(const float* p, float a,
-				float w, float b, float h, bool drive, bool shoot) :
-				Obstacle(p, a, w, b, h,drive,shoot)
+				float w, float b, float h, bool drive, bool shoot, bool rico) :
+				Obstacle(p, a, w, b, h,drive,shoot,rico)
 {
   finalize();
   return;
@@ -57,7 +57,7 @@ Obstacle* PyramidBuilding::copyWithTransform(const MeshTransform& xform) const
 
   PyramidBuilding* copy =
     new PyramidBuilding(newPos, newAngle, newSize[0], newSize[1], newSize[2],
-			driveThrough, shootThrough);
+			driveThrough, shootThrough, ricochet);
 
   copy->ZFlip = !(getZFlip() == flipped);
 
@@ -426,6 +426,7 @@ void* PyramidBuilding::pack(void* buf) const
   stateByte |= isDriveThrough() ? _DRIVE_THRU : 0;
   stateByte |= isShootThrough() ? _SHOOT_THRU : 0;
   stateByte |= getZFlip() ? _FLIP_Z : 0;
+  stateByte |= canRicochet() ? _RICOCHET : 0;
   buf = nboPackUByte(buf, stateByte);
 
   return buf;
@@ -442,6 +443,7 @@ void* PyramidBuilding::unpack(void* buf)
   buf = nboUnpackUByte(buf, stateByte);
   driveThrough = (stateByte & _DRIVE_THRU) != 0;
   shootThrough = (stateByte & _SHOOT_THRU) != 0;
+  ricochet     = (stateByte & _RICOCHET)   != 0;
   ZFlip = (stateByte & _FLIP_Z) != 0;
 
   finalize();
@@ -485,6 +487,9 @@ void PyramidBuilding::print(std::ostream& out, const std::string& indent) const
       out << indent << "  shootthrough" << std::endl;
     }
   }
+  if (canRicochet()) {
+    out << indent << "  ricochet" << std::endl;
+  }
   out << indent << "end" << std::endl;
   return;
 }
@@ -498,7 +503,7 @@ static void outputFloat(std::ostream& out, float value)
   return;
 }
 
-void PyramidBuilding::printOBJ(std::ostream& out, const std::string& /*indent*/) const
+void PyramidBuilding::printOBJ(std::ostream& out, const std::string& UNUSED(indent)) const
 {
   int i;
   float verts[5][3] = {
