@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993-2010 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -32,8 +32,8 @@ Teleporter::Teleporter()
 
 Teleporter::Teleporter(const float* p, float a, float w,
 		       float b, float h, float _border, bool _horizontal,
-		       bool drive, bool shoot) :
-			 Obstacle(p, a, w, b, h, drive, shoot),
+		       bool drive, bool shoot, bool rico) :
+			 Obstacle(p, a, w, b, h, drive, shoot, rico),
 			 border(_border), horizontal(_horizontal)
 {
   finalize();
@@ -62,7 +62,7 @@ Obstacle* Teleporter::copyWithTransform(const MeshTransform& xform) const
 
   Teleporter* copy =
     new Teleporter(newPos, newAngle, newSize[0], newSize[1], newSize[2],
-		   border, horizontal, driveThrough, shootThrough);
+		   border, horizontal, driveThrough, shootThrough, ricochet);
 
   copy->setName(name);
 
@@ -158,7 +158,7 @@ void Teleporter::makeLinks()
       bvrts[i][2] = p[2] + ((h - br) * params[i][1]);
     }
     backLink = new MeshFace(NULL, 4, bvrts, NULL, btxcds,
-			    NULL, -1, false, false, true, true);
+			    NULL, -1, false, false, true, true, false);
 
     for (i = 0; i < 4 ;i++) {
       fvrts[i][0] = p[0] - (wlen[0] + (blen[0] * params[i][0]));
@@ -166,7 +166,7 @@ void Teleporter::makeLinks()
       fvrts[i][2] = p[2] + ((h - br) * params[i][1]);
     }
     frontLink = new MeshFace(NULL, 4, fvrts, NULL, ftxcds,
-			     NULL, -1, false, false, true, true);
+			     NULL, -1, false, false, true, true, false);
   }
   else {
     float xlen = w - br;
@@ -184,14 +184,14 @@ void Teleporter::makeLinks()
     bvrts[3][1] = p[1] + ((cos_val * ylen) + (sin_val * -xlen));
     bvrts[3][2] = p[2] + h - br;
     backLink = new MeshFace(NULL, 4, bvrts, NULL, btxcds,
-			    NULL, -1, false, false, true, true);
+			    NULL, -1, false, false, true, true, false);
 
     for (i = 0; i < 4; i++) {
       memcpy(fvrts[i], bvrts[3 - i], sizeof(float[3])); // reverse order
       fvrts[i][2] = p[2] + h; // change the height
     }
     frontLink = new MeshFace(NULL, 4, fvrts, NULL, ftxcds,
-			     NULL, -1, false, false, true, true);
+			     NULL, -1, false, false, true, true, false);
   }
 
   return;
@@ -290,7 +290,7 @@ bool Teleporter::inBox(const float* p, float a,
 }
 
 
-bool Teleporter::inMovingBox(const float* oldP, float /*oldAngle */,
+bool Teleporter::inMovingBox(const float* oldP, float UNUSED(oldAngle),
 			     const float* p, float a,
 			     float dx, float dy, float dz) const
 {
@@ -441,72 +441,68 @@ void Teleporter::getPointWRT(const Teleporter& t2, int face1, int face2,
 			     const float* pIn, const float* dIn, float aIn,
 			     float* pOut, float* dOut, float* aOut) const
 {
-  const float x1 = pIn[0] - getPosition()[0];
-  const float y1 = pIn[1] - getPosition()[1];
-  const float a = t2.getRotation() - getRotation() +
-			(face1 == face2 ? (float)M_PI : 0.0f);
-  const float c = cosf(a), s = sinf(a);
-  const float x2 = c * x1 - s * y1;
-  const float y2 = c * y1 + s * x1;
+  // transforming pIn to pOut, going from tele1(this) to tele2
+  const Teleporter& tele1 = *this;
+  const Teleporter& tele2 = t2;
 
-  /*
-	Here's what the next statements do:
+  const float* p1 = tele1.getPosition();
+  const float* p2 = tele2.getPosition();
+  const float* s1 = tele1.getSize();
+  const float* s2 = tele2.getSize();
 
-  In order to account for different-size teleporters, each of the dimensions
-  is expressed a a ratio of the length of the transporter in the dimension
-  divided by position of the tank relative to the transporter in that dimension, and
-  is proportional to the width of the target teleporter in that dimension.
-  Here is the formula, with example lengths:
+  const fvec3  pos1 (p1[0], p1[1], p1[2]);
+  const fvec3 size1 (s1[0], s1[1], s1[2]);
+  const fvec3  pos2 (p2[0], p2[1], p2[2]);
+  const fvec3 size2 (s2[0], s2[1], s2[2]);
 
-  W1/W2 = T1/T2
+  const float bord1 = tele1.getBorder();
+  const float bord2 = tele2.getBorder();
 
+  // y & z axis scaling factors  (relative active areas)
+  const fvec2 dims1(size1.y - bord1, size1.z - bord1);
+  const fvec2 dims2(size2.y - bord2, size2.z - bord2);
+  const fvec2 dimsScale = (dims2 / dims1);
 
-  |--------|	Tank Pos (T1)
-  |----------------------------------------------------------|	Transport width (W1)
+  // adjust the angles for the faces
+  // NOTE: if (face1 == face2), there's an extra 180 degrees spin
+  const float pi = float(M_PI);
+  const float radians1 = tele1.getRotation() + ((face1 == 0) ? 0.0f : pi);
+  const float radians2 = tele2.getRotation() + ((face2 == 1) ? 0.0f : pi);
 
+  // intermediate output position
+  fvec3 p(pIn[0], pIn[1], pIn[2]);
 
-  |-|	New tank Pos (T2)
-  |---------|	New Transport width (W2)
+  // translate to origin, and revert rotation
+  p -= pos1;
+  p = p.rotateZ(-radians1);
 
-  We are looking for T2, and simple algebra tells us that T2 = (W2 * T1) / W1
+  // fixed x offset, and scale y & z coordinates
+  p.x = -size2.x;
+  p.yz() *= dimsScale; // note the .yz()
 
-  Now, we can correctly position the tank.
+  // apply rotation, translate to new position
+  p = p.rotateZ(+radians2);
+  p += pos2;
 
-  Note that this is only the position relative to the transporter, to get the real position,
-  it is added to the rest.  Since I'm not 100% sure of my work, I am leaving the old code
-  commented above.
-  */
-
-  //T1 = x2 and y2
-  //W2 = t2.getWidth()
-  //W1 = getWidth()
-
-  //pOut[0] = x2 + t2.getPosition()[0];
-  //pOut[1] = y2 + t2.getPosition()[1];
-  pOut[0] = t2.getPosition()[0] + (x2 * (t2.getBreadth() - t2.getBorder())) / getBreadth();
-  pOut[1] = t2.getPosition()[1] + (y2 * (t2.getBreadth() - t2.getBorder())) / getBreadth();
-  //T1 = pIn[2] - getPosition()[2]
-  //W2 = t2.getHeight()
-  //W1 = getHeight
-
-  //(t2.getPosition()[2] - getPosition()[2]) adds the height differences between the
-  //teleporters so that teleporters can be off of the ground at different heights.
-
-  //pOut[2] = pIn[2] + t2.getPosition()[2] - getPosition()[2];
-  pOut[2] = t2.getPosition()[2]
-	  + ((pIn[2] - getPosition()[2]) * (t2.getHeight() - t2.getBorder()))/getHeight();
-
-  if (dOut && dIn) {
-    const float dx = dIn[0];
-    const float dy = dIn[1];
-    dOut[0] = c * dx - s * dy;
-    dOut[1] = c * dy + s * dx;
-    dOut[2] = dIn[2];
-  }
+  // final output position
+  pOut[0] = p.x;
+  pOut[1] = p.y;
+  pOut[2] = p.z;
 
+  // fill in output angle and direction variables, if requested
+  const float a = radians2 - radians1;
   if (aOut) {
     *aOut = aIn + a;
   }
+  if (dOut && dIn) {
+    const float c = cosf(a);
+    const float s = sinf(a);
+    const float dx = dIn[0];
+    const float dy = dIn[1];
+    dOut[0] = (c * dx) - (s * dy);
+    dOut[1] = (c * dy) + (s * dx);
+    dOut[2] = dIn[2];
+  }
 }
 
 
@@ -538,6 +534,7 @@ void* Teleporter::pack(void* buf) const
   unsigned char stateByte = 0;
   stateByte |= isDriveThrough() ? _DRIVE_THRU : 0;
   stateByte |= isShootThrough() ? _SHOOT_THRU : 0;
+  stateByte |= canRicochet()    ? _RICOCHET   : 0;
   buf = nboPackUByte(buf, stateByte);
 
   return buf;
@@ -561,6 +558,7 @@ void* Teleporter::unpack(void* buf)
   buf = nboUnpackUByte(buf, stateByte);
   driveThrough = (stateByte & _DRIVE_THRU) != 0;
   shootThrough = (stateByte & _SHOOT_THRU) != 0;
+  ricochet     = (stateByte & _RICOCHET)   != 0;
 
   finalize();
 
@@ -600,6 +598,9 @@ void Teleporter::print(std::ostream& out, const std::string& indent) const
   if (horizontal) {
     out << indent << "  horizontal" << std::endl;
   }
+  if (ricochet) {
+    out << indent << "  ricochet" << std::endl;
+  }
   out << indent << "end" << std::endl;
   return;
 }
@@ -613,7 +614,7 @@ static void outputFloat(std::ostream& out, float value)
   return;
 }
 
-void Teleporter::printOBJ(std::ostream& out, const std::string& /*indent*/) const
+void Teleporter::printOBJ(std::ostream& out, const std::string& UNUSED(indent)) const
 {
   int i;
   float verts[8][3] = {
