@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -275,7 +275,7 @@ void OpenGLTexture::setFilter(Filter _filter)
   glBindTexture(GL_TEXTURE_2D, list);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, minifyFilter[filterIndex]);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, magnifyFilter[filterIndex]);
-#ifdef HAVE_GLEW  
+#ifdef HAVE_GLEW
   if (GLEW_EXT_texture_filter_anisotropic) {
     GLint aniso = BZDB.evalInt("aniso");
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, aniso);
@@ -358,19 +358,8 @@ int OpenGLTexture::getBestFormat(int _width, int _height, const GLvoid* pixels)
       break;
   const bool useAlpha = (i != size);
 
-  // intensity format defined in 1.1 and an extension in 1.0
-  static const bool hasTextureExt = true;
-
-  // see if all pixels are r=g=b=a.  if so return intensity format.
-  // SGI IMPACT and 3Dfx systems don't support GL_INTENSITY.
-  const char* const glRenderer = (const char*)glGetString(GL_RENDERER);
-  static bool noIntensity =
-    ((glRenderer == NULL) ||
-     (strncmp(glRenderer, "IMPACT", 6) == 0) ||
-     (strncmp(glRenderer, "3Dfx", 4) == 0));
-  if (!noIntensity) {
     bool useIntensity = false;
-    if (hasTextureExt && useLuminance) {
+    if (useLuminance) {
       scan = (const GLubyte*)pixels;
       for (i = 0; i < size; scan += 4, i++)
 	if (scan[3] != scan[0])
@@ -379,7 +368,6 @@ int OpenGLTexture::getBestFormat(int _width, int _height, const GLvoid* pixels)
     }
     if (useIntensity)
       return GL_INTENSITY;
-  }
 
   // pick internal format
   return (useLuminance ?
