@@ -1,13 +1,13 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
- * named LICENSE that should have accompanied this file.
+ * named COPYING that should have accompanied this file.
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* BeOS GUI backend by François Revol
@@ -284,7 +284,7 @@ void					BeOSWindow::setMinSize(int width, int height)
   bWindow->Unlock();
 }
 
-void					BeOSWindow::setFullscreen()
+void					BeOSWindow::setFullscreen(bool on)
 {
   bWindow->Lock();
   // FIXME
