@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -19,6 +19,9 @@
 // Don't try compile this if we don't have an up-to-date, working DX
 #if defined(USE_DINPUT)
 
+#pragma comment(lib, "dinput8.lib")
+#pragma comment(lib, "dxguid.lib")
+
 /* system headers */
 #include <vector>
 #include <string>
@@ -36,7 +39,7 @@ std::map<std::string, LPDIRECTINPUTEFFECT> DXJoystick::effectDatabase;
 DXJoystick::DXJoystick() : device(NULL)
 {
   HINSTANCE hinst = GetModuleHandle(NULL);
-  HRESULT success = DirectInputCreateEx(hinst, DIRECTINPUT_VERSION, IID_IDirectInput7,
+  HRESULT success = DirectInput8Create(hinst, DIRECTINPUT_VERSION, IID_IDirectInput8,
 					(void**)&directInput, NULL);
 
   if (success != DI_OK) {
@@ -77,14 +80,12 @@ void	      DXJoystick::initJoystick(const char* joystickName)
    */
   GUID thisDevice;
   for (unsigned int i = 0; i < devices.size(); i++) {
-    if (strcmp(joystickName, devices[i].tszProductName) == 0) {
-      thisDevice = devices[i].guidInstance;
-      break;
-    }
+	  if (strcmp(joystickName, devices[i].tszProductName) == 0) {
+		  thisDevice = devices[i].guidInstance;
+		  break;
+	  }
   }
-  HRESULT success = directInput->CreateDeviceEx(thisDevice,
-						IID_IDirectInputDevice7,
-						(void**)&device, NULL);
+  HRESULT success = directInput->CreateDevice(thisDevice,&device, NULL);
 
   if (success != DI_OK) {
     DXError("Could not initialize device", success);
@@ -359,7 +360,7 @@ void	      DXJoystick::ffRumble(int count, float delay, float duration,
   HRESULT success = DI_OK;
 
   // Generate a string to identify a specific rumble effect,
-  // based on the paramaters of the rumble
+  // based on the parameters of the rumble
   std::string effectType = TextUtils::format("R%d|%d|%d|%d|%d", count, delay, duration, strong_motor, weak_motor);
 
   // Check if we need to create the effect
@@ -439,7 +440,7 @@ void	DXJoystick::ffDirectionalConstant(int count, float delay, float duration,
   HRESULT success = DI_OK;
 
   // Generate a string to identify a specific constant effect,
-  // based on the paramaters of the effect
+  // based on the parameters of the effect
   std::string effectType = TextUtils::format("C%d|%d|%d|%d|%d|%d", count, delay, duration, x_direction, y_direction, strength);
 
   // Check if we need to create the effect
@@ -523,7 +524,7 @@ void	DXJoystick::ffDirectionalPeriodic(int count, float delay, float duration,
   HRESULT success = DI_OK;
 
   // Generate a string to identify a specific periodic effect,
-  // based on the paramaters of the effect
+  // based on the parameters of the effect
   std::string effectType = TextUtils::format("P%d|%d|%d|%d|%d|%d|%d|%d", count, delay, duration, x_direction, y_direction, amplitude, period, type);
 
   // Check if we need to create the effect
@@ -704,7 +705,7 @@ void DXJoystick::enumerateDevices()
 
   devices.clear();
 
-  HRESULT success = directInput->EnumDevices(DIDEVTYPE_JOYSTICK,
+  HRESULT success = directInput->EnumDevices(DI8DEVCLASS_GAMECTRL,
 					     &deviceEnumCallback, NULL,
 					     DIEDFL_ATTACHEDONLY);
 
@@ -801,7 +802,7 @@ void DXJoystick::DXError(const char* situation, HRESULT problem)
 
 /* Nasty callbacks 'cause DirectX sucks */
 
-BOOL CALLBACK DXJoystick::deviceEnumCallback(LPCDIDEVICEINSTANCE device, void* /*pvRef*/)
+BOOL CALLBACK DXJoystick::deviceEnumCallback(LPCDIDEVICEINSTANCE device, void* UNUSED(pvRef))
 {
   if (!device)
     return DIENUM_STOP;
