@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -83,7 +83,8 @@ SDLDisplay::SDLDisplay() : fullScreen(false), base_width(640),
   }
 
   // limit us to the main display
-  putenv("SDL_SINGLEDISPLAY=1");
+  static char singleDisplayEnv[] = "SDL_SINGLEDISPLAY=1";
+  putenv(singleDisplayEnv);
 
   // register modes
   initResolutions(_resolutions, _numResolutions, defaultResolutionIndex);
@@ -476,7 +477,7 @@ bool SDLDisplay::getKey(const SDL_Event& sdlEvent, BzfKeyEvent& key) const
       key.button = BzfKeyEvent::NoButton;
   }
 
-  if (key.button == BzfKeyEvent::NoButton)
+  if (key.button == BzfKeyEvent::NoButton) {
     if (unicode) {
       if ((unicode & 0xFF80))
 	return false;
@@ -491,6 +492,7 @@ bool SDLDisplay::getKey(const SDL_Event& sdlEvent, BzfKeyEvent& key) const
       else
 	return false;
     }
+  }
 
   key.shift = 0;
   if (mod & KMOD_SHIFT)
@@ -530,6 +532,10 @@ bool SDLDisplay::createWindow() {
     OpenGLGState::initContext();
     return true;
   }
+
+//   /* Enable vsync */
+//   long sync = 1;
+//   CGLSetParameter (ctx, kCGLCPSwapInterval, &sync);
 }
 
 void SDLDisplay::setFullscreen(bool on) {
