@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2006 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -138,11 +138,15 @@ void		    SDLJoystick::getJoyDeviceAxes(std::vector<std::string> &list) const
 
 void		    SDLJoystick::setXAxis(const std::string axis)
 {
+  // unset
+  if (axis == "") return;
   xAxis = atoi(axis.c_str());
 }
 
 void		    SDLJoystick::setYAxis(const std::string axis)
 {
+  // unset
+  if (axis == "") return;
   yAxis = atoi(axis.c_str());
 }
 
