@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2005 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -7,7 +7,7 @@
  *
  * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
- * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 /* WinDisplay:
@@ -29,18 +29,21 @@ class WinDisplay : public BzfDisplay {
 				const char* videoFormat);
 			~WinDisplay();
 
-    bool		isValid() const;
+    virtual bool	isValid() const;
     bool		isEventPending() const;
     bool		getEvent(BzfEvent&) const;
     bool		hasGetKeyMode() {return true;};
     void		getModState(bool &shift, bool &control, bool &alt);
 
-    bool		setDefaultResolution();
+    bool		setDefaultResolution() const;
 
     bool		isFullScreenOnly() const;
     int			getFullWidth() const;
     int			getFullHeight() const;
 
+	bool		peekEvent(BzfEvent& event) const;
+
+
     // for other Windows stuff
     class Rep {
       public:
@@ -72,12 +75,13 @@ class WinDisplay : public BzfDisplay {
     ResInfo**		getVideoFormats(int& num, int& current);
     static bool		canChangeDepth();
 
+	bool windowsEventToBZFEvent ( MSG &msg, BzfEvent& event ) const;
+
   private:
     Rep*		rep;
 
     // resolution info
     HWND		hwnd;
-    bool		using3Dfx;
     int			fullWidth;
     int			fullHeight;
     Resolution*		resolutions;
