@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2008 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -37,7 +37,6 @@ WinWindow::WinWindow(const WinDisplay* _display, WinVisual* _visual) :
 				inactiveDueToDeactivateAll(false),
 				useColormap(false),
 				hasGamma(false),
-				has3DFXGamma(false),
 				gammaVal(1.0f),
 				prev(NULL),
 				next(NULL),
@@ -186,13 +185,13 @@ void			WinWindow::setFullscreen(bool on)
       int w, h, x, y, count;
       char xs, ys;
       count = sscanf(BZDB.get("geometry").c_str(),
-    		 "%dx%d%c%d%c%d", &w, &h, &xs, &x, &ys, &y);
+		 "%dx%d%c%d%c%d", &w, &h, &xs, &x, &ys, &y);
       if (w < 256) w = 256;
       if (h < 192) h = 192;
       if (count == 6) {
-        if (xs == '-') x = display->getWidth() - x - w;
-        if (ys == '-') y = display->getHeight() - y - h;
-        setPosition(x, y);
+	if (xs == '-') x = display->getWidth() - x - w;
+	if (ys == '-') y = display->getHeight() - y - h;
+	setPosition(x, y);
       }
       setSize(w, h);
     } else {
@@ -250,7 +249,7 @@ void			WinWindow::grabMouse()
   DWORD style = GetWindowLong(hwnd, GWL_STYLE);
   // don't compensate for window trimmings if they're turned off
   if (!((style & (WS_BORDER | WS_CAPTION | WS_DLGFRAME))
-        == (WS_BORDER | WS_CAPTION | WS_DLGFRAME)))
+	== (WS_BORDER | WS_CAPTION | WS_DLGFRAME)))
     xborder = yborder = titlebar = 0;
 
   RECT rect;
@@ -289,7 +288,7 @@ void			WinWindow::hideMouse()
 
 void			WinWindow::setGamma(float newGamma)
 {
-  if (!useColormap && !hasGamma && !has3DFXGamma)
+  if (!useColormap && !hasGamma)
     return;
 
   // save gamma
@@ -318,7 +317,7 @@ float			WinWindow::getGamma() const
 
 bool			WinWindow::hasGammaControl() const
 {
-  return useColormap || hasGamma || has3DFXGamma;
+  return useColormap || hasGamma;
 }
 
 void			WinWindow::makeCurrent()
@@ -433,20 +432,12 @@ void			WinWindow::destroyChild()
   }
 }
 
-typedef BOOL		(*GammaRamp3DFX)(HDC, LPVOID);
 
 void			WinWindow::getGammaRamps(WORD* ramps)
 {
   if (hDCChild == NULL)
     return;
 
-  // see if we've got the 3Dfx gamma ramp extension
-  PROC proc = wglGetProcAddress("wglGetDeviceGammaRamp3DFX");
-  if (proc != NULL) {
-    GammaRamp3DFX wglGetDeviceGammaRamp3DFX = (GammaRamp3DFX)proc;
-    has3DFXGamma = wglGetDeviceGammaRamp3DFX(hDCChild, ramps + 3 * 256) != FALSE;
-  }
-
   // get device gamma ramps
   hasGamma = GetDeviceGammaRamp(hDCChild, ramps) != FALSE;
 }
@@ -456,14 +447,6 @@ void			WinWindow::setGammaRamps(const WORD* ramps)
   if (hDCChild == NULL)
     return;
 
-  if (has3DFXGamma) {
-    PROC proc = wglGetProcAddress("wglSetDeviceGammaRamp3DFX");
-    if (proc != NULL) {
-      GammaRamp3DFX wglSetDeviceGammaRamp3DFX = (GammaRamp3DFX)proc;
-      wglSetDeviceGammaRamp3DFX(hDCChild, (LPVOID)(ramps + 3 * 256));
-    }
-  }
-
   if (hasGamma)
     SetDeviceGammaRamp(hDCChild, (LPVOID)ramps);
 }
