@@ -1,5 +1,5 @@
 /* bzflag
- * Copyright (c) 1993 - 2007 Tim Riker
+ * Copyright (c) 1993-2011 Tim Riker
  *
  * This package is free software;  you can redistribute it and/or
  * modify it under the terms of the license found in the file
@@ -90,7 +90,7 @@ void ZSceneDatabase::finalizeStatics()
 }
 
 
-bool ZSceneDatabase::addStaticNode(SceneNode* object, bool /*dontFree*/)
+bool ZSceneDatabase::addStaticNode(SceneNode* object, bool UNUSED(dontFree))
 {
   if (staticCount == staticSize) {
     if (staticSize == 0) staticSize = 15;
